/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_1)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.dataobjects;

import java.awt.Dimension;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.geom.Area;
import java.io.File;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import javax.media.j3d.PolygonAttributes;
import javax.swing.JOptionPane;
import javax.vecmath.Matrix3f;

import org.cellmicrocosmos.cm2.constants.CM2_CONSTANTS;
import org.cellmicrocosmos.cm2.engine.DataPool;
import org.cellmicrocosmos.cm2.functionality3d.IntersectManager;
import org.cellmicrocosmos.cm2.functionality3d.Membrane3D;
import org.cellmicrocosmos.cm2.io.PdbReader_3D;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.Lipid;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.MembranePackingAlgorithm;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.MicroDomain;
import org.cellmicrocosmos.cm2.misc.Cm2_Toolkit;


/**
 * /**
 * This class represents the Membrane that all parts of the Application need to
 * work with. Contains all its Molecules seperated into Extracellular Lipids,
 * Intracellular Lipids and Proteins and provides different ways to access them.
 * Does also contain and provide all necessary Metadata of the Membrane
 * 
 * @author Tim Dingersen dackelnose@gmx.net, 
 * 	       Sebastian Rubert, Bjoern Sommer bjoern@CELLmicrocosmos.org
 *
 */
public class MembraneModel {

	/*
	 * indicates if the MembraneModel has been edited since last Fileoperation
	 */
	private boolean membraneHasBeenEditedAgain = true;
	
	// contains all Proteins in the Membrane
	private final Vector<Protein_3D> allProteins = new Vector<Protein_3D>();

	// contains all Lipids in the Membrane
	private final Vector<Lipid_3D> allLipidsExtra = new Vector<Lipid_3D>();

	private final Vector<Lipid_3D> allLipidsIntra = new Vector<Lipid_3D>();

	/*
	 * contains abstract Lipids with the percent of the lipid allocation
	 */

	private final Vector<Lipid_3D> exampleLipidsExtra = new Vector<Lipid_3D>();

	private final Vector<Lipid_3D> exampleLipidsIntra = new Vector<Lipid_3D>();

	public Vector<?>[] allVectors = { this.allProteins, this.allLipidsExtra,
			this.allLipidsIntra, this.exampleLipidsExtra,
			this.exampleLipidsIntra };
	
	private boolean sortByType = false; 
	
	private boolean computeAbsoluteValues = false;
	
	/*
	 * text for the title section
	 */
	private String header, Code, title, date, author, remark, source, keywds,
			expdta, jrnl;

	/*
	 * Size of the membrane in angstrom
	 */
	private Dimension size = new Dimension(
			CM2_CONSTANTS.MEMBRANE_SIZES.DEFAULT_SIZE_OF_MEMBRANE);

	/*
	 * Info about the algorithm used to calculate the bilayer with
	 */
	private String algorithmname, algorithmdescription,
			algorithmstringrepresentation;

	/*
	 * the file that the membrane was generated into last time
	 */
	private File lastGeneratedMembraneFile = null;

	private final Vector<MicroDomain_3D> microdomains = new Vector<MicroDomain_3D>();

	private int decimalplaces = CM2_CONSTANTS.MISC.LIPID_PERCENTAGE_DIVISOR;

	private final Vector<Layer> layers = new Vector<Layer>();

	/**
	 * Constructs a new Membrane Model. All stored atoms in the Datapool get
	 * deleted here because they are not needed anymore. The membrane gets
	 * initialized with the current date, all other fields are empty at startup
	 * 
	 */
	public MembraneModel() {

		this.setDate(Cm2_Toolkit.createCurrentDate());
		if (System.getProperty("user.name") != null)
			this.setAuthor("    " + System.getProperty("user.name"));

	}
	
	public Molecule_3D[] getAllMoleculesInMembrane(boolean interimSortByType) {
		return (interimSortByType?this.getAllMoleculesInMembraneSortedByType():this.getAllMoleculesInMembraneSortedBySide());
	}
	
	public Molecule_3D[] getAllMoleculesInMembrane() {
		if (this.sortByType())
			return this.getAllMoleculesInMembraneSortedByType();
		else
			return this.getAllMoleculesInMembraneSortedBySide();
	}

	public Molecule_3D[] getAllMoleculesInMembraneSortedBySide() {
		final Vector<Molecule_3D> ret = new Vector<Molecule_3D>();

		for (int i = 0; i < this.allProteins.size(); i++)
			ret.addElement(this.allProteins.get(i));

		for (int i = 0; i < this.allLipidsExtra.size(); i++)
			ret.addElement(this.allLipidsExtra.get(i));

		for (int i = 0; i < this.allLipidsIntra.size(); i++)
			ret.addElement(this.allLipidsIntra.get(i));
		return ret.toArray(new Molecule_3D[0]);
	}
	
	public Molecule_3D[] getAllMoleculesInMembraneSortedByType() {
		
		final Vector<Molecule_3D> ret = new Vector<Molecule_3D>();
		for (int i = 0; i < this.allProteins.size(); i++)
			ret.addElement(this.allProteins.get(i));
		
		String startPdbCodeLipidExtra = "";
		String startPdbCodeLipidIntra = "";
		
		if (this.allLipidsExtra.size()>0)
			startPdbCodeLipidExtra = this.allLipidsExtra.get(0).getPDBcode();
		
		if (this.allLipidsIntra.size()>0)
			startPdbCodeLipidIntra = this.allLipidsIntra.get(0).getPDBcode();
		
		if (startPdbCodeLipidExtra.isEmpty()) {
			for (int i = 0; i < this.allLipidsIntra.size(); i++)
				ret.addElement(this.allLipidsIntra.get(i));
			return ret.toArray(new Molecule_3D[0]);
		}
		
		if (startPdbCodeLipidIntra.isEmpty()) {
			for (int i = 0; i < this.allLipidsExtra.size(); i++)
				ret.addElement(this.allLipidsExtra.get(i));
			return ret.toArray(new Molecule_3D[0]);
		}
		
		String actualPdbCode = "";
		int currentLipidExtra 	= 0;
		int currentLipidIntra 	= 0;
		
		if (startPdbCodeLipidExtra.compareToIgnoreCase(startPdbCodeLipidIntra)>=0) {
			actualPdbCode = startPdbCodeLipidExtra;
			while (currentLipidExtra < this.allLipidsExtra.size() || currentLipidIntra < this.allLipidsIntra.size()) {
				for (; currentLipidExtra < this.allLipidsExtra.size(); currentLipidExtra++)
					if (this.allLipidsExtra.get(currentLipidExtra).getPDBcode().equalsIgnoreCase(actualPdbCode))
						ret.addElement(this.allLipidsExtra.get(currentLipidExtra));
					else
						break;
				
				for (; currentLipidIntra < this.allLipidsIntra.size(); currentLipidIntra++)
					if (this.allLipidsIntra.get(currentLipidIntra).getPDBcode().equalsIgnoreCase(actualPdbCode))
						ret.addElement(this.allLipidsIntra.get(currentLipidIntra));
					else
						break;
				
				if (currentLipidExtra < this.allLipidsExtra.size())
					actualPdbCode = this.allLipidsExtra.get(currentLipidExtra).getPDBcode();
				else if (currentLipidIntra < this.allLipidsIntra.size())
					actualPdbCode = this.allLipidsIntra.get(currentLipidIntra).getPDBcode();
				
			}
			
		} else {
			actualPdbCode = startPdbCodeLipidIntra;
			while (currentLipidExtra < this.allLipidsExtra.size() || currentLipidIntra < this.allLipidsIntra.size()) {
				for (; currentLipidIntra < this.allLipidsIntra.size(); currentLipidIntra++)
					if (this.allLipidsIntra.get(currentLipidIntra).getPDBcode().equalsIgnoreCase(actualPdbCode))
						ret.addElement(this.allLipidsIntra.get(currentLipidIntra));
					else
						break;

				for (; currentLipidExtra < this.allLipidsExtra.size(); currentLipidExtra++)
					if (this.allLipidsExtra.get(currentLipidExtra).getPDBcode().equalsIgnoreCase(actualPdbCode))
						ret.addElement(this.allLipidsExtra.get(currentLipidExtra));
					else
						break;
				
				actualPdbCode = this.allLipidsIntra.get(currentLipidIntra).getPDBcode();
				
				if (currentLipidIntra < this.allLipidsIntra.size())
					actualPdbCode = this.allLipidsIntra.get(currentLipidIntra).getPDBcode();
				else if (currentLipidExtra < this.allLipidsExtra.size())
					actualPdbCode = this.allLipidsExtra.get(currentLipidExtra).getPDBcode();
				
			}
		
		}
		return ret.toArray(new Molecule_3D[0]);
	}

	/**
	 * Indicates wether or not the current MembraneModel has been edited again
	 * since last load or save
	 * 
	 * @return
	 */
	public boolean getMembraneHasBeenEditedAgain() {
		return this.membraneHasBeenEditedAgain;
	}

	/**
	 * sets the state of membraneHasBeenEditedAgain
	 * 
	 * @param membraneHasBeenEditedAgain
	 */
	public void setMembraneHasBeenEditedAgain(
			final boolean membraneHasBeenEditedAgain) {
		this.membraneHasBeenEditedAgain = membraneHasBeenEditedAgain;
	}

	/**
	 * Returns the code string
	 * 
	 * @return
	 */
	public String getCode() {
		return this.Code;
	}

	/**
	 * Sets the code string
	 * 
	 * @param code
	 */
	public void setCode(final String code) {
		this.Code = code;
		if (this.Code != null)
			this.Code = this.Code.toUpperCase();
		this.membraneHasBeenEditedAgain = true;
	}

	/**
	 * Returns the header string
	 * 
	 * @return
	 */
	public String getHeader() {
		return this.header;
	}

	/**
	 * Sets the header string
	 * 
	 * @param header
	 */
	public void setHeader(final String header) {
		this.header = header;
		if (this.header != null)
			this.header = this.header.toUpperCase();
		this.membraneHasBeenEditedAgain = true;
	}

	/**
	 * returns the date String
	 * 
	 * @return
	 */
	public String getDate() {
		return this.date;
	}

	/**
	 * sets the date string
	 * 
	 * @param date
	 */
	public void setDate(final String date) {
		this.date = date;
		this.membraneHasBeenEditedAgain = true;
	}

	/**
	 * Returns the area of the MembraneModel.
	 * @return int
	 */
	public int getArea() {
		return this.getSize().width * this.getSize().height;
	}
	
	/**
	 * Returns the author string
	 * 
	 * @return
	 */
	public String getAuthor() {
		return this.author;
	}

	/**
	 * Sets the author string
	 * 
	 * @param author
	 */
	public void setAuthor(final String author) {
		this.author = author;
		if (this.author != null)
			this.author = this.author.toUpperCase();
		this.membraneHasBeenEditedAgain = true;
	}

	/**
	 * sets a list of species
	 * 
	 * @param species
	 */
	public void setSOURCE(final String species) {
		this.source = species;
		if (this.source != null)
			this.source = this.source.toUpperCase();
		this.membraneHasBeenEditedAgain = true;
	}

	/**
	 * returns a list of entered species
	 * 
	 * @return
	 */
	public String getSOURCE() {
		return this.source;
	}

	/**
	 * Returns the title String
	 * 
	 * @return
	 */
	public String getTitle() {
		return this.title;
	}

	/**
	 * Sets the title String
	 * 
	 * @param title
	 */
	public void setTitle(final String title) {
		this.title = title;
		if (this.title != null)
			this.title = this.title.toUpperCase();
		this.membraneHasBeenEditedAgain = true;
	}

	/**
	 * returns the comment String
	 * 
	 * @return
	 */
	public String getRemark() {
		return this.remark;
	}

	/**
	 * sets the comment string
	 * 
	 * @param comment
	 */
	public void setRemark(final String comment) {
		this.remark = comment;
		if (this.remark != null)
			this.remark = this.remark.toUpperCase();
		this.membraneHasBeenEditedAgain = true;
	}

	public String getKEYWDS() {

		return this.keywds;
	}

	public void setKeywds(final String keywds) {
		this.keywds = keywds;
		if (this.keywds != null)
			this.keywds = this.keywds.toUpperCase();
		this.membraneHasBeenEditedAgain = true;
	}

	public String getEXPDTA() {

		return this.expdta;
	}

	public void setExpdta(final String expdta) {
		this.expdta = expdta;
		if (this.expdta != null)
			this.expdta = this.expdta.toUpperCase();
		this.membraneHasBeenEditedAgain = true;
	}

	public String getJRNL() {

		return this.jrnl;
	}

	public void setJrnl(final String jrnl) {
		this.jrnl = jrnl;
		if (this.jrnl != null)
			this.jrnl = this.jrnl.toUpperCase();
		this.membraneHasBeenEditedAgain = true;
	}

	/*
	 * Those 5 getters return the entire list of a molecule group
	 */
	public Vector<Protein_3D> getAllProteins() {
		return this.allProteins;
	}

	public Vector<Lipid_3D> getAllLipidsExtra() {
		return this.allLipidsExtra;
	}

	public Vector<Lipid_3D> getAllLipidsIntra() {
		return this.allLipidsIntra;
	}

	public Vector<Lipid_3D> getAllExampleLipidsExtra() {
		return this.exampleLipidsExtra;
	}

	public Vector<Lipid_3D> getAllExampleLipidsIntra() {
		return this.exampleLipidsIntra;
	}

	/**
	 * adds a single Protein into the Model
	 * 
	 * @param pro
	 * @param layerID
	 */
	public void addProtein(final Protein_3D pro, final int layerID) {
		if (pro.getPDBcode() == null) {
			pro.setPDBcode(Cm2_Toolkit.getSimpleName(pro.getPDBFile())
					.toUpperCase());
		}
		pro.setLayerID(layerID);
		this.allProteins.add(pro);
		this.membraneHasBeenEditedAgain = true;

	}

	/**
	 * gets the protein at the specifed index
	 * 
	 * @param index
	 * @return
	 */
	public Protein_3D getProteinAt(final int index) {
		return this.allProteins.get(index);
	}

	/**
	 * deletes the passed Protein
	 * 
	 * @param molecule_3D
	 */
	public void deleteProtein(final Protein_3D molecule_3D) {
		this.allProteins.remove(molecule_3D);
		this.allProteins.trimToSize();
		this.membraneHasBeenEditedAgain = true;

	}

	/**
	 * deletes the Protein at specified index
	 * 
	 * @param index
	 */
	public void deleteProteinAt(final int index) {
		this.allProteins.remove(index);
		this.allProteins.trimToSize();
		this.membraneHasBeenEditedAgain = true;

	}

	/**
	 * returns the current number of Proteins in the model
	 * 
	 * @return
	 */
	public int getProteinCount() {
		return this.allProteins.size();
	}

	/**
	 * adds a Lipid to the Model
	 * 
	 * @param lip
	 * @param domainID
	 */
	public void addLipidExtra(final Lipid_3D lip, final int domainID,
			final int layerID) {
		if (lip.getPDBcode() == null) {
			lip.setPDBcode(Cm2_Toolkit.getSimpleName(lip.getPDBFile())
					.toUpperCase());
		}
		lip.setDomainID(domainID);
		lip.setLayerID(layerID);
		lip.setMembraneSide(Membrane3D.EXTRA);
		this.allLipidsExtra.addElement(lip);
		this.membraneHasBeenEditedAgain = true;

	}

	/**
	 * adds a Lipid to the Model
	 * 
	 * @param lip
	 */
	public void addLipidIntra(final Lipid_3D lip, final int domainID,
			final int layerID) {
		if (lip.getPDBcode() == null) {
			lip.setPDBcode(Cm2_Toolkit.getSimpleName(lip.getPDBFile())
					.toUpperCase());
		}
		lip.setDomainID(domainID);
		lip.setLayerID(layerID);
		lip.setMembraneSide(Membrane3D.INTRA);
		this.allLipidsIntra.addElement(lip);
		this.membraneHasBeenEditedAgain = true;

	}

	/**
	 * returns the lipid at the specified index
	 * 
	 * @param index
	 * @return
	 */
	public Lipid_3D getLipidAtExtra(final int index) {
		return this.allLipidsExtra.get(index);
	}

	/**
	 * returns the lipid at the specified index
	 * 
	 * @param index
	 * @return
	 */
	public Lipid_3D getLipidAtIntra(final int index) {
		return this.allLipidsIntra.get(index);
	}

	/**
	 * deletes the specified lipid
	 * 
	 * @param todelete
	 */
	public void deleteLipidExtra(final Lipid_3D todelete) {
		this.allLipidsExtra.remove(todelete);
		this.allLipidsExtra.trimToSize();
		this.membraneHasBeenEditedAgain = true;

	}

	/**
	 * deletes the specified lipid
	 * 
	 * @param todelete
	 */
	public void deleteLipidIntra(final Lipid_3D todelete) {
		this.allLipidsIntra.remove(todelete);
		this.allLipidsIntra.trimToSize();
		this.membraneHasBeenEditedAgain = true;

	}

	/**
	 * deletes the Lipid that has the given index
	 * 
	 * @param index
	 */
	public void deleteLipidAtExtra(final int index) {
		this.allLipidsExtra.remove(index);
		this.allLipidsExtra.trimToSize();
		this.membraneHasBeenEditedAgain = true;

	}

	public void deleteExampleLipidExtra(final Lipid_3D lip) {
		this.exampleLipidsExtra.remove(lip);
		this.exampleLipidsExtra.trimToSize();
		this.membraneHasBeenEditedAgain = true;

	}

	/**
	 * deletes the Lipid that has the given index
	 * 
	 * @param index
	 */
	public void deleteLipidAtIntra(final int index) {
		this.allLipidsIntra.remove(index);
		this.allLipidsIntra.trimToSize();
		this.membraneHasBeenEditedAgain = true;

	}

	public void deleteExampleLipidIntra(final Lipid_3D lip) {
		this.exampleLipidsIntra.remove(lip);
		this.exampleLipidsIntra.trimToSize();
		this.membraneHasBeenEditedAgain = true;

	}

	/**
	 * deletes the passed Molecule of any kind
	 * 
	 * @param molecule_3D
	 */
	public void deleteMolecule(final Molecule_3D mol) {

		for (int i = 0; i < this.allVectors.length; i++) {
			if (this.allVectors[i].contains(mol)) {
				this.allVectors[i].remove(mol);
				this.allVectors[i].trimToSize();
				this.membraneHasBeenEditedAgain = true;

				return;
			}
		}

	}

	/**
	 * returns the current number of lipids in the model
	 * 
	 * @return
	 */
	public int getLipidCountExtra() {
		return this.allLipidsExtra.size();
	}

	/**
	 * returns the current number of lipids in the model
	 * 
	 * @return
	 */
	public int getLipidCountIntra() {
		return this.allLipidsIntra.size();
	}

	/**
	 * returns all Protein individues which are of the specified type. the type
	 * is specified by its pdb code (because all individues of a type will have
	 * the same)
	 * 
	 * @param PDBCode
	 *            the pdb code of all proteins that should be returned
	 * @return an array of proteins
	 */
	public Protein_3D[] getProteinsByPDBCode(final String PDBCode) {
		final Vector<Protein_3D> vec = new Vector<Protein_3D>();
		for (int i = 0; i < this.allProteins.size(); i++) {
			if (this.allProteins.get(i).getPDBcode().equals(PDBCode))
				vec.add(this.allProteins.get(i));
		}

		return vec.toArray(new Protein_3D[0]);

	}

	/**
	 * returns all Lipid individues which are of the specified type. the type is
	 * specified by its pdb code (because all individues of a type will have the
	 * same)
	 * 
	 * @param PDBCode
	 *            the pdb code of all lipids that should be returned
	 * @return an array of lipids
	 */
	public Lipid_3D[] getLipidsByPDBCodeExtra(final String PDBCode) {
		final Vector<Lipid_3D> vec = new Vector<Lipid_3D>();
		for (int i = 0; i < this.allLipidsExtra.size(); i++) {
			if (this.allLipidsExtra.get(i).getPDBcode().equals(PDBCode))
				vec.add(this.allLipidsExtra.get(i));
		}

		return vec.toArray(new Lipid_3D[0]);

	}

	/**
	 * returns all Lipid individues which are of the specified type. the type is
	 * specified by its pdb code (because all individues of a type will have the
	 * same)
	 * 
	 * @param PDBCode
	 *            the pdb code of all lipids that should be returned
	 * @return an array of lipids
	 */
	public Lipid_3D[] getLipidsByPDBCodeIntra(final String PDBCode) {
		final Vector<Lipid_3D> vec = new Vector<Lipid_3D>();
		for (int i = 0; i < this.allLipidsIntra.size(); i++) {
			if (this.allLipidsIntra.get(i).getPDBcode().equals(PDBCode))
				vec.add(this.allLipidsIntra.get(i));
		}

		return vec.toArray(new Lipid_3D[0]);

	}

	/**
	 * returns all Sample Lipid individues which are of the specified type. the
	 * type is specified by its pdb code (because all individues of a type will
	 * have the same)
	 * 
	 * @param PDBCode
	 *            the pdb code of all lipids that should be returned
	 * @return an array of lipids
	 */
	public Lipid_3D[] getExampleLipidsByPDBCodeIntra(final String PDBCode) {
		final Vector<Lipid_3D> vec = new Vector<Lipid_3D>();
		for (int i = 0; i < this.exampleLipidsIntra.size(); i++) {
			if (this.exampleLipidsIntra.get(i).getPDBcode().equals(PDBCode))
				vec.add(this.exampleLipidsIntra.get(i));
		}

		return vec.toArray(new Lipid_3D[0]);

	}

	/**
	 * returns all Sample Lipid individues which are of the specified type. the
	 * type is specified by its pdb code (because all individues of a type will
	 * have the same)
	 * 
	 * @param PDBCode
	 *            the pdb code of all lipids that should be returned
	 * @return an array of lipids
	 */
	public Lipid_3D[] getExampleLipidsByPDBCodeExtra(final String PDBCode) {
		final Vector<Lipid_3D> vec = new Vector<Lipid_3D>();
		for (int i = 0; i < this.exampleLipidsExtra.size(); i++) {
			if (this.exampleLipidsExtra.get(i).getPDBcode().equals(PDBCode))
				vec.add(this.exampleLipidsExtra.get(i));
		}

		return vec.toArray(new Lipid_3D[0]);

	}

	/**
	 * returns a list of all types of proteins that occur in the membrane at
	 * least once
	 * 
	 * @return a string array that contains the pdb-codes,
	 */
	public String[] getAllProteinTypes() {

		final Vector<String> vec = new Vector<String>();
		for (int i = 0; i < this.allProteins.size(); i++) {
			if (vec.contains(this.allProteins.get(i).getPDBcode()) == false)
				vec.add(this.allProteins.get(i).getPDBcode());
		}

		final String[] retval = vec.toArray(new String[0]);

		Arrays.sort(retval);

		return retval;
	}

	/**
	 * returns a list of all types of sample lipids that occur in the membrane
	 * at least once
	 * 
	 * @return a string array that contains the pdb-codes, null if no lipids are
	 *         available
	 */
	public String[] getAllExampleLipidTypesExtra() {
		final Vector<String> vec = new Vector<String>();
		for (int i = 0; i < this.exampleLipidsExtra.size(); i++) {
			if (vec.contains(this.exampleLipidsExtra.get(i).getPDBcode()) == false)
				vec.add(this.exampleLipidsExtra.get(i).getPDBcode());
		}

		final String[] retval = vec.toArray(new String[0]);

		Arrays.sort(retval);

		return retval;

	}

	/**
	 * returns a list of all types of sample lipids that occur in the membrane
	 * at least once
	 * 
	 * @return a string array that contains the pdb-codes, null if no lipids are
	 *         available
	 */
	public String[] getAllExampleLipidTypesIntra() {
		final Vector<String> vec = new Vector<String>();
		for (int i = 0; i < this.exampleLipidsIntra.size(); i++) {
			if (vec.contains(this.exampleLipidsIntra.get(i).getPDBcode()) == false)
				vec.add(this.exampleLipidsIntra.get(i).getPDBcode());
		}

		final String[] retval = vec.toArray(new String[0]);

		Arrays.sort(retval);

		return retval;

	}

	/**
	 * returns a list of all types of proteins that occur in the extracellular membrane at
	 * least once
	 * 
	 * @return a string array that contains the pdb-codes,
	 */
	public String[] getAllLipidTypesInMembraneExtra() {
		final Vector<String> vec = new Vector<String>();
		for (int i = 0; i < this.allLipidsExtra.size(); i++) {
			if (vec.contains(this.allLipidsExtra.get(i).getPDBcode()) == false)
				vec.add(this.allLipidsExtra.get(i).getPDBcode());
		}

		final String[] retval = vec.toArray(new String[0]);

		Arrays.sort(retval);

		return retval;

	}

	/**
	 * returns a list of all types of lipids that occur in the intracellular membrane at least
	 * once
	 * 
	 * @return a string array that contains the pdb-codes, null if no lipids are
	 *         available
	 */
	public String[] getAllLipidTypesInMembraneIntra() {
		final Vector<String> vec = new Vector<String>();
		for (int i = 0; i < this.allLipidsIntra.size(); i++) {
			if (vec.contains(this.allLipidsIntra.get(i).getPDBcode()) == false)
				vec.add(this.allLipidsIntra.get(i).getPDBcode());
		}

		final String[] retval = vec.toArray(new String[0]);

		Arrays.sort(retval);

		return retval;

	}
	
	/**
	 * returns a list of all types of lipids that occur in the membrane at least
	 * once
	 * 
	 * @return a string array that contains the pdb-codes, null if no lipids are
	 *         available
	 */
	public String[] getAllLipidTypes() {
		final Vector<String> vec = new Vector<String>();
		for (int i = 0; i < this.allLipidsIntra.size(); i++) {
			if (vec.contains(this.allLipidsIntra.get(i).getPDBcode()) == false)
				vec.add(this.allLipidsIntra.get(i).getPDBcode());
		}
		for (int i = 0; i < this.allLipidsExtra.size(); i++) {
			if (vec.contains(this.allLipidsExtra.get(i).getPDBcode()) == false)
				vec.add(this.allLipidsExtra.get(i).getPDBcode());
		}

		final String[] retval = vec.toArray(new String[0]);

		Arrays.sort(retval);

		return retval;

	}

	/**
	 * Tests if a Protein with the passed PdbCode is already inside the Membrane
	 */
	public boolean getProteinTypeAlreadyInside(final String PDBCode) {

		for (int i = 0; i < this.allProteins.size(); i++) {
			if (this.allProteins.get(i).getPDBcode().equalsIgnoreCase(PDBCode)) {

				return true;
			}
		}
		return false;

	}

	/**
	 * Tests if a Lipid with the passed PdbCode is already inside the Membrane
	 */
	public boolean getLipidTypeAlreadyInside(final String PDBCode) {

		for (int i = 0; i < this.allLipidsExtra.size(); i++) {
			if (this.allLipidsExtra.get(i).getPDBcode().equalsIgnoreCase(
					PDBCode))
				return true;
		}
		for (int i = 0; i < this.allLipidsIntra.size(); i++) {
			if (this.allLipidsIntra.get(i).getPDBcode().equalsIgnoreCase(
					PDBCode))
				return true;
		}
		return false;

	}

	/**
	 * Tests if a ExampleLipid with the passed PdbCode is already added
	 */
	public boolean getExampleLipidTypeAlreadyInside(final String PDBCode) {

		for (int i = 0; i < this.exampleLipidsExtra.size(); i++) {
			if (this.exampleLipidsExtra.get(i).getPDBcode().equalsIgnoreCase(
					PDBCode))
				return true;
		}
		for (int i = 0; i < this.exampleLipidsIntra.size(); i++) {
			if (this.exampleLipidsIntra.get(i).getPDBcode().equalsIgnoreCase(
					PDBCode))
				return true;
		}
		return false;

	}

	/**
	 * Returns the size of the membrane
	 * 
	 * @return
	 */
	public Dimension getSize() {
		return this.size;
	}

	/**
	 * Sets the size of the membrane
	 * 
	 * @param size
	 */
	public void setSize(final Dimension size) {
		this.size = size;
	}

	/**
	 * Returns the total molecule count. If argument is 0, then the proteincount
	 * is returned else, the count of both intra and extracellular lipids is
	 * returned
	 * 
	 * @param group
	 * @return
	 */
	public int getTotalMoleculeCount(final int group) {
		if (group == 0)
			return this.getProteinCount();
		else if (group == 1) {
			return this.getLipidCountExtra() + this.getLipidCountIntra();
		} else
			return 0;
	}

	/**
	 * Removes all lipids from the membrane model (not the sample lipids) Also
	 * clears the info about the last algorithm used
	 * 
	 */
	public void clearLipids() {
		this.allLipidsExtra.clear();
		this.allLipidsIntra.clear();
		this.membraneHasBeenEditedAgain = true;

		this.algorithmdescription = null;
		this.algorithmname = null;
		this.algorithmstringrepresentation = null;
	}

	public void clearSampleLipids() {
		this.exampleLipidsExtra.clear();
		this.exampleLipidsIntra.clear();
		this.membraneHasBeenEditedAgain = true;

	}

	/**
	 * Removes all Proteins from the model
	 * 
	 */
	public void clearProteins() {
		this.allProteins.clear();
		this.membraneHasBeenEditedAgain = true;

	}

	/**
	 * Removes all sample lipids from the model
	 * 
	 */
	public void clearExampleLipids() {
		this.exampleLipidsExtra.clear();
		this.exampleLipidsIntra.clear();
		this.membraneHasBeenEditedAgain = true;

	}

	/**
	 * Adds an sample lipid to the membrane
	 * 
	 * @param lip
	 * @param domainID
	 */
	public void addExampleLipidExtra(final Lipid_3D actLip, final int domainID,
			final int layerID) {
		boolean insertion = false;
		if (actLip.getPDBcode() == null) {
			actLip.setPDBcode(Cm2_Toolkit.getSimpleName(actLip.getPDBFile())
					.toUpperCase());
		}
		actLip.setDomainID(domainID);
		actLip.setLayerID(layerID);
		actLip.setMembraneSide(Membrane3D.EXTRA);
		// sorting of the added lipids in lexicographic order
		for (Lipid_3D exaLip : this.exampleLipidsExtra) {
			// the value of compareTo() becomes positiv, if extLip is lexicographically
			// greater then actLip
			if (exaLip.getPDBcode().compareTo(actLip.getPDBcode()) > 0) {
				this.exampleLipidsExtra.add(this.exampleLipidsExtra.indexOf(exaLip), actLip);
				// when an insertion is done the boolean parameter will be set 'true'
				insertion = true;
				break;
			}
		}
		// if there was no insertion before, the Vector of ExampleLipids is empty or the
		// actLip has to be put at the end		
		if (insertion == false) {
			this.exampleLipidsExtra.add(actLip);
		}
		this.membraneHasBeenEditedAgain = true;
		actLip.move();
	}

	/**
	 * Adds an sample lipid to the membrane
	 * 
	 * @param lip
	 * @param domainID
	 */
	public void addExampleLipidIntra(final Lipid_3D actLip, final int domainID,
			final int layerID) {
		if (actLip.getPDBcode() == null) {
			actLip.setPDBcode(Cm2_Toolkit.getSimpleName(actLip.getPDBFile())
					.toUpperCase());
		}
		boolean insertion = false;
		actLip.setDomainID(domainID);
		actLip.setLayerID(layerID);
		actLip.setMembraneSide(Membrane3D.INTRA);
		// sorting of the added lipids in lexicographic order
		for (Lipid_3D exaLip : this.exampleLipidsIntra) {
			// the value of compareTo() becomes positiv, if extLip is lexicographically
			// greater then actLip
			if (exaLip.getPDBcode().compareTo(actLip.getPDBcode()) > 0) {
				this.exampleLipidsIntra.add(this.exampleLipidsIntra.indexOf(exaLip), actLip);
				// when an insertion is done the boolean parameter will be set 'true'
				insertion = true;
				break;
			}
		}
		// if there was no insertion before, the Vector of ExampleLipids is empty or the
		// actLip has to be put at the end
		if (insertion == false) {
			this.exampleLipidsIntra.add(actLip);
		}
		this.membraneHasBeenEditedAgain = true;
		actLip.move();
	}

	/**
	 * Returns the sample lipid at the specified index
	 * 
	 * @param index
	 * @return
	 */
	public Lipid_3D getExampleLipidAtExtra(final int index) {
		if (this.exampleLipidsExtra.size() == 0)
			return null;
		return this.exampleLipidsExtra.get(index);
	}

	/**
	 * Returns the sample lipid at the specified index
	 * 
	 * @param index
	 * @return
	 */
	public Lipid_3D getExampleLipidAtIntra(final int index) {
		if (this.exampleLipidsIntra.size() == 0)
			return null;
		return this.exampleLipidsIntra.get(index);
	}

	/**
	 * Returns the count of example lipids
	 * 
	 * @return
	 */
	public int getExampleLipidsCountExtra() {
		return this.exampleLipidsExtra.size();
	}

	/**
	 * Returns the count of example lipids
	 * 
	 * @return
	 */
	public int getExampleLipidsCountIntra() {
		return this.exampleLipidsIntra.size();
	}

	/**
	 * Removes the example lipid at specified index
	 * 
	 * @return
	 */
	public void deleteExampleLipidAtExtra(final int index) {
		this.exampleLipidsExtra.remove(index);
		this.membraneHasBeenEditedAgain = true;

	}

	/**
	 * Removes the example lipid at specified index
	 * 
	 * @return
	 */
	public void deleteExampleLipidAtIntra(final int index) {
		this.exampleLipidsIntra.remove(index);
		this.membraneHasBeenEditedAgain = true;

	}

	/**
	 * Returns the first Molecule found that has the passed file in the
	 * specified group of molecules
	 * 
	 * @param file
	 * @param group
	 * @param modelnumber
	 * @return
	 */
	public Molecule_3D getFirstMoleculeThatHasFile(final File file, int group,
			final int modelnumber) {

		if (group > 0)
			group = 1;

		switch (group) {

		case 0:
			for (int i = this.getProteinCount() - 1; i > -1; i--) {
				if (this.getProteinAt(i).getPDBFile().equals(file)
						&& this.getProteinAt(i).getModelSet() == modelnumber) {

					return this.getProteinAt(i);

				}
			}
			break;

		case 1:

			for (int i = this.getExampleLipidsCountExtra() - 1; i > -1; i--) {
				if (this.getExampleLipidAtExtra(i).getPDBFile().equals(file)
						&& this.getExampleLipidAtExtra(i).getModelSet() == modelnumber) {
					return this.getExampleLipidAtExtra(i);
				}
			}
			for (int i = this.getExampleLipidsCountIntra() - 1; i > -1; i--) {
				if (this.getExampleLipidAtIntra(i).getPDBFile().equals(file)
						&& this.getExampleLipidAtIntra(i).getModelSet() == modelnumber) {
					return this.getExampleLipidAtIntra(i);
				}
			}
			for (int i = this.getLipidCountExtra() - 1; i > -1; i--) {
				if (this.getLipidAtExtra(i).getPDBFile().equals(file)
						&& this.getLipidAtExtra(i).getModelSet() == modelnumber) {
					return this.getLipidAtExtra(i);
				}
			}
			for (int i = this.getLipidCountIntra() - 1; i > -1; i--) {
				if (this.getLipidAtIntra(i).getPDBFile().equals(file)
						&& this.getLipidAtIntra(i).getModelSet() == modelnumber) {
					return this.getLipidAtIntra(i);
				}
			}

			break;

		}

		return null;

	}

	/**
	 * Returns the lipid with the higest percentage in the specified group
	 * 
	 * @param mode
	 *            0 extra, 1 intra
	 * @return
	 */
	public Lipid_3D getHighestExampleLipid(final int mode) {
		Vector<Lipid_3D> tmpVec;
		Lipid_3D tmp = null;
		int percent = 0;

		if (mode == 0) {
			tmpVec = this.getAllLipidsExtra();
		} else
			tmpVec = this.getAllLipidsIntra();

		for (int i = 0; i < tmpVec.size(); i++) {
			if (tmpVec.get(i).getPercent() > percent) {
				percent = tmpVec.get(i).getPercent();
				tmp = tmpVec.get(i);
			}
		}

		return tmp;
	}

	/**
	 * Returns the maximal FileSize of the current Membrane
	 * 
	 * @return
	 */
	public long calculateTotalFileSize(final int domainID, final int layerID) {
		long size = 0;

		Molecule_3D[] mols = this.getAllProteinsInDomain(domainID, layerID);
		for (int i = 0; i < mols.length; i++) {
			size += mols[i].getPDBFile().length();
		}
		mols = this.getAllLipidsExtraInDomain(domainID, layerID);
		for (int i = 0; i < mols.length; i++) {
			size += mols[i].getPDBFile().length();
		}
		mols = this.getAllLipidsIntraInDomain(domainID, layerID);
		for (int i = 0; i < mols.length; i++) {
			size += mols[i].getPDBFile().length();
		}

		return size;

	}

	/**
	 * Tests, if the passed protein is currently in the model
	 * 
	 * @param mol
	 * @return
	 */
	public boolean isMoleculeStillIn(final Molecule_3D mol) {
		return this.getAllProteins().contains(mol)
				|| this.getAllExampleLipidsExtra().contains(mol)
				|| this.getAllExampleLipidsIntra().contains(mol);
	}

	/**
	 * Test, wheter the current percentage settings of the lipids on both sides
	 * have an overall of 100%
	 * 
	 * @return
	 */
	public boolean areLipidPercentagesOk() {

		// for(int a=-1;a<this.getLayerCount();a++) {
		// int layerid;
		// if(a==-1)layerid=0;
		// else {
		// try{layerid=this.getLayer(a).getLayerId();
		// }catch(Exception e) {
		// continue;
		// }
		// }

		for (int m = 0; m < this.countMicroDomains() + 1; m++) {
			final Lipid_3D[] lipsEC = this.getAllExampleLipidsExtraInDomain(m,
					DataPool.getCurrentLayerID());

			int count = 0;
			if (lipsEC.length < 1)
				count = 100;
			for (int i = 0; i < lipsEC.length; i++) {
				count += lipsEC[i].getPercent();
			}

			final MembraneModel model = DataPool.getMembraneModel();
			final int divisor = (int) Math.pow(10, model.getDecimalPlaces());
			if (lipsEC.length > 0 && count != 100 * divisor)
				return false;

			final Lipid_3D[] lipsIC = this.getAllExampleLipidsIntraInDomain(m,
					DataPool.getCurrentLayerID());
			count = 0;
			if (lipsIC.length < 1)
				count = 100 * divisor;
			for (int i = 0; i < lipsIC.length; i++) {
				count += lipsIC[i].getPercent();
			}
			if (lipsIC.length > 0 && count != 100 * divisor)
				return false;

		}
		// }
		return true;
	}
	
	/**
	 * Test, wheter the current absolute value settings have at least one
	 * lipid per side.
	 * 
	 * @return
	 */
	public boolean areLipidAbsoluteValuesOk() {

		return true;

//		for (int m = 0; m < this.countMicroDomains() + 1; m++) {
//			final Lipid_3D[] lipsEC = this.getAllExampleLipidsExtraInDomain(m,
//					DataPool.getCurrentLayerID());
//
//			int count = 0;
//			if (lipsEC.length < 1)
//				count = 100;
//			for (int i = 0; i < lipsEC.length; i++) {
//				count += lipsEC[i].getPercent();
//			}
//
//			final MembraneModel model = DataPool.getMembraneModel();
//			final int divisor = (int) Math.pow(10, model.getDecimalPlaces());
//			if (lipsEC.length > 0 && count != 100 * divisor)
//				return false;
//
//			final Lipid_3D[] lipsIC = this.getAllExampleLipidsIntraInDomain(m,
//					DataPool.getCurrentLayerID());
//			count = 0;
//			if (lipsIC.length < 1)
//				count = 100 * divisor;
//			for (int i = 0; i < lipsIC.length; i++) {
//				count += lipsIC[i].getPercent();
//			}
//			if (lipsIC.length > 0 && count != 100 * divisor)
//				return false;
//
//		}
//		// }
//		return true;
	}

	/**
	 * Tests, wheter a Protein with the passed ID is currently present in the
	 * model
	 * 
	 * @param id
	 * @return
	 */
	public boolean hasProteinWithId(final String id) {
		final int length = this.getProteinCount();

		for (int i = 0; i < length; i++) {

			if (this.getProteinAt(i).getID().equals(id)) {

				return true;
			}

		}
		return false;
	}

	/**
	 * Changes the shininess for all Molecules during 3d runtime
	 * 
	 * @param value
	 * @param mode
	 *            0 PROTEINS, 1 LIPIDS
	 */
	public void setShininess(final int value, final int mode) {
		if (mode == 0) {
			for (int i = 0; i < this.allProteins.size(); i++) {
				this.allProteins.get(i).getShape().getAppearance()
						.getMaterial().setShininess(value);
			}
		}
		if (mode == 1) {
			for (int i = 0; i < this.allLipidsExtra.size(); i++) {
				this.allLipidsExtra.get(i).getShape().getAppearance()
						.getMaterial().setShininess(value);
			}
			for (int i = 0; i < this.allLipidsIntra.size(); i++) {
				this.allLipidsIntra.get(i).getShape().getAppearance()
						.getMaterial().setShininess(value);
			}
			for (int i = 0; i < this.exampleLipidsExtra.size(); i++) {
				this.exampleLipidsExtra.get(i).getShape().getAppearance()
						.getMaterial().setShininess(value);
			}
			for (int i = 0; i < this.exampleLipidsIntra.size(); i++) {
				this.exampleLipidsIntra.get(i).getShape().getAppearance()
						.getMaterial().setShininess(value);
			}
		}

	}

	/**
	 * Changes the PolygonMode for all Molecules during 3d runtime
	 * 
	 * @param polygon_mode
	 */
	public void setPolygonMode(final int polygon_mode) {

		for (int i = 0; i < this.allProteins.size(); i++) {
			if (this.allProteins.get(i).isPicked() == true
					&& polygon_mode == PolygonAttributes.POLYGON_LINE)
				this.allProteins.get(i).getShape().getAppearance()
						.getPolygonAttributes().setPolygonMode(
								PolygonAttributes.POLYGON_FILL);
			else
				this.allProteins.get(i).getShape().getAppearance()
						.getPolygonAttributes().setPolygonMode(polygon_mode);
		}

		for (int i = 0; i < this.allLipidsExtra.size(); i++) {
			this.allLipidsExtra.get(i).getShape().getAppearance()
					.getPolygonAttributes().setPolygonMode(polygon_mode);
		}
		for (int i = 0; i < this.allLipidsIntra.size(); i++) {
			this.allLipidsIntra.get(i).getShape().getAppearance()
					.getPolygonAttributes().setPolygonMode(polygon_mode);
		}
		for (int i = 0; i < this.exampleLipidsExtra.size(); i++) {
			this.exampleLipidsExtra.get(i).getShape().getAppearance()
					.getPolygonAttributes().setPolygonMode(polygon_mode);
		}
		for (int i = 0; i < this.exampleLipidsIntra.size(); i++) {
			this.exampleLipidsIntra.get(i).getShape().getAppearance()
					.getPolygonAttributes().setPolygonMode(polygon_mode);
		}

	}

	/**
	 * Creates and adds a new sample Lipid with the passed parameters
	 * 
	 * @param code
	 * @param file
	 * @param domainID
	 * @param membraneSide
	 * @return
	 */
	public boolean createAndAddANewExampleLipid(final String code,
			final String file, final int modelnumber, final int domainID,
			final int membraneSide, final int layerID) {

		Lipid_3D newone;
		try {
			final PdbReader_3D reader = new PdbReader_3D();
			newone = reader.readLipid(new File(file), modelnumber, null);
			if (newone.getPDBcode() == null) {
				newone.setPDBcode(Cm2_Toolkit
						.getSimpleName(newone.getPDBFile()).toUpperCase());
			}
			newone.setPDBcode(code);
			final boolean ischolesterol = newone.getPDBcode().toUpperCase()
					.contains("CHOL")
					|| newone.getHeader().toUpperCase().contains("CHOL");
			newone.setColor(ischolesterol ? Cm2_Toolkit
					.createRandomCholesterolColor() : Cm2_Toolkit
					.getRandomLipidColor());
			newone.setRotationMatrix(PdbReader_3D.autorotateUpwards(newone));

		} catch (final Exception e) {
			JOptionPane.showMessageDialog(null,
					"An Error occured during reading!\nReason:\n\n"
							+ e.getMessage(), "ERROR",
					JOptionPane.ERROR_MESSAGE);
			return false;
		}

		newone.setPercent(1);

		if (membraneSide == -1 || membraneSide == 0) {

			this.addExampleLipidExtra(newone, domainID, layerID);
		}
		if (membraneSide == -1 || membraneSide == 1) {

			final Lipid_3D lip = (Lipid_3D) newone.copy(true, false);
			final Matrix3f newmatrix = lip.getRotationMatrix();
			Cm2_Toolkit.rotate_Z_Axis_Of_CoordinateSystem(newmatrix, 180);
			lip.setRotationMatrix(newmatrix);

			this.addExampleLipidIntra(lip, domainID, layerID);

		}
		return true;

	}

	/**
	 * Counts the atoms for the specified group of molecules. If the passed
	 * number is not 0,1,2, then the entire atom count of molecules in the
	 * membrane is returned
	 * 
	 * @param group
	 * @return
	 */
	public int countAtoms(final int group, final int domainID, int layerID,
			boolean withOPMAtoms) {
		int c = 0;
		if (domainID > 0)
			layerID = -1;
		if (group == 0 || group < 0 || group > 2) {

			for (int i = 0; i < this.allProteins.size(); i++) {
				final Protein_3D p = this.allProteins.get(i);
				if ((domainID < 0 ? true : p.getDomainID() == domainID)
						&& (layerID < 0 ? true : p.getLayerID() == layerID))
					c += (this.allProteins.get(i).getAtomCount() + (withOPMAtoms ? this.allProteins
							.get(i).opmAtoms.length
							: 0));
			}

		}

		if (group == 1 || group < 0 || group > 2) {

			for (int i = 0; i < this.allLipidsExtra.size(); i++) {
				final Lipid_3D p = this.allLipidsExtra.get(i);
				if ((domainID < 0 ? true : p.getDomainID() == domainID)
						&& (layerID < 0 ? true : p.getLayerID() == layerID))
					c += (this.allLipidsExtra.get(i).getAtomCount() + (withOPMAtoms ? this.allLipidsExtra
							.get(i).opmAtoms.length
							: 0));
			}

		}
		if (group == 2 || group < 0 || group > 2) {

			for (int i = 0; i < this.allLipidsIntra.size(); i++) {
				final Lipid_3D p = this.allLipidsIntra.get(i);
				if ((domainID < 0 ? true : p.getDomainID() == domainID)
						&& (layerID < 0 ? true : p.getLayerID() == layerID))
					c += (this.allLipidsIntra.get(i).getAtomCount() + (withOPMAtoms ? this.allLipidsIntra
							.get(i).opmAtoms.length
							: 0));
			}

		}
		return c;
	}

	/**
	 * Counts the chains for the specified group of molecules. If the passed
	 * number is not 0,1,2, then the entire chain count of molecules in the
	 * membrane is returned
	 * 
	 * @param group
	 * @return
	 */
	public int countChains(final int group, final int domainID, int layerID) {
		int c = 0;
		if (domainID > 0)
			layerID = -1;
		if (group == 0 || group < 0 || group > 2) {

			for (int i = 0; i < this.allProteins.size(); i++) {
				final Protein_3D p = this.allProteins.get(i);
				if ((domainID < 0 ? true : p.getDomainID() == domainID)
						&& (layerID < 0 ? true : p.getLayerID() == layerID))
					c += this.allProteins.get(i).getChainCount();
			}

		}

		if (group == 1 || group < 0 || group > 2) {

			for (int i = 0; i < this.allLipidsExtra.size(); i++) {
				final Lipid_3D p = this.allLipidsExtra.get(i);
				if ((domainID < 0 ? true : p.getDomainID() == domainID)
						&& (layerID < 0 ? true : p.getLayerID() == layerID))
					c += this.allLipidsExtra.get(i).getChainCount();
			}

		}
		if (group == 2 || group < 0 || group > 2) {

			for (int i = 0; i < this.allLipidsIntra.size(); i++) {
				final Lipid_3D p = this.allLipidsIntra.get(i);
				if ((domainID < 0 ? true : p.getDomainID() == domainID)
						&& (layerID < 0 ? true : p.getLayerID() == layerID))
					c += this.allLipidsIntra.get(i).getChainCount();
			}

		}
		return c;
	}

	/**
	 * Tests, wether the Membrane is completely Empty or not
	 * 
	 * @return
	 */
	public boolean isEmpty() {

		return this.allProteins.isEmpty() && this.allLipidsExtra.isEmpty()
				&& this.allLipidsIntra.isEmpty();
	}

	/*
	 * Some getters and setters concerning the last used algorithm
	 */

	public String getAlgorithmName() {
		return this.algorithmname;
	}

	public String getAlgorithmDescription() {
		return this.algorithmdescription;
	}

	public String getAlgorithmStringRepresentation() {
		return this.algorithmstringrepresentation;
	}

	public void setAlgorithmdescription(final String algorithmdescription) {
		this.algorithmdescription = algorithmdescription;
	}

	public void setAlgorithmname(final String algorithmname) {
		this.algorithmname = algorithmname;
	}

	public void setAlgorithmStringRepresentation(
			final String algorithmstringrepresentation) {
		this.algorithmstringrepresentation = algorithmstringrepresentation;
	}

	public void addAlgorithmStringRepresentation(
			final String algorithmstringrepresentation) {
		if (this.algorithmstringrepresentation== null) {
			this.algorithmstringrepresentation = algorithmstringrepresentation;
		} else {
			String oldStreingRepresentation = this.algorithmstringrepresentation;
			this.algorithmstringrepresentation = algorithmstringrepresentation + 
													"\n_____________________________________________________\n\n" +
													oldStreingRepresentation;
		}
	}
	
	public File getLastGeneratedMembraneFile() {
		return this.lastGeneratedMembraneFile;
	}

	public void setLastGeneratedMembraneFile(
			final File lastGeneratedMembraneFile) {
		this.lastGeneratedMembraneFile = lastGeneratedMembraneFile;
	}

	public void addMicroDomain(final MicroDomain_3D drawingDomain) {
		this.membraneHasBeenEditedAgain = true;

		this.microdomains.add(drawingDomain);
	}

	public MicroDomain_3D getMicroDomain(final int index) {
		return this.microdomains.get(index);
	}

	public void removeMicroDomain(final MicroDomain_3D toremove) {
		this.membraneHasBeenEditedAgain = true;

		toremove.kill();
		this.microdomains.remove(toremove);
		this.microdomains.trimToSize();

	}

	// public void clearMicroDomains() {
	// this.membraneHasBeenEditedAgain = true;
	// this.setConsistence(false);
	//
	// for (int i = 0; i < this.countMicroDomains(); i++) {
	//
	// MicroDomain_3D current = this.getMicroDomain(i);
	//
	// current.kill();
	//
	// }
	//
	// this.microdomains.clear();
	// this.microdomains.trimToSize();
	//
	// }

	public int countMicroDomains() {

		return this.microdomains.size();
	}

	public void organizeDomains() {

		final MembraneModel model = DataPool.getMembraneModel();

		for (int newid = 0; newid < this.countMicroDomains(); newid++) {

			final MicroDomain_3D current = this.getMicroDomain(newid);

			for (int i = 0; i < model.getExampleLipidsCountExtra(); i++) {
				final Lipid_3D mol = model.getExampleLipidAtExtra(i);
				if (mol.getDomainID() == current.id)
					mol.setDomainID(newid + 1);

			}
			for (int i = 0; i < model.getExampleLipidsCountIntra(); i++) {
				final Lipid_3D mol = model.getExampleLipidAtIntra(i);
				if (mol.getDomainID() == current.id)
					mol.setDomainID(newid + 1);

			}
			current.id = newid + 1;

		}

		for (int i = 0; i < this.getProteinCount(); i++) {
			final Protein_3D prot = this.getProteinAt(i);

			prot.setDomainID(this.getMicroDomainAtPoint(new Point((int) prot
					.getX(), (int) prot.getZ()), prot.getLayerID()));

		}

		this.membraneHasBeenEditedAgain = true;

	}

	public boolean containsExampleLipidExtra(final String code,
			final int domainID, final int layerID) {

		for (int i = 0; i < this.getExampleLipidsCountExtra(); i++) {
			if (this.getExampleLipidAtExtra(i).getLayerID() == layerID
					&& this.getExampleLipidAtExtra(i).getPDBcode()
							.equalsIgnoreCase(code)
					&& this.getExampleLipidAtExtra(i).getDomainID() == domainID) {
				return true;

			}
		}
		return false;
	}

	public boolean containsExampleLipidIntra(final String code,
			final int domainID, final int layerID) {

		for (int i = 0; i < this.getExampleLipidsCountIntra(); i++) {
			if (this.getExampleLipidAtIntra(i).getLayerID() == layerID
					&& this.getExampleLipidAtIntra(i).getPDBcode()
							.equalsIgnoreCase(code)
					&& this.getExampleLipidAtIntra(i).getDomainID() == domainID) {
				return true;

			}
		}
		return false;
	}

	public boolean canAddMoreExampleLipidsExtra(final int domainID,
			final int layerID) {
		int c = 0;

		for (int i = 0; i < this.getExampleLipidsCountExtra(); i++) {
			if (this.getExampleLipidAtExtra(i).getLayerID() == layerID
					&& this.getExampleLipidAtExtra(i).getDomainID() == domainID)
				c++;
		}
		return c < 100;
	}

	public boolean canAddMoreExampleLipidsIntra(final int domainID,
			final int layerID) {
		int c = 0;

		for (int i = 0; i < this.getExampleLipidsCountIntra(); i++) {
			if (this.getExampleLipidAtIntra(i).getLayerID() == layerID
					&& this.getExampleLipidAtIntra(i).getDomainID() == domainID)
				c++;
		}
		return c < 100;
	}

	public int getMicroDomainAtPoint(final Point point, final int layerID) {
		for (int j = 0; j < this.countMicroDomains(); j++) {

			if (this.getMicroDomain(j).layerId != layerID)
				continue;
			if (new Area(this.getMicroDomain(j).geoObject).contains(point)) {

				// System.out.println(this.getMicroDomain(j).id);
				return this.getMicroDomain(j).id;
			}
		}
		if (new Rectangle(0, 0, this.getSize().width, this.getSize().height)
				.contains(point))
			return 0;
		else
			return -1;
	}

	public void reshapeDomains() {
		final Rectangle mbounds = new Rectangle(0, 0, this.getSize().width,
				this.getSize().height);

		for (int i = this.countMicroDomains() - 1; i > -1; i--) {

			final MicroDomain_3D micro = this.getMicroDomain(i);
			if (!mbounds.contains(micro.getBoundingBox())) {
				micro.kill();
				this.microdomains.remove(i);
			}
		}

	}

	public Molecule_3D[] getAllMoleculesInMembrane(int domainID, int layerID) {
		Molecule_3D[] mols = this.getAllMoleculesInMembrane();

		if (domainID > 0)
			layerID = -1;
		if (domainID < 0 && layerID < 0)
			return mols;
		final Vector<Molecule_3D> ret = new Vector<Molecule_3D>();
		for (int i = 0; i < mols.length; i++) {

			if ((domainID < 0 ? true : mols[i].getDomainID() == domainID)
					&& (layerID < 0 ? true : mols[i].getLayerID() == layerID))
				ret.add(mols[i]);
		}
		return ret.toArray(new Molecule_3D[0]);
	}

	public Protein_3D[] getAllProteinsInDomain(final int domainID, int layerID) {
		if (domainID > 0)
			layerID = -1;
		if (domainID < 0 && layerID < 0)
			return this.getAllProteins().toArray(new Protein_3D[0]);
		final Vector<Protein_3D> ret = new Vector<Protein_3D>();
		for (int i = 0; i < this.getProteinCount(); i++) {
			final Protein_3D mol = this.getProteinAt(i);
			if ((domainID < 0 ? true : mol.getDomainID() == domainID)
					&& (layerID < 0 ? true : mol.getLayerID() == layerID))
				ret.add(mol);
		}
		return ret.toArray(new Protein_3D[0]);

	}

	public Lipid_3D[] getAllExampleLipidsExtraInDomain(final int domainID,
			int layerID) {
		if (domainID > 0)
			layerID = -1;
		if (domainID < 0 && layerID < 0)
			return this.getAllExampleLipidsExtra().toArray(new Lipid_3D[0]);
		final Vector<Lipid_3D> ret = new Vector<Lipid_3D>();
		for (int i = 0; i < this.getExampleLipidsCountExtra(); i++) {
			final Lipid_3D mol = this.getExampleLipidAtExtra(i);
			if ((domainID < 0 ? true : mol.getDomainID() == domainID)
					&& (layerID < 0 ? true : mol.getLayerID() == layerID))
				ret.add(mol);
		}
		return ret.toArray(new Lipid_3D[0]);

	}

	public Lipid_3D[] getAllExampleLipidsIntraInDomain(final int domainID,
			int layerID) {
		if (domainID > 0)
			layerID = -1;
		if (domainID < 0 && layerID < 0)
			return this.getAllExampleLipidsIntra().toArray(new Lipid_3D[0]);
		final Vector<Lipid_3D> ret = new Vector<Lipid_3D>();
		for (int i = 0; i < this.getExampleLipidsCountIntra(); i++) {

			final Lipid_3D mol = this.getExampleLipidAtIntra(i);
			if ((domainID < 0 ? true : mol.getDomainID() == domainID)
					&& (layerID < 0 ? true : mol.getLayerID() == layerID))
				ret.add(mol);
		}
		return ret.toArray(new Lipid_3D[0]);

	}

	public Lipid_3D[] getAllLipidsExtraInDomain(final int domainID, int layerID) {
		if (domainID > 0)
			layerID = -1;
		if (domainID < 0 && layerID < 0)
			return this.getAllLipidsExtra().toArray(new Lipid_3D[0]);
		final Vector<Lipid_3D> ret = new Vector<Lipid_3D>();
		for (int i = 0; i < this.getLipidCountExtra(); i++) {
			final Lipid_3D mol = this.getLipidAtExtra(i);
			if ((domainID < 0 ? true : mol.getDomainID() == domainID)
					&& (layerID < 0 ? true : mol.getLayerID() == layerID))
				ret.add(mol);
		}
		return ret.toArray(new Lipid_3D[0]);

	}

	public Lipid_3D[] getAllLipidsIntraInDomain(final int domainID, int layerID) {
		if (domainID > 0)
			layerID = -1;
		if (domainID < 0 && layerID < 0)
			return this.getAllLipidsIntra().toArray(new Lipid_3D[0]);
		final Vector<Lipid_3D> ret = new Vector<Lipid_3D>();
		for (int i = 0; i < this.getLipidCountIntra(); i++) {
			final Lipid_3D mol = this.getLipidAtIntra(i);
			if ((domainID < 0 ? true : mol.getDomainID() == domainID)
					&& (layerID < 0 ? true : mol.getLayerID() == layerID))
				ret.add(mol);
		}
		return ret.toArray(new Lipid_3D[0]);

	}

	private String[] getAllTypesInDomain(final int type, final int domainID,
			final int layerID) {
		final Hashtable<String, String> ret = new Hashtable<String, String>();
		Molecule_3D[] mols;

		if (type == 0)
			mols = this.getAllProteinsInDomain(domainID, layerID);
		else if (type == 1)
			mols = this.getAllExampleLipidsExtraInDomain(domainID, layerID);
		else if (type == 2)
			mols = this.getAllExampleLipidsIntraInDomain(domainID, layerID);
		else if (type == 3)
			mols = this.getAllLipidsExtraInDomain(domainID, layerID);
		else
			mols = this.getAllLipidsIntraInDomain(domainID, layerID);

		for (int i = 0; i < mols.length; i++) {
			ret.put(mols[i].getPDBcode(), "");

		}
		final Enumeration<String> en = ret.keys();

		final Vector<String> strings = new Vector<String>();
		while (en.hasMoreElements())
			strings.add(en.nextElement());

		final String[] returns = strings.toArray(new String[0]);

		Arrays.sort(returns);

		return returns;

	}

	public String[] getAllProteinTypesInDomain(final int domainID,
			final int layerID) {
		return this.getAllTypesInDomain(0, domainID, layerID);
	}

	public String[] getAllExampleLipidTypesExtraInDomain(final int domainID,
			final int layerID) {
		return this.getAllTypesInDomain(1, domainID, layerID);
	}

	public String[] getAllExampleLipidTypesIntraInDomain(final int domainID,
			final int layerID) {
		return this.getAllTypesInDomain(2, domainID, layerID);
	}

	public String[] getAllLipidTypesExtraInDomain(final int domainID,
			final int layerID) {
		return this.getAllTypesInDomain(3, domainID, layerID);
	}

	public String[] getAllLipidTypesIntraInDomain(final int domainID,
			final int layerID) {
		return this.getAllTypesInDomain(4, domainID, layerID);
	}

	public Protein_3D[] getAllProteinsOfType(final String code,
			final int domainID, final int layerID) {
		final Protein_3D[] mols = this
				.getAllProteinsInDomain(domainID, layerID);
		final Vector<Protein_3D> ret = new Vector<Protein_3D>();
		for (int i = 0; i < mols.length; i++) {
			if (mols[i].getPDBcode().equalsIgnoreCase(code))
				ret.add(mols[i]);
		}
		return ret.toArray(new Protein_3D[0]);
	}

	public Lipid_3D[] getAllExampleLipidsOfTypeExtra(final String code,
			final int domainID, final int layerID) {
		final Lipid_3D[] mols = this.getAllExampleLipidsExtraInDomain(domainID,
				layerID);
		final Vector<Lipid_3D> ret = new Vector<Lipid_3D>();
		for (int i = 0; i < mols.length; i++) {
			if (mols[i].getPDBcode().equalsIgnoreCase(code))
				ret.add(mols[i]);
		}
		return ret.toArray(new Lipid_3D[0]);
	}

	public Lipid_3D[] getAllExampleLipidsOfTypeIntra(final String code,
			final int domainID, final int layerID) {
		final Lipid_3D[] mols = this.getAllExampleLipidsIntraInDomain(domainID,
				layerID);
		final Vector<Lipid_3D> ret = new Vector<Lipid_3D>();
		for (int i = 0; i < mols.length; i++) {
			if (mols[i].getPDBcode().equalsIgnoreCase(code))
				ret.add(mols[i]);
		}
		return ret.toArray(new Lipid_3D[0]);
	}

	public Lipid_3D[] getAllLipidsOfTypeExtra(final String code,
			final int domainID, final int layerID) {
		final Lipid_3D[] mols = this.getAllLipidsExtraInDomain(domainID,
				layerID);
		final Vector<Lipid_3D> ret = new Vector<Lipid_3D>();
		for (int i = 0; i < mols.length; i++) {
			if (mols[i].getPDBcode().equalsIgnoreCase(code))
				ret.add(mols[i]);
		}
		return ret.toArray(new Lipid_3D[0]);
	}

	public Lipid_3D[] getAllLipidsOfTypeIntra(final String code,
			final int domainID, final int layerID) {
		final Lipid_3D[] mols = this.getAllLipidsIntraInDomain(domainID,
				layerID);
		final Vector<Lipid_3D> ret = new Vector<Lipid_3D>();
		for (int i = 0; i < mols.length; i++) {
			if (mols[i].getPDBcode().equalsIgnoreCase(code))
				ret.add(mols[i]);
		}
		return ret.toArray(new Lipid_3D[0]);
	}

	public void prepare(final MembranePackingAlgorithm memb) {

		for (int i = 0; i < this.countMicroDomains(); i++) {

			memb.microDomains
					.add(new MicroDomain(this.getMicroDomain(i), memb));

		}
		// Lipid_3D[] lip1 = this.getAllExampleLipidsExtraInDomain(0,0);
		// Lipid_3D[] lip2 = this.getAllExampleLipidsIntraInDomain(0,0);
		//
		// for (int i = 0; i < lip1.length; i++) {
		// memb.lipsEC.addElement(new Lipid((Lipid_3D) lip1[i].copy(true)));
		// }
		// for (int i = 0; i < lip2.length; i++) {
		// memb.lipsIC.addElement(new Lipid((Lipid_3D) lip2[i].copy(true)));
		// }

	}

	public void setDecimalPlaces(final int decimalplaces) {
		this.decimalplaces = decimalplaces;
	}

	public int getDecimalPlaces() {
		return this.decimalplaces;
	}

	public Lipid createNextLipid(final int membraneside,
			final Lipid_3D[] sampleLipids, final int domainID,
			final int layerID, MembranePackingAlgorithm algo) {

		// System.out.println(sampleLipids.length+"   "+domainID+"   "+layerID);
		if (sampleLipids.length == 0)
			return null;

		final int[] powers = new int[sampleLipids.length];

		int percentageOfAllSampleLipids = 0;
		
		for (int i = 0; i < sampleLipids.length; i++) {
			powers[i] = sampleLipids[i].getPercent();
			percentageOfAllSampleLipids += powers[i];
		}

		final int divisor = this.getComputeAbsoluteValues()?
				percentageOfAllSampleLipids:
				(100 * (int) Math.pow(10, DataPool.getMembraneModel().getDecimalPlaces()));

		Lipid_3D lip = null;

		if (true) {

			final Lipid_3D[] lipsInMemb = membraneside == MembranePackingAlgorithm.EXTRA ? this
					.getAllLipidsExtraInDomain(domainID, layerID)
					: this.getAllLipidsIntraInDomain(domainID, layerID);

			final double[] totalcounts = new double[sampleLipids.length];
			double totalcount = 0;

			for (int i = 0; i < totalcounts.length; i++) {
				totalcounts[i] = 0;
			}

			for (int i = 0; i < lipsInMemb.length; i++) {
				for (int j = 0; j < sampleLipids.length; j++) {
					if (lipsInMemb[i].getPDBcode().equals(
							sampleLipids[j].getPDBcode())) {
						totalcounts[j]++;
						totalcount++;
						break;
					}
				}
			}

			boolean goon = true;
			for (int i = 0; i < totalcounts.length; i++) {
				if (totalcounts[i] < 1 && !this.getComputeAbsoluteValues()) {
					goon = false;
					lip = (Lipid_3D) sampleLipids[i].copy(true, false);
					break;
				}
			}

			if (goon) {
				final double[] realPercentages = new double[sampleLipids.length];
				final double[] wantedPercentages = new double[sampleLipids.length];

				// compute absolute values
				if (this.getComputeAbsoluteValues()) {
					
					for (int i = 0; i < realPercentages.length; i++) {
						realPercentages[i] = totalcounts[i];
						wantedPercentages[i] = (double) sampleLipids[i]
								.getPercent();
					}
	
					while (true) {
						
						// the divisor is checked, because a values of 0 will make problems 
						// with the random generator
						int rand = (divisor==0)?0:algo.random.nextInt(divisor);
						rand++;
	
						int last = 0;
						int stoppedIndex = 0;
						for (int i = 0; i < powers.length; i++) {
							if (rand > last && rand <= last + powers[i]) {
								stoppedIndex = i;
								break;
							}
							last += powers[i];
						}
	
						if (realPercentages[stoppedIndex]+1
								- wantedPercentages[stoppedIndex] <= 0) {
							lip = (Lipid_3D) sampleLipids[stoppedIndex].copy(true,
									false);
							break;
						} else {
							break;
						}
	
					}
					
				// compute percental values (default)
				} else {
					
					for (int i = 0; i < realPercentages.length; i++) {
						realPercentages[i] = totalcounts[i] / totalcount;
						wantedPercentages[i] = (double) sampleLipids[i]
								.getPercent()
								/ (double) divisor;
					}

					while (true) {
	
						int rand = algo.random.nextInt(divisor);
	
						rand++;
	
						int last = 0;
						int stoppedIndex = 0;
						for (int i = 0; i < powers.length; i++) {
							if (rand > last && rand <= last + powers[i]) {
	
								stoppedIndex = i;
								break;
							}
							last += powers[i];
						}
	
						if (realPercentages[stoppedIndex]
								- wantedPercentages[stoppedIndex] <= 0) {
							lip = (Lipid_3D) sampleLipids[stoppedIndex].copy(true,
									false);
							break;
						}
	
					}
				}

				// System.out.println("Side "+membraneside);
				// for (int i = 0; i < powers.length; i++) {
				// System.out.println(realPercentages[i]+"--"+wantedPercentages[i
				// ]);
				//
				// }
				// System.out.println("------------------------------------");

			}

		}

		if (lip == null)
			return null;

		lip.setYTranslation((MembranePackingAlgorithm.getTotalHeight(lip) / 2)
				/ CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE);

		if (membraneside == MembranePackingAlgorithm.INTRA)
			lip.setYTranslation(lip.getYTranslation() * -1);

		final Lipid newLip = new Lipid(lip, algo);

		return newLip;

	}

	public boolean areaContainsMolecules(final int area, final int layerID) {

		return this.getAllProteinsInDomain(area, layerID).length > 0
				|| this.getAllLipidsExtraInDomain(area, layerID).length > 0
				|| this.getAllLipidsIntraInDomain(area, layerID).length > 0;

	}

	public void sortMolecules() {

		final Vector<Molecule_3D>[] vectors = new Vector[] { this.allProteins,
				this.allLipidsExtra, this.allLipidsIntra };

		final Comparator<Object> c = new Comparator<Object>() {

			public int compare(final Object arg0, final Object arg1) {
				final Molecule_3D mol1 = (Molecule_3D) arg0;
				final Molecule_3D mol2 = (Molecule_3D) arg1;

				if (mol1.getPDBcode() == null && mol2.getPDBcode() == null) {

					return Cm2_Toolkit.getSimpleName(mol1.getPDBFile())
							.compareTo(
									Cm2_Toolkit
											.getSimpleName(mol2.getPDBFile()));

				}

				if (mol1.getPDBcode() != null && mol2.getPDBcode() == null)
					return 1;
				if (mol1.getPDBcode() == null && mol2.getPDBcode() != null)
					return -1;

				return mol1.getPDBcode().compareTo(mol2.getPDBcode());
			}

		};

		for (int v = 0; v < vectors.length; v++) {

			Collections.sort(vectors[v], c);

		}

	}

	public boolean anyCurrentIntersections(final boolean withBorderCheck) {

		final IntersectManager intersector = new IntersectManager();

		final Molecule_3D[] mols = this.getAllMoleculesInMembrane();
		final Vector<?> v = Cm2_Toolkit.fromArrayToVector(mols);
		for (int i = 0; i < mols.length; i++) {
			if (withBorderCheck ? intersector.intercectsBorderOrAnyMolecule(
					mols[i], true) : intersector
					.intersectsMoleculeMoleculesVector(mols[i], v, true))
				return true;
		}

		// for(int i=0;i<this.getProteinCount();i++) {
		// Molecule_3D mol = this.getProteinAt(i);
		// if(intersector.intercectsBorderOrAnyMole(mol,true))
		// ret.add(mol);
		// }
		// for(int i=0;i<this.getLipidCountExtra();i++) {
		// Molecule_3D mol = this.getLipidAtExtra(i);
		// if(intersector.intercects_Molecule_Border(mol,this.getSize())||intersector.intersectsMoleculeAnyMolecules(mol,true))
		// ret.add(mol);
		// }
		// for(int i=0;i<this.getLipidCountIntra();i++) {
		// Molecule_3D mol = this.getLipidAtIntra(i);
		// if(intersector.intercects_Molecule_Border(mol,this.getSize())||intersector.intersectsMoleculeAnyMolecules(mol,true))
		// ret.add(mol);
		// }

		return false;
	}

	public Vector<Molecule_3D> getCurrentIntersections(
			final boolean withBorderCheck) {

		final Vector<Molecule_3D> ret = new Vector<Molecule_3D>();
		final IntersectManager intersector = new IntersectManager();

		final Molecule_3D[] mols = this.getAllMoleculesInMembrane();
		final Vector<?> v = Cm2_Toolkit.fromArrayToVector(mols);
		for (int i = 0; i < mols.length; i++) {
			if (withBorderCheck ? intersector.intercectsBorderOrAnyMolecule(
					mols[i], true) : intersector
					.intersectsMoleculeMoleculesVector(mols[i], v, true))
				ret.addElement(mols[i]);
		}

		// for(int i=0;i<this.getProteinCount();i++) {
		// Molecule_3D mol = this.getProteinAt(i);
		// if(intersector.intercectsBorderOrAnyMole(mol,true))
		// ret.add(mol);
		// }
		// for(int i=0;i<this.getLipidCountExtra();i++) {
		// Molecule_3D mol = this.getLipidAtExtra(i);
		// if(intersector.intercects_Molecule_Border(mol,this.getSize())||intersector.intersectsMoleculeAnyMolecules(mol,true))
		// ret.add(mol);
		// }
		// for(int i=0;i<this.getLipidCountIntra();i++) {
		// Molecule_3D mol = this.getLipidAtIntra(i);
		// if(intersector.intercects_Molecule_Border(mol,this.getSize())||intersector.intersectsMoleculeAnyMolecules(mol,true))
		// ret.add(mol);
		// }

		return ret;
	}

	public boolean containsMoleculeWithAbsoluteFilepath(final String key) {
		for (int i = 0; i < this.getProteinCount(); i++) {
			if (this.getProteinAt(i).getPDBFile().getAbsolutePath().equals(key))
				return true;

		}
		for (int i = 0; i < this.getLipidCountExtra(); i++) {
			if (this.getLipidAtExtra(i).getPDBFile().getAbsolutePath().equals(
					key))
				return true;

		}
		for (int i = 0; i < this.getLipidCountIntra(); i++) {
			if (this.getLipidAtIntra(i).getPDBFile().getAbsolutePath().equals(
					key))
				return true;

		}
		for (int i = 0; i < this.getExampleLipidsCountExtra(); i++) {
			if (this.getExampleLipidAtExtra(i).getPDBFile().getAbsolutePath()
					.equals(key))
				return true;

		}
		for (int i = 0; i < this.getExampleLipidsCountIntra(); i++) {
			if (this.getExampleLipidAtIntra(i).getPDBFile().getAbsolutePath()
					.equals(key))
				return true;

		}
		return false;
	}

	public int getLayerCount() {
		return this.layers.size();
	}

	public Layer getLayer(final int index) {
		return this.layers.get(index);
	}

	public boolean hasMultipleLayers() {
		return !this.layers.isEmpty();
	}

	public Layer getLayerByID(final int layerID) {
		for (int i = 0; i < this.getLayerCount(); i++) {
			if (this.getLayer(i).getLayerId() == layerID)
				return this.getLayer(i);
		}
		return null;
	}

	public void addLayer(final Layer layer) {
		this.layers.addElement(layer);
		this.setMembraneHasBeenEditedAgain(true);

	}

	public void removeLayer(final Layer currentLayer) {
		this.layers.remove(currentLayer);
		this.layers.trimToSize();
		this.setMembraneHasBeenEditedAgain(true);

	}
	
	public void showAllMultilayers(boolean show) {
		final int currentID = DataPool.getCurrentLayerID();
		for (int i = 0; i < this.allProteins.size(); i++) {
			final Protein_3D mol = this.allProteins.get(i);
			final boolean b = mol.getLayerID() == currentID;

			mol.setTransparent((!b  && !show),b);
		}
		for (int i = 0; i < this.allLipidsExtra.size(); i++) {
			final Lipid_3D mol = this.allLipidsExtra.get(i);
			final boolean b = mol.getLayerID() == currentID;

			mol.setTransparent((!b  && !show),b);
		}
		for (int i = 0; i < this.allLipidsIntra.size(); i++) {
			final Lipid_3D mol = this.allLipidsIntra.get(i);
			final boolean b = mol.getLayerID() == currentID;

			mol.setTransparent((!b  && !show),b);
		}
	}

	public void translateAllMolecules(float factor, final boolean move,
			final boolean updatePresentation) {
		final int currentID = DataPool.getCurrentLayerID();
		factor *= -1;
		for (int i = 0; i < this.allProteins.size(); i++) {
			final Protein_3D mol = this.allProteins.get(i);
			mol.setYTranslation(mol.getYTranslation() + factor);

			if (updatePresentation) {
				final boolean b = mol.getLayerID() == currentID;

				mol.getShape().setPickable(b);
				if (b)
					mol.setRenderMode(DataPool.J3DSettings.overallRenderMode);
				else
					mol.setRenderMode(CM2_CONSTANTS.RENDERMODE.RENDER_MODE_POLYGON);
				mol.setTransparent((!b  && !DataPool.J3DSettings.showAllMultiLayers),b);

			}
			if (move)
				mol.move();
		}
		for (int i = 0; i < this.allLipidsExtra.size(); i++) {
			final Molecule_3D mol = this.allLipidsExtra.get(i);
			mol.setYTranslation(mol.getYTranslation() + factor);

			if (updatePresentation) {
				final boolean b = mol.getLayerID() == currentID;

				mol.getShape().setPickable(b);
				if (b)
					mol.setRenderMode(DataPool.J3DSettings.overallRenderMode);
				else
					mol.setRenderMode(CM2_CONSTANTS.RENDERMODE.RENDER_MODE_POLYGON);
				mol.setTransparent((!b  && !DataPool.J3DSettings.showAllMultiLayers),b);
			}
			if (move)
				mol.move();
		}
		for (int i = 0; i < this.allLipidsIntra.size(); i++) {
			final Molecule_3D mol = this.allLipidsIntra.get(i);
			mol.setYTranslation(mol.getYTranslation() + factor);
			if (move)
				mol.move();
			if (updatePresentation) {
				final boolean b = mol.getLayerID() == currentID;

				mol.getShape().setPickable(b);
				if (b)
					mol.setRenderMode(DataPool.J3DSettings.overallRenderMode);
				else
					mol.setRenderMode(CM2_CONSTANTS.RENDERMODE.RENDER_MODE_POLYGON);
				mol.setTransparent((!b  && !DataPool.J3DSettings.showAllMultiLayers),b);
			}
			if (move)
				mol.move();
		}
	}

	public MicroDomain_3D[] getAllMicroDomains(final int currentLayerID) {
		final Vector<MicroDomain_3D> mds = new Vector<MicroDomain_3D>();
		for (int i = 0; i < this.countMicroDomains(); i++) {
			if (this.getMicroDomain(i).layerId == currentLayerID)
				mds.addElement(this.getMicroDomain(i));
		}
		return mds.toArray(new MicroDomain_3D[0]);
	}

	public int countTERs(int group, int domainID, int layerID) {
		int ret = 0;
		Molecule_3D[] mols;
		if (group == 0)
			mols = this.getAllProteinsInDomain(domainID, layerID);
		else if (group == 1)
			mols = this.getAllLipidsExtraInDomain(domainID, layerID);
		else if (group == 2)
			mols = this.getAllLipidsIntraInDomain(domainID, layerID);
		else
			return 0;
		for (int i = 0; i < mols.length; i++) {
			ret += mols[i].terCount;
		}

		return ret;
	}

	/**
	 * Check if the lipids in the membrane PDB file is sorted by type during export.
	 * @return boolean sortByType
	 */
	public boolean sortByType() {
		return sortByType;
	}

	/**
	 * Set if the lipids in the membrane PDB file are sorted by type during export.
	 * @param boolean sortByType
	 */
	public void setSortByType(boolean sortByType) {
		this.sortByType = sortByType;
	}

	/**
	 * Check if absolute values are used during the computation process instead of
	 * percental values.
	 * @return boolean computeAbsoluteValues
	 */
	public boolean getComputeAbsoluteValues() {
		return computeAbsoluteValues;
	}

	/**
	 * Set if absolute values are used during the computation process instead of
	 * percental values.
	 * @param boolean computeAbsoluteValues
	 */
	public void setComputeAbsoluteValues(boolean computeAbsoluteValues) {
		this.computeAbsoluteValues = computeAbsoluteValues;
	}
	
}
