/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_1)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.dataobjects;

import java.awt.Color;
import java.awt.Point;
import java.awt.Polygon;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Ellipse2D.Double;

import javax.media.j3d.Appearance;
import javax.media.j3d.BranchGroup;
import javax.media.j3d.ColoringAttributes;
import javax.media.j3d.GeometryArray;
import javax.media.j3d.LineAttributes;
import javax.media.j3d.LineStripArray;
import javax.media.j3d.Shape3D;
import javax.vecmath.Color3f;
import javax.vecmath.Matrix3f;
import javax.vecmath.Point3f;
import javax.vecmath.Vector3f;

import org.cellmicrocosmos.cm2.constants.CM2_CONSTANTS;
import org.cellmicrocosmos.cm2.engine.DataPool;
import org.cellmicrocosmos.cm2.misc.Cm2_Toolkit;

/**
 * This class represents the microdomains/lipid rafts objects.
 *  
 * @author Tim Dingersen dackelnose@gmx.net
 * 
 */
public class MicroDomain_3D {

	public final static int RECTANGULAR = 0;

	public final static int ELLIPSE = 1;

	public final static int POLYGON = 2;

	public int id = -1;

	public int layerId = 0;

	public int geometry = MicroDomain_3D.RECTANGULAR;

	public Shape3D shape = new Shape3D();

	public Shape geoObject = new Rectangle();

	public LineStripArray geo;

	public BranchGroup branch;

	public boolean isHigh = false;

	private boolean active = true;

	public MicroDomain_3D() {

		this.shape.addGeometry(this.geo);
		this.shape.setCapability(Shape3D.ALLOW_GEOMETRY_WRITE);
		this.shape.setCapability(Shape3D.ALLOW_APPEARANCE_WRITE);
		this.shape.setPickable(false);

		this.branch = new BranchGroup();
		this.branch.setCapability(BranchGroup.ALLOW_CHILDREN_EXTEND);
		this.branch.setCapability(BranchGroup.ALLOW_CHILDREN_READ);
		this.branch.setCapability(BranchGroup.ALLOW_CHILDREN_WRITE);
		this.branch.setCapability(BranchGroup.ALLOW_PARENT_READ);
		this.branch.setCapability(BranchGroup.ALLOW_DETACH);

		this.shape.setAppearance(this.createAppearance(2,
				CM2_CONSTANTS.J3D.DEFAULTMICRODOMAINCOLOR));
		this.branch.addChild(this.shape);
		this.branch.setUserData(this);

	}

	/**
	 * Return the type of the Microdomain. This can be:
	 * - MicroDomain_3D.RECTANGULAR
	 * - MicroDomain_3D.ELLIPSE
	 * - MicroDomain_3D.POLYGON
	 * @return int
	 */
	public int getGeometryType() {
		return this.geometry;

	}
	
	/**
	 * Returns the area of the actual geometry shape.
	 * @return double
	 */
	public double getArea() {
		if (this.geometry == MicroDomain_3D.RECTANGULAR) {
//			System.out.println("MD area rectangle "+((Rectangle)this.geoObject).getHeight()*((Rectangle)this.geoObject).getWidth());
			return ((Rectangle)this.geoObject).getHeight()*((Rectangle)this.geoObject).getWidth();
		} else if (this.geometry == MicroDomain_3D.ELLIPSE) {
//			System.out.println("MD area ellipse "+((Ellipse2D.Double)this.geoObject).getHeight()/2*((Ellipse2D.Double)this.geoObject).getWidth()/2*Math.PI);
			return ((Ellipse2D.Double)this.geoObject).getHeight()/2*((Ellipse2D.Double)this.geoObject).getWidth()/2*Math.PI;
		// Adopted from:
		// Below is the syntax highlighted version of Polygon.java from 3.5 Inheritance. 
		// Copyright 2007, Robert Sedgewick and Kevin Wayne.
		// Last updated: Tue Oct 20 19:50:35 EDT 2009. 
		} else { //if (this.geometry == MicroDomain_3D.POLYGON) {

	        double sum = 0.0;
	        for (int i = 0; i < ((Polygon)this.geoObject).npoints; i++) {
	        	int j = (i+1 < ((Polygon)this.geoObject).npoints)?i+1:0;
	            sum = sum + (	( ((Polygon)this.geoObject).xpoints[i] * ((Polygon)this.geoObject).ypoints[j] ) -
	                         	( ((Polygon)this.geoObject).ypoints[i] * ((Polygon)this.geoObject).xpoints[j] )   );
	        }
//	        System.out.println("MD area polygon "+0.5 * sum);
	        return ((sum<0)?-0.5:0.5)*sum;
		}
	}
	
	public void setGeometryType(final int mode) {
		if (mode == MicroDomain_3D.RECTANGULAR) {
			this.geoObject = new Rectangle();
		}
		if (mode == MicroDomain_3D.ELLIPSE) {
			this.geoObject = new Ellipse2D.Double();
		}
		if (mode == MicroDomain_3D.POLYGON) {
			this.geoObject = new Polygon();
		}

		this.geometry = mode;

	}

	public void startPainting(final Point point) {
		this.setGeometryType(this.geometry);
		this.updateShape3D();

		this.cutOffBorders(point);

		if (this.geometry == MicroDomain_3D.RECTANGULAR)
			this.geoObject = new Rectangle(point.x, point.y, 0, 0);

		else if (this.geometry == MicroDomain_3D.ELLIPSE)
			this.geoObject = new Ellipse2D.Double(point.x, point.y, 0, 0);

		else {
			this.geoObject = new Polygon();
			((Polygon) this.geoObject).addPoint(point.x, point.y);
			((Polygon) this.geoObject).addPoint(point.x, point.y);
		}

		this.updateShape3D();

	}

	public void continuePainting(final Point point) {

		this.cutOffBorders(point);

		if (this.geometry == MicroDomain_3D.RECTANGULAR) {
			final Rectangle rect = (Rectangle) this.geoObject;

			final int x = rect.x;
			final int y = rect.y;
			final int w = point.x - x;
			final int h = point.y - y;

			if (w >= 0 && h >= 0)
				this.geoObject = new Rectangle(x, y, w, h);
			else if (w < 0 && h >= 0)
				this.geoObject = new Rectangle(x, y, 0, h);
			else if (w >= 0 && h < 0)
				this.geoObject = new Rectangle(x, y, w, 0);
			else
				this.geoObject = new Rectangle(x, y, 0, 0);

		}

		else if (this.geometry == MicroDomain_3D.ELLIPSE) {
			final Ellipse2D.Double ellipse = (Ellipse2D.Double) this.geoObject;

			final double x = ellipse.x;
			final double y = ellipse.y;
			final double w = point.x - x;
			final double h = point.y - y;

			if (w >= 0 && h >= 0)
				this.geoObject = new Ellipse2D.Double(x, y, w, h);
			else if (w < 0 && h >= 0)
				this.geoObject = new Ellipse2D.Double(x, y, 0, h);
			else if (w >= 0 && h < 0)
				this.geoObject = new Ellipse2D.Double(x, y, w, 0);
			else
				this.geoObject = new Ellipse2D.Double(x, y, 0, 0);

		} else {

			final Polygon poly = (Polygon) this.geoObject;
			poly.xpoints[poly.npoints - 1] = point.x;
			poly.ypoints[poly.npoints - 1] = point.y;
			poly.addPoint(poly.xpoints[0], poly.ypoints[0]);

		}

		this.updateShape3D();

	}

	private void cutOffBorders(final Point point) {
		if (point.x < 0)
			point.x = 0;
		if (point.y < 0)
			point.y = 0;
		if (point.x > DataPool.getMembraneModel().getSize().width)
			point.x = DataPool.getMembraneModel().getSize().width;
		if (point.y > DataPool.getMembraneModel().getSize().height)
			point.y = DataPool.getMembraneModel().getSize().height;

	}

	public void updateShape3D() {

		float[] vertices;

		if (this.geometry == MicroDomain_3D.RECTANGULAR) {
			final Rectangle rect = (Rectangle) this.geoObject;

			final float x = rect.x * CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE;
			final float y = rect.y * CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE;
			final float width = rect.width
					* CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE;
			final float height = rect.height
					* CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE;

			vertices = new float[] { x, 0, y, x + width, 0, y, x + width, 0,
					y + height, x, 0, y + height, x, 0, y };

			this.geo = new LineStripArray(5, GeometryArray.COORDINATES,
					new int[] { 5 });

			this.geo.setCoordinates(0, vertices);
			this.geo.setCapability(GeometryArray.ALLOW_COORDINATE_READ);
			this.geo.setCapability(GeometryArray.ALLOW_COORDINATE_WRITE);
			this.geo.setCapability(GeometryArray.ALLOW_COLOR_WRITE);
			this.shape.setGeometry(this.geo);
		} else if (this.geometry == MicroDomain_3D.ELLIPSE) {

			final Ellipse2D.Double ellipse = (Ellipse2D.Double) this.geoObject;

			vertices = this.createEllipse3D(ellipse);

			this.geo = new LineStripArray(vertices.length / 3,
					GeometryArray.COORDINATES,
					new int[] { vertices.length / 3 });
			this.geo.setCoordinates(0, vertices);
			this.geo.setCapability(GeometryArray.ALLOW_COORDINATE_READ);
			this.geo.setCapability(GeometryArray.ALLOW_COORDINATE_WRITE);
			this.geo.setCapability(GeometryArray.ALLOW_COLOR_WRITE);
			this.shape.setGeometry(this.geo);
		}

		else {
			final Polygon poly = (Polygon) this.geoObject;
			if (poly.npoints < 2)
				return;
			vertices = new float[poly.npoints * 3];

			int c = 0;
			for (int i = 0; i < poly.npoints; i++) {
				vertices[c++] = poly.xpoints[i]
						* CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE;
				vertices[c++] = 0f;
				vertices[c++] = poly.ypoints[i]
						* CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE;

			}

			this.geo = new LineStripArray(poly.npoints,
					GeometryArray.COORDINATES, new int[] { poly.npoints });
			this.geo.setCoordinates(0, vertices);
			this.geo.setCapability(GeometryArray.ALLOW_COORDINATE_READ);
			this.geo.setCapability(GeometryArray.ALLOW_COORDINATE_WRITE);
			this.geo.setCapability(GeometryArray.ALLOW_COLOR_WRITE);
			this.shape.setGeometry(this.geo);

		}

	}

	private float[] createEllipse3D(final Double ellipse) {

		float w = (float) ellipse.width / 2f;
		float h = (float) ellipse.height / 2f;

		w *= CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE;
		h *= CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE;

		final float[] ret = new float[273];
		int c = 0;
		for (int i = 0; i <= 360; i += 4) {
			final Matrix3f matrix = Cm2_Toolkit.createAUnitMatrix();
			final Vector3f v = new Vector3f(0, 0, 1);
			matrix.rotY((float) Math.toRadians(i));
			final Point3f point = Cm2_Toolkit.transformToNewPoint(matrix,
					new Point3f(v));

			point.x *= w;
			point.z *= h;

			point.add(new Point3f((float) ellipse.x
					* CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE, 0f,
					(float) ellipse.y * CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE));

			point.add(new Point3f(w, 0, h));

			ret[c++] = point.x;
			ret[c++] = 0;
			ret[c++] = point.z;

		}

		return ret;
	}

	public void kill() {
		final MembraneModel model = DataPool.getMembraneModel();

		for (int i = 0; i < model.getExampleLipidsCountExtra(); i++) {
			final Lipid_3D mol = model.getExampleLipidAtExtra(i);
			if (mol.getDomainID() == this.id)
				model.deleteExampleLipidExtra(mol);

		}
		for (int i = 0; i < model.getExampleLipidsCountIntra(); i++) {
			final Lipid_3D mol = model.getExampleLipidAtIntra(i);
			if (mol.getDomainID() == this.id)
				model.deleteExampleLipidIntra(mol);

		}
		// if(this.branch.getParent()instanceof BranchGroup) {
		// ((BranchGroup)this.branch.getParent()).removeChild(this.branch);
		// }
	}

	public void setValid(final boolean valid) {

		if (valid) {

			this.shape.setAppearance(this.createAppearance(2, Color.MAGENTA));
		} else
			this.shape.setAppearance(this.createAppearance(2, Color.RED));

	}

	public Rectangle getBoundingBox() {

		if (this.geoObject instanceof Rectangle
				|| this.geoObject instanceof Ellipse2D.Double) {

			return this.geoObject.getBounds();
		} else {

			final Polygon poly = (Polygon) this.geoObject;
			int maxx = 0;
			int maxy = 0;
			int minx = DataPool.getMembraneModel().getSize().width;
			int miny = DataPool.getMembraneModel().getSize().height;
			for (int i = 0; i < poly.npoints; i++) {
				maxx = Math.max(maxx, poly.xpoints[i]);
				maxy = Math.max(maxy, poly.ypoints[i]);

				minx = Math.min(minx, poly.xpoints[i]);
				miny = Math.min(miny, poly.ypoints[i]);
			}

			return new Rectangle(minx, miny, maxx - minx, maxy - miny);

		}

	}

	public void setHighlighted(final boolean b) {
		if (b == this.isHigh)
			return;
		this.isHigh = b;
		if (b) {

			this.shape.setAppearance(this.createAppearance(8, Color.GREEN));
		} else {

			this.shape.setAppearance(this.createAppearance(2,
					CM2_CONSTANTS.J3D.DEFAULTMICRODOMAINCOLOR));
		}

	}

	private Appearance createAppearance(final int lineWidth, final Color color) {
		
		final Appearance appearance = new Appearance();
		
		final ColoringAttributes coloringAttrs = new ColoringAttributes(
				new Color3f(color), ColoringAttributes.SHADE_GOURAUD);
		appearance.setColoringAttributes(coloringAttrs);
		
		final LineAttributes la = new LineAttributes();
		la.setPatternMask(LineAttributes.PATTERN_DOT);
		la.setLineWidth(lineWidth);
		appearance.setLineAttributes(la);
		
		Cm2_Toolkit.setRenderingAttributes(appearance);
		
		return appearance;
	}

	public Protein_3D[] getProteins() {
		return DataPool.getMembraneModel().getAllProteinsInDomain(this.id,
				this.layerId);

	}

	public Lipid_3D[] getExampleLipidsExtra() {
		return DataPool.getMembraneModel().getAllExampleLipidsExtraInDomain(
				this.id, this.layerId);

	}

	public Lipid_3D[] getExampleLipidsIntra() {
		return DataPool.getMembraneModel().getAllExampleLipidsIntraInDomain(
				this.id, this.layerId);
	}

	public String[] getAllProteinTypes() {
		return DataPool.getMembraneModel().getAllProteinTypesInDomain(this.id,
				this.layerId);
	}

	public String[] getAllExampleLipidTypesExtra() {
		return DataPool.getMembraneModel()
				.getAllExampleLipidTypesExtraInDomain(this.id, this.layerId);
	}

	public String[] getAllExampleLipidTypesIntra() {
		return DataPool.getMembraneModel()
				.getAllExampleLipidTypesIntraInDomain(this.id, this.layerId);
	}

	public Protein_3D[] getAllProteinsOfType(final String code) {
		return DataPool.getMembraneModel().getAllProteinsOfType(code, this.id,
				this.layerId);
	}

	public Lipid_3D[] getAllExampleLipidsOfTypeExtra(final String code) {
		return DataPool.getMembraneModel().getAllExampleLipidsOfTypeExtra(code,
				this.id, this.layerId);
	}

	public Lipid_3D[] getAllExampleLipidsOfTypeIntra(final String code) {
		return DataPool.getMembraneModel().getAllExampleLipidsOfTypeIntra(code,
				this.id, this.layerId);
	}

	public boolean isEnabled() {
		return this.active;
	}

	public void setEnabled(final boolean b) {
		this.active = b;
	}

	public boolean containsMolecules() {
		final MembraneModel model = DataPool.getMembraneModel();

		for (int i = 0; i < model.getProteinCount(); i++) {
			if (model.getProteinAt(i).getDomainID() == this.id)
				return true;
		}

		for (int i = 0; i < model.getLipidCountExtra(); i++) {
			if (model.getLipidAtExtra(i).getDomainID() == this.id)
				return true;
		}
		for (int i = 0; i < model.getLipidCountIntra(); i++) {
			if (model.getLipidAtIntra(i).getDomainID() == this.id)
				return true;
		}
		return false;
	}

}
