/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_1)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.engine;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.Polygon;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.geom.Ellipse2D;
import java.awt.image.BufferedImage;
import java.awt.image.RenderedImage;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Vector;

import javax.imageio.ImageIO;
import javax.media.j3d.BranchGroup;
import javax.media.j3d.Canvas3D;
import javax.media.j3d.ImageComponent;
import javax.media.j3d.ImageComponent2D;
import javax.media.j3d.View;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPopupMenu;
import javax.swing.SwingUtilities;
import javax.vecmath.Matrix3f;
import javax.vecmath.Point3d;
import javax.vecmath.Point3f;

import org.cellmicrocosmos.cm2.coarsegrainsupport.MoleculeEditor;
import org.cellmicrocosmos.cm2.constants.CM2_CONSTANTS;
import org.cellmicrocosmos.cm2.dataobjects.Layer;
import org.cellmicrocosmos.cm2.dataobjects.Lipid_3D;
import org.cellmicrocosmos.cm2.dataobjects.MembraneModel;
import org.cellmicrocosmos.cm2.dataobjects.MicroDomain_3D;
import org.cellmicrocosmos.cm2.dataobjects.Molecule_3D;
import org.cellmicrocosmos.cm2.dataobjects.Protein_3D;
import org.cellmicrocosmos.cm2.engine.DataPool.APLVoroSettings;
import org.cellmicrocosmos.cm2.functionality3d.CameraMode;
import org.cellmicrocosmos.cm2.functionality3d.Membrane3D;
import org.cellmicrocosmos.cm2.gui.GlobalDatabaseBrowser;
import org.cellmicrocosmos.cm2.gui.MainGui;
import org.cellmicrocosmos.cm2.gui.dialogs.ChangeSizeDialog;
import org.cellmicrocosmos.cm2.gui.dialogs.HelpDialog;
import org.cellmicrocosmos.cm2.gui.dialogs.ManageTitleSectionDialog;
import org.cellmicrocosmos.cm2.gui.dialogs.NewMembraneDialog;
import org.cellmicrocosmos.cm2.gui.dialogs.ProtocolDialog;
import org.cellmicrocosmos.cm2.gui.dialogs.StatusDialog;
import org.cellmicrocosmos.cm2.gui.elements.ComponentView;
import org.cellmicrocosmos.cm2.gui.elements.DatabaseView;
import org.cellmicrocosmos.cm2.gui.elements.MembraneView;
import org.cellmicrocosmos.cm2.gui.elements.PropertiesView;
import org.cellmicrocosmos.cm2.gui.elements.StatisticsView;
import org.cellmicrocosmos.cm2.interfaces.DatabaseViewListener;
import org.cellmicrocosmos.cm2.interfaces.GuiListener;
import org.cellmicrocosmos.cm2.interfaces.MembraneViewListener;
import org.cellmicrocosmos.cm2.io.Compatibility;
import org.cellmicrocosmos.cm2.io.DatabaseIO;
import org.cellmicrocosmos.cm2.io.FileIOHandler;
import org.cellmicrocosmos.cm2.io.Installer;
import org.cellmicrocosmos.cm2.io.InvalidEntriesException;
import org.cellmicrocosmos.cm2.io.MembraneXmlReader;
import org.cellmicrocosmos.cm2.io.MembraneXmlWriter;
import org.cellmicrocosmos.cm2.io.PDBWriter;
import org.cellmicrocosmos.cm2.io.PdbReader_3D;
import org.cellmicrocosmos.cm2.io.ReversePDBParser;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.CoordinateTransformationHints;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.MembranePackingAlgorithm;
import org.cellmicrocosmos.cm2.misc.Cm2_IconDepot;
import org.cellmicrocosmos.cm2.misc.Cm2_Toolkit;
import org.cellmicrocosmos.cm2.misc.ConfigParser;
import org.cellmicrocosmos.cm2.pluginmanager.CalculationManager;
import org.cellmicrocosmos.cm2.pluginmanager.PluginManager;
import org.cellmicrocosmos.cmX.mrna.MRNA;
import org.jdom.Document;

import com.sun.j3d.utils.universe.SimpleUniverse;


/**
 * This class is the basement of the whole application; 
 * the basic usecases will be handled by this class.
 * 
 * @author Tim Dingersen, Sebastian Schneider, Christian Gamroth, Bjoern Sommer bjoern@CELLmicrocosmos.org
 */
public class Engine {

	public static final long VERSION_IMAGES = 1217;

	public static final long VERSION_HELP = 1222;

	public static final long VERSION_SETTINGS = 1214;
	
	public static final long VERSION_JMOL = 1213;

	/*
	 * the gui components
	 */
	
	// this thread is only needed if an external application runs Cm2
	static Thread ownThread = null;
	
	MainGui gui = null;

	ComponentView componentview = null;

	public MembraneView membraneview = null;

	PropertiesView propertiesview = null;

	StatisticsView statisticsview = null;

	DatabaseView databaseview = null;

	/*
	 * Creating functionality classes that implement the listeners
	 */

	final BasicOperations basicOperations = new BasicOperations();

	final DatabaseOperations databaseOperations = new DatabaseOperations();

	public final MembraneOperations membraneOperations = new MembraneOperations();

	final J3DOperations j3doperations = new J3DOperations();

	final PluginManager pluginmanager = new PluginManager();

	private CalculationManager manager;

	public static boolean stereoAvailable = false;
	
	private boolean isActive = true;
	
	private static MRNA cmX_MRNA = null;
	
	private String cm1TempName2Edit = "null";

	private MoleculeEditor molEditView;

	/*
	 * start of the constructor
	 */

	public Engine(final StatusDialog welcomescreen, final File file2open, final String cm1TempName2Edit) {
		System.out.print(".. ");
		System.out.println("Done!");
		DataPool.iAmWebstart = Cm2_Toolkit.amIRunningOnWebstart();
		TestArea.engine=this;
		
		// this is only needed if a new membrane file from Cm1 has been opened
		// (not for editing an existing file)
		this.cm1TempName2Edit = cm1TempName2Edit;

		// set the security manager in order to allow the plugins to access the
		// application
		System.setSecurityManager(null);

		/*
		 * At first, the appearing statusbar gets a status to show. This will
		 * happen several times while task is working through this constructor
		 */
		welcomescreen.setProgress("Installing Files...", 0, false);

		Installer.installSettings();

		welcomescreen.setProgress("Installing Files...", 5, false);

		Installer.installJmol();
		
		Installer.installHelpfiles();

		welcomescreen.setProgress("Installing Files...", 10, false);

		DataPool.initMIFHandler();
		
		Installer.initWorkspace();

		welcomescreen.setProgress("Restoring Previous Settings", 20, false);

		// restoring j3d settings
		DataPool.J3DSettings.restoreSettings();

		DataPool.LimitationSettings.restoreSettings();

		DataPool.CoordinateTransformationSettings.restoreSettings();
		
		DataPool.PPESettings.restoreSettings();

		DataPool.InternetSettings.restoreSettings();
		
		DataPool.JmolSettings.restoreSettings();
		
		DataPool.APLVoroSettings.restoreSettings();
		
		DataPool.AlgorithmSettings.restoreSettings();

		ConfigParser.getConfigFile().getParentFile();

		try {
			DataPool.AtomValues.load();
		} catch (final Exception e1) {

		}

		// doing internet settings
		if (DataPool.InternetSettings.proxyuse)
			this.basicOperations.doProxySettings();

		welcomescreen.setProgress("Caching Icons", 30, false);
		Cm2_IconDepot.readIcons();

		welcomescreen.setProgress("Initializing Components", 40, false);
		/*
		 * creating all of our components
		 */

		this.gui = new MainGui();
		this.molEditView = null;
		
		this.componentview = new ComponentView();
		this.databaseview = new DatabaseView();
		
		welcomescreen.setProgress("Check Java3D Version", 40, false);
		this.checkJava3DVersion();
		
		this.membraneview = new MembraneView();

		this.statisticsview = new StatisticsView();
		this.propertiesview = new PropertiesView();

		/*
		 * adding Interfaces to the Components
		 */
		this.gui.addGuiListener(this.basicOperations);
		this.databaseview.addDatabaseViewListener(this.databaseOperations);
		this.membraneview.addMembraneViewListener(this.membraneOperations);
		this.componentview.addMembraneViewListener(this.membraneOperations);

		this.propertiesview.addDatabaseViewListener(this.databaseOperations);
		this.propertiesview.addMembraneListener(this.membraneOperations);
		/*
		 * adding our components to the Gui
		 */
		this.gui.addView("Overview", Cm2_IconDepot
				.getSpecifiedIcon("Frame_Overview"), this.componentview);
		this.gui.addView("Local Database", Cm2_IconDepot
				.getSpecifiedIcon("Frame_Database"), this.databaseview);
		this.gui.addView("Membrane View", Cm2_IconDepot
				.getSpecifiedIcon("Frame_Membrane_View"), this.membraneview);

		this.gui.addView("Statistics", Cm2_IconDepot
				.getSpecifiedIcon("Frame_Statistics"), this.statisticsview);
		this.gui.addView("Properties", Cm2_IconDepot
				.getSpecifiedIcon("Frame_Properties"), this.propertiesview);

		// setting the program's name as title
		this.gui
				.setMainFrameTitle(CM2_CONSTANTS.GENERAL.NAME_OF_THIS_PROGRAM
						+ " "
						+ (CM2_CONSTANTS.GUI_SETTINGS_AND_CONTROLS.DEVELOPER_EDITION ? CM2_CONSTANTS.GENERAL.DEVELOPER_EDITION_NAME
								: CM2_CONSTANTS.GENERAL.STANDARD_EDITION_NAME)
						+ " "
						+ (Cm2_Toolkit.amIRunningOnWebstart() ? CM2_CONSTANTS.GENERAL.WEBSTART_NAME
								: CM2_CONSTANTS.GENERAL.STANDALONE_NAME));

		// tells the gui in which file it may find/write the layout
		this.gui.setLayoutFile(CM2_CONSTANTS.IO.LAYOUT_FILE);

		welcomescreen.setProgress("Building Framework", 60, false);

		// after all views are added, gui may assemble itself

		this.gui.generateGui();

		welcomescreen.setProgress("Reading Algorithms", 80, false);
		this.pluginmanager.init();

		// initializing the database
		welcomescreen.setProgress("Initializing Local PDB Database", 90, false);

		this.basicOperations.initializeDB();

		this.databaseview.configureTables();

		/*
		 * pops up the helper if the program gets started for the very first
		 * time. As soon as the config file exists, this wont happen again
		 */

		SwingUtilities.invokeLater(new Thread() {
			@Override
			public void run() {

				Engine.this.gui.restoreLayout();

				Engine.this.gui.getFrame().setVisible(true);
				SwingUtilities.invokeLater(new Thread() {
					@Override
					public void run() {
						HelpDialog.popUpHelpIfItsApplicationsFirstRun();
					}
				});
			}
		});

		if (file2open == null) {

			final MembraneModel model = new MembraneModel();

			DataPool.replaceMembraneModel(model);
			this.membraneview.setEnabled(true);

			DataPool.setCurrentMembraneFile(null);
			this.gui
			.setMainFrameTitle(CM2_CONSTANTS.GENERAL.NAME_OF_THIS_PROGRAM
					+ " "
					+ (CM2_CONSTANTS.GUI_SETTINGS_AND_CONTROLS.DEVELOPER_EDITION ? CM2_CONSTANTS.GENERAL.DEVELOPER_EDITION_NAME
							: CM2_CONSTANTS.GENERAL.STANDARD_EDITION_NAME)
					+ " "
					+ (Cm2_Toolkit.amIRunningOnWebstart() ? CM2_CONSTANTS.GENERAL.WEBSTART_NAME
							: CM2_CONSTANTS.GENERAL.STANDALONE_NAME));
			this.gui.enableMembraneButtons(true);
			this.gui.enableSaveButtons(true);
			this.databaseview.activateAddOptions(true);
			this.componentview.setEnabled(true);
			if (this.membraneview.getMembrane3D() != null) {
				this.membraneview.getMembrane3D().clearSelection();
				(this.membraneview.getMembrane3D())
						.repaintByMembraneModel(model);
				(this.membraneview.getMembrane3D()).getCamera().setViewByMode(
						Membrane3D.EXTRA, null);
			}

			this.membraneOperations.updateMicroDomainsIn3D();

			this.membraneOperations.highlightMicroDomain(-1);
			this.propertiesview.showMembraneInfo();
			this.membraneOperations.refreshViews(true, true, true, true, true,
					true);
			model.setMembraneHasBeenEditedAgain(false);

		} else
			this.basicOperations.userChoseQuickLoadMembrane(file2open);
				
		welcomescreen.dispose();

	}

	/**
	 * provides the basic file Operations for the application.
	 * 
	 * @author Tim Dingersen dackelnose@gmx.net
	 * 
	 */
	public class BasicOperations implements GuiListener {
	
		/**
		 * Called when user chose to create a new Membrane
		 */
		public void userChoseNewMembrane() {

			final FileIOHandler filehandler = new FileIOHandler(Engine.this.gui
					.getFrame());

			if (filehandler.authorizeDiscargingOfCurrentMembrane() == false)
				return;

			else {

				if (Engine.this.membraneview.isInSpecialMode())
					Engine.this.membraneview.finishSpecialMode(false);

				final MembraneModel model = new MembraneModel();

				if (new NewMembraneDialog(model).popUp() == NewMembraneDialog.CANCEL)
					return;

				DataPool.replaceMembraneModel(model);
				Engine.this.membraneview.setEnabled(true);

				DataPool.setCurrentMembraneFile(null);
				Engine.this.gui
						.setMainFrameTitle(CM2_CONSTANTS.GENERAL.NAME_OF_THIS_PROGRAM);
				Engine.this.gui.enableMembraneButtons(true);
				Engine.this.gui.enableSaveButtons(true);
				Engine.this.databaseview.activateAddOptions(true);
				Engine.this.componentview.setEnabled(true);
				if (Engine.this.membraneview.getMembrane3D() != null) {
					Engine.this.membraneview.getMembrane3D().clearSelection();
					(Engine.this.membraneview.getMembrane3D())
							.repaintByMembraneModel(model);
					(Engine.this.membraneview.getMembrane3D()).getCamera()
							.setViewByMode(Membrane3D.EXTRA, null);
				}

				

				Engine.this.membraneOperations.updateMicroDomainsIn3D();
				Engine.this.membraneOperations.highlightMicroDomain(0);
				Engine.this.membraneOperations.highlightMicroDomain(-1);

				Engine.this.propertiesview.showMembraneInfo();
				Engine.this.membraneOperations.refreshViews(true, true, true,
						true, true, true);

			}
		}

		/**
		 * Called when user chose to load a Membrane
		 */
		public void userChoseLoadMembrane() {

			final FileIOHandler filehandler = new FileIOHandler(Engine.this.gui
					.getFrame());

			if (filehandler.authorizeDiscargingOfCurrentMembrane() == false)
				return;

			final File file = filehandler
					.requestUsertoChooseFile(filehandler.LOADMEMBRANE);

			if (file == null)
				return;

			this.load(filehandler, file);

		}

		/**
		 * Called when user clicked on one of the quickloaditems
		 */
		public void userChoseQuickLoadMembrane(final File file) {
			final FileIOHandler filehandler = new FileIOHandler(Engine.this.gui
					.getFrame());

			if (filehandler.authorizeDiscargingOfCurrentMembrane() == false)
				return;

			this.load(filehandler, file);

		}

		/**
		 * Called when user chose to save a Membrane
		 */
		public void userChoseSaveMembrane() {
			
			if (Engine.this.membraneview.isInSpecialMode()) {
				JOptionPane.showMessageDialog(null,
						"You have to exit molecule alignment first!",
						"Saving not possible", JOptionPane.ERROR_MESSAGE);
				return;
			}

			final FileIOHandler filehandler = new FileIOHandler(Engine.this.gui
					.getFrame());

			final File file = filehandler
					.requestUsertoChooseFile(filehandler.SAVEMEMBRANE);

			if (file == null)
				return;

			if (filehandler.authorizeSaving(file)) {
				DataPool.getMembraneModel().translateAllMolecules(
						-DataPool.getCurrentLayerHeight(), false, false);
				try {

					new MembraneXmlWriter(DataPool.workspace).writeMembrane(
							file, DataPool.getMembraneModel());
					DataPool.setCurrentMembraneFile(file);

					Engine.this.gui
							.setMainFrameTitle(CM2_CONSTANTS.GENERAL.SHORT_NAME_OF_THIS_PROGRAM
									+ " "
									+ DataPool.getCurrentMembraneFile()
											.getName());
					DataPool.setMembraneHasBeenEditedAgain(false);
					Engine.this.gui.addQuickloadLink(file);
					Engine.this.propertiesview.showMembraneInfo();

					final Thread T = new Thread() {
						@Override
						public void run() {

							final StatusDialog dialog = new StatusDialog(false);
							dialog.setProgress("File Saved Successfully!", 0,
									false);
							try {
								Thread.sleep(2000);
							} catch (final InterruptedException e) {
								dialog.dispose();
							}
							dialog.dispose();
						}
					};
					T.setPriority(Thread.MAX_PRIORITY);
					T.start();

				} catch (final IOException e) {
					filehandler.reportMembraneSavingException(e);
				}
				DataPool.getMembraneModel().translateAllMolecules(
						DataPool.getCurrentLayerHeight(), false, false);
			}
		}

		/**
		 * Called when user chose to quicksave current Membrane
		 */
		public void userChoseQuickSaveMembrane() {

			if (Engine.this.membraneview.isInSpecialMode()) {
				JOptionPane.showMessageDialog(null,
						"You have to exit molecule alignment first!",
						"Saving not possible", JOptionPane.ERROR_MESSAGE);
				return;
			}

			final FileIOHandler filehandler = new FileIOHandler(Engine.this.gui
					.getFrame());

			// load the last saved or loaded file
			if (DataPool.getCurrentMembraneFile() != null) {
				
				// save Cm2 file if it has been opened or saved before
				if (DataPool.getCurrentMembraneFile().getName().substring(DataPool.getCurrentMembraneFile().getName().lastIndexOf(".")+1).toLowerCase().equals("cm2")) {

					final File file = DataPool.getCurrentMembraneFile();
					DataPool.getMembraneModel().translateAllMolecules(
							-DataPool.getCurrentLayerHeight(), false, false);
					try {
	
						new MembraneXmlWriter(DataPool.workspace).writeMembrane(
								file, DataPool.getMembraneModel());
						DataPool.setMembraneHasBeenEditedAgain(false);
	
						final Thread T = new Thread() {
							@Override
							public void run() {
	
								final StatusDialog dialog = new StatusDialog(false);
	
								dialog.setProgress("File Saved Successfully!", 0,
										false);
	
								try {
									Thread.sleep(2000);
								} catch (final InterruptedException e) {
									dialog.dispose();
								}
								dialog.dispose();
							}
						};
						T.setPriority(Thread.MAX_PRIORITY);
						T.start();
	
					} catch (final IOException e) {
						filehandler.reportMembraneSavingException(e);
					}
					DataPool.getMembraneModel().translateAllMolecules(
							DataPool.getCurrentLayerHeight(), false, false);
				// save PDB file if it has been opened before
				} else if (DataPool.getCurrentMembraneFile().getName().substring(DataPool.getCurrentMembraneFile().getName().lastIndexOf(".")+1).toLowerCase().equals("pdb")) {
					
					this.userChoseGeneratePDBFile(DataPool.getCurrentMembraneFile());
					
				}
			// if no file to quicksave to is determined yet, normal saving
			// is performed
			} else {
				 if (Engine.this.getCm1TempName2Edit().equals("null")) 
					 this.userChoseSaveMembrane();
				 else
					 this.userChoseGeneratePDBFile(null);
			}

		}
		
		/**
		 * Called when user chose to save current Screenshot.
		 * 
		 * @author Jonas Osterloff
		 */
		public void userChoseSaveImage() {
			if (Engine.this.membraneview.getMembrane3D() != null) {
				if (Engine.this.membraneview.getMembrane3D().getCanvas() != null) {
					
					int width = Engine.this.membraneview.getMembrane3D().getCanvas().getSize().width;
					int height = Engine.this.membraneview.getMembrane3D().getCanvas().getSize().height;
					
					BufferedImage image = null;
					
					if (width > 0 && height > 0) {
						
						// generate the screenshot
						
						// First we create a Canvas3D and specify that it is to be used
						// for offscreen rendering.
						Canvas3D offScreenCanvas3D = new Canvas3D(SimpleUniverse.getPreferredConfiguration() , true );
						
						Engine.this.membraneview.getMembrane3D().getUniverse().getViewer().getView().addCanvas3D(offScreenCanvas3D);

						// We then need to explicitly set the size of the off screen
						// Canvas3D.
						offScreenCanvas3D.getScreen3D().setSize(width, height);

						// This calculation returns the physical size of the screen and
						// is based on 90 display pixels per inch
						offScreenCanvas3D.getScreen3D().setPhysicalScreenHeight(0.0254 / 90 * height);
						offScreenCanvas3D.getScreen3D().setPhysicalScreenWidth(0.0254 / 90 * width);

						// We then create an AWT RenderedImage that the Canvas3D will
						// render into. We create a simple 3 Byte RGB format image.
						RenderedImage renderedImage = new BufferedImage(width,
								height, BufferedImage.TYPE_3BYTE_BGR);

						// The AWT RenderedImage needs to be wrapped in a Java 3D
						// ImageComponent2D before it can be assigned to the
						// Canvas3D for rendering
						ImageComponent2D imageComponent = new ImageComponent2D(ImageComponent.FORMAT_RGB8, renderedImage);

						// This call notifies Java 3D that we require read-access to the
						// ImageComponent2D. We will be reading the pixels in the image
						// when we output it to disk.
						imageComponent.setCapability(ImageComponent2D.ALLOW_IMAGE_READ);

						// Finally, we assign the ImageComponent2D to the offscreen
						// Canvas3D for rendering
						offScreenCanvas3D.setOffScreenBuffer(imageComponent);
						
						
						// Render the offscreen screen shot
						offScreenCanvas3D.renderOffScreenBuffer();
						offScreenCanvas3D.waitForOffScreenRendering();
						image =  offScreenCanvas3D.getOffScreenBuffer().getImage();
						
						
						// save the image
						final FileIOHandler filehandler = new FileIOHandler(Engine.this.gui.getFrame());

						final File file = filehandler.requestUsertoChooseFile(filehandler.SAVEIMAGE);

						if (file == null)
							return;

						if (filehandler.authorizeSaving(file)) {
							try {

								ImageIO.write(image, "jpg", file);
								
								final Thread T = new Thread() {
									@Override
									public void run() {

										final StatusDialog dialog = new StatusDialog(false);
										dialog.setProgress("File Saved Successfully!", 0,
												false);
										try {
											Thread.sleep(2000);
										} catch (final InterruptedException e) {
											dialog.dispose();
										}
										dialog.dispose();
									}
								};
								T.setPriority(Thread.MAX_PRIORITY);
								T.start();

							} catch (final IOException e) {
								filehandler.reportImageSavingException(e);
							}
						}
					}
				}
			}
		}
		
		/**
		 * Called when user chose to save a Time Protocol
		 */
		public void userChoseSaveTimeProtocol(Vector<String> timeProtocol) {
			
			final FileIOHandler filehandler = new FileIOHandler(Engine.this.gui
					.getFrame());

			final File file = filehandler
					.requestUsertoChooseFile(filehandler.SAVEPROTOCOL);

			if (file == null)
				return;
			if (filehandler.authorizeSaving(file)) {
				BufferedWriter bw;
				try {
					// extract a number from the file name for the set index
					//  the numbers of the file are sub by 1, because xmgrace needs 
					//  to start at set 0, with followers +1 each:
					//  New_Protocol -> 0; New_Protocol_2 -> 1; New_Protocol_3 -> 2; etc.
					String fileName2Number = file.getName().substring(0, file.getName().lastIndexOf('.'));
					fileName2Number = fileName2Number.replaceAll("\\D", "");
					if (fileName2Number.isEmpty())
						fileName2Number = "0";
					else if (!fileName2Number.equals("0"))
						fileName2Number = Integer.toString((Integer.parseInt(fileName2Number)-1));
					
					// print the time statistic
					String strTimeStatistic = "# CELLmicrocosmos 2.2 MembraneEditor Time Protocol for xmgrace\n"
										+"# Algorithm: "+DataPool.getMembraneModel().getAlgorithmName()+"\n"
										+(DataPool.getMembraneModel().getLastGeneratedMembraneFile()!=null?
												"# Membrane : "+DataPool.getMembraneModel().getLastGeneratedMembraneFile().toString()+"\n":"")
										+"# Author   : "+DataPool.getMembraneModel().getAuthor()+"\n"
										+"# Protocol :\n"
										+"# "+DataPool.getMembraneModel().getAlgorithmStringRepresentation().replaceAll("\n", "\n# ");
					strTimeStatistic += "\n@ title \"Computation Time\"\n"
										+"@ xaxis  label \"Time (ns)\"\n"
										+"@ yaxis  label \"Total Lipid Number\"\n"
										+"@ TYPE xy\n"
										+"@ view 0.15, 0.15, 0.75, 0.85\n"
										+"@ legend on\n"
										+"@ legend box on\n"
										+"@ legend loctype view\n"
										+"@ legend 0.78, 0.8\n"
										+"@ legend length 2\n"
										+"@ s"+fileName2Number+" legend \""+DataPool.getMembraneModel().getAlgorithmName()+"\"\n";
					
					bw = new BufferedWriter(new FileWriter(file));
					bw.write(strTimeStatistic);
					for(String s:timeProtocol) {
						bw.write(s);
						bw.newLine();
					}
					
					bw.close();
				} catch (IOException e) {
					filehandler.reportProtocolSavingException(e);
				}

//				DataPool.getMembraneModel().translateAllMolecules(
//						-DataPool.getCurrentLayerHeight(), false, false);
//				try {
//
//					new MembraneXmlWriter(DataPool.workspace).writeMembrane(
//							file, DataPool.getMembraneModel());
//					DataPool.setCurrentMembraneFile(file);
//
//					Engine.this.gui
//							.setMainFrameTitle(CM2_CONSTANTS.GENERAL.SHORTNAMEOFTHISPROGRAM
//									+ " "
//									+ DataPool.getCurrentMembraneFile()
//											.getName());
//					DataPool.setMembraneHasBeenEditedAgain(false);
//					Engine.this.gui.addQuickloadLink(file);
//					Engine.this.propertiesview.showMembraneInfo();
//
//					final Thread T = new Thread() {
//						@Override
//						public void run() {
//
//							final StatusDialog dialog = new StatusDialog(false);
//							dialog.setProgress("File Saved Successfully!", 0,
//									false);
//							try {
//								Thread.sleep(2000);
//							} catch (final InterruptedException e) {
//								dialog.dispose();
//							}
//							dialog.dispose();
//						}
//					};
//					T.setPriority(Thread.MAX_PRIORITY);
//					T.start();
//
//				} catch (final IOException e) {
//					filehandler.reportMembraneSavingException(e);
//				}
//				DataPool.getMembraneModel().translateAllMolecules(
//						DataPool.getCurrentLayerHeight(), false, false);
			}
		}

		/**
		 * Called when user chose to show the current Algorithm Time Protocol
		 */
		public void userChoseShowTimeProtocol() {
			
			new ProtocolDialog();
			
		}
		
		/**
		 * Called at startup. Makes the Database start with the proper contents
		 * 
		 */
		private void initializeDB() {

			// trying to retrieve the file of the DB last used
			try {
				Engine.this.databaseview.setCurrentFile(new File(ConfigParser
						.getValue("currentDatabase")[0]));
			} catch (final Exception e) {
				// if not available, setting DB to the Default DB
				Engine.this.databaseview
						.setCurrentFile(DataPool.IO.Default_DB_File);

			}

			final FileIOHandler filehandler = new FileIOHandler(Engine.this.gui
					.getFrame());

			// trying to load the xml that is set to DB
			try {
				final Document data = new DatabaseIO()
						.loadFromFile(Engine.this.databaseview.getCurrentFile());
				Engine.this.databaseview.setContents(data);
			} catch (final Exception e) {
				// show an Error only if the loaded File wasnt the default DB
				if (Engine.this.databaseview.getCurrentFile().equals(
						DataPool.IO.Default_DB_File) == false)
					filehandler.reportDBLoadingException(e);

				// now, after last DB cannot be loaded, default DB File is
				// definitly set
				Engine.this.databaseview
						.setCurrentFile(DataPool.IO.Default_DB_File);

				// trying to load again
				try {
					final Document data = new DatabaseIO()
							.loadFromFile(Engine.this.databaseview
									.getCurrentFile());
					Engine.this.databaseview.setContents(data);

				} catch (final Exception e1) {
					/*
					 * nothing to do, if even default DB cannot be opened, no
					 * contents are set and DB start empty
					 */
				}

			}

			if (!CM2_CONSTANTS.IO.CONFIG_FILE.exists()
					&& Engine.this.databaseview.isCurrentlyEmpty()) {
				Engine.this.databaseview.autofill();
			}
			Engine.this.databaseview.scr2.setVisible(false);

		}

		/**
		 * Called when user chooses to create a new DataBase
		 */
		public void userChoseNewDB() {

			final FileIOHandler filehandler = new FileIOHandler(Engine.this.gui
					.getFrame());
			// saving the old one automatically first
			final File dbfile = Engine.this.databaseview.getCurrentFile();
			final Document dbdata = Engine.this.databaseview.getContents();
			try {
				new DatabaseIO().writeToFile(dbfile, dbdata);

			} catch (final IOException e) {
				// return if old one shouldnt be discarded
				if (filehandler.reportDBSavingException(e) == JOptionPane.NO_OPTION)
					return;

			}

			final File file = filehandler
					.requestUsertoChooseFile(filehandler.NEWDATABASE);

			if (file == null)
				return;

			if (filehandler.authorizeSaving(file))
				try {
					/*
					 * creating and saving an Empty but valid DB file. If any
					 * trouble occur writing a file to destination, user will be
					 * informed now and not bothered later
					 */
					final Document data = DatabaseIO.getAnEmptyDocument();

					new DatabaseIO().writeToFile(file, data);
					Engine.this.databaseview.setCurrentFile(file);
					Engine.this.databaseview.resetModels();
					Engine.this.databaseview.setContents(data);

				} catch (final IOException e) {
					filehandler.reportDBSavingException(e);

				}

		}

		/**
		 * called when user chooses to load a DB
		 */
		public void userchoseLoadDB() {

			final FileIOHandler filehandler = new FileIOHandler(Engine.this.gui
					.getFrame());

			final File dbfile = Engine.this.databaseview.getCurrentFile();
			final Document dbdata = Engine.this.databaseview.getContents();
			try {
				new DatabaseIO().writeToFile(dbfile, dbdata);

			} catch (final IOException e) {

				if (filehandler.reportDBSavingException(e) == JOptionPane.NO_OPTION)
					return;

			}

			final File file = filehandler
					.requestUsertoChooseFile(filehandler.LOADDATABASE);

			if (file == null)
				return;

			try {
				final Document data = new DatabaseIO().loadFromFile(file);
				Engine.this.databaseview.setContents(data);
				Engine.this.databaseview.setCurrentFile(file);
			} catch (final Exception e) {
				e.printStackTrace();
				filehandler.reportDBLoadingException(e);
			}
		}

		/**
		 * Called when User chooses to generate a PDB file
		 * (Uses the current membrane if the last opened file was a restored PDB file
		 * called when user chooses to generate the Membrane
		 */
		public void userChoseGeneratePDBFile(File pdbFile) {

			final int highlightedDomain = Engine.this.componentview
					.getHighlightedDomainID();

			if (!DataPool.getMembraneModel().areaContainsMolecules(
					highlightedDomain, DataPool.getCurrentLayerID())) {

				JOptionPane.showMessageDialog(null,
						"The selected area is empty!", "Unable to Proceed",
						JOptionPane.ERROR_MESSAGE);

				return;
			}

		
				
			

			if (DataPool.getMembraneModel().anyCurrentIntersections(true)) {

				final int val = JOptionPane
						.showOptionDialog(
								null,
								"Some molecules are currently intersecting!\n\nDo you wish to proceed?"

								, "Current Intersections Detected!",
								JOptionPane.YES_NO_OPTION,
								JOptionPane.WARNING_MESSAGE, null,
								new String[] { "Proceed", "Cancel" }, null);

				if (val != JOptionPane.OK_OPTION)
					return;

			}

		

			final int layerID;

			if (highlightedDomain >= 0
					&& DataPool.getMembraneModel().getAllMicroDomains(
							DataPool.getCurrentLayerID()).length > 0) {
				layerID = DataPool.getCurrentLayerID();
				final int val = JOptionPane
						.showOptionDialog(
								null,
								"You are about to write only the contents of the highlighted part\n"
										+ "of the membrane! If you intend otherwise, you must switch your\n"
										+ "selection to 'All Areas' !"

								, "Information!", JOptionPane.YES_NO_OPTION,
								JOptionPane.WARNING_MESSAGE, null,
								new String[] { "Proceed", "Cancel" }, null);

				if (val != JOptionPane.OK_OPTION)
					return;

			}

			else if (DataPool.getMembraneModel().hasMultipleLayers()
					&& highlightedDomain < 0) {

				final int val = JOptionPane.showOptionDialog(null,
						"Do you want to consider all layers or\n"
								+ "just the one that is active currently?\n"

						, "Multiple Layers", JOptionPane.YES_NO_OPTION,
						JOptionPane.QUESTION_MESSAGE, null, new String[] {
								"All Layers", "Only the Active One" }, null);

				if (val == JOptionPane.OK_OPTION)
					layerID = -1;
				else
					layerID = DataPool.getCurrentLayerID();

			} else {
				layerID = -1;
			}

			
			// This method is used in conjunction with Tim Dingersen's original PDBWriter. 
			// It is still contained out-commented in the new PDBWriter class. 
//			if (PDBWriter.coordinatesWouldExceedValidNumberRangeCurrently(highlightedDomain,layerID,DataPool.CoordinateTransformationSettings.shiftXZ,DataPool.CoordinateTransformationSettings.shiftY,DataPool.CoordinateTransformationSettings.swapYZ,DataPool.PPESettings.saveOPMDummyLayersInPDB)) {
//				JOptionPane
//						.showMessageDialog(
//								null,
//								"Some atom coordinate numbers would be outside the valid number range!\n" +
//								"(that is -999.999 to 1000.000)\n\n"+
//								"Please remove all current border intersections and, if neccessary,\n" +
//								"choose different coordinate transformation settings!","Unable to proceed",
//								JOptionPane.ERROR_MESSAGE);
//
//				return;
//			}
			
			
			final FileIOHandler filehandler = new FileIOHandler(Engine.this.gui
					.getFrame());

			final File file = (pdbFile==null)?
								filehandler.requestUsertoChooseFileForGeneratingMembrane():
								pdbFile;

			if (file == null)
				return;

			if (pdbFile == null && filehandler.authorizeSaving(file) == false)
				return;

			final StatusDialog dialog = new StatusDialog(true);

			final Thread T = new Thread() {
				
				public void run() {
					PDBWriter writer = null;
					try {

						writer = new PDBWriter(file, dialog);
						CoordinateTransformationHints hints=new CoordinateTransformationHints();
						hints.shiftXZ=DataPool.CoordinateTransformationSettings.shiftXZ;
						hints.shiftY=DataPool.CoordinateTransformationSettings.shiftY;
						hints.swapYZ=DataPool.CoordinateTransformationSettings.swapYZ;
						hints.sortByType=DataPool.CoordinateTransformationSettings.sortByType;
						
						writer.writeMembrane(highlightedDomain, layerID,hints);
						dialog.dispose();
						if(DataPool.LimitationSettings.reverseParsing) {
							DataPool.setCurrentMembraneFile(file.getAbsoluteFile());
							Engine.this.gui.setMainFrameTitle(CM2_CONSTANTS.GENERAL.SHORT_NAME_OF_THIS_PROGRAM
																+ " "
																+ DataPool.getCurrentMembraneFile()
																		.getName());
						}
						if (Engine.getMRNA()!=null) {
							Engine.getMRNA().refreshMembranesInCm1(DataPool.getCurrentMembraneFile().getAbsolutePath(), Engine.this.getCm1TempName2Edit());
							Engine.this.setCm1TempName2Edit("null");
						}
						
					} catch (final Exception e) {
						e.printStackTrace();
						dialog.dispose();
						filehandler.reportMembraneGeneratingException(e);
//						if (writer != null)
//							writer.closeBufferedWriter();
						return;
					}

				}
			};

			T
					.setPriority(CM2_CONSTANTS.THREADS.PRIORITY_GENERATE_MEMBRANE_THREAD);
			T.start();

		}

		/**
		 * exits the Application, and causes the current settings to be saved
		 * 
		 */
		@SuppressWarnings("deprecation")
		public synchronized void exitApplication() {

			if(Engine.this.manager!=null) {
				if(Engine.this.manager.T!=null&&Engine.this.manager.T.isAlive()) {
					return;
				}
			}
			
			final FileIOHandler filehandler = new FileIOHandler(Engine.this.gui
					.getFrame());

			if (filehandler.authorizeDiscargingOfCurrentMembrane() == false)
				return;

			final File dbfile = Engine.this.databaseview.getCurrentFile();
			final Document dbdata = Engine.this.databaseview.getContents();
			try {
				new DatabaseIO().writeToFile(dbfile, dbdata);

			} catch (final IOException e) {
				if (filehandler.reportDBSavingException(e) == JOptionPane.NO_OPTION)
					return;

			}

			
			Engine.this.databaseview.storeSizes();
			Engine.this.componentview.storeCheckBoxValues();
			ConfigParser.deleteAllAt("currentDatabase");
			ConfigParser.addValue("currentDatabase", Engine.this.databaseview
					.getCurrentFile().getPath());
			// moleculeview.storeSettings();
			DataPool.J3DSettings.storeSettings();
			Engine.this.membraneview.storeSettings();
			DataPool.LimitationSettings.storeSettings();
			DataPool.CoordinateTransformationSettings.storeSettings();
			DataPool.PPESettings.storeSettings();
			DataPool.InternetSettings.storeSettings();
			DataPool.JmolSettings.storeSettings();
			DataPool.APLVoroSettings.storeSettings();
			DataPool.AlgorithmSettings.storeSettings();
			try {
				ConfigParser.writeDataToDisc();

			} catch (final Exception e) {
				// if the data cant be saved, programm will restart with
				// defaults next time
				e.printStackTrace();
				System.err.println("can't save Config File!");
			}

			new File(CM2_CONSTANTS.IO.TEMP_DIRECTORY + File.separator
					+ CM2_CONSTANTS.IO.TEMP_MEMBRANE_FILE.getName()).deleteOnExit();
			
			new File(FileIOHandler.getTempJar()).deleteOnExit();

			// Exit the standalone application
			if (Engine.getOwnThread()==null)
				System.exit(0);
			// Exit the external started application
			else {
				// suspend the thread (maybe this is not needed)
				Engine.getOwnThread().interrupt();
//				Engine.getOwnThread().stop();
				// TODO Find here a better way instead of the deprecated suspend?
				Engine.getOwnThread().suspend();
			
				// clear MembraneView
				if (Engine.this.membraneview.isInSpecialMode())
					Engine.this.membraneview.finishSpecialMode(false);
				final MembraneModel model = new MembraneModel();
				DataPool.replaceMembraneModel(model);
				Engine.this.membraneview.setEnabled(true);
				DataPool.setCurrentMembraneFile(null);
				if (Engine.this.membraneview.getMembrane3D() != null) {
					Engine.this.membraneview.getMembrane3D().clearSelection();
					(Engine.this.membraneview.getMembrane3D())
							.repaintByMembraneModel(model);
					(Engine.this.membraneview.getMembrane3D()).getCamera()
							.setViewByMode(Membrane3D.EXTRA, null);
				}			
				Engine.this.membraneOperations.updateMicroDomainsIn3D();
				Engine.this.membraneOperations.highlightMicroDomain(0);
				Engine.this.membraneOperations.highlightMicroDomain(-1);
				Engine.this.propertiesview.showMembraneInfo();
				Engine.this.membraneOperations.refreshViews(true, true, true,
						true, true, true);
				Engine.this.membraneview.removeAll();
				Engine.this.gui.getFrame().removeAll();
				Engine.this.gui.getFrame().invalidate();
				Engine.this.gui.getFrame().dispose();
				Engine.this.setActive(false);
				
				// ask for garbage collection
				System.gc();
				
				notify();
			}

		}

		/**
		 * Invoked when user wants to open the SearchView
		 */
		public void userClickedOpenRCSB() {
			Engine.this.databaseOperations.userOpensGlobalDBView();

		}
		
		/**
		 * performs loading in a new Thread
		 * 
		 * @param filehandler
		 * @param file
		 */
		private void load(final FileIOHandler filehandler, final File file) {
			final Thread T = new Thread() {
				@Override
				public void run() {
					DataPool.J3DSettings.overallRenderMode = CM2_CONSTANTS.RENDERMODE.RENDER_MODE_POLYGON;

					if (Engine.this.membraneview.isInSpecialMode())
						Engine.this.membraneview.finishSpecialMode(false);

					// moleculeview.setEnabled(false);
					final StatusDialog dialog = new StatusDialog(true);

					MembraneModel loadedmembrane;
					final MembraneXmlReader reader = new MembraneXmlReader(
							dialog);
					try {
					
					
//						DataPool.J3DSettings.overallRenderMode = Molecule_3D.RENDER_MODE_POLYGON;
//
//						if (Engine.this.membraneview.isInSpecialMode())
//							Engine.this.membraneview.finishSpecialMode(false);
							
						if (file.getName().toLowerCase().contains(".xml") ||
							file.getName().toLowerCase().contains(".cm2")) {
							try {
								reader.readMembrane(file);
							} catch (final InvalidEntriesException ipe) {
								filehandler.reportMembraneLoadingException(ipe);
							}
							loadedmembrane = reader.getResultingMembraneModel();
							
						} else if (file.getName().toLowerCase().contains(".pdb")) {
							
							final ReversePDBParser parser = new ReversePDBParser(new File(
									FileIOHandler.home),true);
							loadedmembrane = parser.restoreMembraneModelFromPdbFile(file);
							if(loadedmembrane==null)throw new Exception("CANCELLED");
							else DataPool.setCurrentMembraneFile(file);
							
						} else {
							return;
						}

						dialog
								.setProgress("Refreshing Views...", 100,
										true);
						
						DataPool.replaceMembraneModel(loadedmembrane);
						DataPool.setCurrentMembraneFile(file);

						Engine.this.gui.addQuickloadLink(file);
						Engine.this.gui
								.setMainFrameTitle(CM2_CONSTANTS.GENERAL.SHORT_NAME_OF_THIS_PROGRAM
										+ " "
										+ DataPool.getCurrentMembraneFile()
												.getName());

						Engine.this.membraneview.setEnabled(true);

						Engine.this.gui.enableMembraneButtons(true);
						Engine.this.gui.enableSaveButtons(true);
						Engine.this.databaseview.activateAddOptions(true);

						Engine.this.componentview.setEnabled(true);

						if (Engine.this.membraneview.getMembrane3D() != null)
							Engine.this.membraneview.getMembrane3D()
									.clearSelection();

						DataPool.getMembraneModel().organizeDomains();
						

						// membraneview.getMembrane3D().getMicroDomainManager()
						// .updateDisplayOfMicroDomains();

						(Engine.this.membraneview.getMembrane3D())
								.repaintByMembraneModel(DataPool
										.getMembraneModel());
						(Engine.this.membraneview.getMembrane3D())
								.setViewByMode(Membrane3D.EXTRA, null);
						DataPool.setMembraneHasBeenEditedAgain(false);

						// if (DataPool.J3DSettings.enableFastTransitions)
						// membraneview.getMembrane3D().dockScreen(false);

						Engine.this.membraneOperations.refreshViews(true, true,
								true, true, false, true);

						// final Vector<Molecule_3D>collided =
						// DataPool.getMembraneModel().getCurrentIntersections();

						Engine.this.propertiesview.showMembraneInfo();
						dialog.dispose();

						SwingUtilities.invokeLater(new Thread() {
							@Override
							public void run() {
								Engine.this.membraneview.switchToLayer(0);
								Engine.this.membraneOperations
										.highlightMicroDomain(0);
								Engine.this.membraneOperations
										.highlightMicroDomain(-1);
							}
						});

						// if (DataPool.J3DSettings.enableFastTransitions)
						// membraneview.getMembrane3D().dockScreen(true);

					} catch (final Exception e) {
						e.printStackTrace();
						filehandler.reportMembraneLoadingException(e);

					}

				}
			};

			T.setPriority(CM2_CONSTANTS.THREADS.PRIORITY_LOAD_MEMBRANE_THREAD);
			T.start();
		}

		/**
		 * Called, when its necessary to rebuild all shapes in the mebrane, for
		 * instance when another detail level is chosen. Saves the current
		 * Membrane into a tempfile, and reloads this one, without setting it as
		 * a new one. Delets the tempfile afterwards.
		 * 
		 * 
		 */
		public void reloadGeometry() {
			final boolean membraneWasEditedBefore = DataPool
					.getMembraneHasBeenEditedAgain();
			final File file = new File(CM2_CONSTANTS.IO.TEMP_DIRECTORY
					+ File.separator
					+ CM2_CONSTANTS.IO.TEMP_MEMBRANE_FILE.getName());
			DataPool.getMembraneModel().translateAllMolecules(
					-DataPool.getCurrentLayerHeight(), false, false);
			try {
				new MembraneXmlWriter(DataPool.workspace).writeMembrane(file,
						DataPool.getMembraneModel());
				DataPool.getMembraneModel().translateAllMolecules(
						DataPool.getCurrentLayerHeight(), false, false);
			} catch (final IOException e) {
				JOptionPane.showMessageDialog(Engine.this.gui.getFrame(),
						"Rebuilding of the Membrane Failed!\n\nReason:\n"
								+ e.toString(), "Error",
						JOptionPane.ERROR_MESSAGE);
				DataPool.getMembraneModel().translateAllMolecules(
						DataPool.getCurrentLayerHeight(), false, false);
				return;
			}

			final Thread T = new Thread() {
				@Override
				public void run() {

					final StatusDialog dialog = new StatusDialog(true);

					MembraneModel loadedmembrane;
					final MembraneXmlReader reader = new MembraneXmlReader(
							dialog);
					try {
						try {

							reader.readMembrane(file);

						} catch (final InvalidEntriesException ipe) {
							JOptionPane.showMessageDialog(Engine.this.gui
									.getFrame(),
									"Rebuilding of the Membrane Failed!\n\nReason:\n"
											+ ipe.getMessageString(), "Error",
									JOptionPane.ERROR_MESSAGE);

						}

						SwingUtilities.invokeLater(new Thread() {
							@Override
							public void run() {
								dialog.setProgress("Refreshing Views...", 100,
										false);
							}
						});

						// if (DataPool.J3DSettings.enableFastTransitions)
						// membraneview.getMembrane3D().dockScreen(false);

						loadedmembrane = reader.getResultingMembraneModel();

						DataPool.replaceMembraneModel(loadedmembrane);

						Engine.this.componentview.setEnabled(true);
						// moleculeview.setEnabled(false);
						if (Engine.this.membraneview.getMembrane3D() != null)
							Engine.this.membraneview.getMembrane3D()
									.clearSelection();
						
						Engine.this.membraneOperations.refreshViews(false,
								false, false, false, false, true);

						(Engine.this.membraneview.getMembrane3D())
								.repaintByMembraneModel(DataPool
										.getMembraneModel());

						DataPool
								.setMembraneHasBeenEditedAgain(membraneWasEditedBefore);

						file.delete();

						dialog.dispose();

						Engine.this.membraneview.switchToLayer(0);
						if (Engine.this.membraneview.getMembrane3D()
								.getPickNDragManager().getSelected().isEmpty())
							Engine.this.membraneOperations
									.highlightMicroDomain(-1);
						else {
							final Molecule_3D selected = Engine.this.membraneview
									.getMembrane3D().getPickNDragManager()
									.getSelected().firstElement();
							Engine.this.membraneOperations
									.highlightMicroDomain(DataPool
											.getMembraneModel()
											.getMicroDomainAtPoint(
													new Point(
															(int) selected
																	.getPosition().x,
															(int) selected
																	.getPosition().z),
													selected.getLayerID()));
						}

						// if (DataPool.J3DSettings.enableFastTransitions)
						// membraneview.getMembrane3D().dockScreen(true);

						/** @TODO test and insert again */

					} catch (final Exception e) {
						dialog.dispose();
						e.printStackTrace();
						JOptionPane.showMessageDialog(Engine.this.gui
								.getFrame(),
								"Rebuilding of the Membrane Failed!\n\nReason:\n"
										+ e.toString(), "Error",
								JOptionPane.ERROR_MESSAGE);
						return;

					}

				}
			};

			T.setPriority(CM2_CONSTANTS.THREADS.PRIORITY_LOAD_MEMBRANE_THREAD);
			T.start();
		}

		public void doProxySettings() {

			DataPool.InternetSettings.doProxySettings();

		}

		public void popUpAttrsDialog() {

			new ManageTitleSectionDialog().popUp();

			Engine.this.statisticsview.updateMembraneStatistics(DataPool
					.getMembraneModel(), Engine.this.membraneOperations
					.getHighlightedMicroDomain());
			if (Engine.this.propertiesview.isMembraneAttrsShowing())
				Engine.this.propertiesview.showProperties(null);

		}

		public void userChosePreviewMembrane(final int mode) {

			final StatusDialog dialog = new StatusDialog(true);
			
			
			dialog.setProgress("Checking Area", 0, true);
			
			final int highlightedDomain = Engine.this.componentview
					.getHighlightedDomainID();
			if (!DataPool.getMembraneModel().areaContainsMolecules(
					highlightedDomain, DataPool.getCurrentLayerID())) {

				JOptionPane.showMessageDialog(null,
						"The selected Area is Empty!", "Unable to proceed",
						JOptionPane.ERROR_MESSAGE);

				dialog.dispose();
				
				return;
			}

			
			dialog.setProgress("Checking Area", 0, true);
			
			
			// This method is used in conjunction with Tim Dingersen's original PDBWriter. 
			// It is still contained out-commented in the new PDBWriter class.
//			if (PDBWriter.coordinatesWouldExceedValidNumberRangeCurrently(highlightedDomain,DataPool
//					.getCurrentLayerID(),DataPool.CoordinateTransformationSettings.shiftXZ,DataPool.CoordinateTransformationSettings.shiftY,DataPool.CoordinateTransformationSettings.swapYZ,DataPool.PPESettings.saveOPMDummyLayersInPDB)) {
//				JOptionPane
//						.showMessageDialog(
//								null,
//								"Some atom coordinate numbers would be outside the valid number range!\n" +
//								"(that is -999.999 to 1000.000)\n\n"+
//								"Please remove all current border intersections and, if neccessary,\n" +
//								"choose different coordinate transformation settings!","Unable to proceed",
//								JOptionPane.ERROR_MESSAGE);
//				
//				dialog.dispose();
//
//				return;
//			}
			
			
			dialog.setProgress("Preparing File", 0, true);
			
			final File file = new File(CM2_CONSTANTS.IO.TEMP_DIRECTORY
					+ File.separator
					+ CM2_CONSTANTS.IO.TEMP_MEMBRANE_FILE.getName());

			
			final Thread t = new Thread() {

				public void run() {

					PDBWriter writer = null;
					try {
						
						CoordinateTransformationHints hints=new CoordinateTransformationHints();
						hints.shiftXZ=DataPool.CoordinateTransformationSettings.shiftXZ;
						hints.shiftY=DataPool.CoordinateTransformationSettings.shiftY;
						hints.swapYZ=DataPool.CoordinateTransformationSettings.swapYZ;
						hints.sortByType=DataPool.CoordinateTransformationSettings.sortByType;
						writer = new PDBWriter(file, dialog);
						
						boolean writingSucceeded = false;
						
						if((DataPool.getMembraneModel().hasMultipleLayers()||DataPool.getMembraneModel().countMicroDomains()>0)) {
							if (JOptionPane.showConfirmDialog(
											null,
											"Do you want to preview the whole membrane?",
											"Whole Membrane",
											JOptionPane.YES_NO_OPTION,
											JOptionPane.WARNING_MESSAGE) == JOptionPane.NO_OPTION)
								writingSucceeded = writer.writeMembrane(highlightedDomain, DataPool
													.getCurrentLayerID(),hints);
							else
								writingSucceeded = writer.writeMembrane(-1, -1,hints);
						}else {
							writingSucceeded = writer.writeMembrane(-1, -1,hints);
						}
						dialog.dispose();
						
						if (writingSucceeded)
							if (mode==1)
								DataPool.openPdbFileBrowser(file);
							else
								Engine.openJmolWithFile(file);
					
					} catch (final Exception e) {
						e.printStackTrace();
						dialog.dispose();
						final FileIOHandler filehandler = new FileIOHandler();
						filehandler.reportMembraneGeneratingException(e);
//						if (writer != null)
//							writer.closeBufferedWriter();
						return;
					}
				}
			};
			t.setPriority(Thread.MAX_PRIORITY);
			t.start();
		}
		
		public void userChoseAnalyzeMembrane() {

			if (APLVoroSettings.aplVoroExecutionLink == null || APLVoroSettings.aplVoroExecutionLink.isEmpty())
				return;
			
			final StatusDialog dialog = new StatusDialog(true);
			
			
			dialog.setProgress("Checking Area", 0, true);
			
			final int highlightedDomain = Engine.this.componentview
					.getHighlightedDomainID();
			if (!DataPool.getMembraneModel().areaContainsMolecules(
					highlightedDomain, DataPool.getCurrentLayerID())) {

				JOptionPane.showMessageDialog(null,
						"The selected Area is Empty!", "Unable to proceed",
						JOptionPane.ERROR_MESSAGE);

				dialog.dispose();
				
				return;
			}

			
			dialog.setProgress("Checking Area", 0, true);
			
			
			// This method is used in conjunction with Tim Dingersen's original PDBWriter. 
			// It is still contained out-commented in the new PDBWriter class.
//			if (PDBWriter.coordinatesWouldExceedValidNumberRangeCurrently(highlightedDomain,DataPool
//					.getCurrentLayerID(),DataPool.CoordinateTransformationSettings.shiftXZ,DataPool.CoordinateTransformationSettings.shiftY,DataPool.CoordinateTransformationSettings.swapYZ,DataPool.PPESettings.saveOPMDummyLayersInPDB)) {
//				JOptionPane
//						.showMessageDialog(
//								null,
//								"Some atom coordinate numbers would be outside the valid number range!\n" +
//								"(that is -999.999 to 1000.000)\n\n"+
//								"Please remove all current border intersections and, if neccessary,\n" +
//								"choose different coordinate transformation settings!","Unable to proceed",
//								JOptionPane.ERROR_MESSAGE);
//				
//				dialog.dispose();
//
//				return;
//			}
			
			
			dialog.setProgress("Preparing File", 0, true);
			
			final File pdbFile = new File(CM2_CONSTANTS.IO.TEMP_DIRECTORY
					+ File.separator
					+ CM2_CONSTANTS.IO.TEMP_MEMBRANE_FILE.getName());
			
			final File aplFile = new File(CM2_CONSTANTS.IO.TEMP_DIRECTORY
					+ File.separator
					+ CM2_CONSTANTS.IO.TEMP_APLVORO_FILE.getName());

			
			final Thread t = new Thread() {

				public void run() {

					PDBWriter writer = null;
					try {
						
//						public Boolean assignResSeqNumbers() {
//							return true;
//						}

						CoordinateTransformationHints hints=new CoordinateTransformationHints();
						hints.shiftXZ=Compatibility.TRANSFORM_NONE;
						hints.shiftY=DataPool.CoordinateTransformationSettings.shiftY;
						hints.swapYZ=true;
						hints.sortByType=true;
						hints.exportAPLVoroFile=true;
						writer = new PDBWriter(pdbFile, dialog);
						
						boolean writingSucceeded = false;
						
						if((DataPool.getMembraneModel().hasMultipleLayers()||DataPool.getMembraneModel().countMicroDomains()>0)) {
							if (JOptionPane.showConfirmDialog(
											null,
											"Do you want to preview the whole membrane?",
											"Whole Membrane",
											JOptionPane.YES_NO_OPTION,
											JOptionPane.WARNING_MESSAGE) == JOptionPane.NO_OPTION)
								writingSucceeded = writer.writeMembrane(highlightedDomain, DataPool
													.getCurrentLayerID(),hints);
							else
								writingSucceeded = writer.writeMembrane(-1, -1,hints);
						}else {
							writingSucceeded = writer.writeMembrane(-1, -1,hints);
						}
						dialog.dispose();
						
						if (writingSucceeded)
							Engine.openAPLVoroWithFile(pdbFile, aplFile);
					
					} catch (final Exception e) {
						e.printStackTrace();
						dialog.dispose();
						final FileIOHandler filehandler = new FileIOHandler();
						filehandler.reportMembraneGeneratingException(e);
//						if (writer != null)
//							writer.closeBufferedWriter();
						return;
					}
				}
			};
			t.setPriority(Thread.MAX_PRIORITY);
			t.start();
		}

		public void userClickedOpenExternal() {
			final FileIOHandler filehandler = new FileIOHandler(Engine.this.gui
					.getFrame());
			final File file = filehandler.askPDBFileForExternalPreview();
			if (file != null)
				Engine.this.databaseOperations.userWantToPreviewMolecule(file,
						0);

		}

		public void userClicked3DSettings(final Point location) {
			Engine.this.membraneOperations.popup3DSettingsDialog(location);

		}

		public void userClickedEditorMode(final boolean bright_App) {
			DataPool.J3DSettings.bright_appearance = bright_App;
			if (bright_App) {
				MainGui.setBrightLookAndFeel();
			} else
				MainGui.setDarkLookAndFeel();

			(Engine.this.membraneview.getMembrane3D())
					.setBrightEditorAppearance(bright_App);
			// moleculeview.setBrightEditorAppearance(bright_App);
			this.updateTextColors();
		}
		
		public void userClickedUniformBackground() {
			(Engine.this.membraneview.getMembrane3D())
				.setBrightEditorAppearance(DataPool.J3DSettings.bright_appearance);
		}

		private void updateTextColors() {
			if (DataPool.useSystemLookandFeel)
				return;

			
			Engine.this.statisticsview.updateColors();
			// SwingUtilities.invokeLater(new Thread() {
			// public void run() {
			Engine.this.componentview.updateColors();
			// }
			// });

		}

		public void popUpChangeSize(final Point locationOnScreen) {

			Molecule_3D[]mols=DataPool.getMembraneModel().getAllMoleculesInMembrane();
			
			Dimension oldSize=new Dimension(DataPool.getMembraneModel().getSize().width,DataPool.getMembraneModel().getSize().height);
			Point3f[] oldPositions=new Point3f[mols.length];
			Shape[] oldMDGeometries=new Shape[DataPool.getMembraneModel().countMicroDomains()];
			
			for(int i=0;i<mols.length;i++) {
				oldPositions[i]=new Point3f(mols[i].getPosition());
			}
			for(int i=0;i<oldMDGeometries.length;i++) {
				if(DataPool.getMembraneModel().getMicroDomain(i).geometry==MicroDomain_3D.RECTANGULAR) {
					final Rectangle geo = (Rectangle) DataPool.getMembraneModel().getMicroDomain(i).geoObject;
					oldMDGeometries[i]=new Rectangle(geo.x,geo.y,geo.width,geo.height);
				}else if(DataPool.getMembraneModel().getMicroDomain(i).geometry==MicroDomain_3D.ELLIPSE) {
					final Ellipse2D.Double geo = (Ellipse2D.Double) DataPool.getMembraneModel().getMicroDomain(i).geoObject;
					oldMDGeometries[i]=new Ellipse2D.Double(geo.x,geo.y,geo.width,geo.height);
				}else {
					final Polygon poly = (Polygon) DataPool.getMembraneModel().getMicroDomain(i).geoObject;
					
					Polygon newPoly=new Polygon();
					newPoly.xpoints=new int[poly.xpoints.length];
					newPoly.ypoints=new int[poly.ypoints.length];
					newPoly.npoints=poly.npoints;
					
					for(int j=0;j<poly.xpoints.length;j++) {
						newPoly.xpoints[j]=poly.xpoints[j];
						
					}
					for(int j=0;j<poly.ypoints.length;j++) {
						newPoly.ypoints[j]=poly.ypoints[j];
						
					}
					newPoly.invalidate();
					
					oldMDGeometries[i]=newPoly;
					
				}
				
			}
			
			final ChangeSizeDialog dia = new ChangeSizeDialog(locationOnScreen,Engine.this,oldPositions,oldMDGeometries,oldSize);
			final int val = dia.popUp();
			if (val == ChangeSizeDialog.APPROVE) {
				if(dia.checkAfterwards()) {
					DataPool.getMembraneModel().reshapeDomains();
					DataPool.getMembraneModel().organizeDomains();
					Engine.this.membraneOperations.refreshViews(true, true, true, true, true, true);
					Engine.this.membraneOperations.updateMicroDomainsIn3D();
					Engine.this.statisticsview.updateMembraneStatistics(DataPool
							.getMembraneModel(), Engine.this.membraneOperations
							.getHighlightedMicroDomain());
					DataPool.setMembraneHasBeenEditedAgain(true);
					if (Engine.this.propertiesview.isMembraneAttrsShowing())
						Engine.this.propertiesview.showProperties(null);
					
				}else {
					dia.discardChanges();
				}
				
				
			}else {
				dia.discardChanges();


			
				
				

				
				
			}

		}

		public void setAllEnabled(final boolean b) {

			Engine.this.databaseview.setEnabled(b);
			Engine.this.componentview.setEnabled(b);
			Engine.this.membraneview.getMembrane3D().getPickNDragManager()
					.setActive(b);
			Engine.this.membraneview.setToolbarsEnabled(b);

			Engine.this.gui.setEnabled(b);

		}

		public void userClickedOpenRestrictionDialog() {
			DataPool.openPdbSettingsWindow(Engine.this.membraneOperations);

		}

		public void startDomainMode(final MicroDomain_3D domainToChange) {
			Engine.this.gui.restoreView(2);
			Engine.this.gui.focusView(2);
			Engine.this.membraneview.startDomainMode(domainToChange);

		}

		public void userChangedStereoEnable(final boolean stereoEnabled) {
			Engine.this.membraneview.getMembrane3D().getCanvas()
					.setStereoEnable(stereoEnabled);
		}

		public void userChangedStereoSlider() {
			Engine.this.gui.getStereoSliderPosition().setText(
					Engine.this.j3doperations.getStereoValueString());
			Engine.this.membraneOperations.autoFocus(1.0);
			// this is from the old version without autofocus
			// Engine.this.j3doperations.setEyeDistance(Engine.this.j3doperations
			//		.getStereoValue());
		}

		public PluginManager getPluginManager() {
			return Engine.this.pluginmanager;
		}

		public void addMultilayerPressed() {

			final MembraneModel model = DataPool.getMembraneModel();

			final Float height = Engine.this.membraneview.askForYHeight(CM2_CONSTANTS.GUI_SETTINGS_AND_CONTROLS.STANDARD_MULTILAYER_Y_TRANSLATION);
			if (height == null)
				return;

			int a = 2;
			for (int i = 0; i < model.getLayerCount(); i++) {
				if (model.getLayer(i).getLayerId() == a) {
					a++;
					i = 0;
				}

			}

			// if (DataPool.J3DSettings.enableFastTransitions)
			// membraneview.getMembrane3D().dockScreen(false);

			model.addLayer(new Layer(height, a));
			final float oldY = DataPool.getCurrentLayerHeight();
			DataPool.setSelectedLayer(a);
			DataPool.getMembraneModel().translateAllMolecules(
					-oldY + DataPool.getCurrentLayerHeight(), true, true);
			Engine.this.membraneview.getMembrane3D().updateAfterLayerChange();

			Engine.this.membraneOperations.refreshViews(true, true, true, true,
					true, true);

			// if (DataPool.J3DSettings.enableFastTransitions)
			// membraneview.getMembrane3D().dockScreen(true);

		}
		
		/**
		 * Show the dialog to restore a membrane from a PDB file.
		 * The program does not wait for the reverse parsing thread to end. 
		 * Because of this the return value is invalid. The GUI reports the success. 
		 */
		public boolean restoreMembraneFromPDBPressed() {
			return restoreMembraneFromPDB(null, null, null, true, false);
		}

		/**
		 * Restore a membrane from a PDB file with the following options and
		 * wait until the reverse parsing status is returned.
		 * (Set optional parameters to null if not needed):
		 * 
		 * @param String absolutePathOfPDB mandatory: the absolute path of the PDB 
		 * @param String absolutePathOfCm2 optional: the absolute path of the Cm2 against which the PDB has to be parsed
		 * @param CoordinateTransformationHints overwriteHints optional: the transformation hints containing information about how to transform the membrane
		 * @param boolean checkMembraneState mandatory: true if a dialog should appear if there were changes on the membrane, false if the process should be started immediately
		 */
		public boolean restoreMembraneFromPDB(final String absolutePathOfPDB, final String absolutePathOfCm2, 
											final CoordinateTransformationHints overwriteHints, final boolean checkMembraneState) {
			return restoreMembraneFromPDB(absolutePathOfPDB, absolutePathOfCm2, overwriteHints, checkMembraneState, true);
		}
		/**
		 * Restore a membrane from a PDB file with the following options
		 * (Set optional parameters to null if not needed):
		 * 
		 * @param String absolutePathOfPDB mandatory: the absolute path of the PDB 
		 * @param String absolutePathOfCm2 optional: the absolute path of the Cm2 against which the PDB has to be parsed
		 * @param CoordinateTransformationHints overwriteHints optional: the transformation hints containing information about how to transform the membrane
		 * @param boolean checkMembraneState mandatory: true if a dialog should appear if there were changes on the membrane, false if the process should be started immediately
		 * @param boolean waitForStatus mandatory: forces the program to wait until the reverse parsing thread has finished and reported the status
		 */
		public boolean restoreMembraneFromPDB(final String absolutePathOfPDB, final String absolutePathOfCm2, 
											final CoordinateTransformationHints overwriteHints, final boolean checkMembraneState, final boolean waitForStatus) {

			File fileToLoad = null;
			
			final FileIOHandler io = new FileIOHandler(Engine.this.gui
					.getFrame());
			if (checkMembraneState && io.authorizeDiscargingOfCurrentMembrane() == false)
				return false;
			
			if (absolutePathOfPDB==null) {
				fileToLoad = io.askPDBFileForRestoring();
			} else {
				fileToLoad = new File(absolutePathOfPDB);
			}
			
			final File filePDB = fileToLoad;
			
			if (filePDB == null || !filePDB.exists())
				return false;
			
			final File fileCm2 = absolutePathOfCm2!=null?new File(absolutePathOfCm2):null;
			
			if (absolutePathOfCm2 != null && !fileCm2.exists())
				return false;
			
			Thread T = new Thread() {
				public void run() {
								
					setAllEnabled(false);
					
					final ReversePDBParser parser = new ReversePDBParser(new File(
							FileIOHandler.home),true);
					try {
		
						DataPool.J3DSettings.overallRenderMode = CM2_CONSTANTS.RENDERMODE.RENDER_MODE_POLYGON;
		
						if (Engine.this.membraneview.isInSpecialMode())
							Engine.this.membraneview.finishSpecialMode(false);
		
					
		
						final MembraneModel newModel = (absolutePathOfCm2==null)?
								parser.restoreMembraneModelFromPdbFile(filePDB):
								parser.restoreMembraneModelFromPdbFile(filePDB,fileCm2,overwriteHints);
		
						if(newModel==null)throw new Exception("CANCELLED");
						DataPool.replaceMembraneModel(newModel);
						DataPool.setCurrentMembraneFile(null);
						Engine.this.gui
						.setMainFrameTitle(CM2_CONSTANTS.GENERAL.SHORT_NAME_OF_THIS_PROGRAM
								+ " " + (DataPool.getMembraneModel().getTitle()==null?filePDB.getName():DataPool.getMembraneModel().getTitle()));
						DataPool.setMembraneHasBeenEditedAgain(false);
						this.setName("SUCCESS");
		
					} catch (final Exception e) {
						if(parser.dia!=null)parser.dia.dispose();
						if(e.getMessage()==null||(!e.getMessage().equals("CANCELLED"))) {
							e.printStackTrace();
							JOptionPane.showMessageDialog(null,
									"The following Error occured while parsing:\n\n"
											+ e.getMessage(), "Error",
									JOptionPane.ERROR_MESSAGE);
							this.setName("ERROR");
						}
					}
		
					if (Engine.this.membraneview.getMembrane3D() != null)
						Engine.this.membraneview.getMembrane3D().clearSelection();
		
					DataPool.getMembraneModel().organizeDomains();
				
		
					Engine.this.membraneview.getMembrane3D().getMicroDomainManager()
							.updateDisplayOfMicroDomains();
		
					(Engine.this.membraneview.getMembrane3D())
							.repaintByMembraneModel(DataPool.getMembraneModel());
					(Engine.this.membraneview.getMembrane3D()).setViewByMode(
							Membrane3D.EXTRA, null);
				
		
					
					
		
					setAllEnabled(true);
					Engine.this.membraneOperations.refreshViews(true, true, true, true,
							false, true);
		
					Engine.this.propertiesview.showMembraneInfo();
					Engine.this.membraneview.switchToLayer(0);
					Engine.this.membraneOperations.highlightMicroDomain(0);
					Engine.this.membraneOperations.highlightMicroDomain(-1);
					Engine.this.membraneview.setEnabled(true);
					
					DataPool.setCurrentMembraneFile(filePDB);
					
				}
			};
			
			T.start();
			
			if (waitForStatus) {
				while(T.isAlive()) {
					try {
						Thread.sleep(1000);
					} catch (InterruptedException e) {
						e.printStackTrace();
					}
				}
			}
			return (T.getName().equals("SUCCESS"))?true:false;
		}
		
		/**
		 * Set percentages of the ComponentView.
		 */
		public void setPercentages(int mode, int newval, String code, int domainID){
			Engine.this.componentview.setPercentages(mode, newval, code, domainID);
		}
		
		/**
		 * The ComponentView uses absolute values instead of percental values to compute
		 * the lipid distribution.
		 */
		public void setComputeAbsoluteValues(boolean computeAbsoluteValues) {
			Engine.this.componentview.setComputeAbsoluteValues(computeAbsoluteValues);
		}

		/**
		 * Check if the ComponentView uses absolute values instead of percental values 
		 * to compute the lipid distribution.
		 * @return
		 */
		public boolean getComputeAbsoluteValues() {
			return Engine.this.componentview.getComputeAbsoluteValues();
		}
		
		/**
		 * Called when user chose generate a PDB-File using Gromacs
		 */
		public void userChoseGromacsCall() {
			
			
			
		}

	}

	/**
	 * Provides all Operations that may be started from the databaseview
	 * 
	 * @author Tim Dingersen dackelnose@gmx.net
	 * 
	 */
	public class DatabaseOperations implements DatabaseViewListener {

		public void userAddsLipid(final String code, final String file,
				final int modelnumber, final int DomainID,
				final int membraneSide) {

			if (DataPool.getMembraneModel().createAndAddANewExampleLipid(code,
					file, modelnumber, DomainID, membraneSide,
					DataPool.getCurrentLayerID()) == false) {

				return;
			}

			
			Engine.this.componentview.setPercentages(1, 1, code, DomainID);
			Engine.this.componentview.setPercentages(2, 1, code, DomainID);

			Engine.this.membraneOperations.refreshViews(true, true, true, true,
					false, false);
			Engine.this.membraneview.setCalculationEnabled(true);

		}

		public void userAddsProtein(final String code, final String file,
				final int modelnumber, final int DomainID) {

			if ((Engine.this.membraneview.getMembrane3D())
					.addANewProteinAtRandomPosition( code, file,
							modelnumber, DomainID) == false)
				JOptionPane.showMessageDialog(Engine.this.gui.getFrame(),
						"Cannot add any more Proteins of this Type.",
						"Not Possible", JOptionPane.INFORMATION_MESSAGE);
			else {
				DataPool.getMembraneModel().organizeDomains();
				Engine.this.membraneOperations.refreshViews(true, true, true,
						true, true, true);
			}
		}

		public void userWantToSeeProperties(final File file, final int type) {
//			Engine.this.gui.focusView(4);

			Engine.this.propertiesview.showProperties(file, type);

		}

		public void userOpensGlobalDBView() {
			this.userOpensGlobalDBView("null", "");
		}
		
		public void userOpensGlobalDBView(String searchString, String searchTitle) {
			final GlobalDatabaseBrowser browser = new GlobalDatabaseBrowser(searchString, searchTitle);
			browser.setDatabaseViewListener(this);

			Engine.this.gui.getFrame().setEnabled(false);
		}

		public void userClosesGlobalDBView() {
			Engine.this.gui.getFrame().setEnabled(true);

		}

		public void addEntryToDB(final File file) {

			Engine.this.databaseview.addFromRSCB(file);

		}

		public void addEntryToDB(final File file, final int group) {

			Engine.this.databaseview.addFromRSCB(file, group);

		}
		
		public void userWantToPreviewMolecule(final File file, final int model) {

			Engine.openJmolWithFile(file);
		}

		public void DnDEventOccured(final Point loc) {
			Engine.this.membraneview.getMembrane3D().DnDEventOccured(loc);

		}

		public void DNDFinished(final boolean cancelled) {

			Engine.this.membraneview.getMembrane3D().DnDFinished(cancelled);

		}

		public void userWantToChangeModel(final File file,
				final int initialModel) {

			Protein_3D prot;
			try {
				prot = new PdbReader_3D().readProtein(file, initialModel, null);
			} catch (final Exception e) {
				JOptionPane.showMessageDialog(Engine.this.gui.getFrame(),
						"The file cannot be read.", "ERROR",
						JOptionPane.ERROR_MESSAGE);
				return;
			}
			Engine.this.membraneview.startSpecialMode(prot,
					MembraneView.GUI_MODE_MODEL_CHANGE);

		}

		public String retrieveUserCommentFromDB(final File file, final int mode) {

			return Engine.this.databaseview.retrieveUserCommentFromDB(file,
					mode);
		}

		public void highlightMicroDomain(final int domainID) {
			Engine.this.membraneOperations.highlightMicroDomain(domainID);

		}

		public boolean dbContainsCode(final Object code) {

			return Engine.this.databaseview
					.dbContainsCode(code instanceof String ? (String) code
							: null);
		}

		public void openMoleculeEditorFromDB(File file,int mode) {
			
			PdbReader_3D reader = new PdbReader_3D();
			
			Molecule_3D mol;
			try {
				mol = (mode==0?reader.readProtein(file,0,null):reader.readLipid(file,0,null));
				if(mol.getPDBcode()==null) {
					mol.setPDBcode(Cm2_Toolkit.getSimpleName(mol.getPDBFile()));
				}
				if(mol instanceof Lipid_3D) {
					Matrix3f m = PdbReader_3D.autorotateUpwards((Lipid_3D)mol);
					mol.setRotationMatrix(m);
				}else {
					
					
					// molecule is a protein
					// =================================================================
					// TODO: PPE Update 03_2012											|
					// =================================================================
					if(DataPool.J3DSettings.enablePPEAutomatic) {
						// PPE automatic is enabled
						// -> automatic OPM adjustment
						// -> automatic PDBTM adjustment
						if(mol.opmAtoms.length > 0) {
							// protein is a OPM File
							DataPool.getMifHandler().applyOPMAlignmentTo(mol);
						} else {
							// molecule isn't a OPM file.
							// this also works when molecule isn't a PDBTM file
							// because of its default values stored in
							// Y-translation and rotation matrix.
							mol.setRotationMatrix(mol.getPDBTMRotation());
							mol.setYTranslation(mol.getPDBTMTranslation());				}
					} else {
						// PPE automatic is disabled
						mol.setRotationMatrix(Cm2_Toolkit.createAUnitMatrix());
					}
					
					
				}
				new MoleculeEditor(mol, false, Engine.this, null, Engine.this.j3doperations.getStereoValue());
			} catch (Exception e) {

				JOptionPane.showMessageDialog(Engine.this.gui
						.getFrame(),
						"Cannot open the file!\n\nReason:\n"
								+ e.toString(), "Error",
						JOptionPane.ERROR_MESSAGE);
				e.printStackTrace();
				return;

			}
			
			
		}

	}

	/**
	 * Operations regarding the current Membrane
	 * 
	 * @author Tim Dingersen dackelnose@gmx.net
	 * @author Sebastian Schneider
	 */
	public class MembraneOperations implements MembraneViewListener {

		private MembranePackingAlgorithm currentAlgorithm;
		private double currentEyeDistance = 0.0021; 
		

		public void setCalculationEnabled() {
			Engine.this.membraneview.setCalculationEnabled(true);
		}

		public void alignSingleProtein(final Molecule_3D proToAlign) {

			Engine.this.membraneview.startSpecialMode(proToAlign,
					MembraneView.GUI_MODE_ALIGN_PROTEIN);
		}
		
		public void alignSingleProteinPlus(final Molecule_3D proToAlign) {

			Engine.this.membraneview.startSpecialMode(proToAlign,
					MembraneView.GUI_MODE_ALIGN_PROTEIN_PLUS);
		}

		public void pickMolecules(final Molecule_3D[] proPicked) {

			Engine.this.membraneview.getMembrane3D().getPickNDragManager()
					.setPickedMolecules(proPicked);

			// if(proPicked==null) {
			// Molecule_3D[] mols =
			// DataPool.getMembraneModel().getAllMolecules();
			// for(int i=0;i<mols.length;i++) {
			// if(mols[i].isPicked())mols[i].setPicked(false);
			// }
			// return;
			// }

			if (proPicked != null) {
				if (proPicked.length < 1 || proPicked[0] == null) {
					this.refreshViews(false, true, false, false, false, false);
					return;
				}

				if (Engine.this.propertiesview.getCurrentMolecules() != null) {
					if (proPicked.length == 1
							&& Engine.this.propertiesview.getCurrentMolecules().length == 1) {
						if (proPicked[0] == Engine.this.propertiesview
								.getCurrentMolecules()[0]) {
							this.refreshViews(false, true, false, false, false,
									false);
							return;
						}
					}
				}

				// for(int i=0;i<proPicked.length;i++) {
				//					
				// proPicked[i].setPicked(true);
				// }
				Engine.this.componentview
						.select(proPicked[0],
								proPicked[0] instanceof Protein_3D ? 0
										: ((Lipid_3D) proPicked[0])
												.getMembraneSide() + 1);

				Engine.this.propertiesview.showProperties(proPicked);

			} else {

				Engine.this.componentview.select(null, 0);
				Engine.this.propertiesview.showProperties(null);

			}
			this.refreshViews(false, true, false, false, false, false);
		}

		public synchronized void refreshViews(final boolean compView,
				final boolean memViewPanels, final boolean database,
				final boolean stats, final boolean props,
				final boolean intersections) {

			// if (DataPool.J3DSettings.enableFastTransitions)
			// membraneview.getMembrane3D().dockScreen(false);
			DataPool.getMembraneModel().sortMolecules();
			if (compView) {
				Engine.this.componentview.upDateAll();
			}
			if (memViewPanels) {
				Engine.this.membraneview.updateLayers();
				Engine.this.membraneview.updatePanels();

				Engine.this.gui.enableGenerateButton(!DataPool
						.getMembraneModel().isEmpty()
						&& Engine.this.membraneview.mayGenerate());
				
				

			}
			if (stats)
				Engine.this.statisticsview.updateMembraneStatistics(DataPool
						.getMembraneModel(), this.getHighlightedMicroDomain());

			if (props) {
				if (Engine.this.propertiesview.isMembraneAttrsShowing())
					Engine.this.propertiesview.showMembraneInfo();
				else
					Engine.this.propertiesview.updateCurrentView();
			}

			if (database)
				Engine.this.databaseview.fireUpdateTablesOnMembraneModel();

			DataPool.cleanUpRawData();

			// if (DataPool.J3DSettings.enableFastTransitions)
			// membraneview.getMembrane3D().dockScreen(true);

			if (intersections)
				Engine.this.membraneview.getIntersectionsDisplayer()
						.updateIntersectionDispay();
			
			Molecule_3D[]mols = DataPool.getMembraneModel().getAllMoleculesInMembrane();
			boolean anyOPM=false;
			for(int i=0;i<mols.length;i++) {
				if(!mols[i].isTransparent())
					mols[i].displayOPMAtoms(membraneview.toggleOPMLayersButton.isSelected());
				if(mols[i].opmAtoms.length>0)
					anyOPM=true;
			}
			membraneview.toggleOPMLayersButton.setVisible(anyOPM);
		}

		public void displayedLayerChanged(final int mode) {
			Engine.this.statisticsview.changeLayer(mode);

		}

		public void selectProteinsByCode(final String code, final int domainID) {

			Engine.this.membraneview.getMembrane3D().selectByCode(code,
					domainID);

		}

		public boolean addANewProteinAtRandomPosition(
				final String code, final String file, final int modelnumber,
				final int DomainID) {

			return (Engine.this.membraneview.getMembrane3D())
					.addANewProteinAtRandomPosition(code, file,
							modelnumber, DomainID);

		}

		

		public void requestFocusOnView(final int view) {
			Engine.this.gui.focusView(view);

		}

		public void removeProteinsFrom3D(final Protein_3D[] prots2remove) {
			(Engine.this.membraneview.getMembrane3D())
					.removeMoleculesFrom3D(prots2remove);

		}

		public void clearAllLipidsFrom3DandModel(final int domainID,
				final int layerID) {
			(Engine.this.membraneview.getMembrane3D()).clearLipids(domainID,
					layerID);

		}

		public JPopupMenu createCanvasPopUpMenu() {
			return Engine.this.membraneview.createCanvasPopUpMenu();
		}

		public void popup3DSettingsDialog(final Point location) {
			DataPool.openJ3DSettingsWindow(Engine.this.j3doperations, location);
		}

		public boolean isProperties_Membrane_Atrribus_Showing() {
			return Engine.this.propertiesview.isMembraneAttrsShowing();
		}

		public void updateProperties() {
			Engine.this.propertiesview.updateCurrentView();

		}

		public void userChangedModelForFile(final File file, final int model) {
			Engine.this.databaseview.changeModelForFile(file, model);

		}

		public boolean areAxisVisible() {
			return Engine.this.membraneview.getMembrane3D().areAxisVisible();
		}

		public void setShowAxis(final boolean b) {
			Engine.this.membraneview.getMembrane3D().setShowAxis(b);

		}

		public void addLipidExtra(final Lipid_3D lip2add) {
			DataPool.getMembraneModel().addLipidExtra(lip2add,
					lip2add.getDomainID(), lip2add.getLayerID());
			Engine.this.membraneview.getMembrane3D()
					.addMolecule(lip2add, false);

		}

		public void addLipidIntra(final Lipid_3D lip2add) {
			DataPool.getMembraneModel().addLipidIntra(lip2add,
					lip2add.getDomainID(), lip2add.getLayerID());
			Engine.this.membraneview.getMembrane3D()
					.addMolecule(lip2add, false);

		}

		public void removeLipidExtra(final Lipid_3D lip2remove) {
			Engine.this.membraneview.getMembrane3D().removeMoleculesFrom3D(
					new Lipid_3D[] { lip2remove });
			DataPool.getMembraneModel().deleteLipidExtra(lip2remove);
		}

		public void removeLipidIntra(final Lipid_3D lip2remove) {
			Engine.this.membraneview.getMembrane3D().removeMoleculesFrom3D(
					new Lipid_3D[] { lip2remove });
			DataPool.getMembraneModel().deleteLipidIntra(lip2remove);

		}

		public void calculate(final MembranePackingAlgorithm algo) {

					this.currentAlgorithm=algo;
					int higlightedMicroDomain=this.getHighlightedMicroDomain();
					algo.CurrentMicroDomain=higlightedMicroDomain;
			this.pickMolecules(null);
			this.highlightMicroDomain(-1);
			 manager = new CalculationManager(this,
					algo, Engine.this.membraneview,
					Engine.this.basicOperations, Engine.this.databaseOperations, 
					Engine.this.statisticsview, Engine.this.propertiesview);

			if (!manager.performLipidCheck())
				return;

			if (!manager.prepare())
				return;

			manager.start();

		}

		public void displayAlgorithmProgress(final int progress) {
			Engine.this.membraneview.setAlgoritmProgress(progress);

		}

		public void displayAlgoritmInfo(final String info) {
			Engine.this.membraneview.displayAlgorithmInfo(info);

		}

		public void alignExampleLipid(final Lipid_3D lip, final int side) {

			Engine.this.membraneview.startSpecialMode(lip,
					MembraneView.GUI_MODE_ALIGN_SAMPPLELIPID);
		}

		public void setAllEnabled(final boolean b) {
			Engine.this.basicOperations.setAllEnabled(b);

		}

		public void updateMicroDomainsIn3D() {

			// if (DataPool.J3DSettings.enableFastTransitions)
			// membraneview.getMembrane3D().dockScreen(false);

			Engine.this.membraneview.getMembrane3D().getMicroDomainManager()
					.updateDisplayOfMicroDomains();

			// if (DataPool.J3DSettings.enableFastTransitions)
			// membraneview.getMembrane3D().dockScreen(true);

		}

		public void highlightMicroDomain(final int domainID) {

			if (this.getHighlightedMicroDomain() == domainID)
				return;

			Engine.this.componentview.highlightMicroDomain(domainID);
			// if (DataPool.J3DSettings.enableFastTransitions)
			// membraneview.getMembrane3D().dockScreen(false);
			Engine.this.membraneview.getMembrane3D().getMicroDomainManager()
					.higlightMicroDomainIn3D(domainID);

			Engine.this.membraneview.updatePanels();
			Engine.this.gui.enableGenerateButton(!DataPool.getMembraneModel()
					.isEmpty()
					&& Engine.this.membraneview.mayGenerate());
			Engine.this.propertiesview.updateCurrentView();
			Engine.this.statisticsview.updateMembraneStatistics(DataPool
					.getMembraneModel(), this.getHighlightedMicroDomain());

			// if (DataPool.J3DSettings.enableFastTransitions)
			// membraneview.getMembrane3D().dockScreen(true);

		}

		public void removeMicroDomain(final MicroDomain_3D domainToDelete) {

			// if (DataPool.J3DSettings.enableFastTransitions)
			// membraneview.getMembrane3D().dockScreen(false);

			this.highlightMicroDomain(-1);
			this.pickMolecules(null);

			Engine.this.membraneview.getMembrane3D().clearLipids(
					domainToDelete.id, DataPool.getCurrentLayerID());
			DataPool.getMembraneModel().removeMicroDomain(domainToDelete);
			DataPool.getMembraneModel().organizeDomains();
			this.updateMicroDomainsIn3D();

			this.refreshViews(true, true, false, false, false, true);

			// if (DataPool.J3DSettings.enableFastTransitions)
			// membraneview.getMembrane3D().dockScreen(true);

		}

		public void startDomainChange(final MicroDomain_3D domainToChange) {

			Engine.this.membraneview.startDomainMode(domainToChange);

		}

		public void selectFromCV(final Molecule_3D mol, final int type) {
			if (type == 0) {
				this.selectProteinsByCode(mol.getPDBcode(), mol.getDomainID());
			} else if (type > 0) {
				this.selectProteinsByCode(null, 0);
				Engine.this.componentview.select(mol, type);
				Engine.this.propertiesview.showLipid((Lipid_3D) mol);
			} else {
				this.selectProteinsByCode(null, 0);
				Engine.this.componentview.select(mol, type);
			}

		}

		public Vector<Molecule_3D> getCurrentSelection() {

			return Engine.this.membraneview.getMembrane3D()
					.getSelectedMolecules();
		}

		public void showStereoView() {
			Engine.this.j3doperations.showStereoView();
			Engine.this.basicOperations.userChangedStereoSlider();
		}

		public int getHighlightedMicroDomain() {

			return Engine.this.membraneview.getMembrane3D()
					.getMicroDomainManager().getHighlightedID();
		}

		public void reloadGeometry() {
			Engine.this.basicOperations.reloadGeometry();

		}

		public PluginManager getPluginManager() {
			return Engine.this.pluginmanager;
		}

		public void removeLipidsFrom3D(final Lipid_3D[] toremove) {
			(Engine.this.membraneview.getMembrane3D())
					.removeMoleculesFrom3D(toremove);

		}

		public void startModelChange(final Molecule_3D mol) {
			Engine.this.membraneview.startSpecialMode(mol,
					MembraneView.GUI_MODE_MODEL_CHANGE);

		}

		public void DnDEventOccured(final Point loc) {
			Engine.this.databaseOperations.DnDEventOccured(loc);

		}

		public void DNDFinished(final boolean cancelled) {
			Engine.this.databaseOperations.DNDFinished(cancelled);

		}

		public JFrame getMainFrame() {
			return Engine.this.gui.getFrame();
		}

		

		

		public void openMoleculeEditorFromCV(Molecule_3D mol) {

			if (DataPool.J3DSettings.enableAtomPerformanceMode) {
				JOptionPane.showMessageDialog(Engine.this.gui
						.getFrame(),
						"Select Profile/the Molecule Editor will not work in Atom Performance Mode!\n"
						+ "Just deactivate the Atom Performance Mode and reload the membrane!", 
						"Warning",
						JOptionPane.WARNING_MESSAGE);
				return;
			}

			Molecule_3D copiedMolecule = mol.copy(true,true);
			try {
				
				Engine.this.molEditView =  
					new MoleculeEditor(copiedMolecule, true, Engine.this,
							copiedMolecule.getCurrentProfileName(), Engine.this.j3doperations.getStereoValue());			
			} catch (Exception e) {

				JOptionPane.showMessageDialog(Engine.this.gui
						.getFrame(),
						"Cannot open the file!\n\nReason:\n"
								+ e.toString(), "Error",
						JOptionPane.ERROR_MESSAGE);
				e.printStackTrace();
				return;
				
			}
			
		}

		public void requestRenderModeChange() {
			this.currentAlgorithm.requestChangeOfRenderMode();
			
		}

		public void changeRenderModeFromAlgorithm() {
			membraneview.switchRenderMode_While_Algorithm_Runs();
			
		}

		public void dockScreen(boolean dockScreen) {
			membraneview.getMembrane3D().dockScreen(dockScreen);
			
		}

		/**
		 * AutoFocus for the stereo mode.
		 * 
		 * @author Christian Gamroth
		 */
		public void autoFocus(double ratio) {
			// setting up the basis value
			if(Double.isInfinite(ratio)) {
				ratio = 1.0;
			}
			final double baseValue = Engine.this.gui.getStereoSlider().getValue() / CM2_CONSTANTS.J3D.STEREOVIEW_SCALE;
//			System.out.println("SliderValue: " + basisValue);
//			System.out.println();
			// Calculating new EyeDistance
//			System.out.println("oldEyeDistance: " + this.currentEyeDistance);
			this.currentEyeDistance *= ratio;
			double newValue = baseValue + this.currentEyeDistance;
			// if value getting negative, this avoids ghosts in the front
			if(newValue < 0.00001) {
				newValue = 0.00001;
			}
//			System.out.println("newValue: " + newValue);
//			System.out.println("currentEyeDistance: " + this.currentEyeDistance);
//			System.out.println();
			// Setting the new EyeDistance
			if (Engine.this.membraneview.getMembrane3D() != null) {
				if (Engine.this.membraneview.getMembrane3D().getCanvas() != null) {
					final Canvas3D canvas = Engine.this.membraneview
							.getMembrane3D().getCanvas();
					canvas.getView().setWindowEyepointPolicy(
							View.RELATIVE_TO_FIELD_OF_VIEW);
					final double eyeDistance = newValue / 2;
					final Point3d left = new Point3d();
					canvas.getView().getPhysicalBody().getLeftEyePosition(left);
					left.x = - eyeDistance;
					final Point3d right = new Point3d();
					canvas.getView().getPhysicalBody().getRightEyePosition(right);
					right.x = eyeDistance;
					// wird ignoriert von policy "RELATIVE_TO_FIELD_OF_VIEW"
					// canvas.setLeftManualEyeInImagePlate(left);
					// canvas.setRightManualEyeInImagePlate(right);
					canvas.getView().getPhysicalBody().setLeftEyePosition(left);
					canvas.getView().getPhysicalBody().setRightEyePosition(right);
					try{
						Engine.this.molEditView.getDisplay().setEyePosition(left, right);
					} catch (NullPointerException e) {
						// handle exception here
						
					}
				}
			}
			
		}

		public BranchGroup getJ3DBranchGroup() {
			return membraneview.getMembrane3D().publicBranchGroup;
		}
	}

	/**
	 * Operations concerning Detail Settings in the 3D scene.
	 * 
	 * @author Tim Dingersen dackelnose@gmx.net
	 * 
	 */
	public class J3DOperations {

		public Component getRealtiveComponent() {
			return Engine.this.membraneview;
		}

		public void setBrightnessForAmbientLight(final float f) {
			(Engine.this.membraneview.getMembrane3D()).setAmbientBrightness(f);

		}

		public void setLight1Enabled(final boolean b) {
			(Engine.this.membraneview.getMembrane3D()).setEnableLight1(b);

		}

		public void setLight2Enabled(final boolean b) {
			(Engine.this.membraneview.getMembrane3D()).setEnableLight2(b);

		}

		public void setViewToMode(final int i) {
			Engine.this.membraneview.getMembrane3D().setViewByMode(i, null);

		}

		public void setForbidDraggingInFreeLookMode(final boolean b) {
			if (Engine.this.membraneview.getMembrane3D().getCamera().getMode() == CameraMode.FREE)
				Engine.this.membraneview.getMembrane3D().getPickNDragManager()
						.allowDragging(!b);

		}

		public void setMouseSensitivity(final double value) {
			Engine.this.membraneview.getMembrane3D().getCamera()
					.setMouseSensitivity(value);

		}

		public void showStereoView() {
			Engine.this.gui.getStereoEnable().setVisible(true);
			Engine.this.gui.getStereoEnable().setEnabled(true);
			Engine.this.gui.getStereoSlider().setVisible(true);
			Engine.this.gui.getStereoSliderPosition().setVisible(true);
		}

		public void hideStereoView() {
			Engine.this.gui.getStereoEnable().setVisible(false);
			Engine.this.gui.getStereoEnable().setEnabled(false);
			Engine.this.gui.getStereoSlider().setVisible(false);
			Engine.this.gui.getStereoSliderPosition().setVisible(false);
		}

		public double getStereoValue() {
			double value;
			value = (Engine.this.gui.getStereoSlider().getValue())
					/ CM2_CONSTANTS.J3D.STEREOVIEW_SCALE;
			return value;
		}

		public String getStereoValueString() {
			return String.format("%.3f", this.getStereoValue());
		}

		public void setEyeDistance(final double d) {
			if (Engine.this.membraneview.getMembrane3D() != null) {
				if (Engine.this.membraneview.getMembrane3D().getCanvas() != null) {
					final Canvas3D canvas = Engine.this.membraneview
							.getMembrane3D().getCanvas();
					canvas.getView().setWindowEyepointPolicy(
							View.RELATIVE_TO_FIELD_OF_VIEW);

					final double lx = -d / 2;
//					/*IO*/ System.out.println("EyeDistance to set: " + d);
					final Point3d left = new Point3d();
					canvas.getView().getPhysicalBody().getLeftEyePosition(left);
//					/*IO*/ System.out.println("Left Eye: " + left.x + ", " + left.y + ", " + left.z);
					left.x = lx;

					final double rx = d / 2;
					final Point3d right = new Point3d();
					canvas.getView().getPhysicalBody().getRightEyePosition(right);
//					/*IO*/ System.out.println("Right Eye: " + right.x + ", " + right.y + ", " + right.z);
					right.x = rx;
					
//					/*IO*/ final Point3d before_left = new Point3d();
//					/*IO*/ canvas.getLeftManualEyeInImagePlate(before_left);
//					/*IO*/ final Point3d before_right = new Point3d();
//					/*IO*/ canvas.getRightManualEyeInImagePlate(before_right);
//					/*IO*/ System.out.println("EyeInImagePlate before_left: " + before_left.x + ", " + before_left.y + ", " + before_left.z);
//					/*IO*/ System.out.println("EyeInImagePlate before_right: " + before_right.x + ", " + before_right.y + ", " + before_right.z);
					/*wird ignoriert von policy "RELATIVE_TO_FIELD_OF_VIEW"*/
					canvas.setLeftManualEyeInImagePlate(left);
					canvas.setRightManualEyeInImagePlate(right);
//					/*IO*/ final Point3d after_left = new Point3d();
//					/*IO*/ canvas.getLeftManualEyeInImagePlate(after_left);
//					/*IO*/ final Point3d after_right = new Point3d();
//					/*IO*/ canvas.getRightManualEyeInImagePlate(after_right);
//					/*IO*/ System.out.println("EyeInImagePlate after_left: " + after_left.x + ", " + after_left.y + ", " + after_left.z);
//					/*IO*/ System.out.println("EyeInImagePlate after_left: " + after_right.x + ", " + after_right.y + ", " + after_right.z);

					canvas.getView().getPhysicalBody().setLeftEyePosition(left);
					canvas.getView().getPhysicalBody().setRightEyePosition(right);
					/*is still != null after exit! */
					try{
						Engine.this.molEditView.getDisplay().setEyePosition(left, right);
					} catch (NullPointerException e) {
						// handle exception here						
					}
				}
			}
		}

		public void setAntialiasingEnabled(final boolean b) throws Throwable {
			Engine.this.membraneview.getMembrane3D().getCanvas().getView()
					.setSceneAntialiasingEnable(b);

		}
		
		public void setParallelProjection(final boolean b) throws Throwable {
			if (b) {
				Engine.this.membraneview.getMembrane3D().getCanvas().getView().setProjectionPolicy(View.PARALLEL_PROJECTION);
				Engine.this.membraneview.getMembrane3D().getCanvas().getView().setScreenScalePolicy(View.SCALE_EXPLICIT);
				Engine.this.membraneview.getMembrane3D().getCanvas().getView().setBackClipDistance(CM2_CONSTANTS.J3D.VIEW_BACK_CLIP_DISTANCE_PARALLEL);
			} else {
				Engine.this.membraneview.getMembrane3D().getCanvas().getView().setProjectionPolicy(View.PERSPECTIVE_PROJECTION);
				Engine.this.membraneview.getMembrane3D().getCanvas().getView().setScreenScalePolicy(View.SCALE_SCREEN_SIZE);
				Engine.this.membraneview.getMembrane3D().getCanvas().getView().setBackClipDistance(CM2_CONSTANTS.J3D.VIEW_BACK_CLIP_DISTANCE);
			}

		}

		public void setUniformBackground(final boolean b) {
			Engine.this.membraneview.getMembrane3D().setUniformBackground(b);
		}
		
		public void showAllMultilayers(final boolean b) {
//			Engine.this.membraneview.getMembrane3D().showAllMultilayers(b);
			//TODO
			DataPool.getMembraneModel().showAllMultilayers(b);
//			Engine.this.membraneOperations.refreshViews(true, true, true, true, true, true);
		}

		public void refreshViews(final boolean compView,
				final boolean memViewPanels, final boolean database,
				final boolean stats, final boolean props,
				final boolean intersections) {
			Engine.this.membraneOperations.refreshViews(
					compView,memViewPanels,database,
					stats,props,intersections);

		}

	}
	
	/**
	 * This method loads a PDB-Membrane into the running MembraneEditor.
	 * @param absolutePathOfPDB
	 */
	public boolean restoreMembraneFromPDB(String absolutePathOfPDB, String absolutePathToCm2, CoordinateTransformationHints overwriteHints, boolean checkMembraneState) {
		return this.gui.listener.restoreMembraneFromPDB(absolutePathOfPDB, absolutePathToCm2, overwriteHints, checkMembraneState);
	}

	/**
	 * This opens the specified file in Jmol.
	 * This method is static, because Jmol may be started without running the main application. 
	 * @param file
	 */
	public static void openJmolWithFile(final File file) {
		
		String infoString = "Execution of:\n "+DataPool.JmolSettings.jmolExecutionLink+" \""+file.getAbsolutePath()+"\""+
							"\nOS: "+System.getProperty("os.name");
		
		// start on Windows Systems
		if (System.getProperty("os.name").toLowerCase().contains("windows")) {
			String commandLine = "rundll32 shell32.dll,ShellExec_RunDLL " +
								DataPool.JmolSettings.jmolExecutionLink+" \""+file.getAbsolutePath()+"\"";
			
			String[] args = commandLine.split(" ");
			ProcessBuilder pb = new ProcessBuilder(args);
			try {
				pb.start();
			} catch (Exception e) {
				e.printStackTrace();
				JOptionPane.showMessageDialog(null, e.getMessage()+"\n"+infoString,
						"Jmol does not start, Sorry. Please report this error to bjoern@CELLmicrocosmos.org", JOptionPane.ERROR_MESSAGE);
			}
		// start on other Systems
		} else {
			String[] commands = (DataPool.JmolSettings.jmolExecutionLink+" "+file.getAbsolutePath()).replaceAll("\"", "").replaceAll("'", "").split("\\s");
            ProcessBuilder pb = new ProcessBuilder(commands);
            pb.directory(file.getParentFile());
			try {
				pb.start();
			} catch (Exception e) {
				e.printStackTrace();
				JOptionPane.showMessageDialog(null, e.getMessage()+"\n"+infoString,
					"Jmol does not start, Sorry. Please report this error to bjoern@CELLmicrocosmos.org", JOptionPane.ERROR_MESSAGE);
			}
		}
		 
	}
	
	/**
	 * This opens Jmol.
	 * This method is static, because Jmol may be started without running the main application. 
	 */
	public static void openJmol() {
		
		String infoString = "Execution of:\n "+DataPool.JmolSettings.jmolExecutionLink+
							"\nOS: "+System.getProperty("os.name");
		
		System.out.println(infoString);
		// start on Windows Systems
		if (System.getProperty("os.name").toLowerCase().contains("windows")) {
			String commandLine = "rundll32 shell32.dll,ShellExec_RunDLL " +
								DataPool.JmolSettings.jmolExecutionLink;
			
			String[] args = commandLine.split(" ");
			ProcessBuilder pb = new ProcessBuilder(args);
			try {
				pb.start();
			} catch (Exception e) {
				e.printStackTrace();
				JOptionPane.showMessageDialog(null, e.getMessage()+"\n"+infoString,
						"Jmol does not start, Sorry. Please report this error to bjoern@CELLmicrocosmos.org", JOptionPane.ERROR_MESSAGE);
			}
		// start on other Systems
		} else {
			String[] commands = (DataPool.JmolSettings.jmolExecutionLink.split("\\s"));
            ProcessBuilder pb = new ProcessBuilder(commands);
			try {
				pb.start();
			} catch (Exception e) {
				e.printStackTrace();
				JOptionPane.showMessageDialog(null, e.getMessage()+"\n"+infoString,
					"Jmol does not start, Sorry. Please report this error to bjoern@CELLmicrocosmos.org", JOptionPane.ERROR_MESSAGE);
			}
		}
		 
	}
	
	/**
	 * This opens the specified file in Jmol.
	 * This method is static, because Jmol may be started without running the main application. 
	 * @param file
	 */
	public static void openAPLVoroWithFile(final File pdbFile, final File aplFile) {
		
		String infoString = "Execution of:\n "+DataPool.APLVoroSettings.aplVoroExecutionLink+" \""+pdbFile.getAbsolutePath()+"\""+
							"\nOS: "+System.getProperty("os.name");
		
		// start on Windows Systems
		// remark: Did somebody ever compile APL@Voro for Windows? We never did this and so we never tested this section. Please use a Unix-compatible system.
		if (System.getProperty("os.name").toLowerCase().contains("windows")) {
			String commandLine = "rundll32 shell32.dll,ShellExec_RunDLL " +
						DataPool.APLVoroSettings.aplVoroExecutionLink+" -f "+pdbFile.getAbsolutePath()+" -c "+aplFile.getAbsolutePath();
			
			String[] args = commandLine.split(" ");
			ProcessBuilder pb = new ProcessBuilder(args);
			try {
				pb.start();
			} catch (Exception e) {
				e.printStackTrace();
				JOptionPane.showMessageDialog(null, e.getMessage()+"\n"+infoString,
						"APL@Voro does not start, Sorry. Please report this error to bjoern@CELLmicrocosmos.org", JOptionPane.ERROR_MESSAGE);
			}
		// start on other Systems
		} else {
			String[] commands = (DataPool.APLVoroSettings.aplVoroExecutionLink+" -f "+pdbFile.getAbsolutePath()+" -c "+aplFile.getAbsolutePath()).replaceAll("\"", "").replaceAll("'", "").split("\\s");

            ProcessBuilder pb = new ProcessBuilder(commands);
            pb.directory(pdbFile.getParentFile());
			try {
				pb.start();
			} catch (Exception e) {
				e.printStackTrace();
				JOptionPane.showMessageDialog(null, e.getMessage()+"\n"+infoString,
					"APL@Voro does not start, Sorry. Please report this error to bjoern@CELLmicrocosmos.org", JOptionPane.ERROR_MESSAGE);
			}
		}
		 
	}
	
	public String getTitle() {
		return this.gui.getFrame().getTitle();
	}

	
	// the following methods are only needed for the CmX_MRNA interface 
	
	public static Thread getOwnThread() {
		return ownThread;
	}

	public static void setOwnThread(Thread getOwnThread) {
		ownThread = getOwnThread;
	}
	
	public static MRNA getMRNA() {
		return cmX_MRNA;
	}

	public static void setMRNA(MRNA cmX_MRNA) {
		Engine.cmX_MRNA = cmX_MRNA;
	}

	
	public static void setStereoAvailable(boolean available) {
		stereoAvailable = available;
	}
	
	public static boolean getStereoAvailable() {
		return stereoAvailable;
	}
	
	public boolean isActive() {
		return isActive;
	}

	public void setActive(boolean isActive) {
		this.isActive = isActive;
	}

	protected String getCm1TempName2Edit() {
		return cm1TempName2Edit;
	}

	protected void setCm1TempName2Edit(String cm1TempName2Edit) {
		this.cm1TempName2Edit = cm1TempName2Edit;
	}

	public void performSearchInDownloadAPDB(String searchString, String searchTitle) {
		this.databaseOperations.userOpensGlobalDBView(searchString, searchTitle);	
	}

	/**
	 * Check if the correct Java3D Version is installed.
	 *
	 */
	public void checkJava3DVersion() {
		
		try {
			
			String j3dVersion = SimpleUniverse.getProperties().get("j3d.version").toString();
			if (!j3dVersion.contains("1.5") && !j3dVersion.contains("1.6")) {
				final int val = JOptionPane
					.showOptionDialog(
						null,
						"Your system has installed an old Java3D version, which is not supported!\n"+
						"Do you wish to exit the application and install an actual version (Java3D 1.5+)\n" +
						"or do you want to try to proceed (this will surely cause errors)?\n\n"+
						"Note: If you run CmME on MacOS X, you have to manually install some actual Java3D files.\n"+
						"This is a problem of MacOS X, which can easily be resolved (no administrator rights needed!).\n"+
						"Please refer to the Cm2.2 MembraneEditor install notes in the forum at:\n"+
						"http://forum.CELLmicrocosmos.org"
						, "Java3D Version Warning",
						JOptionPane.YES_NO_OPTION,
						JOptionPane.WARNING_MESSAGE, null,
						new String[] { "Proceed", "Exit MembraneEditor" }, null);
				if (val != JOptionPane.OK_OPTION)
					System.exit(0);
			}

		} catch (final Throwable e) {
			
			e.printStackTrace();
			JOptionPane
					.showMessageDialog(
							null,
							"The following Error occured:\n"
									+ e.getMessage()
									+ "\n\n"
									+ "Maybe your video driver is not working properly or\n "
									+ "Java3D was not installed correctly to your machine.\n\n" 
									+ "Hint: If this happens during the initial installation process,\n" 
									+ "this might be a known Java WebStart bug applying to some \n"
									+ "special Java distributions.\n" 
									+ "Close this program now and check, if the Icon of this program \n" 
									+ "was already installed to your desktop.\n" 
									+ "If so, double-click this icon, then the error might be solved.\n\n" 
									+ "If the error persists, please try another Java distribution or \n"
									+ "check the forum at http://Cm2.CELLmicrocosmos.org",
							"ERROR", JOptionPane.ERROR_MESSAGE);
			System.exit(0);
			
		}
		
	}

}
