/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_1)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.functionality3d;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.geom.Area;
import java.io.File;
import java.util.Enumeration;
import java.util.Random;
import java.util.Vector;

import javax.media.j3d.AmbientLight;
import javax.media.j3d.Background;
import javax.media.j3d.BoundingSphere;
import javax.media.j3d.BranchGroup;
import javax.media.j3d.Canvas3D;
import javax.media.j3d.ImageComponent;
import javax.media.j3d.ImageComponent2D;
import javax.media.j3d.Light;
import javax.media.j3d.PointLight;
import javax.media.j3d.VirtualUniverse;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;
import javax.vecmath.Color3f;
import javax.vecmath.Point3d;
import javax.vecmath.Point3f;

import org.cellmicrocosmos.cm2.constants.CM2_CONSTANTS;
import org.cellmicrocosmos.cm2.dataobjects.Lipid_3D;
import org.cellmicrocosmos.cm2.dataobjects.MembraneModel;
import org.cellmicrocosmos.cm2.dataobjects.MicroDomain_3D;
import org.cellmicrocosmos.cm2.dataobjects.Molecule_3D;
import org.cellmicrocosmos.cm2.dataobjects.Protein_3D;
import org.cellmicrocosmos.cm2.engine.DataPool;
import org.cellmicrocosmos.cm2.engine.DataPool.J3DSettings;
import org.cellmicrocosmos.cm2.engine.Engine;
import org.cellmicrocosmos.cm2.gui.MyGradientBufferedImage;
import org.cellmicrocosmos.cm2.interfaces.MembraneViewListener;
import org.cellmicrocosmos.cm2.io.PdbReader_3D;
import org.cellmicrocosmos.cm2.misc.Cm2_Toolkit;
import org.cellmicrocosmos.cm2.misc.ConfigParser;

import com.sun.j3d.utils.picking.PickCanvas;
import com.sun.j3d.utils.universe.SimpleUniverse;

/**
 * This is the component that displays the Membrane in 3D. Can also be regarded
 * as the core of the 3d funcionality because this class inherits the other 3d
 * classes too. Does also manage some parts of adding/removing of molecules in
 * the model and on screen
 * 
 * @author Sebastian Schneider, Tim Dingersen dackelnose@gmx.net, Philipp Unruh,
 *         Bjoern Sommer bjoern@CELLmicrocosmos.org
 */
public class Membrane3D extends JPanel {

	private static final long serialVersionUID = 1L;

	// the canvas for displaying
	private Canvas3D canvas;

	// the coordinates room
	private final SimpleUniverse universe;

	private final CameraMode cameraMode;

	public final IntersectManager intersector;

	private final PickNDragManager PNDmanager;

	private final MicroDomainManager MDmanager;

	public Hud hud;

	public static final int EXTRA = 0;

	public static final int INTRA = 1;

	public static final int FREE = -1;

	private final BranchGroup mainBranchGroup;

	private final BranchGroup basementBranchGroup;
	
	// this BranchGroup should only be used by Plugin developers, 
	// therefore, it has to be empty and unused in the MembraneEditor
	// (moreover, it is not possible to access the original membrane objects
	//  by the Plugin developer)
	public final BranchGroup publicBranchGroup;

	private final CoordinateBar coordinateBar;

	private final Raster raster;

	private final PickCanvas MyPick;

	public boolean axisVisible = true;

	public boolean rasterVisible = true;

	private final PointLight light1;

	private final PointLight light2;

	private final PointLight light3;

	private final AmbientLight ambientLight;

	public final Background background;

	private ImageComponent2D ic;

	private MembraneViewListener listener;

	public final MyGradientBufferedImage BIdark = new MyGradientBufferedImage(
			new Color(53, 56, 53), new Color(5, 10, 5), 0);

	public final MyGradientBufferedImage BIbright = new MyGradientBufferedImage(
			new Color(255, 255, 255), new Color(225, 230, 230), 0);

	private final MyGradientBufferedImage BIbright_Wet = new MyGradientBufferedImage(
			new Color(245, 245, 255), new Color(210, 210, 255), 0);

	private final MyGradientBufferedImage BIdark_Wet = new MyGradientBufferedImage(
			new Color(50, 45, 70), new Color(15, 10, 20), 0);
	
	public final MyGradientBufferedImage BIdarkUni = new MyGradientBufferedImage(
			new Color(0, 0, 0), new Color(0, 0, 0), 0);

	public final MyGradientBufferedImage BIbrightUni = new MyGradientBufferedImage(
			new Color(255, 255, 255), new Color(255, 255, 255), 0);

	private final MyGradientBufferedImage BIbrightUni_Wet = new MyGradientBufferedImage(
			new Color(255, 255, 255), new Color(255, 255, 255), 0);

	private final MyGradientBufferedImage BIdarkUni_Wet = new MyGradientBufferedImage(
			new Color(0, 0, 0), new Color(0, 0, 0), 0);

	private boolean notWet = true;

	public MicroDomain_3D defaultArea = null;

	/**
	 * Initializes the 3d classes
	 * 
	 */
	public Membrane3D() {

		super();

		try {
			this.axisVisible = Boolean.parseBoolean(ConfigParser
					.getValue("ShowAxis3D")[0]);
		} catch (final Exception e) {
		}
		try {
			this.rasterVisible = Boolean.parseBoolean(ConfigParser
					.getValue("ShowRaster3D")[0]);
		} catch (final Exception e) {
		}

		//
		// canvas
		//

		try {
			this.canvas = new Canvas3D(
					SimpleUniverse.getPreferredConfiguration()) {

				private static final long serialVersionUID = 1L;

				@Override
				public void postRender() {

					super.postRender();
					try {
						Membrane3D.this.hud.drawHud(Membrane3D.this.canvas
								.getGraphics2D());
						Membrane3D.this.canvas.getGraphics2D().flush(true);
					} catch (final Throwable e) {
						// System.out.println("Error bei canvas3d");
					}

					Membrane3D.this.manipulateAmbience();

				}

			};
			
		} catch (final Throwable e) {
			
			e.printStackTrace();
			JOptionPane
					.showMessageDialog(
							null,
							"The following Error occured:\n"
									+ e.getMessage()
									+ "\n\n"
									+ "Maybe your video driver is not working properly or\n "
									+ "Java3D was not installed correctly to your machine.\n\n" 
									+ "Hint: If this happens during the initial installation process,\n" 
									+ "this might be a known Java WebStart bug applying to some \n"
									+ "special Java distributions.\n" 
									+ "Close this program now and check, if the Icon of this program \n" 
									+ "was already installed to your desktop.\n" 
									+ "If so, double-click this icon, then the error might be solved.\n\n" 
									+ "If the error persists, please try another Java distribution or \n"
									+ "check the forum at http://Cm2.CELLmicrocosmos.org",
							"ERROR", JOptionPane.ERROR_MESSAGE);
			System.exit(0);
			
		}

		// check for Stereo3D devices
		System.out.println();

		if (this.canvas.getStereoAvailable()) {
			Engine.setStereoAvailable(true);
			System.out
					.println("StereoView enabled.\n Please use the StereoView window to adjust the settings.\n");
		} else {
			Engine.setStereoAvailable(false);
			System.out
					.println("StereoView disabled.\n If you have a compatible stereo device like the\n nVidia Quadro FX cards and installed an actual driver,\n please add to your VM arguments:\n -Dj3d.stereo=PREFERRED\n and enable stereo at your graphic driver.\n");
		}

		this.add(this.canvas);

		//
		// universe
		//
		this.universe = new SimpleUniverse(this.canvas);
		VirtualUniverse.setJ3DThreadPriority(Thread.MAX_PRIORITY);

		this.mainBranchGroup = new BranchGroup();
		this.basementBranchGroup = new BranchGroup();
		this.publicBranchGroup = new BranchGroup();
		

		Cm2_Toolkit.setAllCapabilitysForBranchGroup(this.mainBranchGroup);
		Cm2_Toolkit.setAllCapabilitysForBranchGroup(this.basementBranchGroup);
		Cm2_Toolkit.setAllCapabilitysForBranchGroup(this.publicBranchGroup);

		this.defaultArea = new MicroDomain_3D() {

			public void setHighlighted(final boolean b) {
			
				if (b) {
					if (((BranchGroup) this.branch.getParent()) == null)
						Membrane3D.this.mainBranchGroup.addChild(this.branch);
			
					super.setHighlighted(b);
				} else {
					if (((BranchGroup) this.branch.getParent()) != null)
						Membrane3D.this.mainBranchGroup
								.removeChild(this.branch);

					super.setHighlighted(b);
				}
			}
		};

		this.defaultArea.setGeometryType(MicroDomain_3D.RECTANGULAR);

		this.mainBranchGroup.addChild(this.defaultArea.branch);
		this.basementBranchGroup.addChild(this.mainBranchGroup);
		this.basementBranchGroup.addChild(this.publicBranchGroup);

		final BoundingSphere worldBounds = new BoundingSphere(new Point3d(0.0,
				0.0, 0.0), 100000.0);

		this.background = new Background() {
			public void setImage(ImageComponent2D ic) {
				
				super.setColor(new Color3f(new JPanel().getBackground()));
				if (DataPool.J3DSettings.noBackground) {
					super.setImage(null);
				} else {
					super.setImage(ic);
				}
			}
		};
		this.background.setImageScaleMode(Background.SCALE_FIT_ALL);
		this.background.setCapability(Background.ALLOW_IMAGE_WRITE);

		// if (CM2_CONSTANTS.IMAGES_AND_ICONS.M3D_BACKGROUND_IMAGE_BRIGHT !=
		// null
		// || CM2_CONSTANTS.IMAGES_AND_ICONS.M3D_BACKGROUND_IMAGE_DARK != null)
		// {

		// try {
		// BIbright = ImageIO
		// .read(new File(
		// CM2_CONSTANTS.IMAGES_AND_ICONS.M3D_BACKGROUND_IMAGE_BRIGHT));
		//
		// } catch (Exception e) {
		// BIbright = null;
		// }
		//
		// try {
		// BIdark = ImageIO
		// .read(new File(
		// CM2_CONSTANTS.IMAGES_AND_ICONS.M3D_BACKGROUND_IMAGE_DARK));
		// } catch (Exception e) {
		// BIdark = null;
		// }

		if (this.BIdark != null)
			this.ic = new ImageComponent2D(ImageComponent.FORMAT_RGB,
					J3DSettings.uniformBackground ? this.BIdarkUni
							: this.BIdark);
		else if (this.BIbright != null)
			this.ic = new ImageComponent2D(ImageComponent.FORMAT_RGB,
					J3DSettings.uniformBackground ? this.BIbrightUni
							: this.BIbright);
		// } else
		// ic = null;

		// try {
		// BIdark_Wet = ImageIO
		// .read(new File(
		// CM2_CONSTANTS.IMAGES_AND_ICONS.M3D_BACKGROUND_IMAGE_DARK_Wet));
		// } catch (Exception e) {
		// BIdark_Wet = null;
		// }
		//
		// try {
		// BIbright_Wet = ImageIO
		// .read(new File(
		// CM2_CONSTANTS.IMAGES_AND_ICONS.M3D_BACKGROUND_IMAGE_BRIGHT_Wet));
		// } catch (Exception e) {
		// BIbright_Wet = null;
		// }
		if (this.ic != null) {
			this.background.setImage(this.ic);
			this.background.setImageScaleMode(Background.SCALE_FIT_ALL);
			this.background.setCapability(Background.ALLOW_IMAGE_WRITE);
		}

		this.background.setCapability(Background.ALLOW_COLOR_WRITE);
		this.background.setApplicationBounds(worldBounds);
		this.basementBranchGroup.addChild(this.background);

		this.light1 = new PointLight();
		this.light2 = new PointLight();
		this.light3 = new PointLight();
		this.ambientLight = new AmbientLight();

		final Color3f lightColor = new Color3f(1f, 1f, 1f);
		final Color3f lightColor2 = new Color3f(0.5f, 0.5f, 0.5f);
		final Color3f ambientColor = new Color3f(
				DataPool.J3DSettings.ambientlight,
				DataPool.J3DSettings.ambientlight,
				DataPool.J3DSettings.ambientlight);

		this.light1.setColor(lightColor);
		this.light2.setColor(lightColor2);
		this.light3.setColor(lightColor2);
		this.ambientLight.setColor(ambientColor);

		final BoundingSphere bounds = new BoundingSphere(new Point3d(0.0, 0.0,
				0.0), 100000.0);

		this.light1.setInfluencingBounds(bounds);
		this.light2.setInfluencingBounds(bounds);
		this.light3.setInfluencingBounds(bounds);
		this.ambientLight.setInfluencingBounds(bounds);

		this.light1.setCapability(Light.ALLOW_STATE_WRITE);
		this.light1.setCapability(PointLight.ALLOW_POSITION_WRITE);

		this.light2.setCapability(Light.ALLOW_STATE_WRITE);
		this.light2.setCapability(PointLight.ALLOW_POSITION_WRITE);

		this.light3.setCapability(Light.ALLOW_STATE_WRITE);
		this.light3.setCapability(PointLight.ALLOW_POSITION_WRITE);

		this.ambientLight.setCapability(Light.ALLOW_COLOR_WRITE);
		this.ambientLight.setCapability(Light.ALLOW_STATE_WRITE);

		this.light1.setEnable(DataPool.J3DSettings.light1);
		this.light2.setEnable(DataPool.J3DSettings.light2);
		this.light3.setEnable(DataPool.J3DSettings.light2);

		final BranchGroup lbg1 = new BranchGroup();
		final BranchGroup lbg2 = new BranchGroup();
		final BranchGroup lbg3 = new BranchGroup();

		lbg1.addChild(this.light1);
		lbg2.addChild(this.light2);
		lbg3.addChild(this.light3);

		this.basementBranchGroup.addChild(lbg1);
		this.basementBranchGroup.addChild(lbg2);
		this.basementBranchGroup.addChild(lbg3);

		this.basementBranchGroup.addChild(this.ambientLight);

		this.coordinateBar = new CoordinateBar();

		this.raster = new Raster();

		this.mainBranchGroup.addChild(this.coordinateBar.getBranchGroup());
		this.mainBranchGroup.addChild(this.raster.getBranchGroup());

		this.setBrightEditorAppearance(DataPool.J3DSettings.bright_appearance);

		this.universe.addBranchGraph(this.basementBranchGroup);

		this.MyPick = new PickCanvas(this.canvas, this.mainBranchGroup);

		this.cameraMode = new CameraMode(this.canvas, this.universe);
		this.intersector = new IntersectManager();

		this.hud = new Hud(this.canvas, this.MyPick);
		this.PNDmanager = new PickNDragManager(this.canvas, this.intersector,
				this.cameraMode, this.MyPick, this.light1, this.light2,
				this.light3, this.hud);

		this.hud.setPickNDragManager(this.PNDmanager);
		this.MDmanager = new MicroDomainManager(this.canvas,
				this.mainBranchGroup, this.PNDmanager, this.defaultArea);

		this.universe.getCanvas().getView()
				.setSceneAntialiasingEnable(DataPool.J3DSettings.antialiasing);
		if (((BranchGroup) this.defaultArea.branch.getParent()) != null)
			((BranchGroup) this.defaultArea.branch.getParent())
					.removeChild(this.defaultArea.branch);

		this.setVisible(true);

	}

	/**
	 * Establishes access to major functionalities for the 3d classes
	 * 
	 * @param listener
	 */
	public void addMembraneViewListener(final MembraneViewListener listener) {
		this.listener = listener;
		this.cameraMode.addMembraneViewListener(listener);
		this.PNDmanager.addMembraneViewListener(listener);
	}

	/**
	 * returns the 3D universe
	 * 
	 * @return
	 */
	public SimpleUniverse getUniverse() {
		return this.universe;
	}

	/**
	 * adds a Molecule to the 3d view if the boolean is set, the axis of the 3D
	 * modell will be repainted.
	 * 
	 * 
	 * @param prot
	 * @param rebuildAxis
	 */

	public void addMolecule(final Molecule_3D mol, final boolean rebuildAxis) {

		mol.move();
		this.mainBranchGroup.addChild(mol.getBranchGroup());

		if (rebuildAxis == true)
			this.setShowAxis(this.axisVisible);

		mol.setRenderMode(DataPool.J3DSettings.overallRenderMode);

	}

	/**
	 * Tries to add the passed Protein to the Membrane. Returns true, if
	 * successful, false otherwise
	 * 
	 * @param newProt
	 *            or null if newone should be created
	 * @param code
	 *            - can be null if passed protein !=null
	 * @param file
	 *            - can be null if passed protein !=null
	 * @param domainID
	 * @return
	 */
	public boolean addANewProteinAtRandomPosition(final String code,
			final String file, final int modelnumber, final int domainID) {

		final MembraneModel model = DataPool.getMembraneModel();

		Protein_3D newProt;

		Protein_3D other = null;
		for (int i = 0; i < model.getProteinCount(); i++) {
			other = (Protein_3D) model.getFirstMoleculeThatHasFile(new File(
					file), 0, modelnumber);
			if (other != null) {

				break;
			}
		}

		// if (other != null) {
		// final Atom[] altered = DataPool
		// .getAtomsWithAlteredPositions(other);
		// if (altered != null && altered.length > 0)
		// other = null;
		// }

		if (other != null) {

			newProt = (Protein_3D) other.copy(false, true);
		} else {

			try {
				final PdbReader_3D reader = new PdbReader_3D();
				newProt = reader.readProtein(new File(file), modelnumber, null);
				newProt.setPDBcode(code);
				newProt.setColor(Cm2_Toolkit.getRandomProteinColor());

			} catch (final Exception e) {
				JOptionPane.showMessageDialog(
						null,
						"An Error occured during reading!\nReason:\n\n"
								+ e.getMessage(), "ERROR",
						JOptionPane.ERROR_MESSAGE);
				return true;
			}

		}
		newProt.setID(DataPool.getNextFreeProteinId());

		// ===============================================================
		// TODO: PPE Update 03_2012 |
		// ===============================================================
		if (DataPool.J3DSettings.enablePPEAutomatic) {
			// PPE automatic is enabled
			// -> automatic OPM adjustment
			// -> automatic PDBTM adjustment
			if (newProt.opmAtoms.length > 0) {
				// molecule is a OPM File
				DataPool.getMifHandler().applyOPMAlignmentTo(newProt);
			} else {
				// molecule isn't a OPM file.
				// this also works when molecule isn't a PDBTM file
				// because of its default values stored in
				// Y-translation and rotation matrix.
				newProt.setRotationMatrix(newProt.getPDBTMRotation());
				newProt.setYTranslation(newProt.getPDBTMTranslation());
			}
		} else {
			// PPE automatic is disabled
			newProt.setRotationMatrix(Cm2_Toolkit.createAUnitMatrix());
		}

		MicroDomain_3D micro;
		Rectangle dim;
		if (domainID > 0) {
			micro = model.getMicroDomain(domainID - 1);
			dim = micro.getBoundingBox();
		} else {
			dim = new Rectangle(0, 0, model.getSize().width,
					model.getSize().height);
			micro = null;
		}

		final double H = dim.getHeight();
		final double W = dim.getWidth();

		for (int j = 0; j < CM2_CONSTANTS.MISC.RETRIES_AT_RANDOM_PROTEIN_PLACING; j++) {

			int xpos = new Random().nextInt((int) W);
			int zpos = new Random().nextInt((int) H);

			if (micro != null) {
				xpos += dim.x;
				zpos += dim.y;
				if (!new Area(micro.geoObject).contains(new Point(xpos, zpos))) {
					continue;
				}
			} else {
				boolean b = false;
				for (int i = 0; i < model.countMicroDomains(); i++) {
					if (new Area(model.getMicroDomain(i).geoObject)
							.contains(new Point(xpos, zpos))) {
						b = true;
						break;
					}
				}
				if (b)
					continue;
			}

			newProt.setPosition(xpos, zpos);

			if (this.intersector
					.intercectsWithBorderOrAnyProtein(newProt, true)) {

				continue;
			} else {

				this.addMolecule(newProt, false);
				model.addProtein(newProt, DataPool.getCurrentLayerID());

				return true;
			}

		}

		return false;
	}

	/**
	 * clears the selection. after this, no protein is selected
	 */
	public void clearSelection() {
		this.PNDmanager.getSelected().clear();

	}

	/**
	 * clears the lipid Vector. Deletes all lipids from the membrane model and
	 * from the 3d view
	 * 
	 * @param domainID
	 * @param layerID
	 * 
	 */
	public void clearLipids(final int domainID, final int layerID) {

		if (DataPool.J3DSettings.enableFastTransitions)
			this.dockScreen(false);

		final MembraneModel model = DataPool.getMembraneModel();

		final Lipid_3D[] lipsEx = model.getAllLipidsExtraInDomain(domainID,
				layerID);
		final Lipid_3D[] lipsIn = model.getAllLipidsIntraInDomain(domainID,
				layerID);
		this.removeMoleculesFrom3D(lipsEx);
		this.removeMoleculesFrom3D(lipsIn);

		for (int i = 0; i < lipsEx.length; i++)
			model.deleteLipidExtra(lipsEx[i]);

		for (int i = 0; i < lipsIn.length; i++)
			model.deleteLipidIntra(lipsIn[i]);

		this.listener.setShowAxis(this.listener.areAxisVisible());
		this.listener.refreshViews(false, true, false, true, true, true);

		if (DataPool.J3DSettings.enableFastTransitions)
			this.dockScreen(true);
	}

	/**
	 * clears all Vectors. Delets Lipids and Proteins from the membrane model
	 * and from the view
	 * 
	 * @param clearSampleLipidsToo
	 * @param clearSampleLipidsToo
	 * @param b
	 * 
	 */
	public void clearAll(final boolean clearProteins,
			final boolean clearLipidsInMembrane,
			final boolean clearSampleLipids, final int layerID) {
		final MembraneModel model = DataPool.getMembraneModel();
		if (clearLipidsInMembrane) {

			if (DataPool.J3DSettings.enableFastTransitions)
				this.dockScreen(false);

			Vector<Lipid_3D> toremove = new Vector<Lipid_3D>();

			for (int i = 0; i < model.getLipidCountExtra(); i++) {
				final Lipid_3D lip = model.getLipidAtExtra(i);
				if (layerID < 0 || lip.getLayerID() == layerID)
					toremove.addElement(lip);
			}

			this.removeMoleculesFrom3D(toremove.toArray(new Lipid_3D[0]));
			for (int i = 0; i < toremove.size(); i++) {
				model.deleteLipidExtra(toremove.get(i));
			}

			toremove = new Vector<Lipid_3D>();

			for (int i = 0; i < model.getLipidCountIntra(); i++) {
				final Lipid_3D lip = model.getLipidAtIntra(i);
				if (layerID < 0 || lip.getLayerID() == layerID)
					toremove.addElement(lip);
			}

			this.removeMoleculesFrom3D(toremove.toArray(new Lipid_3D[0]));
			for (int i = 0; i < toremove.size(); i++) {
				model.deleteLipidIntra(toremove.get(i));
			}

			if (DataPool.J3DSettings.enableFastTransitions)
				this.dockScreen(true);

		}
		if (clearProteins) {
			final Vector<Protein_3D> toremove = new Vector<Protein_3D>();

			for (int i = 0; i < model.getProteinCount(); i++) {
				final Protein_3D prot = model.getProteinAt(i);
				if (layerID < 0 || prot.getLayerID() == layerID)
					toremove.addElement(prot);
			}

			this.removeMoleculesFrom3D(toremove.toArray(new Protein_3D[0]));
			for (int i = 0; i < toremove.size(); i++) {
				model.deleteProtein(toremove.get(i));
			}
		}
		if (clearSampleLipids) {
			Vector<Lipid_3D> toremove = new Vector<Lipid_3D>();

			for (int i = 0; i < model.getExampleLipidsCountExtra(); i++) {
				final Lipid_3D lip = model.getExampleLipidAtExtra(i);
				if (layerID < 0 || lip.getLayerID() == layerID)
					toremove.addElement(lip);
			}

			for (int i = 0; i < toremove.size(); i++) {
				model.deleteExampleLipidExtra(toremove.get(i));
			}

			toremove = new Vector<Lipid_3D>();

			for (int i = 0; i < model.getExampleLipidsCountIntra(); i++) {
				final Lipid_3D lip = model.getExampleLipidAtIntra(i);
				if (layerID < 0 || lip.getLayerID() == layerID)
					toremove.addElement(lip);
			}

			for (int i = 0; i < toremove.size(); i++) {
				model.deleteExampleLipidIntra(toremove.get(i));
			}

		}

		this.listener.setShowAxis(this.listener.areAxisVisible());
		DataPool.getMembraneModel().organizeDomains();
		this.listener.refreshViews(true, true, true, true, true, true);
	}

	/**
	 * Returns the selected Proteins from the PNDmanager
	 * 
	 * @return
	 */
	public Vector<Molecule_3D> getSelectedMolecules() {
		return this.PNDmanager.getSelected();
	}

	/**
	 * removes given proteins from the 3D model
	 * 
	 * @param protVec
	 */
	public void removeMoleculesFrom3D(final Molecule_3D[] mols) {

		boolean rebuildaxis = false;
		for (int i = 0; i < mols.length; i++) {

			if (mols[i] instanceof Protein_3D)
				rebuildaxis = true;

			this.mainBranchGroup.removeChild(mols[i].getBranchGroup());

		}

		if (rebuildaxis) {
			this.setShowAxis(this.axisVisible);
			this.setShowRaster(this.rasterVisible);

		}

	}

	/**
	 * repaints the 3D model by using the informations from the model. clears
	 * the current 3D model.
	 * 
	 * 
	 * 
	 * @param model
	 */
	public void repaintByMembraneModel(final MembraneModel model) {

		this.basementBranchGroup.removeChild(this.mainBranchGroup);

		final Enumeration<?> en = this.mainBranchGroup.getAllChildren();

		while (en.hasMoreElements()) {
			final Object o = en.nextElement();
			if (o instanceof BranchGroup
					&& ((BranchGroup) o).getUserData() instanceof Molecule_3D)
				this.mainBranchGroup.removeChild((BranchGroup) o);
		}

		for (int i = 0; i < model.getProteinCount(); i++) {

			final Protein_3D protS = model.getProteinAt(i);
			this.addMolecule(protS, false);

		}

		for (int i = 0; i < model.getLipidCountExtra(); i++) {
			final Lipid_3D lipEx = model.getLipidAtExtra(i);
			this.addMolecule(lipEx, false);
		}
		for (int i = 0; i < model.getLipidCountIntra(); i++) {
			final Lipid_3D lipIn = model.getLipidAtIntra(i);
			this.addMolecule(lipIn, false);
		}

		this.getMicroDomainManager().updateDisplayOfMicroDomains();
		this.setShowAxis(this.axisVisible);
		this.setShowRaster(this.rasterVisible);

		this.basementBranchGroup.addChild(this.mainBranchGroup);

		Molecule_3D[] mols = model.getAllMoleculesInMembrane();
		for (int i = 0; i < mols.length; i++) {
			mols[i].updateVisualAtomPositionsOnly();
		}
	}

	/**
	 * En- or disables the 3d display
	 */
	@Override
	public void setEnabled(final boolean b) {

		super.setEnabled(b);
		this.canvas.setEnabled(b);
		this.canvas.setVisible(b);

		if (this.canvas.getStereoAvailable()) {
			try {
				this.listener.showStereoView();
			} catch (final Exception e) {
			}
		}

		if (b == true) {
			final MembraneModel model = DataPool.getMembraneModel();
			if (model == null)
				return;

		}
	}

	/**
	 * Sets the preferred size for the 3d display
	 */
	@Override
	public void setPreferredSize(final Dimension d) {
		super.setPreferredSize(d);
		this.canvas.setSize(d);
	}

	/**
	 * En-or disables the axis bars in the view. If those are activated while
	 * still active, this serves as a refresh method in case the bounds of the
	 * membrane changed
	 * 
	 * @param b
	 */
	public synchronized void setShowAxis(final boolean b) {
		if (this.isEnabled() == false)
			return;
		this.axisVisible = b;
		this.coordinateBar.setBarsVisible(b);

	}

	/**
	 * En-or disables the Raster painted on the Plane in the view. If activated
	 * while still active, this serves as a refresh method in case the bounds of
	 * the membrane changed
	 * 
	 * @param b
	 */
	public void setShowRaster(final boolean b) {
		if (this.isEnabled() == false)
			return;

		this.defaultArea.geoObject = new Rectangle(-1, -1, DataPool
				.getMembraneModel().getSize().width + 2, DataPool
				.getMembraneModel().getSize().height + 2);
		this.defaultArea.updateShape3D();
		this.rasterVisible = b;
		this.raster.setRasterVisible(b);

	}

	/**
	 * Returns if the axis are visible currently
	 * 
	 * @return
	 */
	public boolean areAxisVisible() {
		return this.axisVisible;
	}

	/**
	 * Returns if the raster is visible currently
	 * 
	 * @return
	 */
	public boolean isRasterVisible() {
		return this.rasterVisible;
	}

	// /**
	// * Updates the alignment of a molecule. The referring 3d object in the
	// * view will be aligned the way the passed molecule is
	// * @param currentMolecule
	// */
	// public void updateAlignment3D(Molecule_3D currentMolecule) {
	//
	// TransformGroup tg = currentMolecule.getTransformGroup();
	// Transform3D t3d = new Transform3D();
	//
	// Vector3f v3f = new Vector3f(currentMolecule.getX()
	// * CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE, currentMolecule
	// .getYTranslation()
	// * CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE, currentMolecule.getZ()
	// * CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE);
	// t3d.setTranslation(v3f);
	// t3d.setRotation(currentMolecule.getRotationMatrix());
	// tg.setTransform(t3d);
	//
	// Bounds bnds = currentMolecule.getBounds();
	//
	// bnds.transform(t3d);
	// currentMolecule.setCollisionBounds(bnds);
	//
	// }

	/**
	 * Sets the ambient colors brightness
	 * 
	 * @param f
	 *            values between 0f and 1f
	 */
	public void setAmbientBrightness(final float f) {

		final Color3f color = new Color3f(f, f, f);
		this.ambientLight.setColor(color);
	}

	/**
	 * En-or disables light1
	 * 
	 * @param b
	 */
	public void setEnableLight1(final boolean b) {
		this.light1.setEnable(b);
		this.getPickNDragManager().updateLights();
	}

	/**
	 * En-or disables light2 (containing of 2 light sources that can only be
	 * activated together)
	 * 
	 * @param b
	 */
	public void setEnableLight2(final boolean b) {
		this.light2.setEnable(b);
		this.light3.setEnable(b);
		this.getPickNDragManager().updateLights();
	}

	/**
	 * Sets the appearance of the editor. If true is passed, editor will get a
	 * bright appearance, otherwise a dark one
	 * 
	 * @param bright_App
	 */
	public void setBrightEditorAppearance(final boolean bright_App) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				try {
					// this.background.setColor(null);

					if (bright_App) {
						Membrane3D.this.ic = new ImageComponent2D(
								ImageComponent.FORMAT_RGB,
								Membrane3D.this.notWet ? J3DSettings.uniformBackground ? Membrane3D.this.BIbrightUni
										: Membrane3D.this.BIbright
										: J3DSettings.uniformBackground ? Membrane3D.this.BIbrightUni_Wet
												: Membrane3D.this.BIbright_Wet);

						if (Membrane3D.this.ic != null) {
							Membrane3D.this.background
									.setImage(Membrane3D.this.ic);

						}

					} else {
						Membrane3D.this.ic = new ImageComponent2D(
								ImageComponent.FORMAT_RGB,
								Membrane3D.this.notWet ? J3DSettings.uniformBackground ? Membrane3D.this.BIdarkUni
										: Membrane3D.this.BIdark
										: J3DSettings.uniformBackground ? Membrane3D.this.BIdarkUni_Wet
												: Membrane3D.this.BIdark_Wet);
						if (Membrane3D.this.ic != null) {
							Membrane3D.this.background
									.setImage(Membrane3D.this.ic);

						}
					}
				} catch (final Exception e) {
					e.printStackTrace();
				}
			}
		});

		if (DataPool.getMembraneModel() == null)
			return;

		Color3f color;
		if (DataPool.J3DSettings.bright_appearance)
			color = new Color3f(
					this.notWet ? CM2_CONSTANTS.J3D.COLOR_COORDINATEBARS_BRIGHT
							: CM2_CONSTANTS.J3D.COLOR_COORDINATEBARS_BRIGHT_WET);
		else
			color = new Color3f(
					this.notWet ? CM2_CONSTANTS.J3D.COLOR_COORDINATEBARS_DARK
							: CM2_CONSTANTS.J3D.COLOR_COORDINATEBARS_DARK_WET);

		this.coordinateBar.setColor(color);
		this.raster.setColor(color);

		this.setShowAxis(this.axisVisible);
		this.setShowRaster(this.rasterVisible);
	}

	public void setUniformBackground(boolean uniform) {
		if (J3DSettings.bright_appearance) {
			Membrane3D.this.ic = new ImageComponent2D(
					ImageComponent.FORMAT_RGB,
					Membrane3D.this.notWet ? 
							uniform ? Membrane3D.this.BIbrightUni
									: Membrane3D.this.BIbright : 
							uniform ? Membrane3D.this.BIbrightUni_Wet
									: Membrane3D.this.BIbright_Wet);

			if (Membrane3D.this.ic != null) {
				Membrane3D.this.background.setImage(Membrane3D.this.ic);

			}

		} else {
			Membrane3D.this.ic = new ImageComponent2D(
					ImageComponent.FORMAT_RGB,
					Membrane3D.this.notWet ? 
							uniform ? Membrane3D.this.BIdarkUni
									: Membrane3D.this.BIdark : 
							uniform ? Membrane3D.this.BIdarkUni_Wet
									: Membrane3D.this.BIdark_Wet);
			if (Membrane3D.this.ic != null) {
				Membrane3D.this.background.setImage(Membrane3D.this.ic);

			}
		}
	}

	/**
	 * Returns the camera
	 * 
	 * @return
	 */
	public CameraMode getCamera() {

		return this.cameraMode;
	}

	/**
	 * Selects all protein instances that have the passed pdb code
	 * 
	 * @param code
	 * @param domainID
	 */
	public void selectByCode(final String code, final int domainID) {
		this.PNDmanager.selectByCode(code, domainID);

	}

	// /DND section //////////////

	/**
	 * Called, when a drag event is recognized in the database table. The mouse
	 * position on screen gets delivered here, and will be used for determining
	 * the location on the canvas
	 * 
	 * If position enters the canvas, the dragged object will be added into
	 * universe. If position exits, it will be removed. While moving inside
	 * canvas range, the object gets moved like all other objects
	 */
	public void DnDEventOccured(final Point loc) {
		if (DataPool.DNDBridge.getCurrentMolecule() == null)
			return;

		final Rectangle b = new Rectangle(this.canvas.getLocationOnScreen().x,
				this.canvas.getLocationOnScreen().y, this.canvas.getWidth(),
				this.canvas.getHeight());

		if (b.contains(loc)) {
			if (DataPool.DNDBridge.getCurrentMolecule().getBranchGroup()
					.getParent() == null) {
				this.addMolecule(DataPool.DNDBridge.getCurrentMolecule(), false);
			}
			SwingUtilities.convertPointFromScreen(loc, this.canvas);

			this.PNDmanager.moveDNDObject(
					DataPool.DNDBridge.getCurrentMolecule(), loc);

		} else {
			if (DataPool.DNDBridge.getCurrentMolecule().getBranchGroup()
					.getParent() != null) {
				this.removeMoleculesFrom3D(new Molecule_3D[] { DataPool.DNDBridge
						.getCurrentMolecule() });
				// happens because a used shape cannot be added twice. So
				// it gets replaced

				if (DataPool.DNDBridge.getCurrentMolecule() instanceof Protein_3D)
					DataPool.DNDBridge
							.overwriteCurrentMolecule(DataPool.DNDBridge
									.getCurrentMolecule().copy(true, true));
				else
					DataPool.DNDBridge
							.overwriteCurrentMolecule(DataPool.DNDBridge
									.getCurrentMolecule().copy(true, false));

				DataPool.DNDBridge.getCurrentMolecule()
						.setPositionIsInAValidSpace(false);
			}

		}

	}

	/**
	 * Finishes the DND. If cancelled is true, or if the dragged Object is not
	 * in valid space, it simply gets removed. Otherwise, a new copy of the
	 * dragged Object will be integrated properly into the mebrane
	 * 
	 * @param cancelled
	 */
	public void DnDFinished(boolean cancelled) {

		if (DataPool.DNDBridge.getCurrentMolecule() == null)
			return;

		final int domainID = DataPool.getMembraneModel().getMicroDomainAtPoint(
				new Point((int) DataPool.DNDBridge.getCurrentMolecule().getX(),
						(int) DataPool.DNDBridge.getCurrentMolecule().getZ()),
				DataPool.getCurrentLayerID());

		if (domainID < 0 && cancelled == false) {
			JOptionPane
					.showMessageDialog(
							this,
							"The center point of the molecule must\nbe placed inside membrane bounds!",
							"Illega Position", JOptionPane.ERROR_MESSAGE);
			cancelled = true;
		} else if ((DataPool.DNDBridge.getCurrentMolecule() instanceof Lipid_3D && domainID != DataPool.DNDBridge
				.getCurrentMolecule().getDomainID()) && cancelled == false) {
			JOptionPane
					.showMessageDialog(
							this,
							"The lipid can only be placed\ninside its associated area!",
							"Illega Position", JOptionPane.ERROR_MESSAGE);
			cancelled = true;
		}

		final boolean live = DataPool.DNDBridge.getCurrentMolecule()
				.getBranchGroup().isLive();

		this.removeMoleculesFrom3D(new Molecule_3D[] { DataPool.DNDBridge
				.getCurrentMolecule() });

		if (cancelled || !live) {

			// listener.highlightMicroDomain(0);
		} else {

			final Molecule_3D mol = DataPool.DNDBridge.getCurrentMolecule()
			// .copy(true)
			;

			if (mol instanceof Protein_3D) {
				DataPool.getMembraneModel().addProtein((Protein_3D) mol,
						DataPool.getCurrentLayerID());
			} else {
				if (((Lipid_3D) mol).getMembraneSide() == Membrane3D.EXTRA) {
					DataPool.getMembraneModel().addLipidExtra((Lipid_3D) mol,
							((Lipid_3D) mol).getDomainID(),
							DataPool.getCurrentLayerID());
				} else {
					DataPool.getMembraneModel().addLipidIntra((Lipid_3D) mol,
							((Lipid_3D) mol).getDomainID(),
							DataPool.getCurrentLayerID());
				}
			}
			this.addMolecule(mol, true);
			DataPool.setMembraneHasBeenEditedAgain(true);

			DataPool.getMembraneModel().organizeDomains();
			this.listener.refreshViews(true, true, true, true, true, true);

			this.listener.pickMolecules(new Molecule_3D[] { mol });

		}
		DataPool.DNDBridge.deactivate();
	}

	// END DND section
	/**
	 * Returns the canvas
	 * 
	 * @return
	 */
	public Canvas3D getCanvas() {

		return this.canvas;
	}

	/**
	 * Returns the PND manager
	 * 
	 * @return
	 */
	public PickNDragManager getPickNDragManager() {
		return this.PNDmanager;

	}

	public Hud getHud() {
		return this.hud;

	}

	/**
	 * Switches the camera to the requested view.
	 * 
	 * @param mode
	 */
	public void setViewByMode(final int mode, final Point3f center) {
		this.cameraMode.setViewByMode(mode, center);
		if (mode == CameraMode.FREE)
			if (DataPool.J3DSettings.forbidDraggingInFreeLookMode == false)
				this.PNDmanager.allowDragging(true);
			else
				this.PNDmanager.allowDragging(false);
		else
			this.PNDmanager.allowDragging(true);

		final Thread t = new Thread() {
			@Override
			public void run() {
				try {
					Thread.sleep(100);
				} catch (final InterruptedException e) {
					return;
				}
				Membrane3D.this.PNDmanager.updateLights();
			}
		};
		t.start();

	}

	public MicroDomainManager getMicroDomainManager() {
		return this.MDmanager;
	}

	public synchronized void dockScreen(final boolean b) {

		if (DataPool.J3DSettings.isScreenDocked == b)
			return;

		if (b) {
			if (this.mainBranchGroup.getParent() == null)
				this.basementBranchGroup.addChild(this.mainBranchGroup);
		} else if (!DataPool.J3DSettings.enableAtomPerformanceMode) {
			 if (this.mainBranchGroup.getParent() == this.basementBranchGroup) {
				 try {
					 this.mainBranchGroup.detach();
				 } catch(NullPointerException e) {
					 return;
				 }
			 }
		 }
		DataPool.J3DSettings.isScreenDocked = b;
	}

	public IntersectManager getIntersectManager() {
		return this.intersector;
	}

	public void updateAfterLayerChange() {

		this.listener.pickMolecules(null);
		this.listener.highlightMicroDomain(-1);

		final MembraneModel model = DataPool.getMembraneModel();

		int layerId = 0;
		if (DataPool.getCurrentLayer() != null)
			layerId = DataPool.getCurrentLayer().getLayerId();

		// float y=DataPool.getCurrentLayerHeight();

		// float yHeight=0;
		// Layer layer = DataPool.getCurrentLayer();
		// if(layer!=null)
		// yHeight=layer.getYHeight();
		if (this.axisVisible) {
			this.setShowAxis(true);
		}
		if (this.rasterVisible) {
			this.setShowRaster(true);
		}

		for (int i = 0; i < model.countMicroDomains(); i++) {

			final MicroDomain_3D md = model.getMicroDomain(i);

			if (md.layerId == layerId) {

				// md.branch.detach();
				md.setEnabled(true);
				// microBG.addChild(md.branch);

			} else {

				md.setEnabled(false);
				// microBG.removeChild(md.branch);
			}
		}

		this.listener.updateMicroDomainsIn3D();

	}

	public BranchGroup getMainBranchGroup() {
		return this.mainBranchGroup;

	}

	public BranchGroup getBasementBranchGroup() {
		return this.basementBranchGroup;

	}

	private void manipulateAmbience() {

		final Point3f eye = CameraMode.getCurrentEyePosition();

		if (eye.y < 0) {
			if (this.notWet == false)
				return;
			this.notWet = false;
			final float orig = DataPool.J3DSettings.ambientlight;

			if (orig < 0.2f) {
				this.ambientLight.setColor(new Color3f(new Color(
						DataPool.J3DSettings.ambientlight,
						DataPool.J3DSettings.ambientlight,
						DataPool.J3DSettings.ambientlight + 0.15f)));
			} else {
				this.ambientLight.setColor(new Color3f(new Color(
						DataPool.J3DSettings.ambientlight - 0.1f,
						DataPool.J3DSettings.ambientlight - 0.1f,
						DataPool.J3DSettings.ambientlight)));
			}

			this.setBrightEditorAppearance(DataPool.J3DSettings.bright_appearance);

		} else {
			if (this.notWet == true)
				return;
			this.notWet = true;

			this.ambientLight.setColor(new Color3f(new Color(
					DataPool.J3DSettings.ambientlight,
					DataPool.J3DSettings.ambientlight,
					DataPool.J3DSettings.ambientlight)));

			this.setBrightEditorAppearance(DataPool.J3DSettings.bright_appearance);

		}

	}

}
