/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_1)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.io;

import java.awt.Component;
import java.io.File;
import java.io.IOException;

import javax.swing.Icon;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileView;

import org.cellmicrocosmos.cm2.engine.DataPool;
import org.cellmicrocosmos.cm2.gui.MyFCAccessory;
import org.cellmicrocosmos.cm2.gui.MylFCAccessoryForGenerating;
import org.cellmicrocosmos.cm2.misc.Cm2_IconDepot;
import org.cellmicrocosmos.cm2.misc.ConfigParser;
import org.jdom.JDOMException;


/**
 * This class provides the filechoosing processes. It also awards the clearance
 * for loading or saving an xml file. Can also report Exceptions to the User
 * 
 * @author Tim Dingersen dackelnose@gmx.net
 * 
 */
public class FileIOHandler {

	// the component OptionPanes will popup relative to
	Component componentforoptionpane;

	public final int LOADMEMBRANE = 0;

	public final int SAVEMEMBRANE = 1;
	
	public final int SAVEIMAGE = 2;

	public final int LOADDATABASE = 3;

	public final int NEWDATABASE =  4;
	
	public final int SAVEPROTOCOL = 5;
	

	public static String home;

	/**
	 * Creates a new XmlIOManager. Messages will appear in the middle of the
	 * screen
	 */
	public FileIOHandler() {
		this.componentforoptionpane = null;

	}

	/**
	 * Creates a new XmlIOManager. Messages will appear relative to the passed
	 * Component
	 * 
	 * @param componentforoptionpane
	 */
	public FileIOHandler(final Component componentforoptionpane) {
		this.componentforoptionpane = componentforoptionpane;

	}

	/**
	 * Initializes the specified file Choosing operation. Opens a Filechooser
	 * for the requested operation and if user chose, returns the selected file.
	 * Else null is returned
	 * 
	 * @param which
	 *            - which operation - see fields in this class
	 */
	public File requestUsertoChooseFile(final int which) {

		final JFileChooser fc = new JFileChooser();

		// make sure user can only apply having ONE NON-DIRECTORY file selected
		fc.setMultiSelectionEnabled(false);
		fc.setFileSelectionMode(JFileChooser.FILES_ONLY);
		fc.setAcceptAllFileFilterUsed(false);
		// customizing a filefilter that accepts only .xml's and .cm2's
		final FileFilter filter = new FileFilter() {

			public boolean accept(final File arg0) {
				if (arg0.isDirectory())
					return true;
				if ( ((which == FileIOHandler.this.LOADMEMBRANE || which == FileIOHandler.this.SAVEMEMBRANE) && !arg0.getName().toLowerCase().endsWith(".cm2")) ||
					 ((which == FileIOHandler.this.LOADDATABASE || which == FileIOHandler.this.NEWDATABASE) && !arg0.getName().toLowerCase().endsWith(".xml")) ||
					 (which == FileIOHandler.this.SAVEPROTOCOL && !arg0.getName().toLowerCase().endsWith(".xvg")) ||
					 (which == FileIOHandler.this.SAVEIMAGE && !arg0.getName().toLowerCase().endsWith(".jpg")))
					return false;
				else
					return true;
//				if (which != FileIOHandler.this.SAVEPROTOCOL && !arg0.getName().toLowerCase().endsWith(".cm2"))
//					return false;
//				else if (which == FileIOHandler.this.SAVEPROTOCOL && !arg0.getName().toLowerCase().endsWith(".xvg"))
//					return false;
//				return true;
			}

			public String getDescription() {
				if (which == FileIOHandler.this.LOADMEMBRANE
						|| which == FileIOHandler.this.SAVEMEMBRANE)
					return "Membrane Description (.Cm2)";
				else if (which == FileIOHandler.this.LOADDATABASE
						|| which == FileIOHandler.this.NEWDATABASE)
					return "Database Description (.xml)";
				else if (which == FileIOHandler.this.SAVEPROTOCOL)
					return "Grace Protocol (.xvg)";
				else if (which == FileIOHandler.this.SAVEIMAGE)
						return "JPEG (.jpg)";
				else
					return ".xml";
			}

		};
		// setting the filefilter
		fc.setFileFilter(filter);

		// customizing a fileview
		final FileView fv = new FileView() {

			public Icon getIcon(final File arg0) {

				if (arg0.isDirectory())
					return null;

				if (which == FileIOHandler.this.LOADMEMBRANE
						|| which == FileIOHandler.this.SAVEMEMBRANE)
					return Cm2_IconDepot.getSpecifiedIcon("FC_Membrane");

				else if (which == FileIOHandler.this.LOADDATABASE
						|| which == FileIOHandler.this.NEWDATABASE)
					return Cm2_IconDepot.getSpecifiedIcon("FC_Database");
				
				else if (which == FileIOHandler.this.SAVEIMAGE)
					return Cm2_IconDepot.getSpecifiedIcon("FC_Image");

				else
					return Cm2_IconDepot.getSpecifiedIcon("");
			}
		};

		fc.setFileView(fv);

		if (which == this.LOADMEMBRANE)
			fc.setAccessory(new MyFCAccessory(fc, 0, "Membrane Info"));
		if (which == this.LOADDATABASE)
			fc.setAccessory(new MyFCAccessory(fc, 1, "Database Info"));

		try {
			File filepath;

			/*
			 * trying to get the last used directory for the filechooser
			 */
			if (which == this.LOADMEMBRANE || which == this.SAVEMEMBRANE || 
				which == this.SAVEIMAGE || which == this.SAVEPROTOCOL)
				filepath = new File(ConfigParser
						.getValue("last_membrane_directory")[0]);
			else
				filepath = new File(ConfigParser
						.getValue("last_database_directory")[0]);

			// checks if the retrieved file is a valid directory
			if (!filepath.exists() || !filepath.isDirectory())
				throw new Exception();

			// ...and if so sets the filechoosers path
			fc.setCurrentDirectory(filepath);

			if (DataPool.getCurrentMembraneFile() != null)
				if (which != this.SAVEPROTOCOL)
					fc.setSelectedFile(DataPool.getCurrentMembraneFile());
				else
					fc.setSelectedFile(new File(DataPool.getCurrentMembraneFile().toString().toLowerCase().replaceAll(".cm2", ".xvg")));
			else
				this.setFileChooserSelection(fc, which);

		} catch (final Exception e) {

			// setting the default paths
			if ((which == this.LOADMEMBRANE || which == this.SAVEMEMBRANE || 
				which == this.SAVEIMAGE || which == this.SAVEPROTOCOL)
					&& DataPool.IO.DEFAULTMEMBRANEDIRECTORY.exists())
				fc.setCurrentDirectory(DataPool.IO.DEFAULTMEMBRANEDIRECTORY);
			else if ((which == this.LOADDATABASE || which == this.NEWDATABASE)
					&& DataPool.IO.DEFAULTDBDIRECTORY.exists())
				fc.setCurrentDirectory(DataPool.IO.DEFAULTDBDIRECTORY);

			this.setFileChooserSelection(fc, which);

		}

		/*
		 * here, a proper Dialog is opened and if the user applies
		 * successfully,the file will be returned the last used path will also
		 * be updated
		 */

		switch (which) {

		case 0:
			fc.setDialogTitle("Open Membrane");
			if (fc.showOpenDialog(this.componentforoptionpane) == JFileChooser.APPROVE_OPTION) {

				ConfigParser.deleteAllAt("last_membrane_directory");
				ConfigParser.addValue("last_membrane_directory", new File(fc
						.getSelectedFile().getParent()).getPath());
				return fc.getSelectedFile();
			}

			break;
		case 1:
			fc.setDialogTitle("Save Membrane As...");
			if (fc.showSaveDialog(this.componentforoptionpane) == JFileChooser.APPROVE_OPTION) {

				ConfigParser.deleteAllAt("last_membrane_directory");
				ConfigParser.addValue("last_membrane_directory", new File(fc
						.getSelectedFile().getParent()).getPath());

				File file = fc.getSelectedFile();
				// checking, if the file ends with ".cm2". If not, ".cm2" will
				// be
				// attached
				if (!file.getAbsolutePath().toLowerCase().endsWith(".cm2"))
					file = new File(file.getPath() + ".cm2");

				return file;
			}
			break;
		case 2:
			fc.setDialogTitle("Save Image As...");
			if (fc.showSaveDialog(this.componentforoptionpane) == JFileChooser.APPROVE_OPTION) {

				ConfigParser.deleteAllAt("last_membrane_directory");
				ConfigParser.addValue("last_membrane_directory", new File(fc
						.getSelectedFile().getParent()).getPath());

				File file = fc.getSelectedFile();
				// checking, if the file ends with ".cm2". If not, ".cm2" will
				// be
				// attached
				if (!file.getAbsolutePath().toLowerCase().endsWith(".jpg"))
					file = new File(file.getPath() + ".jpg");

				return file;
			}
			break;
		case 3:
			fc.setDialogTitle("Open Database");
			if (fc.showOpenDialog(this.componentforoptionpane) == JFileChooser.APPROVE_OPTION) {

				ConfigParser.deleteAllAt("last_database_directory");
				ConfigParser.addValue("last_database_directory", new File(fc
						.getSelectedFile().getParent()).getPath());
				return fc.getSelectedFile();
			}
			break;
		case 4:
			fc.setDialogTitle("Create New Database");
			if (fc.showSaveDialog(this.componentforoptionpane) == JFileChooser.APPROVE_OPTION) {

				ConfigParser.deleteAllAt("last_database_directory");
				ConfigParser.addValue("last_database_directory", new File(fc
						.getSelectedFile().getParent()).getPath());

				File file = fc.getSelectedFile();
				// checking, if the file ends with ".xml". If not, ".xml" will
				// be
				// attached
				if (!file.getAbsolutePath().toLowerCase().endsWith(".xml"))
					file = new File(file.getPath() + ".xml");

				return file;
			}
			break;
		case 5:
			fc.setDialogTitle("Save Time Protocol As...");
			if (fc.showSaveDialog(this.componentforoptionpane) == JFileChooser.APPROVE_OPTION) {

				ConfigParser.deleteAllAt("last_membrane_directory");
				ConfigParser.addValue("last_membrane_directory", new File(fc
						.getSelectedFile().getParent()).getPath());

				File file = fc.getSelectedFile();
				// checking, if the file ends with ".xvg". If not, ".xvg" will
				// be
				// attached
				if (!file.getAbsolutePath().toLowerCase().endsWith(".xvg"))
					file = new File(file.getPath() + ".xvg");

				return file;
			}
			break;

		}
		return null;

	}

	/**
	 * Initializes the specified file Choosing operation for the pdb File the
	 * generated Membran will have. Opens a Filechooser and if user chose,
	 * returns the selected file.
	 * 
	 * Else null is returned
	 * 
	 * 
	 */
	public File requestUsertoChooseFileForGeneratingMembrane() {

		final JFileChooser fc = new JFileChooser();
		final MylFCAccessoryForGenerating accessory = new MylFCAccessoryForGenerating();

		// make sure user can only apply having ONE NON-DIRECTORY file selected
		fc.setMultiSelectionEnabled(false);
		fc.setFileSelectionMode(JFileChooser.FILES_ONLY);
		fc.setAcceptAllFileFilterUsed(false);
		// customizing a filefilter that accepts only .pdb's
		final FileFilter filter = new FileFilter() {

			@Override
			public boolean accept(final File arg0) {
				if (arg0.getName().toLowerCase().endsWith(".pdb")
						|| arg0.isDirectory())
					return true;
				else
					return false;
			}

			@Override
			public String getDescription() {
				return ".pdb";
			}

		};
		// setting the filefilter
		fc.setFileFilter(filter);

		// customizing a fileview
		final FileView fv = new FileView() {

			@Override
			public Icon getIcon(final File arg0) {

				if (arg0.isDirectory())
					return null;

				return Cm2_IconDepot.getSpecifiedIcon("FC_Pdb_Ok");

			}
		};

		fc.setFileView(fv);
		fc.setAccessory(accessory);
		// fc.setPreferredSize(new
		// Dimension(fc.getPreferredSize().width,accessory.getPreferredSize().height));

		try {
			File filepath = null;

			/*
			 * trying to get the last used directory for the filechooser
			 */
			if (DataPool.getMembraneModel().getLastGeneratedMembraneFile() != null) {
				if (DataPool.getMembraneModel().getLastGeneratedMembraneFile()
						.getParentFile().exists())
					filepath = DataPool.getMembraneModel()
							.getLastGeneratedMembraneFile().getParentFile();
			}
			if (filepath == null) {

				filepath = new File(ConfigParser
						.getValue("last_finished_membrane_output_directory")[0]);

				// checks if the retrieved file is a valid directory
				if (!filepath.exists() || !filepath.isDirectory())
					throw new Exception();
			}
			// ...and if so sets the filechoosers path
			fc.setCurrentDirectory(filepath);

		} catch (final Exception e) {

		}

		if (DataPool.getMembraneModel().getLastGeneratedMembraneFile() != null) {
			
			fc.setSelectedFile(DataPool.getMembraneModel()
					.getLastGeneratedMembraneFile());
		}

		else if (DataPool.getMembraneModel().getCode() != null
				&& DataPool.getMembraneModel().getCode().length() > 0) {

			fc.setSelectedFile(new File(DataPool.getMembraneModel().getCode()));
		} else {

			this.setFileChooserSelection(fc, -1);
		}

		fc.setDialogTitle("Generate Membrane As...");
		if (fc.showSaveDialog(this.componentforoptionpane) == JFileChooser.APPROVE_OPTION) {

			ConfigParser.deleteAllAt("last_finished_membrane_output_directory");
			ConfigParser.addValue("last_finished_membrane_output_directory",
					new File(fc.getSelectedFile().getParent()).getPath());

			File file = fc.getSelectedFile();
			// checking, if the file ends with ".pdb". If not, ".pdb" will
			// be
			// attached
			if (!file.getAbsolutePath().toLowerCase().endsWith(".pdb"))
				file = new File(file.getPath() + ".pdb");

			if (file.equals(DataPool.getMembraneModel()
					.getLastGeneratedMembraneFile()) == false) {
				DataPool.getMembraneModel().setLastGeneratedMembraneFile(file);
				DataPool.setMembraneHasBeenEditedAgain(true);
			}

			return file;
		}

		return null;

	}

	private void setFileChooserSelection(final JFileChooser fc,
			final int operation) {
		
		String newname;
		String ext;

//		public final int LOADMEMBRANE = 0;
//
//		public final int SAVEMEMBRANE = 1;
//		
//		public final int SAVEIMAGE = 2;
//
//		public final int LOADDATABASE = 3;
//
//		public final int NEWDATABASE =  4;
//		
//		public final int SAVEPROTOCOL = 5;
		
		switch (operation) {

			case 1: newname = "New_Membrane"; 
					ext = ".Cm2";
					break;
			case 2: newname = "New_Image";
					ext = ".jpg";
					break;
			case 4: newname = "New_Database";
					ext = ".xml";
					break;
			case 5: newname = "New_Protocol";
					ext = ".xvg";
					break;
			default: newname = "New_Membrane";
					ext = ".pdb";
					break;
					
		}
		
//		if (operation != this.NEWDATABASE)
//			if (operation != this.SAVEPROTOCOL)
//				newname = "New_Membrane";
//			else
//				newname = "New_Protocol";
//		else
//			newname = "New_Database";
//		String ext;
//		if (operation != -1)
//			if (operation != this.NEWDATABASE)
//				if (operation != this.SAVEPROTOCOL)
//					ext = ".Cm2";
//				else
//					ext = ".xvg";
//			else 
//				ext = ".xml";
//		else
//			ext = ".pdb";

		final File dir = fc.getCurrentDirectory();
		int count = 1;
		String suggestion;
		while (true) {

			if (count == 1)
				suggestion = newname + ext;
			else
				suggestion = newname + "_" + count + ext;
			final File[] files = dir.listFiles();
			boolean b = true;
			for (int i = 0; i < files.length; i++) {
				if (files[i].isDirectory() == false) {
					if (files[i].getName().equalsIgnoreCase(suggestion)) {
						b = false;
						break;
					}
				}
			}
			if (b == true)
				break;
			count++;

		}

		fc.setSelectedFile(new File(suggestion));

	}

	/**
	 * Pops up a Filechooser where the User may select a PDB File.
	 * 
	 * @return the file the user selects, null if cancelled
	 * @param dir
	 *            the file initially selected
	 */
	public File askPDBFileForExternalPreview() {

		final JFileChooser fc = new JFileChooser();

		// make sure user can only apply having ONE NON-DIRECTORY PDB file
		// selected
		fc.setMultiSelectionEnabled(false);
		fc.setFileSelectionMode(JFileChooser.FILES_ONLY);
		fc.setAcceptAllFileFilterUsed(false);

		// customizing a filefilter that accepts only .pdb's
		final FileFilter filter = new FileFilter() {

			@Override
			public boolean accept(final File arg0) {
				if (arg0.getName().toLowerCase().endsWith(".pdb")
						|| arg0.isDirectory())
					return true;
				else
					return false;

			}

			@Override
			public String getDescription() {
				return ".pdb";
			}

		};

		/*
		 * customizing a fileview that returns icons
		 */
		final FileView fv = new FileView() {

			@Override
			public Icon getIcon(final File arg0) {

				if (arg0.isDirectory())
					return null;

				return Cm2_IconDepot.getSpecifiedIcon("FC_Pdb_Ok");
			}
		};

		fc.setFileView(fv);

		fc.setAccessory(new MyFCAccessory(fc, 2, "Pdb Info"));

		// setting the filefilter
		fc.setFileFilter(filter);

		try {
			File filepath;

			/*
			 * trying to get the last used directory for the filechooser (only
			 * relevant if no init. selected file is passed
			 */

			filepath = new File(ConfigParser
					.getValue("last_ExternalPreview_directory")[0]);

			// checks if the retrieved file is a valid directory
			if (!filepath.exists() || !filepath.isDirectory())
				throw new Exception();

			// ...and if so sets the filechoosers path
			fc.setCurrentDirectory(filepath);

		} catch (final Exception e) {

		}

		fc.setDialogTitle("Open a PDB from your File System");

		final int val = fc.showOpenDialog(this.componentforoptionpane);
		if (val == JFileChooser.APPROVE_OPTION) {
			final File file = fc.getSelectedFile();

			// storing the new Path
			ConfigParser.deleteAllAt("last_ExternalPreview_directory");
			ConfigParser.addValue("last_ExternalPreview_directory", new File(
					file.getParent()).getPath());
			return file;

		} else {
			// return null if user cancels
			return null;
		}
	}

	/**
	 * Asks if the given File may be written. If it already exists, the User
	 * will be asked to confirm saving
	 * 
	 * @param file
	 * @return
	 */
	public boolean authorizeSaving(final File file) {

		if (file.exists()) {

			final int val = JOptionPane
					.showConfirmDialog(
							this.componentforoptionpane,
							"The file\n"
									+ file.getName()
									+ "\nalready exists.\nDo you wish to overwrite it?",
							"Confirm", JOptionPane.OK_CANCEL_OPTION);
			if (val == JOptionPane.OK_OPTION)
				return true;

			else
				return false;

		}

		else
			return true;

	}

	/**
	 * Ask if the currently opened Membrane may be discarged. If the current
	 * Membrane has not been edited since last loading or saving, it is allowed
	 * to. Otherwise, user will have to confirm
	 * 
	 * @return
	 */
	public boolean authorizeDiscargingOfCurrentMembrane() {

		if (DataPool.getMembraneHasBeenEditedAgain() == false)
			return true;

		else if (JOptionPane.showConfirmDialog(this.componentforoptionpane,
				"Your current Membrane has been changed.\n"
						+ "Do you wish to proceed anyway?", "Confirm",
				JOptionPane.OK_CANCEL_OPTION) == JOptionPane.OK_OPTION)
			return true;
		else
			return false;

	}

	/**
	 * Report an Exception that occured during loading here and the User will be
	 * informed regarding to the type of exception
	 * 
	 * @param e
	 */
	public void reportMembraneLoadingException(final Exception e) {

		if (e instanceof IOException)
			JOptionPane.showMessageDialog(this.componentforoptionpane,
					"Unable to open the Membrane!\n\nReason:\n" + e.toString(),
					"Error", JOptionPane.ERROR_MESSAGE);
		else if (e instanceof JDOMException)
			JOptionPane
					.showMessageDialog(
							this.componentforoptionpane,
							"Unable to open the Membrane!\n"
									+ "The file format seems to be invalid\n\nReason:\n"
									+ e.toString(), "Error",
							JOptionPane.ERROR_MESSAGE);
		else if (e instanceof InvalidEntriesException) {
			final InvalidEntriesException e1 = (InvalidEntriesException) e;
			JOptionPane.showMessageDialog(this.componentforoptionpane,
					"Some of the Entries  seem to be invalid\n"
							+ "The following Entries have been ignored:\n\n"
							+ e1.getMessageString(), "Error",
					JOptionPane.WARNING_MESSAGE);

		} else
			System.err.println(e.toString());
	}

	/**
	 * Report an Exception occured during saving here to inform the User
	 * 
	 * @param e
	 */
	public void reportMembraneSavingException(final Exception e) {

		if (e instanceof IOException)
			JOptionPane.showMessageDialog(this.componentforoptionpane,
					"Unable to save the Membrane!\n\nReason:\n" + e.toString(),
					"Error", JOptionPane.ERROR_MESSAGE);
		else
			System.err.println(e.toString());
	}

	/**
	 * Report an Exception occured during saving here to inform the User
	 * 
	 * @param e
	 */
	public void reportImageSavingException(final Exception e) {

		if (e instanceof IOException)
			JOptionPane.showMessageDialog(this.componentforoptionpane,
					"Unable to save the Screenshot!\n\nReason:\n" + e.toString(),
					"Error", JOptionPane.ERROR_MESSAGE);
		else
			System.err.println(e.toString());
	}
	
	/**
	 * Report an Exception occured during saving here to inform the User
	 * 
	 * @param e
	 */
	public void reportProtocolSavingException(final Exception e) {

		if (e instanceof IOException)
			JOptionPane.showMessageDialog(this.componentforoptionpane,
					"Unable to save the Time Protocol!\n\nReason:\n" + e.toString(),
					"Error", JOptionPane.ERROR_MESSAGE);
		else
			System.err.println(e.toString());
	}

	/**
	 * Report an Exception that occured during loading the DB here and the User
	 * will be informed regarding to the type of exception
	 * 
	 * @param e
	 */
	public void reportDBLoadingException(final Exception e) {
		if (e instanceof IOException)
			JOptionPane.showMessageDialog(this.componentforoptionpane,
					"Unable to open the Database!\n\nReason:\n" + e.toString(),
					"Error", JOptionPane.ERROR_MESSAGE);
		else if (e instanceof JDOMException)
			JOptionPane
					.showMessageDialog(
							this.componentforoptionpane,
							"Unable to open the Database!\n+"
									+ "The file format seems to be invalid\n\nReason:\n"
									+ e.toString(), "Error",
							JOptionPane.ERROR_MESSAGE);
		else
			System.err.println(e.toString());

	}

	/**
	 * Report an Exception occured during saving the DB here to inform the User
	 * 
	 * @param e
	 */
	public int reportDBSavingException(final IOException e) {

		return JOptionPane.showConfirmDialog(this.componentforoptionpane,
				"Unable to Save the current Database!\n\n\nReason:\n"
						+ e.toString() + "\n\nProceed?", "Error",
				JOptionPane.YES_NO_OPTION, JOptionPane.ERROR_MESSAGE);

	}

	/**
	 * Report an Exception occured during Generating the Membrane into a pdb
	 * file
	 * 
	 * @param e
	 */
	public void reportMembraneGeneratingException(final Exception e) {

		JOptionPane
				.showMessageDialog(this.componentforoptionpane,
						"The Membrane cannot be generated!\n\nReason:\n"
								+ e.toString(), "Error",
						JOptionPane.ERROR_MESSAGE);

	}

	public static File askForWorkspace() {

		final JFileChooser chooser = new JFileChooser();
		chooser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
		chooser.setMultiSelectionEnabled(false);
		chooser.setDialogTitle("Select your workspace location");
		chooser.setAcceptAllFileFilterUsed(false);
		chooser.setFileFilter(new FileFilter() {

			@Override
			public boolean accept(final File arg0) {

				return arg0.isDirectory();
			}

			@Override
			public String getDescription() {

				return "Directories";
			}

		});

		chooser.setCurrentDirectory(new File(System.getProperty("user.home")));
		chooser.setSelectedFile(new File("Cm2_Workspace"));

		// MAC OS X does not work with that
//		final int val = chooser.showDialog(null, "Accept");
		
		final int val = chooser.showSaveDialog(null);

		if (val == JFileChooser.CANCEL_OPTION) {
			return null;
		}

		final File f = chooser.getSelectedFile();

		if (!f.exists() && !f.mkdirs()) {
			JOptionPane.showMessageDialog(null, "Directory cannot be created!",
					"ERROR", JOptionPane.ERROR_MESSAGE);
			return null;
		}

		return f;
	}

	public static String getTempJar() {

		return FileIOHandler.home + "temporaryAlgorithm.jar";
	}

	public File askPDBFileForRestoring() {

		final JFileChooser fc = new JFileChooser();

		// make sure user can only apply having ONE NON-DIRECTORY PDB file
		// selected
		fc.setMultiSelectionEnabled(false);
		fc.setFileSelectionMode(JFileChooser.FILES_ONLY);
		fc.setAcceptAllFileFilterUsed(false);

		// customizing a filefilter that accepts only .pdb's
		final FileFilter filter = new FileFilter() {

			@Override
			public boolean accept(final File arg0) {
				if (arg0.getName().toLowerCase().endsWith(".pdb")
						|| arg0.isDirectory())
					return true;
				else
					return false;

			}

			@Override
			public String getDescription() {
				return ".pdb";
			}

		};

		/*
		 * customizing a fileview that returns icons
		 */
		final FileView fv = new FileView() {

			@Override
			public Icon getIcon(final File arg0) {

				if (arg0.isDirectory())
					return null;

				return Cm2_IconDepot.getSpecifiedIcon("FC_Pdb_Ok");
			}
		};

		fc.setFileView(fv);

		fc.setAccessory(new MyFCAccessory(fc, 2, "Pdb Info"));

		// setting the filefilter
		fc.setFileFilter(filter);

		try {
			File filepath;

			/*
			 * trying to get the last used directory for the filechooser (only
			 * relevant if no init. selected file is passed
			 */

			filepath = new File(ConfigParser
					.getValue("last_finished_membrane_output_directory")[0]);

			// checks if the retrieved file is a valid directory
			if (!filepath.exists() || !filepath.isDirectory())
				throw new Exception();

			// ...and if so sets the filechoosers path
			fc.setCurrentDirectory(filepath);

		} catch (final Exception e) {

		}

		fc.setDialogTitle("Restore a membrane from a pdb file");

		final int val = fc.showOpenDialog(this.componentforoptionpane);
		if (val == JFileChooser.APPROVE_OPTION) {
			final File file = fc.getSelectedFile();

			// storing the new Path

			return file;

		} else {
			// return null if user cancels
			return null;
		}

	}

}
