/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_1)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.io;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import javax.media.j3d.Geometry;
import javax.media.j3d.GeometryArray;
import javax.media.j3d.TriangleArray;
import javax.vecmath.Matrix3f;
import javax.vecmath.Point3f;
import javax.vecmath.Vector2f;
import javax.vecmath.Vector3d;
import javax.vecmath.Vector3f;

import org.cellmicrocosmos.cm2.constants.CM2_CONSTANTS;
import org.cellmicrocosmos.cm2.dataobjects.Atom_3D;
import org.cellmicrocosmos.cm2.dataobjects.Constraint_3D;
import org.cellmicrocosmos.cm2.dataobjects.Lipid_3D;
import org.cellmicrocosmos.cm2.dataobjects.Molecule_3D;
import org.cellmicrocosmos.cm2.dataobjects.MyBounds;
import org.cellmicrocosmos.cm2.dataobjects.Protein_3D;
import org.cellmicrocosmos.cm2.engine.DataPool;
import org.cellmicrocosmos.cm2.misc.Cm2_Toolkit;


import com.sun.j3d.utils.geometry.Box;
import com.sun.j3d.utils.geometry.GeometryInfo;
import com.sun.j3d.utils.geometry.NormalGenerator;

/**
 * The new pdb reader. Used for creation of the Molecule instances. The molecule
 * will be set up with some metavalues first before its geometry will be build.
 * 
 * 
 * @author Tim Dingersen dackelnose@gmx.net
 * 
 */
public class PdbReader_3D {

	public int numberOfIntervals;

	private final static int X_HIGH = 1;

	private final static int Y_HIGH = 3;

	private final static int Z_HIGH = 5;

	private static final Vector3f ref = new Vector3f(0, 0, -1);

	private static final Vector3f ref2 = new Vector3f(0, 0, 1);

	private static int directions;

	private static float eachangle;

//	final private IntersectionUtils utils = new IntersectionUtils();
//
//	final private IntersectManager intersector = new IntersectManager();

	public PdbReader_3D() {
		PdbReader_3D.directions = DataPool.J3DSettings.numberofvertices;
		PdbReader_3D.eachangle = ((float) Math.PI * 2f)
				/ PdbReader_3D.directions;
	}

	/**
	 * Constructs a Protein_3D according to the file
	 * 
	 * 
	 * @param file
	 * @param the
	 *            number of the pdb model that should be taken into account.
	 *            Pass "0" for deafult model or if there isnt any!
	 * @return a new Molecule_3D object created from the file
	 * @throws Exception
	 *             - if something went wrong
	 */
	public Protein_3D readProtein(final File file, int modelnumber,
			final Vector<String> alteredAtoms) throws Exception {

		String[] pdbFile = DataPool.getRawPdbData(file.getAbsolutePath());

		if (pdbFile == null) {
			pdbFile = this.readFile(file);
			DataPool.storeRawPdbData(file.getAbsolutePath(), pdbFile);
		}

		final PdbAttributes attrs = this.readAttributes(pdbFile);
		final Protein_3D prot = new Protein_3D();

		prot.setPDBFile(file);
		prot.setHeader(attrs.header);
		prot.setTitle(attrs.getTitles());
		prot.setDate(attrs.getDate());
		prot.setSOURCE(attrs.getSpecies());
		prot.setAuthor(attrs.getAuthor());
		prot.setPDBcode(attrs.getCode());

		if (modelnumber == 0 && this.checkForModels(file))
			modelnumber = 1;

		prot.setModelSet(modelnumber);

		this.readMolecule(prot, modelnumber, alteredAtoms);

		return prot;
	}

	/**
	 * Constructs a Lipid_3D according to the file
	 * 
	 * 
	 * @param file
	 * @param the
	 *            number of the pdb model that should be taken into account.
	 *            Pass "0" for deafult model or if there isnt any!
	 * @return a new Molecule_3D object created from the file
	 * @throws Exception
	 *             - if something went wrong
	 */
	public Lipid_3D readLipid(final File file, final int modelnumber,
			final Vector<String> alteredAtoms) throws Exception {

		String[] pdbFile = DataPool.getRawPdbData(file.getAbsolutePath());

		if (pdbFile == null) {
			pdbFile = this.readFile(file);
			DataPool.storeRawPdbData(file.getAbsolutePath(), pdbFile);
		}
		final PdbAttributes attrs = this.readAttributes(pdbFile);

		final Lipid_3D lip = new Lipid_3D();
		lip.setPDBFile(file);
		lip.setHeader(attrs.header);
		lip.setTitle(attrs.getTitles());
		lip.setDate(attrs.getDate());
		lip.setSOURCE(attrs.getSpecies());
		lip.setAuthor(attrs.getAuthor());
		lip.setPDBcode(attrs.getCode());

		lip.setModelSet(modelnumber);

		this.readMolecule(lip, modelnumber, alteredAtoms);

		return lip;
	}

	/**
	 * Just reads the passed pdb File
	 * 
	 * @param file
	 * @return an Array of Strings; each String contains an unaltered line of
	 *         the pdb File
	 * @throws IOException
	 */
	public String[] readFile(File file) throws IOException {

		if (!file.exists()) {
			final String newone = DataPool.workspace.getAbsolutePath()
					+ File.separator + file.getPath();
			if (new File(newone).exists())
				file = new File(newone);
		}

		if (file.canRead() == false)
			throw new IOException(file.getPath() + " cannot be resolved");
		final Vector<String> v = new Vector<String>();
		final BufferedReader br = new BufferedReader(new FileReader(file));
		while (true) {

			final String s = br.readLine();
			if (s == null)
				break;

			v.addElement(s);
		}

		br.close();
		return v.toArray(new String[0]);
	}

	/**
	 * Reads in some attributes of the passed Pdb File. These are returned as a
	 * PdbAttributes Object, which stores the retrieved information
	 * 
	 * 
	 * @return An PdbAttributes Object
	 * @throws Exception
	 * @throws IOException
	 * @throws IOException
	 */
	public PdbAttributes readAttributes(final File file) throws IOException {

		return this.readAttributes(this.readFile(file));

	}

	/**
	 * Reads in some attributes of the passed pdb file contents (must be the
	 * lines from an Pdb File in Strings). These are returned as a PdbAttributes
	 * Object, which stores the retrieved information
	 * 
	 * @param Lines
	 *            as Strings
	 * @return An PdbAttributes Object
	 * @throws IOException
	 */

	public PdbAttributes readAttributes(final String[] pdbFile)
			throws IOException {

		final PdbAttributes attrs = new PdbAttributes();

		/*
		 * 4 steps of reading. if all are completed, reading will finish
		 */
		boolean HEADERread = false;
		boolean TITLEread = false;
		boolean SOURCEread = false;
		boolean AUTHORread = false;

		// reading lines until all finished or no more lines found
		for (int i = 0; i < pdbFile.length; i++) {

			final String line = pdbFile[i];

			if (line == null)
				break;

			if (line.startsWith("HEADER")) {

				try {
					attrs.header = line.substring(9, 50).trim();
				} catch (final Exception e) {

				}
				// extracting the date
				try {
					attrs.date = line.substring(50, 60).trim();
				} catch (final Exception e) {

				}
				// extracting the pdb code
				try {
					attrs.code = line.substring(62, 66).trim();
				} catch (final Exception e) {

				}

				HEADERread = true;

			}
			// extracting the TITLE
			else if (line.startsWith("TITLE")) {

				final String string = line.replaceAll("TITLE", "");

				attrs.title = attrs.title + string.trim() + '\n';

				TITLEread = true;

				// extracting the AUTHOR
			} else if (line.startsWith("AUTHOR")) {

				final String string = line.replaceAll("AUTHOR", "");

				attrs.author = attrs.author + string.trim() + '\n';

				AUTHORread = true;
			}
			// extracting the SOURCE
			else if (line.startsWith("SOURCE")) {

				if (line.contains("ORGANISM_SCIENTIFIC: ") == false)
					continue;

				final String[] strings = line.split("ORGANISM_SCIENTIFIC: ");
				if (strings.length < 2)
					continue;
				if (strings[1] != null)
					attrs.source = strings[1].trim();
				SOURCEread = true;
			} else if (HEADERread && TITLEread && SOURCEread && AUTHORread) {

				break;
			}

		}

		return attrs;
	}

	/**
	 * A kind of sheet that stores some strings. Used as a return value when pdb
	 * attributes are read.
	 * 
	 * @author Tim Dingersen dackelnose@gmx.net
	 * 
	 */
	public class PdbAttributes {
		private String header = "";

		private String code = "";

		private String title = "";

		private String source = "";

		private String date = "";

		private String author = "";

		/**
		 * Returns the Code that was found or null
		 * 
		 * @return
		 */
		public String getCode() {

			if (this.code.length() < 4)
				return null;

			return this.code;

		}

		/**
		 * Return the date that was found or null
		 * 
		 * @return
		 */
		public String getDate() {
			if (this.date.length() < 1)
				return null;

			return this.date;
		}

		/**
		 * Returns the Header that was found or null
		 * 
		 * @return
		 */
		public String getHeader() {

			if (this.header.length() < 1)
				return null;

			return this.header;
		}

		/**
		 * Returns the Title that was found or null Multiple titles in the pdb
		 * will be concatenated to the returnvalue
		 * 
		 * @return
		 */
		public String getTitles() {

			if (this.title.length() < 1)
				return null;

			return this.title;

		}

		/**
		 * Returns the Species that was found under SOURCE or null
		 * 
		 * @return
		 */
		public String getSpecies() {

			if (this.source.length() < 1)
				return null;

			return this.source;

		}

		/**
		 * returns the AUTHOR
		 * 
		 * @return
		 */
		public String getAuthor() {
			if (this.author.length() < 1)
				return null;

			return this.author;
		}

	}

	/**
	 * Builds up the shape by reading all ATOM or HETATM entries and using their
	 * coordinates.
	 * 
	 * At first, the raw bounds are calculated based on extremevalues.
	 * Afterwards, the points will (probably) be rotated so that the longest
	 * axis will stand in vertical direction (in case of lipids, the side with
	 * the most Oxygen, Nitrogen and Phosphor Atoms will be turned up in a 90
	 * degree precision). This effects that rendering will happen along the
	 * longest axis, what improves precision in this case. Then, moving from top
	 * to bottom, the outer extremepoints will be determined This happens in
	 * intervals. The size of an interval depents on their count, which again
	 * depends on the length of the molecule and the precision modifier set in
	 * the CONSTANTS class. But a minimum number of intervals defined in the
	 * constants is always guaranteed, in case the objects are really small.
	 * When finished, the points in each interval will be connected with points
	 * in the next interval, so that a closed 3d object consisting of
	 * clockwise-constructed triangles can be generated. Afterwards, the
	 * triangle vertices are rotated back to default location. Some other
	 * values, such as the current vertical length, width on x/z axis etc. will
	 * also be set into the object. A standard rotation matrix will be set for
	 * Proteins, the altered one for lipids that will rotate them upright later,
	 * as desired. In the end, the shape3d is fed with the created array of
	 * triangles, the normals used for lightning are generated and the
	 * appearance gets set.
	 * 
	 * 
	 * @param mol
	 *            the molecule object to be formed
	 * @param alteredAtoms
	 * @param mode
	 *            0 for Protein reading mode, 1 for lipid reading mode
	 * @throws Exception
	 */
	private void readMolecule(final Molecule_3D mol, final int modelnumber,
			Vector<String> alteredAtoms) throws Exception {
		// System.out.println(mol.getPDBFile().getName());

		mol.setModelcount(this.countModels(DataPool.getRawPdbData(mol
				.getPDBFile().getAbsolutePath())));
		// reading in the relevant atoms
		final String[][] input = this.readModel(DataPool.getRawPdbData(mol
				.getPDBFile().getAbsolutePath()), modelnumber);

		Vector<Atom_3D> opmAtoms = new Vector<Atom_3D>();
		for (int i = 0; i < input[1].length; i++) {

			try {

				final Atom_3D atom = new Atom_3D(input[1][i]);

				opmAtoms.addElement(atom);

			} catch (final Exception e) {

				continue;
			}

		}
		mol.opmAtoms = opmAtoms.toArray(new Atom_3D[0]);

		String[] alllines = input[0];

		final Vector<Atom_3D> allAtoms = new Vector<Atom_3D>();

		// stores all Atom points

		final Hashtable<String, String> diffChainIDs = new Hashtable<String, String>();

		// reading the pdbFile loop

		// int chaincount=0;

		final Point3f[] alteredPoints = new Point3f[alllines.length];

		if (alteredAtoms != null) {

			for (int i = 0; i < alteredAtoms.size(); i++) {
				try {
					final String[] s = alteredAtoms.get(i).split(" ");
					final int index = Integer.parseInt(s[0].trim());
					final Point3f p = new Point3f(
							Float.parseFloat(s[1].trim()),
							Float.parseFloat(s[2].trim()),
							Float.parseFloat(s[3].trim()));
					alteredPoints[index] = p;
				} catch (final Exception e) {

				}
			}
		}

		int terCount = 0;
		for (int i = 0; i < alllines.length; i++) {

			// if line is relevant, thread passes to here and the coordinates
			// get extracted

			try {

				final Atom_3D atom = new Atom_3D(alllines[i]);

				// Point3f p = new Point3f(atom.position);

				// adding the atom point

				allAtoms.addElement(atom);

			} catch (final Exception e) {

				if (alllines[i].trim().startsWith("TER")) {
					terCount++;
					allAtoms.addElement(null);
					continue;
				}
			}

			try {
				final String chainId = alllines[i].substring(21, 22).trim();

				if (Cm2_Toolkit.getCharacterPosOf(chainId.charAt(chainId
						.length() - 1)) != -1)
					diffChainIDs.put(chainId, "");

			} catch (final Exception e) {

			}

		}
	
		float maxAtomRadius = 0f;

		for (int i = 0; i < allAtoms.size(); i++) {
			final Atom_3D a = allAtoms.get(i);
			if(a==null)continue;
			final String element = a.element.trim().toUpperCase();
			a.color = DataPool.AtomValues.getColor(element);
			a.radiusCov = DataPool.AtomValues.getCovalentRadius(element);
			a.radiusVDW = DataPool.AtomValues.getVanDerWaalsRadius(element);
			a.massValue=DataPool.AtomValues.getMassValue(element);
			maxAtomRadius = Math.max(maxAtomRadius, a.radiusCov);
		}
		mol.maxAtomRadius = maxAtomRadius;
		
		// before trimming the atom list, making a untouched copy for later
		// generateDefaultConstraints() method.
		// this is necessary to maintain the right bonds/connects
		Vector<Atom_3D> copyOfAllAtoms = new Vector<Atom_3D>();
		for(int index = 0; index < allAtoms.size(); ++index) {
			copyOfAllAtoms.add(allAtoms.get(index));
		}
			
		for(int i = allAtoms.size() - 1; i > -1; i--) {
			if(allAtoms.get(i) == null) {
				allAtoms.remove(i);
			}
		}
		allAtoms.trimToSize();

		// now all atoms are stored in an array without terminations (TERs)
		// ================================================================
		// TODO: PPE Update 03_2012										   |
		// ================================================================
		// generating a transformed copy of the current atom vector for 
		// calculating the original PDBTM Y-translation
		try {
			final String pdbcode = mol.getPDBcode().toLowerCase();
			if(DataPool.getMifHandler().isElementOfPDBTM(pdbcode)) {
				// molecule is stored in the local MIFile (PDBTM)
				Vector<Atom_3D> pdbtmAtoms = new Vector<Atom_3D>();
				final Vector3f v = DataPool.getMifHandler().getTransformVectorOf(pdbcode);
				final Matrix3f m = DataPool.getMifHandler().getTransformMatrixOf(pdbcode);
				try {
					for(int i = 0; i < allAtoms.size(); ++i) {
						Atom_3D a = allAtoms.get(i).copy();
						a.position = new Point3f(
								(a.position.x * m.m00) + (a.position.y * m.m01) + (a.position.z * m.m02) + v.x,
								(a.position.x * m.m10) + (a.position.y * m.m11) + (a.position.z * m.m12) + v.y,
								(a.position.x * m.m20) + (a.position.y * m.m21) + (a.position.z * m.m22) + v.z);
						pdbtmAtoms.add(a);
					}
				} catch(Exception e) {
					// PPE works incorrect
					e.printStackTrace();
				}
				// creating the bounds of the transformed molecule
				MyBounds mb = new MyBounds(pdbtmAtoms);
				// setting up the PDBTM y-translation
				mol.setPDBTMTranslation(mb.center.y);
				// setting up the PDBTM rotation
				mol.setPDBTMRotation(m);
			}
		} catch(Exception e) {
			// probably the molecule has no PDB code
			// this happens in case of a OPM file
		}

		
		mol.atoms = allAtoms.toArray(new Atom_3D[0]);
		mol.allAtomsSorted = allAtoms.toArray(new Atom_3D[0]);
		
		for(int i=0;i<mol.atoms.length;i++) {
			mol.atoms[i].comparator=i;
		}

		// cancels all if too few atoms are found
		if (allAtoms.size() < 1)
			throw new Exception(
					"Pdb File is not valid - No Atoms found!");

		MyBounds mybounds = new MyBounds(mol.atoms);
		
		mol.originalConstraints = this
			.generateDefaultConstraints(mol, allAtoms, copyOfAllAtoms, mybounds);
	
		// ================================================================
		// PPE Update 03_2012											   |
		// ================================================================
		for (int i = 0; i < mol.atoms.length; i++) {

			final Atom_3D a = mol.atoms[i];
			a.position.sub(mybounds.center);
		}

		for (int i = 0; i < mol.opmAtoms.length; i++) {
			final String ele = mol.opmAtoms[i].element.trim().toUpperCase();
			mol.opmAtoms[i].color = DataPool.AtomValues.getColor(ele);
			mol.opmAtoms[i].position.sub(mybounds.center);
		}
		
		for (int i = 0; i < mol.allAtomsSorted.length; i++) {
			if (alteredPoints[i] != null)
				mol.allAtomsSorted[i].position = alteredPoints[i];
		}
		mol.updateAllBonds();

		mol.updateGeometry();

		mol.setChainIDs(diffChainIDs.keySet());
		mol.terCount = terCount;

	}

	private int countModels(final String[] rawPdbData) {
		int counter = 0;

		for (int i = 0; i < rawPdbData.length; i++) {

			final String line = rawPdbData[i];

			if ((line.startsWith("MODEL")))
				counter++;

		}
		return counter;
	}

	public void shapeIt(final Molecule_3D mol) {

		MyBounds mybounds = new MyBounds(mol.atoms);

		final int l = mol.atoms.length;

		mybounds.normalize();
		mybounds.scale(CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE);

		final Point3f[] allPoints = new Point3f[l];
		for (int i = 0; i < l; i++) {
			allPoints[i] = new Point3f(mol.atoms[i].position);

			allPoints[i].scale(CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE);
		}

		Vector<Point3f>finals;
		if(mol.atoms.length<4) {
			finals=this.createSimpleVertices(mol,mybounds,allPoints);
		}else {
			finals=this.createVertices(mol,mybounds,allPoints);
		}
		
		/*
		 * ### Now that all data is ready, the shape 3d can finally be set up!
		 * 
		 * The TriangleArray Object used by the shape3d gets created and set up
		 * getting our triangle points as data. The normals are also computed
		 * according to these points
		 */

		// converting the geometry list into an array
		final Point3f[] points = finals.toArray(new Point3f[0]);

		mol.setTrianglePoints(points);

		final TriangleArray geo = new TriangleArray(points.length,
				GeometryArray.NORMALS | GeometryArray.COORDINATES
						| GeometryArray.COLOR_3);

		final GeometryInfo gi = new GeometryInfo(GeometryInfo.TRIANGLE_ARRAY);

		gi.setCoordinates(points);

		new NormalGenerator().generateNormals(gi);

		geo.setCoordinates(0, points);
		geo.setNormals(0, gi.getNormals());
		
		

		geo.setCapability(Geometry.ALLOW_INTERSECT);

		mol.getShape().setGeometry(geo);

		final Point3f lower = mol.getLower(false);
		final Point3f upper = mol.getUpper(false);

		final Box unrotatedBounds = new Box(Math.abs(upper.x - lower.x), Math
				.abs(upper.y - lower.y), Math.abs(upper.z - lower.z), null);

		mol.setUnrotatedBounds(unrotatedBounds);

	}

	private Vector<Point3f> createSimpleVertices(Molecule_3D mol,
			MyBounds mybounds, Point3f[] allPoints) {

		Vector<Point3f>ret = new Vector<Point3f>();
		
		if(allPoints.length==3) {
			
			final GeometryInfo gi = new GeometryInfo(GeometryInfo.TRIANGLE_ARRAY);

			gi.setCoordinates(allPoints);

			new NormalGenerator().generateNormals(gi);
			
			
			
			Point3f[]points= {new Point3f(allPoints[0]),new Point3f(allPoints[1]),new Point3f(allPoints[2])};
			float[]sizes= {mol.atoms[0].radiusCov*CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE,mol.atoms[1].radiusCov*CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE,mol.atoms[2].radiusCov*CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE};
			
			for(int i=0;i<points.length;i++) {
				final Vector3f v = new Vector3f(points[i]);

				v.sub(mybounds.center);
				v.normalize();
				v.scale(CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE);
				v.scale(1.08f+sizes[i]);
				

				points[i].add(v);
			}
			
			
			Point3f A1=new Point3f(points[0]);
			Point3f B1=new Point3f(points[1]);
			Point3f C1=new Point3f(points[2]);
			
			Point3f A2=new Point3f(points[0]);
			Point3f B2=new Point3f(points[1]);
			Point3f C2=new Point3f(points[2]);
			
			Vector3f dir = new Vector3f(gi.getNormals()[0]);
		
			dir.scale(0.004f+sizes[0]);
			
			
			A1.add(dir);
			A2.sub(dir);
			
			dir = new Vector3f(gi.getNormals()[0]);
			
			dir.scale(0.004f+sizes[1]);
			
			B1.add(dir);
			B2.sub(dir);
			
			dir = new Vector3f(gi.getNormals()[0]);
			
			dir.scale(0.004f+sizes[2]);
			
			C1.add(dir);
			C2.sub(dir);
			
			
			
			
			ret.addElement(A1);
			ret.addElement(B1);
			ret.addElement(C1);
			
			ret.addElement(A2);
			ret.addElement(B2);
			ret.addElement(C2);
			
			
			ret.addElement(new Point3f(A1));
			ret.addElement(new Point3f(A2));
			ret.addElement(new Point3f(B2));
			
			ret.addElement(new Point3f(A1));
			ret.addElement(new Point3f(B2));
			ret.addElement(new Point3f(B1));
			
			ret.addElement(new Point3f(B1));
			ret.addElement(new Point3f(B2));
			ret.addElement(new Point3f(C2));
			
			ret.addElement(new Point3f(B1));
			ret.addElement(new Point3f(C2));
			ret.addElement(new Point3f(C1));
			
			ret.addElement(new Point3f(C1));
			ret.addElement(new Point3f(C2));
			ret.addElement(new Point3f(A2));
			
			ret.addElement(new Point3f(C1));
			ret.addElement(new Point3f(A2));
			ret.addElement(new Point3f(A1));
			
		}else if(allPoints.length==2) {
			
			Point3f P1=new Point3f(allPoints[0]);
			Point3f P2=new Point3f(allPoints[1]);
			
			float S1=mol.atoms[0].radiusCov*CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE*2;
			float S2=mol.atoms[1].radiusCov*CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE*2;
		
		
			Vector2f v2=new Vector2f(P1.z,P1.y);
			Matrix3f m=Cm2_Toolkit.createAUnitMatrix();
			
			
			
			
			float rZ=(float)Math.toDegrees(v2.angle(new Vector2f(0,1)));
//			float rZ=0f;
			
			
			if(P1.z<0)
				rZ*=-1f;
//			
			
			
			Cm2_Toolkit.rotate_X_Axis_Matrix(m,-rZ);
			
			
			
			P1=Cm2_Toolkit.transformToNewPoint(m,P1);
			P2=Cm2_Toolkit.transformToNewPoint(m,P2);
			
			Vector2f v1=new Vector2f(P1.x,P1.y);
			
			
			float rY=(float)Math.toDegrees(v1.angle(new Vector2f(0,1)));
//			float rY=0f;
			
			
			
			if(P1.x>0)
				rY*=-1f;
			
			m=Cm2_Toolkit.createAUnitMatrix();
			Cm2_Toolkit.rotate_Z_Axis_Of_CoordinateSystem(m,-rY);

			P1=Cm2_Toolkit.transformToNewPoint(m,P1);
			P2=Cm2_Toolkit.transformToNewPoint(m,P2);
			
			
			
			
			if(P1.y>=0)P1.y+=S1;
			else P1.y-=S1;
			
			if(P2.y>=0)P2.y+=S1;
			else P2.y-=S1;
			
			Point3f p1=new Point3f(0,0,S1);
			Point3f p2=new Point3f(0,0,S2);
			
			Point3f A1=new Point3f(p1.x,P1.y,p1.z);
			Point3f A2=new Point3f(p2.x,P2.y,p2.z);
			
			Matrix3f mm=Cm2_Toolkit.createAUnitMatrix();
			Cm2_Toolkit.rotate_Y_Axis_Matrix(mm,120);
			p1=Cm2_Toolkit.transformToNewPoint(mm,p1);
			p2=Cm2_Toolkit.transformToNewPoint(mm,p2);

			Point3f B1=new Point3f(p1.x,P1.y,p1.z);
			Point3f B2=new Point3f(p2.x,P2.y,p2.z);
			mm=Cm2_Toolkit.createAUnitMatrix();
			p1=new Point3f(0,0,S1);
			p2=new Point3f(0,0,S2);
			Cm2_Toolkit.rotate_Y_Axis_Of_CoordinateSystem(mm,240);
			p1=Cm2_Toolkit.transformToNewPoint(mm,p1);
			p2=Cm2_Toolkit.transformToNewPoint(mm,p2);
			
			Point3f C1=new Point3f(p1.x,P1.y,p1.z);
			Point3f C2=new Point3f(p2.x,P2.y,p2.z);
			
			
			
			
			
			
			ret.addElement(A1);
			ret.addElement(B1);
			ret.addElement(C1);
			
			ret.addElement(A2);
			ret.addElement(B2);
			ret.addElement(C2);
			
			
			ret.addElement(new Point3f(A1));
			ret.addElement(new Point3f(A2));
			ret.addElement(new Point3f(B2));
			
			ret.addElement(new Point3f(A1));
			ret.addElement(new Point3f(B2));
			ret.addElement(new Point3f(B1));
			
			ret.addElement(new Point3f(B1));
			ret.addElement(new Point3f(B2));
			ret.addElement(new Point3f(C2));
			
			ret.addElement(new Point3f(B1));
			ret.addElement(new Point3f(C2));
			ret.addElement(new Point3f(C1));
			
			ret.addElement(new Point3f(C1));
			ret.addElement(new Point3f(C2));
			ret.addElement(new Point3f(A2));
			
			ret.addElement(new Point3f(C1));
			ret.addElement(new Point3f(A2));
			ret.addElement(new Point3f(A1));
			
			
			
			
			m=Cm2_Toolkit.createAUnitMatrix();
			Cm2_Toolkit.rotate_Z_Axis_Of_CoordinateSystem(m,rY);
			for(int i=0;i<ret.size();i++) {
				ret.set(i,Cm2_Toolkit.transformToNewPoint(m,ret.get(i)));
			}
			m=Cm2_Toolkit.createAUnitMatrix();
			Cm2_Toolkit.rotate_X_Axis_Matrix(m,rZ);
			for(int i=0;i<ret.size();i++) {
				ret.set(i,Cm2_Toolkit.transformToNewPoint(m,ret.get(i)));
			}
			
			
			
			
			
			
		}else {
			
			
			
		
		
		float S1=mol.atoms[0].radiusCov*CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE*2;
		
		Point3f p=new Point3f(0,0,S1);
		
			Point3f A1=new Point3f(p.x,0,p.z);
			
			
			Matrix3f mm=Cm2_Toolkit.createAUnitMatrix();
			Cm2_Toolkit.rotate_Y_Axis_Matrix(mm,120);
			p=Cm2_Toolkit.transformToNewPoint(mm,p);

			Point3f B1=new Point3f(p.x,0,p.z);
			
			mm=Cm2_Toolkit.createAUnitMatrix();
			p=new Point3f(0,0,S1);
			Cm2_Toolkit.rotate_Y_Axis_Matrix(mm,240);
			p=Cm2_Toolkit.transformToNewPoint(mm,p);
			
			Point3f C1=new Point3f(p.x,0,p.z);
			
			
			
			float a=S1;
			float c=A1.distance(B1);
			
			float b
			=(float)Math.sqrt((c*c)-(a*a))
				;
			
			
//			b=0.023f;
			
			Point3f B=new Point3f(0,b,0);
			
			b=b/2f;
			b-=a/4f;
			
			ret.addElement(A1);
			ret.addElement(B1);
			ret.addElement(C1);
			
			ret.addElement(new Point3f(B));
			ret.addElement(new Point3f(A1));
			ret.addElement(new Point3f(B1));
			
			ret.addElement(new Point3f(B));
			ret.addElement(new Point3f(B1));
			ret.addElement(new Point3f(C1));
			
			ret.addElement(new Point3f(B));
			ret.addElement(new Point3f(C1));
			ret.addElement(new Point3f(A1));
			
			for(int i=0;i<ret.size();i++) {
				ret.get(i).sub(new Point3f(0,b,0));
				
			}
		}
		
	
	
		
		
		return ret;
	}

	private Vector<Point3f> createVertices(Molecule_3D mol,MyBounds mybounds, Point3f[] allPoints) {



		// the direction to rotate upwards from
		int rotateUpwardsFromPos = -1;

		// in the next 2 lines, the longest axis gets determined
		float f = Math.max(mybounds.widthX, mybounds.widthZ);
		f = Math.max(f, mybounds.widthY);

		// det. the direction to rotate upwards from in case of Protein
		// rendering
		if (mybounds.widthX == f)
			rotateUpwardsFromPos = PdbReader_3D.X_HIGH;
		if (mybounds.widthY == f)
			rotateUpwardsFromPos = PdbReader_3D.Y_HIGH;
		if (mybounds.widthZ == f)
			rotateUpwardsFromPos = PdbReader_3D.Z_HIGH;

		float allLevelsHeight;
		float uppestPoint;
		Matrix3f matrix = Cm2_Toolkit.createAUnitMatrix();
		// rotating the matrix dependant on the side to bring to top
		if (rotateUpwardsFromPos == PdbReader_3D.X_HIGH) {
			Cm2_Toolkit.rotate_Z_Axis_Matrix(matrix, -90);

			// extremevalues get flipped when a 90 degree rotation around
			// Z Axis was performed
			uppestPoint = mybounds.upper.x;
			// miny = minx;
			allLevelsHeight = mybounds.widthX;
		}

		else if (rotateUpwardsFromPos == PdbReader_3D.Y_HIGH) {

			uppestPoint = mybounds.upper.y;
			// miny = minx;
			allLevelsHeight = mybounds.widthY;

		}

		else {

			Cm2_Toolkit.rotate_X_Axis_Matrix(matrix, 90);
			uppestPoint = mybounds.upper.z;
			// miny = minz;
			allLevelsHeight = mybounds.widthZ;
		}
		int l=allPoints.length;
		// Rotating all points around the matrix
		for (int i = 0; i < l; i++) {

			allPoints[i] = Cm2_Toolkit
					.transformToNewPoint(matrix, allPoints[i]);

		}

		// ##########The creation of the geometry data finally begins...#######

		// list that will store the extremepoints for each level (interval)
		final Vector<Point3f[]> allLevels = new Vector<Point3f[]>();

		/*
		 * calculating the total number of intervals to use. The molecule height
		 * gets multiplied with a modifier and rounded afterwards so that larger
		 * molecules will have more vertices than small ones. If too small, at
		 * least the minimum number will be used
		 */
		this.numberOfIntervals = (int) (allLevelsHeight * DataPool.J3DSettings.precision_modifier);

		if (this.numberOfIntervals < DataPool.J3DSettings.minprecision)
			this.numberOfIntervals = DataPool.J3DSettings.minprecision;


		// calculating size of each interval
		// plus the initial start and end of the first interval
		final float step = (allLevelsHeight / (float) this.numberOfIntervals);

		float lbegin = uppestPoint;
		float lend = uppestPoint - step;

		// to close the hull without making it look too flat,
		// a top and a bottom point is needed
		Point3f pTop = new Point3f();
		Point3f pBottom = new Point3f();
		// working through each interval
		for (int level = 0; level < this.numberOfIntervals; level++) {

			final Vector<Point3f> levelpoints = new Vector<Point3f>();
			// passing through all points, ignoring those which aren't located
			// inside of
			// the current interval
			for (int i = 0; i < l; i++) {

				final Point3f current = allPoints[i];

				// System.out.println("X "+current.x);
				// System.out.println("Y "+current.y);
				// System.out.println("Z "+current.z);
				// float increaser = 0.005f;
				// if (current.x>=0) current.x=current.x+increaser; else
				// current.x=current.x-increaser;
				// if (current.y>=0) current.y=current.y+increaser; else
				// current.y=current.y-increaser;
				// if (current.z>=0) current.z=current.z+increaser; else
				// current.z=current.z-increaser;

				// if inside the first level, the point with the highest y value
				// gets set
				// as the top of the molecule
				if (level == 0) {
					if (current.y >= pTop.y)
						pTop = current;

				}

				// if inside the last level, the point with the lowest y value
				// gets set
				// as the bottom of the molecule
				 if (level == this.numberOfIntervals - 1) {
					if (current.y <= pBottom.y)
						pBottom = current;
				}

				// if point isnt iside the current interval, skipping to next
				// one already
				 
				
				 if (current.y > lbegin || current.y < lend)
					continue;
				
				levelpoints.addElement(current);
			}

			// adding the extremes of the current level to the list, but only if
			// at least one point was found for this level.
			// of not, this level will be skipped and points from the level
			// before
			// will be connected directly with points from the next level
			if (levelpoints.isEmpty() == false) {
				
				Point3f[] ps=this.calculateLevelExtremes(levelpoints
						.toArray(new Point3f[0]));
			

				allLevels.addElement(ps);
			}
				

			// moving the interval one level down for the next run
			lbegin -= step;
			lend -= step;

		}
		
		Hashtable<Point3f,String>hash = new Hashtable<Point3f,String>();
		hash.put(pTop,"");
		hash.put(pBottom,"");
		for(int i=0;i<allLevels.size();i++) {
			Point3f[]pp=allLevels.get(i);
			for(int j=0;j<pp.length;j++) {
				if(pp[j]!=null)
					hash.put(pp[j],"");
			}
		}
		
		Enumeration<Point3f> en = hash.keys();
		
		
		
		
		
		
		
		// pTop.y = pTop.y+mcShifter;
		// pBottom.y = pBottom.y-mcShifter;

		// ### Creation of extremepoints, wrapped into levels is complete! #####

		/*
		 * #### Now it is necessary to create the triangles that build the hull
		 * of the shape3D. For each triangle, 3 points have to be determined and
		 * put into clockwise order (regarded from the side the triangle should
		 * be visible) The point of each triangle has to be put into an array,
		 * where each triplet represents one triangle.
		 */

		// the final list of triangle coordinates, it will be used as the
		// geometryArray
		// of the shape3d
		final Vector<Point3f> finals = new Vector<Point3f>();

		
		
		
		
		final int levelcount = allLevels.size();
		
		// working through array of extremepoints
		for (int i = 0; i < levelcount; i++) {
			try {
				final Point3f[] currents = allLevels.get(i);
				
				for (int w = 0; w < currents.length; w++) {
					
					if (currents[w] == null)
						throw new Exception(
								"Pdb_Reader: No levelpoints in level " + i
										+ " !");
				}
				
				// when in last level (making triangles with bottom point)
				if (i == levelcount - 1) {

					// working through each direction
					// adding the bottom, the point from next direction and the
					// point from
					// current direction in this order
					for (int d = 0; d < PdbReader_3D.directions; d++) {

						// determining the next direction. if the current one is
						// the
						// last,
						// the next one will be the first one (in order to
						// connect
						// to a circle)
						int next = d;
						if (next < PdbReader_3D.directions - 1)
							next += 1;
						else
							next = 0;
						try {
							if ((pBottom.equals(currents[next])
									|| pBottom.equals(currents[d]) || currents[next]
									.equals(currents[d])) == false) {

								finals.add(currents[next]);
								finals.add(currents[d]);
								finals.add(pBottom);
							}
						} catch (Exception e) {

						}

					}

					// when thread reaches here, all work is done
					break;

					// when in first level(making triangles with top point)
				} else if (i == 0) {
					// working through each direction
					// adding the top, the point from next direction and the
					// point
					// from
					// current direction in this order
					for (int d = 0; d < PdbReader_3D.directions; d++) {
						
						int next = d;
						if (next < PdbReader_3D.directions - 1)
							next += 1;
						else
							next = 0;
						
						
						try {
							if ((pTop.equals(currents[d])
									|| pTop.equals(currents[next]) || currents[d]
									.equals(currents[next])) == false) {
								finals.add(pTop);
								finals.add(currents[d]);
								finals.add(currents[next]);
							}
						} catch (Exception e) {
							
						}
					}

				}
				if (i < levelcount - 1) {

					// getting the points from the next level too in order to
					// connect to
					// them
					final Point3f[] currentsplus = allLevels.get(i + 1);
					for (int w = 0; w < currentsplus.length; w++) {
						if (currentsplus[w] == null)
							throw new Exception(
									"Pdb_Reader: No levelpoints in level " + i
											+ " !");
					}
					for (int d = 0; d < PdbReader_3D.directions; d++) {
						int next = d;
						if (next < PdbReader_3D.directions - 1)
							next += 1;
						else
							next = 0;

						/*
						 * creating 2 clockwise triangles between 4 points,
						 * where 2 are on the current, and 2 are on the next
						 * level. The points are located like this:
						 * 
						 * currents[next] currents[d]
						 * 
						 * currentsplus[next] currentsplus[d]
						 */

						try {
							if ((currents[next].equals(currents[d])
									|| currents[next].equals(currentsplus[d]) || currents[d]
									.equals(currentsplus[d])) == false) {
								finals.add(currentsplus[d]);
								finals.add(currents[next]);
								finals.add(currents[d]);

							}
							if ((currents[next].equals(currentsplus[d])
									|| currents[next]
											.equals(currentsplus[next]) || currentsplus[d]
									.equals(currentsplus[next])) == false) {
								finals.add(currents[next]);
								finals.add(currentsplus[next]);
								finals.add(currentsplus[d]);
							}
						} catch (Exception e) {

						}
					}
				}

			} catch (final Exception e) {
				// System.out.println("Uncritical PDBReader Exception "+e.getMessage());
//				 e.printStackTrace();
			}

		}
		
		
		
		
		
		
		
		
//		System.out.println(finals.size());
		
		
		
		if(finals.size()<3) {
			finals.clear();
			
			Vector<Point3f>v=new Vector<Point3f>();
			while(en.hasMoreElements()) {
				v.addElement((Point3f)en.nextElement());
//				System.out.println("lala");
			}
			Point3f[]pnts=v.toArray(new Point3f[0]);
			
		
			
			for(int i=0;i<pnts.length;i++) {
				
				
				for(int j=0;j<pnts.length;j++) {
					
					finals.addElement(pnts[i]);
					finals.addElement(pnts[j]);
					if((j+1)<pnts.length)finals.addElement(pnts[j+1]);
					else finals.addElement(pnts[0]);
				}
				
			}
			
			
		}
		
		
		// ## Construction of triangles complete! ####

		/*
		 * A new matrix will be rotated back, to the side it was rotated upwards
		 * from
		 */
		// if (mode == 0) {
		matrix = Cm2_Toolkit.createAUnitMatrix();

		if (rotateUpwardsFromPos == PdbReader_3D.X_HIGH) {
			// matrix.rotZ((float) Math.toRadians(90));
			Cm2_Toolkit.rotate_Z_Axis_Matrix(matrix, 90);
		}

		if (rotateUpwardsFromPos == PdbReader_3D.Z_HIGH) {
			// matrix.rotX((float) Math.toRadians(-90));
			Cm2_Toolkit.rotate_X_Axis_Matrix(matrix, -90);
		}

		// performing the backwards rotation on all points (including the
		// points of
		// the geometry)
		for (int i = 0; i < l; i++) {

			allPoints[i] = Cm2_Toolkit
					.transformToNewPoint(matrix, allPoints[i]);

		}

		for (int i = 0; i < finals.size(); i++) {
			Point3f p = finals.get(i);
			p = Cm2_Toolkit.transformToNewPoint(matrix, p);

			finals.set(i, p);

		}

		// TODO This causes errors when reloading a membrane. This method, generated by Tim, generates
		//      a optimized shape. Especially important when there have been no atom collision computation.
		//      To use this method again, the reloading-problem has to be solved! 
//		if (DataPool.J3DSettings.automaticDetailSettings)
//			this.performCorrections(finals, allPoints);

		// replacing the matrix with a fresh standardMatrix again, so that
		// the object will be initialized unrotated (what it is indeed)

		mol.setRotationMatrix(Cm2_Toolkit.createAUnitMatrix());

		
		
		
		
		
		
		
		
		
		
		
		
		return finals;
	}

	/**
	 * Reads all ATOM/HETATM/TER lines for the specified model
	 * 
	 * @param all
	 *            the lines of the pdb file
	 * 
	 *            first array=normal atoms, second array=opm atoms, third array
	 *            TER Entries
	 * @param modelnumber
	 * @throws IOException
	 */
	public String[][] readModel(final String[] strings, final int modelnumber) {

		final Vector<String> normal = new Vector<String>();
		final Vector<String> opm = new Vector<String>();

		boolean started = false;
		if (modelnumber < 1)
			started = true;
		// reading the pdbFile loop
		for (int i = 0; i < strings.length; i++) {

			final String line = strings[i];

			if (line == null)
				break;

			if (started == false && line.startsWith("MODEL")) {
				if (modelnumber == new Integer(line.replaceAll("MODEL", "")
						.trim()).intValue())
					started = true;
			} else if (started == true && modelnumber > 0
					&& line.startsWith("ENDMDL"))
				break;

			if (started == false)
				continue;

			if (line.startsWith("ATOM") || line.startsWith("HETATM")
					|| line.startsWith("TER")) {

				try {

					String residueName = line.substring(17, 20).trim();

					if (residueName.toUpperCase().equals("DUM"))
						opm.addElement(line);
					else
						normal.addElement(line);

				} catch (final Exception e) {
					normal.addElement(line);

				}

			}

		}

		return new String[][] { normal.toArray(new String[0]),
				opm.toArray(new String[0]) };
	}

	/**
	 * Tests (and only tests!) if the passed file contains multiple models
	 * 
	 * @param file
	 * @throws IOException
	 */
	public boolean checkForModels(final File file) throws IOException {
		final BufferedReader br = new BufferedReader(new FileReader(file));

		int counter = 0;
		while (true) {

			final String line = br.readLine();

			if (line == null)
				break;

			if (counter == 0
					&& (line.startsWith("ATOM") || line.startsWith("HETATM")))
				return false;

			if ((line.startsWith("MODEL") || line.startsWith("ENDMDL")))
				counter++;

			if (counter > 2)
				return true;
		}
		br.close();
		return false;
	}

	/**
	 * Tries to autorotate the passed lipid, so that its head faces upwards
	 * 
	 * @param lip
	 * @return
	 */
	public static Matrix3f autorotateUpwards(final Lipid_3D lip) {

		final Vector<Point3f> NP = new Vector<Point3f>();
		final Vector<Point3f> O = new Vector<Point3f>();

		final Atom_3D[] allatoms = lip.atoms;
		if (allatoms == null || allatoms.length < 1)
			return Cm2_Toolkit.createAUnitMatrix();

		for (int i = 0; i < allatoms.length; i++) {
			final String name = allatoms[i].element;

			if (name.contains("N") || name.contains("P"))
				NP.add(new Point3f(allatoms[i].position));
			else if (name.contains("O"))
				O.add(new Point3f(allatoms[i].position));

		}

		Vector<Point3f> finals;

		if (NP.isEmpty() == false)
			finals = NP;
		else if (O.isEmpty() == false)
			finals = O;
		else
			return Cm2_Toolkit.createAUnitMatrix();

		float angleY = 0;

		final Vector3f vector = new Vector3f(0, 0, -1);

		for (int i = 0; i < finals.size(); i++) {

			final Vector3f currentOnlyY = new Vector3f(finals.get(i).x, 0,
					finals.get(i).z);

			if (currentOnlyY.x <= 0)
				angleY -= (float) Math.toDegrees(vector.angle(currentOnlyY));
			else
				angleY += (float) Math.toDegrees(vector.angle(currentOnlyY));
		}

		angleY /= finals.size();

		final Matrix3f matrix = Cm2_Toolkit.createAUnitMatrix();

		Cm2_Toolkit.rotate_Y_Axis_Of_CoordinateSystem(matrix, angleY);

		float angleZ = 0;
		final Vector3f upvector = new Vector3f(0, 1, 0);
		for (int i = 0; i < finals.size(); i++) {

			finals.set(i, Cm2_Toolkit
					.transformToNewPoint(matrix, finals.get(i)));

			final Vector3f currentOnlyZ = new Vector3f(0, finals.get(i).y,
					finals.get(i).z);

			if (currentOnlyZ.z <= 0)
				angleZ += ((float) Math.toDegrees(upvector.angle(currentOnlyZ)));
			else
				angleZ -= ((float) Math.toDegrees(upvector.angle(currentOnlyZ)));
		}

		angleZ /= finals.size();

		Cm2_Toolkit.rotate_X_Axis_Matrix(matrix, angleZ);

		Cm2_Toolkit.rotate_Y_Axis_Matrix(matrix, angleY);

		return matrix;
	}

	/**
	 * Reads (only!) the lines of a section in a pdb file that uses the passed
	 * tag as recordname
	 * 
	 * @param file
	 * @param the
	 *            tag (usually 6 letters) of the section
	 * @throws IOException
	 */
	public String[] readSection(final File file, final String tag)
			throws IOException {
		final String[] data = DataPool.getRawPdbData(file.getAbsolutePath());

		final Vector<String> all = new Vector<String>();

		boolean started = false;

		for (int i = 0; i < data.length; i++) {
			try {
				final String line = data[i];

				if (line == null)
					break;
				if (started && !line.startsWith(tag))
					break;
				else if (started == false
						&& line.substring(0, 6).trim().equalsIgnoreCase(tag)) {
					started = true;
				}

				if (started == false)
					continue;
				if (started) {
					if (line.substring(0, 6).trim().equalsIgnoreCase(tag)) {
						all.addElement(line);
					}
				}

			} catch (final Exception e) {

			}

		}

		return all.toArray(new String[0]);
	}

	/**
	 * Calculates all extremepoints in XZ directions from the passed list of
	 * points. The number of extremepoints returned depends on the number of
	 * directions that are set in the 3d options
	 * 
	 * @param levelpoints
	 * @return
	 */
	public Point3f[] calculateLevelExtremes(final Point3f[] levelpoints) {

		
		final int l = levelpoints.length;
		final Point3f[] output = new Point3f[PdbReader_3D.directions];
		
		
//		if(levelpoints.length<2) {
//			
//			
//			for (int i = 0; i < output.length; i++) {
//				output[i]=new Point3f(levelpoints[0]);
//			}
//			return output;
//		}
//		
//		
//		boolean allequal=true;
//		Point2f ref=new Point2f(levelpoints[0].x,levelpoints[0].z);
//		
//		for (int i = 1; i < l; i++) {
//			
//			final Point3f p = levelpoints[i];
//			
//			if(ref.distance(new Point2f(p.x,p.z))>0.0001f) {
//				allequal=false;
//				break;
//			}
//		}
//		
//		if(allequal) {
//			
//			for (int i = 0; i < output.length; i++) {
//				output[i]=new Point3f(levelpoints[0]);
//			}
//			return output;
//			
//			
//		}
		
		
		
		float maxx = -1000000, maxz = -1000000, minx = 1000000, minz = 1000000;

		final float[] distances = new float[PdbReader_3D.directions];
	

		for (int i = 0; i < PdbReader_3D.directions; i++) {
			distances[i] = 0;
		}

		

		for (int i = 0; i < l; i++) {
			final Point3f p = levelpoints[i];
			
			minx = Math.min(minx, p.x);

			minz = Math.min(minz, p.z);

			maxx = Math.max(maxx, p.x);

			maxz = Math.max(maxz, p.z);

		}

		final float cx = (minx + (Math.abs(maxx - minx) / 2));

		final float cz = (minz + (Math.abs(maxz - minz) / 2));

		for (int i = 0; i < l; i++) {
			final Point3f p = levelpoints[i];
//System.out.println(p);
			final Vector3f modified = new Vector3f(p.x - cx, 0, p.z - cz);

			float angle;

			if (modified.x > 0)
				angle = modified.angle(PdbReader_3D.ref);
			else
				angle = modified.angle(PdbReader_3D.ref2)
						+ ((float) Math.PI - 0.000001f);
			
			

			final int index = (int) (angle / PdbReader_3D.eachangle);

			final float length = modified.length();

			if (length > distances[index]) {
				output[index] = p;
				// modified.negate();
				// output[index].set(modified.x,output[index].y,modified.z);
				// Math.
				distances[index] = length;
			}
			// System.out.println(index);
			// System.out.println("Point "+p.x+" "+p.z+" ("+p.y+")");
			// System.out.println("Modif "+modified.x+" "+modified.z+" ("+modified.y+")");
		}
//System.out.println();
		for (int i = 0; i < PdbReader_3D.directions; i++) {

			if (i == 0 && output[0] == null) {
				for (int j = 1; j < output.length; j++) {
					if (output[j] != null) {
						output[0] = output[j];
						break;
					}
				}
			}

			if (output[i] == null) {
				if (i > 0)
					output[i] = output[i - 1];
				else
					output[i] = output[PdbReader_3D.directions - 1];

			}
		}

		for(int i=0;i<output.length;i++) {
			if(output[i]==null) {
				for(int j=0;j<levelpoints.length;j++) {
					if(levelpoints[j]!=null) {
				output[i]=levelpoints[j];
					break;
					}
				}
			}
			
		}
		
		
		
		return output;
	}

	// TODO This causes errors when reloading a membrane. This method, generated by Tim, generates
	//      an optimized shape. Especially important when there have been no atom collision computation.
	//      To use this method again, the reloading-problem has to be solved! 
//	/**
//	 * Performs corrections on the mesh of the shape afterwards
//	 * 
//	 * @param finals
//	 * @param allPoints
//	 */
//	private void performCorrections(final Vector<Point3f> finals,
//			final Point3f[] allPoints) {
//
//		for (int i = 0; i < finals.size(); i++) {
//
//			this.stretchPoint(finals.get(i), new Point3f());
//		}
//
//		final Vector<Point3f> noTrianglePoints = new Vector<Point3f>();
//
//		for (int i = 0; i < allPoints.length; i++) {
//			final Point3f p = allPoints[i];
//			if (finals.contains(p) == false) {
//				noTrianglePoints.add(p);
//			}
//		}
//
//		Vector<Point3f> outsiders = new Vector<Point3f>();
//
//		for (int i = 0; i < noTrianglePoints.size(); i++) {
//			final Point3f p = noTrianglePoints.get(i);
//			final float[] hull = this.redoMesh(finals, p);
//			if (!this.intersector.isCompletelyInside(new Point3d(p), hull)) {
//				outsiders.add(p);
//			}
//
//		}
//
//		// ////////////////Sorting
//
//		final ComparableVector[] cv = new ComparableVector[outsiders.size()];
//		for (int i = 0; i < cv.length; i++) {
//			cv[i] = new ComparableVector(outsiders.get(i));
//		}
//		Arrays.sort(cv);
//
//		outsiders = new Vector<Point3f>();
//
//		for (int i = 0; i < cv.length; i++) {
//			outsiders.add(cv[i].originalPoint);
//		}
//
//		// ////////////////////
//
//		this.handleOutsiders(outsiders, finals, new Point3f());
//
//		if (!outsiders.isEmpty()) {
//			// System.out.println(outsiders.size());
//			//			
//			for (int i = 0; i < outsiders.size(); i++) {
//				final Point3f current = outsiders.get(i);
//				final Vector<Point3f> vec = new Vector<Point3f>();
//				vec.add(current);
//
//				final int size = finals.size();
//				final ComparableVector[] comp = new ComparableVector[size];
//				for (int j = 0; j < size; j++) {
//
//					//					
//					comp[j] = new ComparableVector(finals.get(j));
//					comp[j].sub(new Vector3d(current));
//
//				}
//				Arrays.sort(comp);
//				for (int k = comp.length - 1; k > -1; k--) {
//
//					final float[] mesh = this.redoMesh(finals,
//							comp[k].originalPoint);
//					if (this.intersector.isCompletelyInside(new Point3d(
//							comp[k].originalPoint), mesh)) {
//						this
//								.handleOutsiders(vec, finals,
//										comp[k].originalPoint);
//						break;
//					}
//				}
//
//			}
//
//		}
//
//		// for(int i=0;i<finals.size();i++) {
//		// Point3f p = finals.get(i);
//		// float f = 0.5f*CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE;
//		//			
//		// if(p.x>0)p.x+=f;
//		// else if(p.x<0)p.x-=f;
//		//			
//		// if(p.y>0)p.y+=f;
//		// else if(p.y<0)p.y-=f;
//		//			
//		// if(p.z>0)p.z+=f;
//		// else if(p.z<0)p.z-=f;
//		// }
//
//	}
//
//	private void handleOutsiders(final Vector<Point3f> outsiders,
//			final Vector<Point3f> finals, final Point3f origin) {
//		boolean breaked = false;
//
//		for (int o = 0; o < outsiders.size(); o++) {
//
//			final Point3f point = outsiders.get(o);
//			final Vector3d vector = new Vector3d(point);
//			final Point3d center = new Point3d(origin);
//			vector.sub(center);
//
//			final int size = finals.size();
//			breaked = false;
//
//			for (int i = 0; i < size; i += 3) {
//
//				final float[] hull = new float[9];
//				int c = -1;
//
//				final Point3f A = finals.get(i);
//				final Point3f B = finals.get(i + 1);
//				final Point3f C = finals.get(i + 2);
//
//				hull[++c] = A.x;
//				hull[++c] = A.y;
//				hull[++c] = A.z;
//
//				hull[++c] = B.x;
//				hull[++c] = B.y;
//				hull[++c] = B.z;
//
//				hull[++c] = C.x;
//				hull[++c] = C.y;
//				hull[++c] = C.z;
//
//				if (this.utils.rayTriangleArray(center, vector, (float) vector
//						.length(), hull, 1, new Point3d(), true)) {
//
//					finals.remove(i);
//					finals.remove(i);
//					finals.remove(i);
//
//					finals.insertElementAt(point, i);
//					finals.insertElementAt(A, i);
//					finals.insertElementAt(B, i);
//					finals.insertElementAt(point, i);
//					finals.insertElementAt(B, i);
//					finals.insertElementAt(C, i);
//					finals.insertElementAt(point, i);
//					finals.insertElementAt(C, i);
//					finals.insertElementAt(A, i);
//
//					this.stretchPoint(point, new Point3f(center));
//					/*
//					 * point A B
//					 * 
//					 * point B C
//					 * 
//					 * point C A
//					 */
//					outsiders.remove(o);
//					o--;
//					breaked = true;
//					break;
//
//				}
//
//			}
//
//			if (breaked == false) {
//
//				final float[] hull = this.redoMesh(finals, point);
//				if (this.intersector.isCompletelyInside(new Point3d(point),
//						hull)) {
//					outsiders.remove(o);
//					o--;
//				}
//
//			}
//
//		}
//
//	}
//	
//	private void stretchPoint(final Point3f point3f, final Point3f center) {
//	
//		final Vector3f v = new Vector3f(point3f);
//
//		v.sub(center);
//		v.normalize();
//		v.scale(CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE);
//		v.scale(1.01f);
//
//		point3f.add(v);
//
//	}

	public float[] redoMesh(final Vector<Point3f> finals,
			final Point3f tocheckwith) {
		// int size = finals.size();
		//		
		// float[] hull = new float[size * 3];
		// int c = -1;
		// for (int i = 0; i < size; i+=3) {
		//			
		//			
		// Point3f A = finals.get(i);
		// Point3f B = finals.get(i+1);
		// Point3f C = finals.get(i+2);
		//			
		//			
		// Point3d lower = new Point3d(A);
		// Point3d upper = new Point3d(A);
		//
		// Point3f[]p3f={A,B,C};
		//		
		// for(int j=0;j<3;j++) {
		// lower.x = Math.min(lower.x, p3f[j].x);
		// lower.y = Math.min(lower.y, p3f[j].y);
		// lower.z = Math.min(lower.z, p3f[j].z);
		//
		// upper.x = Math.max(upper.x, p3f[j].x);
		// upper.y = Math.max(upper.y, p3f[j].y);
		// upper.z = Math.max(upper.z, p3f[j].z);
		//			
		// }
		//			
		// boolean bx = lower.x<=tocheckwith.x&&upper.x>=tocheckwith.x;
		// boolean by = lower.y<=tocheckwith.y&&upper.y>=tocheckwith.y;
		// boolean bz = lower.z<=tocheckwith.z&&upper.z>=tocheckwith.z;
		//
		// if(bx||by||bz) {
		// hull[++c] = A.x;
		// hull[++c] = A.y;
		// hull[++c] = A.z;
		//			
		// hull[++c] = B.x;
		// hull[++c] = B.y;
		// hull[++c] = B.z;
		//			
		// hull[++c] = C.x;
		// hull[++c] = C.y;
		// hull[++c] = C.z;
		// }
		// }
		// return hull;

		final int size = finals.size();

		final float[] hull = new float[size * 3];
		int c = -1;
		for (int i = 0; i < size; i += 3) {

			final Point3f A = finals.get(i);
			final Point3f B = finals.get(i + 1);
			final Point3f C = finals.get(i + 2);

			hull[++c] = A.x;
			hull[++c] = A.y;
			hull[++c] = A.z;

			hull[++c] = B.x;
			hull[++c] = B.y;
			hull[++c] = B.z;

			hull[++c] = C.x;
			hull[++c] = C.y;
			hull[++c] = C.z;

		}
		return hull;

	}

	private Constraint_3D[] generateDefaultConstraints(Molecule_3D mol,
			Vector<Atom_3D> allAtoms, Vector<Atom_3D> copy, MyBounds mybounds) {

		Vector<Constraint_3D> finalConstraints = new Vector<Constraint_3D>() {
			private static final long serialVersionUID = 1L;

			public synchronized boolean contains(Object o) {
				Constraint_3D c = (Constraint_3D) o;
				for (int i = 0; i < this.size(); i++) {
					Constraint_3D cc = this.get(i);
					if ((cc.atomA == c.atomA && cc.atomB == c.atomB)
							|| (cc.atomB == c.atomA && cc.atomA == c.atomB))
						return true;
				}
				return false;
			}
		};

		try {
			final String[] conects = new PdbReader_3D().readSection(mol
					.getPDBFile(), "CONECT");

			for (int i = 0; i < conects.length; i++) {

				Atom_3D ref = null;
				Vector<Atom_3D> conInds = new Vector<Atom_3D>();
				for (int j = 7; j < 77; j += 5) {
					try {

						int number = Integer.parseInt(conects[i].substring(
								j - 1, j + 4).trim());
						if (number < 1) {
							continue;
						}

						if (j == 7) {
							ref = copy.get(number - 1);
						} else {
							conInds.addElement(copy.get(number - 1));
						}

					} catch (final Exception e) {

					}

				}

				if (ref == null || conInds.isEmpty())
					continue;

				for (int j = 0; j < conInds.size(); j++) {
					Constraint_3D newC = new Constraint_3D(ref, conInds.get(j));
					if (!finalConstraints.contains(newC))
						finalConstraints.addElement(newC);

				}

			}

		} catch (IOException e) {

			e.printStackTrace();
		}

		// ///////////////////////////////////////////////////////////////////

		Vector<MyAtom> myAtoms = new Vector<MyAtom>();

		float biggestDistance=1.2f;
		boolean hasHydros=false;
		for (int i = 0; i < allAtoms.size(); i++) {
			Atom_3D a = allAtoms.get(i);
			if(a.element.equals("H")) {
				hasHydros=true;
			}
			biggestDistance=Math.max(biggestDistance,(a.radiusCov*2)+0.56f);
			if(a!=null)
			myAtoms.addElement(new MyAtom(a));
			
		}

		MyAtom[]MA=myAtoms.toArray(new MyAtom[0]);
		
		
		
		
		
//		Point3f lower=mybounds.lower;
//		Point3f upper=mybounds.upper;
//		
//		 Vector<MyMyBounds>All=new Vector<MyMyBounds>();
		
//		for(float x=lower.x;x<mybounds.upper.x;x+=biggestDistance) {
//			
//			for(float y=lower.y;y<mybounds.upper.y;y+=biggestDistance) {
//				
//				for(float z=lower.z;z<mybounds.upper.z;z+=biggestDistance) {
//					
//					MyMyBounds b = new MyMyBounds();
//					b.lower=new Point3f(x,y,z);
//					b.upper=new Point3f(x+biggestDistance,y+biggestDistance,z+biggestDistance);
//					All.addElement(b);
//					
//					
//				}
//				
//				
//				
//			}
//			
//			
//			
//		}
		
		
//		for(int i=0;i<All.size();i++) {
//			
//			for(int j=0;j<All.size();j++) {
//				
//				MyMyBounds a = All.get(i);
//				MyMyBounds b = All.get(j);
//				
//				if(a.lower.x==b.upper.x||a.lower.y==b.upper.y||a.lower.z==b.upper.z||a.upper.x==b.lower.x||a.upper.y==b.lower.y||a.upper.z==b.lower.z) {
//					a.otherBounds.addElement(b);
//				}
//			}
//		}
		
		
		
		
		
//		
		
		
		
		
		
		
		
		
		
		
		Hashtable<String,Vector<MyAtom>>hash=new Hashtable<String,Vector<MyAtom>>();
		
		
		for (int i = 0; i < MA.length; i++) {
		
		String key = MA[i].atom.chainID;
		if(hash.containsKey(key)) {
			hash.get(key).addElement(MA[i]);
		}else{
			Vector<MyAtom> v = new Vector<MyAtom>();
			v.addElement(MA[i]);
			hash.put(key,v);
		}
			
	}
		
		Enumeration<String> en = hash.keys();
		
		while(en.hasMoreElements()) {
			
			
			MyAtom[]my=hash.get(en.nextElement()).toArray(new MyAtom[0]);
			
		for (int i = 0; i < my.length; i++) {

			final MyAtom[] fin = this.findClosest(my[i], my,hasHydros,biggestDistance);
			
			if (fin.length == 0) {
				my[i] = null;
				continue;
			}

			for (int j = 0; j < fin.length; j++) {
				Constraint_3D c = new Constraint_3D(my[i].atom, fin[j].atom);
				
				
				
				if (!finalConstraints.contains(c))
					finalConstraints.add(c);

			}

		}
		}
		return finalConstraints.toArray(new Constraint_3D[0]);

	}
	
	
//	private class MyMyBounds extends MyBounds{
//		public Vector<MyMyBounds>otherBounds=new Vector<MyMyBounds>();
//		
//		public Vector<MyAtom>atoms=new Vector<MyAtom>();
//		
//		
//	}

	private MyAtom[] findClosest(final MyAtom current, final MyAtom[] others,boolean hasHydros, float biggestDistance) {

		Vector<MyAtom> ret = new Vector<MyAtom>();

		for (int j = 0; j < others.length; j++) {

			if (others[j] == null || others[j].neighbor == current
					|| current.neighbor == others[j] || others[j] == current)
				continue;
			
			
			
			
			
			if ((others[j].atom.position.x < current.atom.position.x - biggestDistance)
					|| (others[j].atom.position.x > current.atom.position.x + biggestDistance)
					|| (others[j].atom.position.z < current.atom.position.z - biggestDistance)
					|| (others[j].atom.position.z > current.atom.position.z + biggestDistance)
					|| (others[j].atom.position.y < current.atom.position.y - biggestDistance)
					|| (others[j].atom.position.y > current.atom.position.y + biggestDistance)) {
				continue;
			}
			
			
	
			

			final Vector3f v_otherone = new Vector3f(others[j].atom.position);
			v_otherone.sub(current.atom.position);

			// if (v_otherone.length() * 0.9f <= current.atom.radiusCov
			// + others[j].atom.radiusCov) {
			// others[j].neighbor = current;
			// current.neighbor = others[j];
			// ret.addElement(others[j]);
			// }

			float distance = v_otherone.length();

			boolean accept = distance <= current.atom.radiusCov
					+ others[j].atom.radiusCov + 0.56f
					&& distance >= 0.4f;
					
					if(hasHydros) {
			if (current.atom.element.equals("H")
					|| others[j].atom.element.equals("H")) {
				if (distance >= 1.2f)
					accept = false;
			}
					}
			if (accept) {
				others[j].neighbor = current;
				current.neighbor = others[j];

				ret.addElement(others[j]);
			}

		}
		return ret.toArray(new MyAtom[0]);

	}

	private class MyAtom {
		public Atom_3D atom;

		public MyAtom neighbor;

		public MyAtom(final Atom_3D atom) {
			this.atom = atom;
		}

	}

	class ComparableVector extends Vector3d implements Comparable<Object> {

		private static final long serialVersionUID = 1L;
		public Point3f originalPoint;

		public ComparableVector(final Point3f originalPoint) {
			super(originalPoint);
			this.originalPoint = originalPoint;
		}

		public int compareTo(final Object arg0) {
			final ComparableVector other = (ComparableVector) arg0;

			if (this.length() < other.length())
				return 1;
			else
				return -1;

		}

	}
}
