/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_1)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.membranepackingalgorithms;

import java.awt.Component;
import java.awt.Dimension;
import java.util.Random;
import java.util.Vector;

import javax.vecmath.Matrix3f;
import javax.vecmath.Point3f;

import org.cellmicrocosmos.cm2.dataobjects.MembraneModel;
import org.cellmicrocosmos.cm2.engine.DataPool;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.IllegalOperationException;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.Lipid;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.MembranePackingAlgorithm;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.Molecule;
import org.cellmicrocosmos.cm2.misc.Cm2_Toolkit;

/**
 * A first approach of a simulation similar to Molecular Dynamics.
 * 
 * @author Tim Dingersen dackelnose@gmx.net
 * 
 */
class MD extends MembranePackingAlgorithm {

	// internal
	boolean stop = false;
	Dimension dim;
	MyRandom random;

	// user
	int seed;
	int cutoff = 30;
	boolean monUp = true;
	boolean monDown = true;
	int firstStateTries = 50;

	public MD() {
	}

	public boolean buildAndShowGui() {

		final MembraneModel model = DataPool.getMembraneModel();
		this.dim = model.getSize();

		return true;
	}

	public void fillWithLipids() {
		this.random = new MyRandom(this.seed);
		this.displayInformation("Setting initial state");
		this.initialRandomState();
		this.displayInformation("finnished initial state");
	}

	public void initialRandomState() {

		if (this.monUp) {

			final int xcross = this.dim.width;
			final int zcross = this.dim.height;

			for (int j = 0; j < this.dim.height; j += zcross) {
				for (int k = 0; k < this.dim.width; k += xcross) {
					for (int i = 0; i < xcross * zcross; i++) {
						final Lipid l = this.createNextExtracellularLipid();
						final Point3f p = new Point3f((float) k
								+ (float) this.random.getNextInt(xcross, true),
								l.getYTranslation(), (float) j
										+ (float) this.random.getNextInt(
												zcross, true));
						l.setPosition(p);
						l.rotY(this.random.getNextInt(360, true));

						this.testForBetterPositionInFirstState(l,
								MembranePackingAlgorithm.EXTRA);

						if (this.intersectsBorders(l) == false
								&& this.intersects(l) == false) {

							try {
								this.addExtracellularLipid(l);
							} catch (final IllegalOperationException e) {
								e.printStackTrace();
							}
						}

						if (this.stop == true)
							return;
					}
				}
			}
		}

		// intra
		if (this.monDown) {
			final int xcross = this.dim.width;
			final int zcross = this.dim.height;

			for (int j = 0; j < this.dim.height; j += zcross) {
				for (int k = 0; k < this.dim.width; k += xcross) {
					for (int i = 0; i < xcross * zcross; i++) {
						final Lipid l = this.createNextIntracellularLipid();
						final Point3f p = new Point3f((float) k
								+ (float) this.random.getNextInt(xcross, true),
								l.getYTranslation(), (float) j
										+ (float) this.random.getNextInt(
												zcross, true));

						l.setPosition(p);
						l.rotY(this.random.getNextInt(360, true));

						if (this.intersects(l) == false) {

							this.testForBetterPositionInFirstState(l,
									MembranePackingAlgorithm.INTRA);

							if (this.intersectsBorders(l) == false
									&& this.intersects(l) == false) {
								try {
									this.addIntracellularLipid(l);
								} catch (final IllegalOperationException e) {
									e.printStackTrace();
								}
							}
						}
						if (this.stop == true)
							return;
					}
				}
			}
		}
		this.displayInformation("finishing initial state");
	}

	public Lipid testForBetterPositionInFirstState(final Lipid lip,
			final int membraneside) {

		for (int tries = 0; tries < this.firstStateTries; tries++) {

			this.getMoleculesInRadiusOnMySide(lip, this.cutoff, membraneside);
			final Vector<Molecule> exclude = new Vector<Molecule>();

			final Molecule mol1 = this.getNearestMoleculeOnMySide(lip,
					membraneside);

			if (mol1 != null) {
				final float d1 = this.distance(lip, mol1, true);

				exclude.add(mol1);

				final Molecule mol2 = this.getNearestMoleculeExceptOnMySide(
						lip, exclude, membraneside);

				if (mol2 != null) {
					final float d2 = this.distance(lip, mol2, true);

					final Point3f p = (new Point3f(lip.getPosition()));
					final Matrix3f m = (Cm2_Toolkit.copyMatrix(lip
							.getRotationMatrix()));

					this.shakeMolecule(lip);

					final Molecule test1 = this.getNearestMoleculeOnMySide(lip,
							membraneside);
					final float testD = this.distance(lip, test1, true);

					final Vector<Molecule> testExclude = new Vector<Molecule>();
					testExclude.add(test1);

					final Molecule test2 = this
							.getNearestMoleculeExceptOnMySide(lip, testExclude,
									membraneside);

					final float testD2 = this.distance(lip, test2, true);

					if (testD + testD2 < d1 + d2
							&& this.intersects(lip) == false) {
						return lip;
					} else {
						lip.setPosition(new Point3f(p));
						lip.setRotationMatrix(Cm2_Toolkit.copyMatrix(m));
						return lip;
					}

				} else {

					final Point3f p = (new Point3f(lip.getPosition()));
					final Matrix3f m = (Cm2_Toolkit.copyMatrix(lip
							.getRotationMatrix()));

					this.shakeMolecule(lip);

					final Molecule test1 = this.getNearestMoleculeOnMySide(lip,
							membraneside);
					final float testD = this.distance(lip, test1, true);

					final Vector<Molecule> testExclude = new Vector<Molecule>();
					testExclude.add(test1);

					if (testD < d1 && this.intersects(lip) == false) {
						return lip;
					} else {
						lip.setPosition(new Point3f(p));
						lip.setRotationMatrix(Cm2_Toolkit.copyMatrix(m));
						return lip;
					}
				}
			} else
				return lip;

		}
		return lip;
	}

	public void shakeMolecule(final Molecule mol) {
		final Point3f p = new Point3f(mol.getPosition());
		p.x += (this.random.getNextInt(100, false)) / 100f;
		p.z += (this.random.getNextInt(100, false)) / 100f;
		mol.setPosition(new Point3f(p));
		mol.rotY(this.random.getNextInt(360, true));
	}

	public String getFullInformation() {
		return "A Molecular Dynamics Simulation with Lennard-Jones Potential and Verlet Algorithm over Newtons equations of motion.";
	}

	public String getName() {
		return "Simple Molecular Dynamics";
	}

	public String getShortInformation() {
		return "A simple Molecular Dynamics approach. (Version 0.91)";
	}

	public String getStringRepresentationOfSetupParameters() {
		return null;
	}

	public void stopAlgorithm() {
		this.stop = true;
	}

	private class MyRandom {

		long seed;
		Random rand;

		/**
		 * Creates new MyRandom instance.
		 * 
		 * @param seed
		 */
		public MyRandom(final long seed) {
			this.seed = seed;
			this.rand = new Random();
			this.rand.setSeed(seed);
		}

		/**
		 * Returns the next int value in the given range. The absval boolean
		 * indicates if the int is in the range from -scale to +scale or the
		 * absolut value.
		 * 
		 * 
		 * @param scale
		 * @param absval
		 * @return
		 */
		public int getNextInt(final int scale, final boolean absval) {
			this.rand.setSeed(this.seed);
			int i = this.rand.nextInt(scale);
			this.seed = this.rand.nextLong();
			if (absval == false) {

				if (this.seed % 2 == 0)
					i = -i;
			}

			return i;
		}

		/**
		 * Returns the next float The absval boolean indicates if the float is
		 * in the range from - to + or the absolut value.
		 * 
		 * 
		 * @return
		 */
		public float getNextFloat(final boolean absval) {
			this.rand.setSeed(this.seed);
			float f = this.rand.nextFloat();

			this.seed = this.rand.nextLong();

			if (absval == false) {
				if (this.seed % 2 == 0)
					f = -f;
			}

			return f;
		}

		/**
		 * Returns the next double The absval boolean indicates if the double is
		 * in the range from - to + or the absolut value.
		 * 
		 * @return
		 */
		public double getNextDouble(final boolean absval) {
			this.rand.setSeed(this.seed);
			double d = this.rand.nextDouble();

			this.seed = this.rand.nextLong();

			if (absval == false) {
				if (this.seed % 2 == 0)
					d = -d;
			}

			return d;
		}

	}
	
	public boolean supportsAbsoluteValues() {
		return false;
	}

	public boolean supportsMicrodomains() {
		return false;
	}

	public boolean overrideExistingLipids() {
		return true;
	}

	public Component getRuntimeControlPanel() {
		return null;
	}
}
