/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_1)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.membranepackingalgorithms;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.Random;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JSlider;
import javax.swing.JSpinner;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.vecmath.Matrix3f;
import javax.vecmath.Point3f;

import org.cellmicrocosmos.cm2.dataobjects.MembraneModel;
import org.cellmicrocosmos.cm2.engine.DataPool;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.Atom;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.IllegalOperationException;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.Lipid;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.MembranePackingAlgorithm;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.MembranePackingAlgorithmGUI;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.Molecule;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.Protein;
import org.cellmicrocosmos.cm2.misc.Cm2_Toolkit;


/**
 * This algorithm represents the power of geometrical based membrane simulations. 
 * It has been introduced by the diploma thesis of Sebastian Schneider. The main 
 * idea is the simulation of a cooling down or heating up process. 
 * The results are a more ordered or a more chaotic state. In both cases, the 
 * spatial optimization of the lipids is intended.
 * 
 * @author Sebastian Schneider, comments and fixes by Bjoern Sommer bjoern@CELLmicrocosmos.org
 *
 */
public class SimulatedAnnealing extends MembranePackingAlgorithm implements
		ItemListener, ChangeListener, ActionListener {

	// PARAMETERS
	long seed = 0; // random seed
	boolean firstState = false; // boolean for advanced initial state
	// configuration
	int firstStateTries = 40; // Number of attempts to find better place
	boolean monUp = true; // Upper membrane side
	boolean monDown = true; // Lower membrane side
	int cutoff = 30; // cutoff for molecular interaction
	int initialTemperature = 42; // The initial Temperature for the System
	int finaleTemperature = 38; // The final Temperature for the System. Cancel
	// criteria
	int plateauLength = 10; // Number of steps on Temperature
	int molVarsPerPlateauStep = 1000; // Number of molecule modifications per
	// Plateau Step
	float temperatureLoweringRatio = 0.2f; // Ratio of Temperature lowering
	boolean annealing = true; // annealing == true, heating == false
	int div = 1;
	boolean newLips = false; // newlips during simulation
	int newLipAddingAttempts = 1000; // newlips to add
	boolean nearProtMove = false;
	boolean pressure = true;
	boolean protPressure = true;

	// GLOBAL VARIABLES
	Dimension dim;
	MyRandom random;
	boolean breake = false;
	JCheckBox chkInSt, noInSt, chkNoSA, chkTemp, chkPlat, chkNewLips, chkProtMove,
			chkPressure, chkProtPressure;
	JSlider intStTries, cutOffSlider;
	JPanel intStSetPanel, intStPanel, mainpanel, complIntStPanel, layerRight,
			newLipNumbPanel;
	JLabel lTries, lTries2, lTries3, lTries4, temp1, temp2, temp3, temp4,
			plat1, plat2, annealLabel, annealLabel2, lextr, lintr, divLabel,
			divLabel2, newLipNumbLabel;
	JSpinner initTemp, endTemp, ratio, plateau, molVars, divSpinner,
			seedSpinner, newLipSpinner;
	JRadioButton anneal, heat, mono, bi, extra, intra;
	MembraneModel model;
	float temp;
	float energyOld;
	float energyNew;
	float nearLipYfact = 0.9f;
	float nearProtYfact = 1.3f;
	float nearLipDistFact = 2f;

	/**
	 * generates a new SimulatedAnneling
	 */
	public SimulatedAnnealing() {
	}

	/**
	 * Sets up the GUI for the settings
	 * 
	 */
	public boolean buildAndShowGui() {
		this.model = DataPool.getMembraneModel();

		this.dim = this.getMembraneSize();
		// the absolute mainpanel
		this.mainpanel = new JPanel();
		this.mainpanel
				.setLayout(new BoxLayout(this.mainpanel, BoxLayout.X_AXIS));
		final JPanel mainpanel1 = new JPanel();
		final JPanel mainpanel2 = new JPanel();
		final JPanel mainpanel3 = new JPanel();
		this.mainpanel.add(mainpanel1);
		this.mainpanel.add(mainpanel2);
		this.mainpanel.add(mainpanel3);
		mainpanel1.setLayout(new BoxLayout(mainpanel1, BoxLayout.Y_AXIS));
		mainpanel2.setLayout(new BoxLayout(mainpanel2, BoxLayout.Y_AXIS));
		mainpanel3.setLayout(new BoxLayout(mainpanel3, BoxLayout.Y_AXIS));

		// layerpanel
		final JPanel layer = new JPanel();
		layer.setLayout(new BoxLayout(layer, BoxLayout.X_AXIS));
		layer.setBorder(BorderFactory.createTitledBorder("Layer choice"));
		final JPanel layerLeft = new JPanel();
		layerLeft.setLayout(new BoxLayout(layerLeft, BoxLayout.Y_AXIS));
		this.layerRight = new JPanel();
		this.layerRight.setLayout(new BoxLayout(this.layerRight,
				BoxLayout.Y_AXIS));
		this.bi = new JRadioButton();
		this.bi.setSelected(true);
		this.mono = new JRadioButton();

		final ButtonGroup bgl = new ButtonGroup();
		bgl.add(this.bi);
		bgl.add(this.mono);

		final JPanel ll1 = new JPanel();
		ll1.setLayout(new BoxLayout(ll1, BoxLayout.X_AXIS));
		ll1.add(new JLabel("Bilayer:      "));
		ll1.add(this.bi);

		final JPanel ll2 = new JPanel();
		ll2.setLayout(new BoxLayout(ll2, BoxLayout.X_AXIS));
		ll2.add(new JLabel("Monolayer:"));
		ll2.add(this.mono);
		layerLeft.add(ll1);
		layerLeft.add(ll2);
		layer.add(layerLeft);

		this.extra = new JRadioButton();
		this.extra.setSelected(true);
		this.intra = new JRadioButton();

		final ButtonGroup bgm = new ButtonGroup();
		bgm.add(this.extra);
		bgm.add(this.intra);

		final JPanel lr1 = new JPanel();
		lr1.setLayout(new BoxLayout(lr1, BoxLayout.X_AXIS));
		lr1.add(this.extra);
		this.lextr = new JLabel("extracellular                     ");
		lr1.add(this.lextr);

		this.lextr.setEnabled(false);
		final JPanel lr2 = new JPanel();
		lr2.setLayout(new BoxLayout(lr2, BoxLayout.X_AXIS));
		lr2.add(this.intra);
		this.lintr = new JLabel("intracellular                      ");
		lr2.add(this.lintr);
		this.lintr.setEnabled(false);

		this.layerRight.add(lr1);
		this.layerRight.add(lr2);

		this.extra.setEnabled(false);
		this.intra.setEnabled(false);
		layer.add(new JPanel());
		layer.add(this.layerRight);
		mainpanel1.add(layer);
		this.bi.addActionListener(this);
		this.mono.addActionListener(this);
		this.extra.addActionListener(this);
		this.intra.addActionListener(this);
		this.layerRight.setVisible(false);
		// end layerpanel

		// mainpanel for initial state
		this.intStPanel = new JPanel();
		this.intStPanel.setLayout(new BoxLayout(this.intStPanel,
				BoxLayout.Y_AXIS));

		// panel for initial state complexity
		this.complIntStPanel = new JPanel();
		this.complIntStPanel.setLayout(new BoxLayout(this.complIntStPanel,
				BoxLayout.Y_AXIS));
		this.chkInSt = new JCheckBox(
				"<html>Choose for a advanced initial state. "
						+ "<br> "
						+ "Lipids will be placed more detailed side by side</html>.");
		this.chkInSt.setEnabled(true);
		this.chkInSt.setSelected(false);
		this.chkInSt.addItemListener(this);

		this.chkNoSA = new JCheckBox(
				"<html>Deactivate Simulated Annealing. Compute initial state only.</html>.");
		this.chkNoSA.setEnabled(true);
		this.chkNoSA.setSelected(false);
		
		this.noInSt = new JCheckBox(
				"<html>Use the lipids that are already added " + "<br> "
						+ "as an initial state</html>.");

		if (this.getAllLipids(MembranePackingAlgorithm.EXTRA).size() > 0
				|| this.getAllLipids(MembranePackingAlgorithm.INTRA).size() > 0) {
			this.noInSt.setVisible(true);
			this.noInSt.setSelected(true);
		} else {
			this.noInSt.setVisible(false);
			this.noInSt.setSelected(false);
		}

		this.noInSt.addActionListener(new ActionListener() {

			public void actionPerformed(final ActionEvent arg0) {
				if (SimulatedAnnealing.this.noInSt.isSelected()) {
					SimulatedAnnealing.this.chkInSt.setSelected(false);
					SimulatedAnnealing.this.chkInSt.setEnabled(false);
				} else {
					SimulatedAnnealing.this.chkInSt.setEnabled(true);
				}

			}

		});
		this.chkInSt.addActionListener(new ActionListener() {

			public void actionPerformed(final ActionEvent arg0) {
				if (SimulatedAnnealing.this.chkInSt.isSelected()) {
					SimulatedAnnealing.this.noInSt.setSelected(false);
					SimulatedAnnealing.this.noInSt.setEnabled(false);
				} else {
					SimulatedAnnealing.this.noInSt.setEnabled(true);
				}

			}

		});

		final JPanel p1 = new JPanel();
		final JPanel p2 = new JPanel();
		final JPanel p3 = new JPanel();

		p1.add(this.noInSt);
		p2.add(this.chkInSt);
		p3.add(this.chkNoSA);

		p1.setLayout(new BoxLayout(p1, BoxLayout.X_AXIS));
		p2.setLayout(new BoxLayout(p2, BoxLayout.X_AXIS));
		p3.setLayout(new BoxLayout(p3, BoxLayout.X_AXIS));

		p1.add(Box.createHorizontalGlue());
		p2.add(Box.createHorizontalGlue());
		p3.add(Box.createHorizontalGlue());

		this.complIntStPanel.add(p1);
		this.complIntStPanel.add(p2);
		this.complIntStPanel.add(p3);

		// complIntStPanel
		// .add(new JLabel(
		// "<html>Choose for a advanced initial state. "
		// + "<br> "
		// + "Lipids will be placed more detailed side by side</html>."));

		// panel for initial state settings

		this.intStSetPanel = new JPanel();

		this.intStSetPanel.setLayout(new BoxLayout(this.intStSetPanel,
				BoxLayout.Y_AXIS));
		this.lTries = new JLabel("Choose the amount ");
		this.lTries2 = new JLabel("of attempts to find");
		this.lTries3 = new JLabel("better placing conditions");
		this.lTries4 = new JLabel("for each lipid.");
		this.intStTries = new JSlider(JSlider.HORIZONTAL, 0, 100, 50);
		this.intStTries.setSnapToTicks(true);
		this.intStTries.setMajorTickSpacing(50);
		this.intStTries.setMinorTickSpacing(10);
		this.intStTries.addChangeListener(this);
		this.intStTries.setMaximumSize(new Dimension(200, 50));
		this.intStTries.setPaintLabels(true);
		this.intStTries.setPaintTicks(true);
		this.intStSetPanel.add(this.lTries);
		this.intStSetPanel.add(this.lTries2);
		this.intStSetPanel.add(this.lTries3);
		this.intStSetPanel.add(this.lTries4);

		final JPanel initLabelAndSlider = new JPanel();
		initLabelAndSlider.setLayout(new BoxLayout(initLabelAndSlider,
				BoxLayout.X_AXIS));
		initLabelAndSlider.add(this.intStSetPanel);
		initLabelAndSlider.add(this.intStTries);

		this.intStPanel.add(this.complIntStPanel);
		this.intStPanel.add(new JPanel());
		this.intStPanel.add(initLabelAndSlider);
		this.intStPanel.setBorder(BorderFactory
				.createTitledBorder("Initial State Settings"));

		this.lTries.setEnabled(false);
		this.lTries2.setEnabled(false);
		this.lTries3.setEnabled(false);
		this.lTries4.setEnabled(false);
		this.intStTries.setEnabled(false);

		final JPanel divPanel = new JPanel();
		divPanel.setLayout(new BoxLayout(divPanel, BoxLayout.X_AXIS));
		this.divLabel = new JLabel("Choose number of partial membrane filling.");
		this.divLabel2 = new JLabel("For small membranes choose low numbers.");
		this.divSpinner = new JSpinner();
		this.divSpinner.setMaximumSize(new Dimension(40, 50));
		this.divSpinner.setMinimumSize(new Dimension(40, 50));
		this.divSpinner.addChangeListener(this);

		this.divLabel.setEnabled(false);
		this.divLabel2.setEnabled(false);
		this.divSpinner.setEnabled(false);
		this.divSpinner.setValue(new Integer(this.div));

		final JPanel lPanel = new JPanel();
		lPanel.setLayout(new BoxLayout(lPanel, BoxLayout.Y_AXIS));
		lPanel.add(this.divLabel);
		lPanel.add(this.divLabel2);
		divPanel.add(lPanel);
		divPanel.add(new JLabel("   "));
		divPanel.add(this.divSpinner);

		this.intStPanel.add(new JPanel());
		this.intStPanel.add(divPanel);

		mainpanel1.add(this.intStPanel);

		// end initial state

		// panel for time settings
		final JPanel tempPanelMain = new JPanel();
		tempPanelMain.setLayout(new BoxLayout(tempPanelMain, BoxLayout.Y_AXIS));
		tempPanelMain.setBorder(BorderFactory
				.createTitledBorder("Temperature Settings"));

		final JPanel tempPanel1 = new JPanel();
		final JPanel tempPanel2 = new JPanel();
		final JPanel tempPanel3 = new JPanel();
		final JPanel tempPanel4 = new JPanel();
		final JPanel tempPanel11 = new JPanel();

		// temppanel1
		tempPanel1.setLayout(new BoxLayout(tempPanel1, BoxLayout.X_AXIS));

		this.chkTemp = new JCheckBox();
		this.chkTemp.setSelected(false);
		this.chkTemp.addItemListener(this);

		this.temp1 = new JLabel(
				"<html>Activate to specify the initial <br> and the end temperature of the system. <br>If not activated, default settings will be used.</html>");
		tempPanel1.add(this.chkTemp);
		tempPanel1.add(this.temp1);
		// end temppanel1
		tempPanel11.setLayout(new BoxLayout(tempPanel11, BoxLayout.X_AXIS));
		final JPanel tmp = new JPanel();
		tmp.setLayout(new BoxLayout(tmp, BoxLayout.Y_AXIS));
		this.anneal = new JRadioButton("anneal");
		this.heat = new JRadioButton("heat");
		this.anneal.addActionListener(this);
		this.heat.addActionListener(this);

		final ButtonGroup bg = new ButtonGroup();
		bg.add(this.anneal);
		bg.add(this.heat);
		this.anneal.setSelected(true);

		this.anneal.setEnabled(false);
		this.heat.setEnabled(false);

		this.annealLabel = new JLabel("Choose whether the system ");
		this.annealLabel2 = new JLabel("should be anneald or heatend.");

		final JPanel annealLabelPanel = new JPanel();
		annealLabelPanel.setLayout(new BoxLayout(annealLabelPanel,
				BoxLayout.Y_AXIS));
		annealLabelPanel.add(this.annealLabel);
		annealLabelPanel.add(this.annealLabel2);
		tmp.add(this.anneal);
		tmp.add(this.heat);

		this.annealLabel.setEnabled(false);
		this.annealLabel2.setEnabled(false);

		tempPanel11.add(annealLabelPanel);
		tempPanel11.add(new JLabel("                                       "));
		tempPanel11.add(tmp);
		// temppanel1.1

		// end temppanel1.1
		// temppanel2
		this.initTemp = new JSpinner();

		this.initTemp.setMaximumSize(new Dimension(60, 50));
		this.initTemp.setMinimumSize(new Dimension(60, 50));
		tempPanel2.setLayout(new BoxLayout(tempPanel2, BoxLayout.X_AXIS));
		this.temp2 = new JLabel(
				"Change the initial temperature:                                    ");
		tempPanel2.add(this.temp2);
		tempPanel2.add(new JLabel(" "));
		tempPanel2.add(this.initTemp);

		this.initTemp.setValue(new Integer(this.initialTemperature));
		this.initTemp.setEnabled(false);
		this.temp2.setEnabled(false);

		this.initTemp.addChangeListener(this);
		// end temppanel2

		// temppanel3
		this.endTemp = new JSpinner();
		this.endTemp.setMaximumSize(new Dimension(60, 50));
		this.endTemp.setMinimumSize(new Dimension(60, 50));
		tempPanel3.setLayout(new BoxLayout(tempPanel3, BoxLayout.X_AXIS));
		this.temp3 = new JLabel(
				"Change the end temperature:                                 "
						+ "     ");
		tempPanel3.add(this.temp3);
		tempPanel3.add(new JLabel(" "));
		tempPanel3.add(this.endTemp);

		this.endTemp.setValue(new Integer(this.finaleTemperature));
		this.endTemp.setEnabled(false);
		this.temp3.setEnabled(false);

		this.endTemp.addChangeListener(this);
		// end temppanel3

		// temppanel4

		this.ratio = new JSpinner();
		this.ratio.setMaximumSize(new Dimension(60, 50));
		this.ratio.setMinimumSize(new Dimension(60, 50));
		tempPanel4.setLayout(new BoxLayout(tempPanel4, BoxLayout.X_AXIS));
		this.temp4 = new JLabel(
				"Change the temperature lowering steps (0.01 per 1):");
		tempPanel4.add(this.temp4);
		tempPanel4.add(new JLabel(" "));
		tempPanel4.add(this.ratio);

		this.temp4.setEnabled(false);
		this.ratio.setEnabled(false);
		this.ratio.setValue(new Integer(
				(int) (this.temperatureLoweringRatio * 100f)));

		this.ratio.addChangeListener(this);
		// end temppanel4

		tempPanelMain.add(tempPanel1);
		tempPanelMain.add(new JLabel(" "));
		tempPanelMain.add(tempPanel11);
		tempPanelMain.add(new JLabel(" "));
		tempPanelMain.add(tempPanel2);
		tempPanelMain.add(new JLabel(" "));
		tempPanelMain.add(tempPanel3);
		tempPanelMain.add(new JLabel(" "));
		tempPanelMain.add(tempPanel4);
		tempPanelMain.add(new JLabel(" "));
		mainpanel2.add(tempPanelMain);

		// end panel time

		// begin panel plataeu steps
		final JPanel plateauPanelMain = new JPanel();
		final JPanel plateauPanel1 = new JPanel();
		final JPanel plateauPanel2 = new JPanel();
		final JPanel plateauPanel3 = new JPanel();

		plateauPanelMain.setLayout(new BoxLayout(plateauPanelMain,
				BoxLayout.Y_AXIS));
		plateauPanelMain.setBorder(BorderFactory
				.createTitledBorder("Plateau Settings"));

		// panel1
		plateauPanel1.setLayout(new BoxLayout(plateauPanel1, BoxLayout.X_AXIS));
		this.chkPlat = new JCheckBox();
		this.chkPlat.setSelected(false);
		this.chkPlat.addItemListener(this);

		plateauPanel1.add(this.chkPlat);
		plateauPanel1
				.add(new JLabel(
						"<html>Activate to specify plateau Settings.<br>If not activated, default settings will be used.</html> "));

		plateauPanelMain.add(plateauPanel1);
		// end panel1

		// panel2
		plateauPanel2.setLayout(new BoxLayout(plateauPanel2, BoxLayout.X_AXIS));
		this.plat1 = new JLabel(
				"Change the amount of steps per timestep:                             ");
		this.plateau = new JSpinner();
		this.plateau.addChangeListener(this);
		this.plateau.setValue(this.plateauLength);
		this.plateau.setMaximumSize(new Dimension(60, 50));
		this.plateau.setMinimumSize(new Dimension(60, 50));

		plateauPanel2.add(this.plat1);
		plateauPanel2.add(this.plateau);
		this.plat1.setEnabled(false);
		this.plateau.setEnabled(false);
		plateauPanelMain.add(new JLabel(" "));

		plateauPanelMain.add(plateauPanel2);
		// end panel2

		// panel3
		plateauPanel3.setLayout(new BoxLayout(plateauPanel3, BoxLayout.X_AXIS));
		this.plat2 = new JLabel(
				"Change the amount of variations per plateaustep (per side): ");
		this.molVars = new JSpinner();
		this.molVars.addChangeListener(this);
		this.molVars.setValue(this.molVarsPerPlateauStep);
		this.molVars.setMaximumSize(new Dimension(60, 50));
		this.molVars.setMinimumSize(new Dimension(60, 50));

		plateauPanel3.add(this.plat2);
		plateauPanel3.add(this.molVars);
		this.plat2.setEnabled(false);
		this.molVars.setEnabled(false);
		plateauPanelMain.add(new JLabel(" "));
		plateauPanelMain.add(plateauPanel3);
		plateauPanelMain.add(new JLabel(" "));

		// end panel3

		mainpanel2.add(plateauPanelMain);
		// end panel plateaus steps

		// advanced settings
		final JPanel advPanel = new JPanel();
		advPanel.setLayout(new BoxLayout(advPanel, BoxLayout.Y_AXIS));
		advPanel.setBorder(BorderFactory
				.createTitledBorder("Advanced Settings"));
		final JPanel adv1 = new JPanel();
		adv1.setLayout(new BoxLayout(adv1, BoxLayout.X_AXIS));
		this.cutOffSlider = new JSlider(JSlider.HORIZONTAL, 10, 110, 30);
		this.cutOffSlider.setSnapToTicks(true);
		this.cutOffSlider.setMajorTickSpacing(20);
		this.cutOffSlider.setMinorTickSpacing(10);
		this.cutOffSlider.addChangeListener(this);
		this.cutOffSlider.setMaximumSize(new Dimension(150, 50));
		this.cutOffSlider.setMinimumSize(new Dimension(150, 50));
		this.cutOffSlider.setSize(new Dimension(150, 50));
		this.cutOffSlider.setPaintLabels(true);
		this.cutOffSlider.setPaintTicks(true);

		final JLabel advLabel1 = new JLabel(
				"<html>Choose the calculation<br>radius (aka cutoff)<br>for a molecule. <br>(In Angstrm)</html>");
		adv1.add(advLabel1);
		adv1.add(this.cutOffSlider);

		final JPanel adv2 = new JPanel();

//		final JPanel adv3 = new JPanel();

		advPanel.add(adv1);

		final JPanel seedPanel = new JPanel();
		seedPanel.setLayout(new BoxLayout(seedPanel, BoxLayout.X_AXIS));
		final JLabel seedLabel = new JLabel(
				"Choose random seed:                                                ");
		this.seedSpinner = new JSpinner();
		this.seedSpinner.setValue(new Integer(0));
		this.seedSpinner.addChangeListener(this);
		this.seedSpinner.setMaximumSize(new Dimension(60, 50));
		this.seedSpinner.setMinimumSize(new Dimension(60, 50));
		seedPanel.add(seedLabel);
		seedPanel.add(this.seedSpinner);

		final JPanel newLipPanel = new JPanel();
		newLipPanel.setLayout(new BoxLayout(newLipPanel, BoxLayout.X_AXIS));
		final JLabel newLipLabel = new JLabel(
				"<html> Activate to add new<br>lipids during simulation.</html>");
		this.chkNewLips = new JCheckBox();
		this.chkNewLips.setSelected(this.newLips);
		this.chkNewLips.addItemListener(this);
		newLipPanel.add(this.chkNewLips);
		newLipPanel.add(newLipLabel);

		this.newLipNumbPanel = new JPanel();
		this.newLipNumbPanel.setLayout(new BoxLayout(this.newLipNumbPanel,
				BoxLayout.X_AXIS));
		this.newLipSpinner = new JSpinner();
		this.newLipSpinner.setValue(new Integer(this.newLipAddingAttempts));
		this.newLipSpinner.addChangeListener(this);
		this.newLipSpinner.setMaximumSize(new Dimension(70, 50));
		this.newLipSpinner.setMinimumSize(new Dimension(70, 50));
		this.newLipNumbLabel = new JLabel("Choose attempts:   ");
		this.newLipNumbPanel.add(this.newLipNumbLabel);
		this.newLipNumbPanel.add(this.newLipSpinner);
		this.newLipNumbPanel.add(new JLabel(
				"                                         "));
		if (this.newLips == false) {
			this.newLipNumbPanel.setVisible(false);
		}
		// newLipPanel.add(newLipNumbPanel);

		final JPanel nearProt = new JPanel();
		nearProt.setLayout(new BoxLayout(nearProt, BoxLayout.X_AXIS));

		this.chkProtMove = new JCheckBox();
		this.chkProtMove.setSelected(this.nearProtMove);
		this.chkProtMove.addItemListener(this);
		nearProt.add(this.chkProtMove);
		nearProt
				.add(new JLabel(
						"<html>Activate to let lipids translate along the y-axis if they are near to proteins <br> or other y-translated lipids.<br>"
								+ "Should only be activated for long time simulations. Otherwise the effect might be unsatisfying </html> "));

		final JPanel pressurePanel1 = new JPanel();
		final JPanel pressurePanel2 = new JPanel();

		pressurePanel1
				.setLayout(new BoxLayout(pressurePanel1, BoxLayout.X_AXIS));
		pressurePanel2
				.setLayout(new BoxLayout(pressurePanel2, BoxLayout.X_AXIS));

		pressurePanel1
				.add(new JLabel(
						"<html>Activate to simulate pressure <br>from all of the four sides.</html>"));
		this.chkPressure = new JCheckBox();
		this.chkPressure.setSelected(true);
		this.chkPressure.addItemListener(this);
		pressurePanel1.add(this.chkPressure);

		pressurePanel2
				.add(new JLabel(
						"<html>Activate protein gravity. <br>Lipids near proteins will preferably move towards them</html>"));
		this.chkProtPressure = new JCheckBox();
		this.chkProtPressure.setSelected(true);
		this.chkProtPressure.addItemListener(this);
		pressurePanel2.add(this.chkProtPressure);

		advPanel.add(new JLabel(" "));
		advPanel.add(new JLabel(" "));
		advPanel.add(seedPanel);
		advPanel.add(new JLabel(" "));
		advPanel.add(new JLabel(" "));
		advPanel.add(newLipPanel);
		advPanel.add(new JLabel(" "));
		advPanel.add(this.newLipNumbPanel);
		advPanel.add(new JLabel(" "));
		advPanel.add(nearProt);
		advPanel.add(new JLabel(" "));
		advPanel.add(pressurePanel1);
		advPanel.add(pressurePanel2);
		advPanel.add(adv2);
		mainpanel3.add(advPanel);
		// end advanced settings

		this.mainpanel.setVisible(true);

		return MembranePackingAlgorithmGUI.popUpGUI(this.mainpanel,
				"Simulated Annealing", "Settings");
	}

	/**
	 * calls methods to fill membrane with lipids. 1. Step is creating an
	 * initial setting of lipids 2. Step is to reorganize lipids via simulated
	 * annealing methods.
	 */
	public void fillWithLipids() {

		this.random = new MyRandom(this.seed);
		// checks Example Lipids

		if (this.hasMembraneAnyExtracellularLipids() == false) {
			this.monUp = false;
		}
		if (this.hasMembraneAnyIntracellularLipids() == false) {
			this.monDown = false;
		}
		this.setDisplayedProgress(0);
		this.displayInformation("Setting initial state");
		if (!this.noInSt.isSelected()) {
			this.clearAllLipids();
			this.setInitialState();
		}
		if (this.breake == true || this.chkNoSA.isSelected())
			return;

		this.setDisplayedProgress(0);
		this.displayInformation("Annealing mode starts");
		this.simulateAnnealing();
		if (this.breake == true)
			return;
	}

	/**
	 * Sets up the initial state. 1. Randomly places lipids 2. If activated it
	 * tries to find better placing for each placed lipid The second step is
	 * called (if activated) after each randomly lipid placing. The amount of
	 * attempts to find better position can be set in the Settings-GUI.
	 * 
	 */
	public void setInitialState() {

		// generation of a ordered state
		if (!this.firstState) {
			
			this.simpleMode();
			
		// generation of a chaotic state	
		} else {
			
			// extra
			if (this.monUp) {
	
				final int xcross = this.dim.width / this.div;
				final int zcross = this.dim.height / this.div;
	
				for (int j = 0; j < this.dim.height; j += zcross) {
					for (int k = 0; k < this.dim.width; k += xcross) {
						for (int i = 0; i < xcross * zcross; i++) {
							final Lipid l = this.createNextExtracellularLipid();
							final Point3f p = new Point3f((float) k
									+ (float) this.random.getNextInt(xcross, true),
									l.getYTranslation(), (float) j
											+ (float) this.random.getNextInt(
													zcross, true));
							l.setPosition(p);
							l.rotY(this.random.getNextInt(360, true));
	
							// NEW>=Version 1.02: added by Bjoern:
							//  the following if operation was missing (causing a bad runtime
							//  during advanced initial state generation), although it was already
							//  contained in the following loop for the intra side
							if (this.intersects(l) == false) {
								this.testForBetterPositionInFirstState(l,
										MembranePackingAlgorithm.EXTRA);
								if (this.intersectsBorders(l) == false
										&& this.intersects(l) == false) {
		
									try {
										this.addExtracellularLipid(l);
									} catch (final IllegalOperationException e) {
										e.printStackTrace();
									}
								}
							}
							
							if (this.breake == true)
								return;
						}
					}
				}
			}
			// intra
			if (this.monDown && this.chkInSt.isSelected()) {
				
				final int xcross = this.dim.width / this.div;
				final int zcross = this.dim.height / this.div;

				for (int j = 0; j < this.dim.height; j += zcross) {
					for (int k = 0; k < this.dim.width; k += xcross) {
						for (int i = 0; i < xcross * zcross; i++) {
							final Lipid l = this.createNextIntracellularLipid();
							final Point3f p = new Point3f((float) k
									+ (float) this.random.getNextInt(xcross, true),
									l.getYTranslation(), (float) j
											+ (float) this.random.getNextInt(
													zcross, true));

							l.setPosition(p);
							l.rotY(this.random.getNextInt(360, true));

							if (this.intersects(l) == false) {
								this.testForBetterPositionInFirstState(l,
										MembranePackingAlgorithm.INTRA);
								if (this.intersectsBorders(l) == false
										&& this.intersects(l) == false) {
									try {
										this.addIntracellularLipid(l);
									} catch (final IllegalOperationException e) {
										e.printStackTrace();
									}
								}
							}
							
							if (this.breake == true)
								return;
						}
					}
				}
			}

		}
		
		this.displayInformation("finishing initial state");
	}

	/**
	 * Tests if a new Random Position is better then the old Position.
	 * 
	 * 
	 * @param lip
	 * @param membraneside
	 * @return
	 */
	public Lipid testForBetterPositionInFirstState(final Lipid lip,
			final int membraneside) {

		for (int tries = 0; tries < this.firstStateTries; tries++) {

//			final Vector<Molecule> mols = this.getMoleculesInRadiusOnMySide(
//					lip, this.cutoff, membraneside);
			final Vector<Molecule> exclude = new Vector<Molecule>();

			final Molecule mol1 = this.getNearestMoleculeOnMySide(lip,
					membraneside);

			if (mol1 != null) {
				final float d1 = this.distance(lip, mol1, true);

				exclude.add(mol1);

				final Molecule mol2 = this.getNearestMoleculeExceptOnMySide(
						lip, exclude, membraneside);

				if (mol2 != null) {
					final float d2 = this.distance(lip, mol2, true);

					final Point3f p = (new Point3f(lip.getPosition()));
					final Matrix3f m = (Cm2_Toolkit.copyMatrix(lip
							.getRotationMatrix()));

					this.shakeMolecule(lip);

					final Molecule test1 = this.getNearestMoleculeOnMySide(lip,
							membraneside);
					final float testD = this.distance(lip, test1, true);

					final Vector<Molecule> testExclude = new Vector<Molecule>();
					testExclude.add(test1);

					final Molecule test2 = this
							.getNearestMoleculeExceptOnMySide(lip, testExclude,
									membraneside);

					final float testD2 = this.distance(lip, test2, true);

					if (testD + testD2 < d1 + d2
							&& this.intersects(lip) == false) {
						return lip;
					} else {
						lip.setPosition(new Point3f(p));
						lip.setRotationMatrix(Cm2_Toolkit.copyMatrix(m));
						return lip;
					}

				} else {

					final Point3f p = (new Point3f(lip.getPosition()));
					final Matrix3f m = (Cm2_Toolkit.copyMatrix(lip
							.getRotationMatrix()));

					this.shakeMolecule(lip);

					final Molecule test1 = this.getNearestMoleculeOnMySide(lip,
							membraneside);
					final float testD = this.distance(lip, test1, true);

					final Vector<Molecule> testExclude = new Vector<Molecule>();
					testExclude.add(test1);

					if (testD < d1 && this.intersects(lip) == false) {
						return lip;
					} else {
						lip.setPosition(new Point3f(p));
						lip.setRotationMatrix(Cm2_Toolkit.copyMatrix(m));
						return lip;
					}
				}
			} else
				return lip;

		}
		return lip;
	}

	/**
	 * Simple method to change position and rotation of a molecule at random.
	 * 
	 * @param mol
	 */
	public void shakeMolecule(final Molecule mol) {
		final Point3f p = new Point3f(mol.getPosition());
		p.x += ((float) this.random.getNextInt(100, false)) / 100f;
		p.z += ((float) this.random.getNextInt(100, false)) / 100f;
		mol.setPosition(new Point3f(p));
		mol.rotY((float) this.random.getNextInt(360, true));
	}

	/**
	 * Fulfills the simulated annealing method. Calls the simulatedAnnealingStep
	 * method which actually performs the single annealing or heating action.
	 * Handles settings for the progress display.
	 * 
	 * The runtime of this algorithm is:
	 * 
	 * p*(m*(2*k)+n), where
	 * 
	 *  p = number of plateau steps, or more precisely
	 * 
	 *      |initTemperature - finaleTemperature|
	 *  p = ------------------------------------- * plateauLength
	 *            temperatureLoweringRatio                        ,
	 *  
	 *  m = molVarsPerPlateauStep,
	 *  
	 *  k = molecules inside cutoff,
	 *  
	 *  2 = k pre (1.) and post (2.) energy function computation,
	 *  
	 *  n = newLipsNumb (optional: placeNewLipidDuringSimulationRandom or 
	 *      placeNewLipidDuringSimulationBorder).
	 *  
	 *  Note: Check the code for RUNTIMEVAR to find the corresponding loops.
	 *        The move() method for example is not taken into account, because it 
	 *        moves only one single molecule. This method additionally checks all
	 *        molecules inside cutoff for collisions. Collision detection is not 
	 *        taken into account for the running time. 
	 *        
	 *        A computation step contains in the worst case 
	 *        - the manipulation of the data pool (adding or moving a molecule)
	 *        - the manipulation of the 3D Universe (adding or moving a molecule)
	 *        - the computation of the distance to all molecules inside the cutoff.
	 *        Repetition of single computation steps (like the adding or moving of 
	 *        a molecule or the collision detection) are not counted, because 
	 *        the formula is an estimation.
	 *        
	 *  An example with the standard settings:
	 *  
	 *  p*(1000*(2*k)+1000), where
	 *  
	 *      |42 - 38|
	 *  p = --------- * 10 = 200
	 *         0.2               ,
	 *           
	 *  k = this.getMoleculesInRadius(actualLipid, 30).
	 *  
	 *  
	 */
	public void simulateAnnealing() {
		this.temp = (float) this.initialTemperature;
		final int platInit = this.plateauLength;

		if (this.model.getAllProteins().size() < 1) {
			this.protPressure = false;
			this.pressure = true;
		}
		// temperature while
		//  RUNTIMEVAR: p
		while (true) {
			int platCountDeath = this.plateauLength;
			int platCount = 0;
			while (platCountDeath > 0) {// plateau step while
				this.setDisplayedProgress(platCount);
				if (this.annealing == true) {
					this.displayInformation("annealing mode at T: " + this.temp
							+ ", IT: " + this.initialTemperature + ", FT: "
							+ this.finaleTemperature + ", Cooling ratio: "
							+ this.temperatureLoweringRatio);
				} else {
					this.displayInformation("annealing mode at T: " + this.temp
							+ ", IT: " + this.initialTemperature + ", FT: "
							+ this.finaleTemperature + ", Heating ratio: "
							+ this.temperatureLoweringRatio);
				}
				if (this.monUp) {// extracellular

					this.simulateAnnealingStep(MembranePackingAlgorithm.EXTRA);

				}// end extracellular

				if (this.monDown) {// intracellular

					this.simulateAnnealingStep(MembranePackingAlgorithm.INTRA);

				}// end intracellular

				if (this.breake == true)
					return;
				platCountDeath--;
				platCount += 100 / platInit;

			}// end plateau step while
			if (this.breake == true)
				return;
			if (this.annealing == true) {
				if (this.temp <= (float) this.finaleTemperature) {
					return;
				} else {
					this.temp -= this.temperatureLoweringRatio;
				}
			} else {
				if (this.temp >= (float) this.finaleTemperature) {
					return;
				} else {
					this.temp += this.temperatureLoweringRatio;
				}
			}
		}// end temperature while
	}

	/**
	 * Tries to find a better place for randomly chosen lipids. The amount of
	 * chosen lipids is set by the user via the Settings-GUI. It compares the
	 * energy of the lipid before and after new random movement. If the energy
	 * given by the energyFunction() method is better for the new position, the
	 * lipid will be placed there (if it doesn't intersect with other molecules
	 * or the border. If the energy is not better, there is a chance to change
	 * position anyway, depending on the distance between the two energy values
	 * and the temperature of the system. With higher temperature, bad movements
	 * will appear more often. This is to get out of local maxima.
	 * 
	 * @param membraneside
	 */
	public void simulateAnnealingStep(final int membraneside) {

		// see simulateAnnealing() for running time and RUNTIMEVARs
		
		//  RUNTIMEVAR: m
		for (int i = 0; i < this.molVarsPerPlateauStep; i++) {

			// choose a lipid randomly
			final Lipid real = this.getRandomLipid(membraneside);
			// get the old position and matrix
			final Point3f realPosition = new Point3f(real.getPosition());
			final Matrix3f realMatrix = Cm2_Toolkit.copyMatrix(real
					.getRotationMatrix());

			// compute the old energy for the chosen lipid by taking only
			//  the lipids inside cutoff into account
			//  RUNTIMEVAR: k (1st)
			final float eOld = this.energyFunction(real, membraneside);

			// move the molecule
			this.moveMolecule(real, membraneside);

			// compute the new energy for the chosen lipid by taking only
			//  the lipids inside cutoff into account
			//  RUNTIMEVAR: k (2nd)
			final float eNew = this.energyFunction(real, membraneside);

			// check all molecules inside cutoff for intersections with the new position
			final Vector<Molecule> intersMols = this.getMoleculesInRadius(real,
					this.cutoff);
			
			// if the new energy is lower than before, check only for intersections
			if (eNew < eOld) {

				if (this.intersectsMoleculeWithMolecules(real, intersMols)
						|| this.intersectsBorders(real)) {
					real.setPosition(new Point3f(realPosition));
					real.setRotationMatrix(Cm2_Toolkit.copyMatrix(realMatrix));
				}

			// if the new energy is greater than before, compute randomly if the new
			//  position should be kept anyway
			} else {
				
				final float r = ((float) this.random.getNextInt(100, true)) / 100f;
				if (r < Math.exp((eOld - eNew) / this.temp)) {

					if (this.intersectsMoleculeWithMolecules(real, intersMols)
							|| this.intersectsBorders(real)) {
						real.setPosition(new Point3f(realPosition));
						real.setRotationMatrix(Cm2_Toolkit
								.copyMatrix(realMatrix));
					}
				// NEW>=Version 1.01: added by Bjoern:
				//  the position has to be reset here to satisfy the SA algorithm
				//  discussed in Sebastians diploma thesis (see p. 38, 41)
				} else {   
					real.setPosition(new Point3f(realPosition));
					real.setRotationMatrix(Cm2_Toolkit
							.copyMatrix(realMatrix));
				}// END NEW
				
			}

			if (this.breake == true)
				return;
			
		}

		// place new lipids
		//  RUNTIMEVAR: n
		if (this.newLips == true) {
			
			for (int i = 0; i < this.newLipAddingAttempts; i++) {

				// new lipids added from the sides if pressure from sides
				//  activated and no protein gravity is activated
				if (this.pressure == true && this.protPressure == false) {
					this.placeNewLipidDuringSimulationBorder(membraneside);
				}

				// new lipids added randomly if no pressure from sides
				//  activated and protein gravity is activated
				if (this.pressure == false && this.protPressure == true) {
					this.placeNewLipidDuringSimulationRandom(membraneside);
				}

				// new lipids added with 50/50 chance from sides or randomly
				//  if pressure from sides and protein gravity is activated
				if (this.pressure == true && this.protPressure == true) {
					final int rr = this.random.getNextInt(100, true);
					if (rr < 50) {
						this.placeNewLipidDuringSimulationBorder(membraneside);
					} else {
						this.placeNewLipidDuringSimulationRandom(membraneside);
					}
				}
				
				// new lipids added randomly if pressure from sides and protein 
				//  gravity is deactivated
				if (this.protPressure == false && this.pressure == false) {
					this.placeNewLipidDuringSimulationRandom(membraneside);
				}
				
			}
			
		}
		
	}

	/**
	 * Try to add new molecules somewhere inside the membrane.
	 * @param membraneside
	 */
	public void placeNewLipidDuringSimulationRandom(final int membraneside) {

		Lipid l;
		if (membraneside == MembranePackingAlgorithm.EXTRA) {
			l = this.createNextExtracellularLipid();
		} else {
			l = this.createNextExtracellularLipid();
		}
		final Point3f p = new Point3f((float) this.random.getNextInt(
				this.dim.width, true), l.getYTranslation(), (float) this.random
				.getNextInt(this.dim.height, true));
		l.setPosition(p);
		l.rotY(this.random.getNextInt(360, true));
		if (this.intersectsBorders(l) == false && this.intersects(l) == false) {
			if (membraneside == MembranePackingAlgorithm.EXTRA) {
				try {
					this.addExtracellularLipid(l);
				} catch (final IllegalOperationException e) {
					e.printStackTrace();
				}
			} else {
				try {
					this.addIntracellularLipid(l);
				} catch (final IllegalOperationException e) {
					e.printStackTrace();
				}
			}
		}
	}

	/**
	 * Try to add new molecules on the north, east, south or west border.
	 * @param membraneside
	 */
	public void placeNewLipidDuringSimulationBorder(final int membraneside) {
		Lipid l;
		if (membraneside == MembranePackingAlgorithm.EXTRA) {
			l = this.createNextExtracellularLipid();
		} else {
			l = this.createNextIntracellularLipid();
		}
		final int r = this.random.getNextInt(100, true);
		if (r < 25) {
			final Point3f p = new Point3f((float) this.random.getNextInt(
					this.dim.width, true), l.getYTranslation(), l
					.getMaxRadius_XZ());
			l.setPosition(p);
			l.rotY(this.random.getNextInt(360, true));
			if (this.intersectsBorders(l) == false
					&& this.intersects(l) == false) {
				if (membraneside == MembranePackingAlgorithm.EXTRA) {
					try {
						this.addExtracellularLipid(l);
					} catch (final IllegalOperationException e) {
						e.printStackTrace();
					}
				} else {
					try {
						this.addIntracellularLipid(l);
					} catch (final IllegalOperationException e) {
						e.printStackTrace();
					}
				}
			}
		} else if (r < 50) {
			final Point3f p = new Point3f((float) this.random.getNextInt(
					this.dim.width, true), l.getYTranslation(),
					(float) this.dim.height - l.getMaxRadius_XZ());
			l.setPosition(p);
			l.rotY(this.random.getNextInt(360, true));
			if (this.intersectsBorders(l) == false
					&& this.intersects(l) == false) {
				if (membraneside == MembranePackingAlgorithm.EXTRA) {
					try {
						this.addExtracellularLipid(l);
					} catch (final IllegalOperationException e) {
						e.printStackTrace();
					}
				} else {
					try {
						this.addIntracellularLipid(l);
					} catch (final IllegalOperationException e) {
						e.printStackTrace();
					}
				}
			}
		} else if (r < 75) {
			final Point3f p = new Point3f(l.getMaxRadius_XZ(), l
					.getYTranslation(), (float) this.random.getNextInt(
					this.dim.height, true));
			l.setPosition(p);
			l.rotY(this.random.getNextInt(360, true));
			if (this.intersectsBorders(l) == false
					&& this.intersects(l) == false) {
				if (membraneside == MembranePackingAlgorithm.EXTRA) {
					try {
						this.addExtracellularLipid(l);
					} catch (final IllegalOperationException e) {
						e.printStackTrace();
					}
				} else {
					try {
						this.addIntracellularLipid(l);
					} catch (final IllegalOperationException e) {
						e.printStackTrace();
					}
				}
			}
		} else {
			final Point3f p = new Point3f((float) this.dim.width
					- l.getMaxRadius_XZ(), l.getYTranslation(),
					(float) this.random.getNextInt(this.dim.height, true));
			l.setPosition(p);
			l.rotY(this.random.getNextInt(360, true));
			if (this.intersectsBorders(l) == false
					&& this.intersects(l) == false) {
				if (membraneside == MembranePackingAlgorithm.EXTRA) {
					try {
						this.addExtracellularLipid(l);
					} catch (final IllegalOperationException e) {
						e.printStackTrace();
					}
				} else {
					try {
						this.addIntracellularLipid(l);
					} catch (final IllegalOperationException e) {
						e.printStackTrace();
					}
				}
			}
		}
	}

	/**
	 * Returns a randomly chosen lipid from all lipids placed on the chosen
	 * membrane side.
	 * 
	 * @param membraneside
	 * @return Lipid
	 */
	public Lipid getRandomLipid(final int membraneside) {
		Lipid l = null;
		if (membraneside == MembranePackingAlgorithm.EXTRA) {
			final int r = this.random.getNextInt(this.model
					.getLipidCountExtra(), true);
			l = new Lipid(this.model.getLipidAtExtra(r), this);
		} else {
			final int r = this.random.getNextInt(this.model
					.getLipidCountIntra(), true);
			l = new Lipid(this.model.getLipidAtIntra(r), this);
		}
		return l;
	}

	/**
	 * Moves the molecule to a new position. Movements are: 
	 * 1. Exchange place with a neighbor lipid if it is from the same type 
	 *    by given probability.
	 * 2. Adaption of the rotation of a neighbor lipid by given probability. 
	 * 3. Moves lipid along the x and z axis in range between -1 and 1 Angstrom.
	 *    Rotates around lipid around y-axis in range between 0 and 360 degree. By a
	 *    given low probability the lipid can be rotated around the z- and x-axis.
	 * 4. If a lipid is very close to a protein, or near a lipid close to a
	 *    protein in a certain range, the lipid may change the y-transition which
	 *    means it can move up and down in the membrane.
	 * 
	 * @param mol
	 * @param ori
	 * @param membraneside
	 */
	public void moveMolecule(final Lipid mol, final int membraneside) {

		final Point3f middle = new Point3f(this.dim.width / 2f, 0f,
				this.dim.height / 2f);
		
		
		// FLUID SIMULATION (Case 1)

		if (this.getNearestMoleculeOnMySide(mol, membraneside)
				.getOriginalMolecule().getPDBcode().equalsIgnoreCase(
						mol.getOriginalMolecule().getPDBcode())) {
			// Probability of lipid exchange
			final int r = this.random.getNextInt(100, true);
			if (r < 5) {
				final Lipid near = this.getNearestLipidOnMySide(mol,
						membraneside);
				final Point3f molPos = new Point3f(mol.getPosition());
				final Matrix3f molM = Cm2_Toolkit.copyMatrix(mol
						.getRotationMatrix());

				final Point3f nearPos = new Point3f(near.getPosition());
				final Matrix3f nearM = Cm2_Toolkit.copyMatrix(near
						.getRotationMatrix());

				near.setPosition(new Point3f(molPos));
				near.setRotationMatrix(Cm2_Toolkit.copyMatrix(molM));

				mol.setPosition(new Point3f(nearPos));
				mol.setRotationMatrix(Cm2_Toolkit.copyMatrix(nearM));

				final Vector<Molecule> molTests = this.getMoleculesInRadius(
						mol, this.cutoff);
				final Vector<Molecule> nearTests = this.getMoleculesInRadius(
						near, this.cutoff);
				if (this.intersectsMoleculeWithMolecules(mol, molTests) == false
						&& this
								.intersectsMoleculeWithMolecules(near,
										nearTests) == false) {
					return;
				} else {

					near.setPosition(new Point3f(nearPos));
					near.setRotationMatrix(Cm2_Toolkit.copyMatrix(nearM));

					mol.setPosition(new Point3f(molPos));
					mol.setRotationMatrix(Cm2_Toolkit.copyMatrix(molM));
				}
			}
		}

		// END FLUID SIMULATION

		
		// ROTATION ADAPTION  (Case 2)
		
		if (this.getNearestMoleculeOnMySide(mol, membraneside)
				.getOriginalMolecule().getPDBcode().equalsIgnoreCase(
						mol.getOriginalMolecule().getPDBcode())) {

			final Molecule near = this.getNearestMoleculeOnMySide(mol,
					membraneside);
			final int r = this.random.getNextInt(100, true);
			if (r < 5) {
				final Matrix3f molM = Cm2_Toolkit.copyMatrix(mol
						.getRotationMatrix());
				mol.setRotationMatrix(Cm2_Toolkit.copyMatrix(near
						.getRotationMatrix()));
				final Vector<Molecule> molTests = this.getMoleculesInRadius(
						mol, this.cutoff);
				if (this.intersectsMoleculeWithMolecules(mol, molTests) == false) {
					return;
				} else {
					mol.setRotationMatrix(Cm2_Toolkit.copyMatrix(molM));
				}
			}
		}
		
		// END ROTATION ADAPTION

		
		// NORMAL MOVEMENT (Case 3)
		
		final Point3f p = new Point3f(mol.getPosition());
		final Point3f oriP = new Point3f(mol.getPosition());
		final float rx = ((float) this.random.getNextInt(100, false)) / 100f;
		final float rz = ((float) this.random.getNextInt(100, false)) / 100f;
		p.x += rx;
		p.z += rz;
		final int middleRandom = this.random.getNextInt(100, true);
		if (this.pressure == true && this.protPressure == false) {

			if (this.distance(p, middle, true) < this.distance(mol
					.getPosition(), middle, true)) {
				mol.setPosition(new Point3f(p));
			} else if (middleRandom < 30) {
				mol.setPosition(new Point3f(p));
			}
		}
		if (this.pressure == true && this.protPressure == true) {
			if (middleRandom < 50) {
				if (this.distance(p, middle, true) < this.distance(mol
						.getPosition(), middle, true)) {
					mol.setPosition(new Point3f(p));
				} else if (middleRandom < 30) {
					mol.setPosition(new Point3f(p));
				}
			} else {
				final Protein prot = this.getNearestProtein(mol);
				if (prot != null
						&& this.distance(p, prot.getPosition(), true) < this
								.distance(mol, prot, true)) {
					mol.setPosition(new Point3f(p));
				} else if (middleRandom > 70) {
					mol.setPosition(new Point3f(p));
				}
			}
		}
		if (this.protPressure == true && this.pressure == false) {
			final Protein prot = this.getNearestProtein(mol);
			if (prot != null
					&& this.distance(p, prot.getPosition(), true) < this
							.distance(mol, prot, true)) {
				mol.setPosition(new Point3f(p));
			} else if (middleRandom < 30) {
				mol.setPosition(new Point3f(p));
			}
		}
		if (this.protPressure == false && this.pressure == false) {

			mol.setPosition(new Point3f(p));

		}
		Vector<Molecule> molTests = this.getMoleculesInRadius(mol, this.cutoff);
		if (this.intersectsMoleculeWithMolecules(mol, molTests)) {
			mol.setPosition(new Point3f(oriP));
		}
		// System.out.println(rx+" "+rz);
		Matrix3f m = Cm2_Toolkit.copyMatrix(mol.getRotationMatrix());
		mol.rotY((float) this.random.getNextInt(360, true));

		if (this.intersectsMoleculeWithMolecules(mol, molTests)) {
			mol.setRotationMatrix(Cm2_Toolkit.copyMatrix(m));
		}

		m = Cm2_Toolkit.copyMatrix(mol.getRotationMatrix());

		final int r1 = this.random.getNextInt(100, true);
		final int r2 = this.random.getNextInt(100, true);

		final int maxRot = 10;

		if (((float) r1) / 10f < 0.1f) {
			mol.rotX((float) this.random.getNextInt(maxRot, false));
		} else if (((float) r2) / 10f < 0.1f) {
			mol.rotZ((float) this.random.getNextInt(maxRot, false));
		} else if ((float) r1 < 5f) {
			mol.rotX((float) this.random.getNextInt((int) (maxRot / 2), false));
		} else if ((float) r2 < 5f) {
			mol.rotZ((float) this.random.getNextInt((int) (maxRot / 2), false));
		}

		if (this.intersectsMoleculeWithMolecules(mol, molTests)) {
			mol.setPosition(new Point3f(oriP));
			mol.setRotationMatrix(Cm2_Toolkit.copyMatrix(m));
		}

		// END NORMAL MOVEMENT
		

		// NEAR PROTEIN (Case 4)
		
		if (this.nearProtMove) {
			final Protein prot = this.getNearestProtein(mol);
			if (prot != null) {
				final int rr = this.random.getNextInt(100, true);
				if (rr < 90) {
					// determining height
					final Atom[] atoms = mol.getAlignedAtomCopies();
					float minY = 10000f;
					float maxY = -10000f;
					for (int k = 0; k < atoms.length; k++) {
						final Atom a = atoms[k];
						final Point3f po = a.position;
						minY = Math.min(minY, po.y);
						maxY = Math.max(maxY, po.y);
					}

					final float height = Math.abs(maxY - minY);

					if (this.distance(mol, prot, true) <= prot
							.getMaxRadius_XZ()) {

						final float yTrans = mol.getYTranslation();

						final float dis = this.nearestTrianglePoints(mol, prot);

						// Lipids near Proteins
						if (dis < this.nearLipDistFact) {

							if (Math.abs(mol.getYTranslation()) < height
									- height / 3f) {
								mol.setYTranslation(mol.getYTranslation()
										* this.nearProtYfact);
								molTests = this.getMoleculesInRadius(mol,
										this.cutoff);
								if (this.intersectsMoleculeWithMolecules(mol,
										molTests)) {
									mol.setYTranslation(yTrans);
								}
								return;
							}
						}
					}
					if (this.distance(mol, prot, true) <= prot
							.getMaxRadius_XZ()
							+ mol.getMaxRadius_XZ() * 4) {
						// Lipids near Lipids near Proteins

						Molecule nearMol = this.getNearestMoleculeOnMySide(mol,
								membraneside);

						if (nearMol instanceof Lipid) {

							nearMol = (Lipid) nearMol;

							final float ytrans = mol.getYTranslation();

							mol.setYTranslation(nearMol.getYTranslation()
									* this.nearLipYfact);

							if (mol.getYTranslation() < height / 2f
									|| mol.getYTranslation() > height - height
											/ 3f) {
								mol.setYTranslation(ytrans);
							}
							molTests = this.getMoleculesInRadius(mol,
									this.cutoff);
							if (this.intersectsMoleculeWithMolecules(mol,
									molTests)) {
								mol.setYTranslation(ytrans);
							}
							return;
						}
					}

				}// if prob
			}// if radius
		}// if null prot
		
		// END NEAR PROTEIN
		

	}

	// /**
	// * The energy function returns a value which specifies the average
	// distance
	// * to the neighbor molecules in the cutoff radius. Counts the number of
	// * lipids the given newMol is better placed (called b) to then the oldMol.
	// * It also counts the number of lipids the given newMol is worse placed
	// * (called c) to then the oldMol. All distances are added up. If b is
	// higher
	// * then c, the added up distances for the better conditions are divided by
	// * b. Which means the average distance of the lipids nearer to newMol then
	// * to oldMol is returned. Else, the added up distance for the worse
	// * conditions are divided by c. Which means the average distance of the
	// * lipids farer away to newMol then to oldMol is returned.
	// *
	// * @param newMol
	// * @param oldMol
	// * @param membraneside
	// * @return
	// */
	// public float energyFunction(Molecule newMol, Molecule oldMol,
	// int membraneside) {
	// float ret = 0;
	// float mis = 0;
	// int b = 0;
	// int c = 0;
	// Vector<Molecule> mols = this.getMoleculesInRadiusOnMySide(oldMol,
	// cutoff, membraneside);
	// for (int i = 0; i < mols.size(); i++) {
	// float f1 = this.distance(oldMol, mols.get(i), true);
	// float f2 = this.distance(newMol, mols.get(i), true);
	// if (f2 < f1) {
	// b++;
	// ret += f2;
	// } else {
	// c++;
	// mis += f2;
	// }
	// }
	//
	// float retVl;
	// if (b > c) {
	// retVl = ret / (float) b;
	// } else {
	// retVl = mis / (float) c;
	// }
	// return retVl;
	// }
	//
	// /**
	// * The energy function returns a value which specifies the average
	// distance
	// * to the neighbor molecules in the cutoff radius. Counts the number of
	// * lipids the given p1 is better placed (called b) to then p2. It also
	// * counts the number of lipids the given p1 is worse placed (called c) to
	// * then p2. All distances are added up. If b is higher then c, the added
	// up
	// * distances for the better conditions are divided by b. Which means the
	// * average distance of the lipids nearer to p1 then to p2 is returned.
	// Else,
	// * the added up distance for the worse conditions are divided by c. Which
	// * means the average distance of the lipids farer away to p1 then to p2 is
	// * returned.
	// *
	// * @param p1
	// * @param p2
	// * @param membraneside
	// * @return
	// */
	// public float energyFunction(Point3f p1, Point3f p2, int membraneside) {
	// float ret = 0;
	// float mis = 0;
	// int b = 0;
	// int c = 0;
	// Vector<Molecule> mols = this.getMoleculesInRadiusOnMySide(p2, cutoff,
	// membraneside);
	// for (int i = 0; i < mols.size(); i++) {
	// float f1 = this.distance(p1, mols.get(i).getPosition(), true);
	// float f2 = this.distance(p2, mols.get(i).getPosition(), true);
	// if (f2 > f1) {
	// b++;
	// ret += f2;
	// } else {
	// c++;
	// mis += f2;
	// }
	// }
	//
	// float retVl;
	// if (b > c) {
	// retVl = ret / (float) b;
	// } else {
	// retVl = mis / (float) c;
	// }
	// return retVl;
	// }

	/**
	 * The energy function returns a value which specifies the average distance
	 * to the neighbor molecules in the cutoff radius. 
	 */
	public float energyFunction(final Molecule mol, final int membraneside) {
		float dist = 0;
		int b = 0; // b in this energyFunction == mols.size()
		final Vector<Molecule> mols = this.getMoleculesInRadiusOnMySide(mol,
				this.cutoff, membraneside);
		for (int i = 0; i < mols.size(); i++) {
			final float f = this.distance(mol, mols.get(i), true);
			dist += f;
			b++;
		}

		dist = dist / (float) b;
		return dist;
	}

	/**
	 * Creates rows of lipids.
	 * 
	 * Note: This algorithm does not check the lipids for collisions.
	 *       Therefore it often generates layers with colliding lipids.
	 *       During SA this problem is solved, but this algorithm
	 *       should not be used for regular membrane generation without
	 *       running SA.
	 */
	public void simpleMode() {
		final MembraneModel model = DataPool.getMembraneModel();
		final Dimension dim = model.getSize();

		int height = 0;
		if (this.monUp) {
			while (height <= dim.getHeight() + 4) {
				if (this.breake == true)
					return;
				Lipid oldLip = null;

				Lipid newLip = this.createNextExtracellularLipid();

				newLip
						.setPosition(new Point3f(newLip.getMaxRadius_XZ(),
								newLip.getPosition().y, height
										+ newLip.getMaxRadius_XZ() + 2));

				if (this.intersectsBorders(newLip)) {
					if (this.intersectsBorderNorth(newLip)) {
						while (this.intersectsBorderNorth(newLip)) {
							final Point3f p = new Point3f((Point3f) newLip
									.getPosition());
							p.z += 0.01f;
							newLip.setPosition(p);

						}
					}
					if (this.intersectsBorderWest(newLip)) {
						while (this.intersectsBorderWest(newLip)) {
							final Point3f p = new Point3f((Point3f) newLip
									.getPosition());
							p.x += 0.01f;
							newLip.setPosition(p);

						}
					}

				}

				while (this.intersectsBorderEast(newLip) == false) {

					if (this.intersectsWithProteins(newLip) == false
							&& this.intersectsBorders(newLip) == false) {
						try {
							this.addExtracellularLipid(newLip);

						} catch (final IllegalOperationException e) {

							e.printStackTrace();
						}
					}

					oldLip = newLip;

					newLip = this.createNextExtracellularLipid();

					final Point3f p1 = new Point3f((Point3f) oldLip
							.getPosition());
					p1.x = p1.x + oldLip.getMaxRadius_XZ()
							+ newLip.getMaxRadius_XZ() + 2;

					newLip.setPosition(p1);

				}

				height += (int) newLip.getMaxRadius_XZ() + 2;

			}
		}

		height = 0;
		if (this.monDown) {
			while (height <= dim.getHeight() + 4) {
				if (this.breake == true)
					return;
				Lipid oldLip = null;

				Lipid newLip = this.createNextIntracellularLipid();
				newLip
						.setPosition(new Point3f(newLip.getMaxRadius_XZ(),
								newLip.getPosition().y, height
										+ newLip.getMaxRadius_XZ() + 2));

				if (this.intersectsBorders(newLip)) {
					if (this.intersectsBorderNorth(newLip)) {
						while (this.intersectsBorderNorth(newLip)) {
							final Point3f p = new Point3f((Point3f) newLip
									.getPosition());
							p.z += 0.01f;
							newLip.setPosition(p);

						}
					}
					if (this.intersectsBorderWest(newLip)) {
						while (this.intersectsBorderWest(newLip)) {
							final Point3f p = new Point3f((Point3f) newLip
									.getPosition());
							p.x += 0.01f;
							newLip.setPosition(p);
						}
					}

				}

				while (this.intersectsBorderEast(newLip) == false) {

					if (this.intersectsWithProteins(newLip) == false
							&& this.intersectsBorders(newLip) == false) {
						try {
							this.addIntracellularLipid(newLip);

						} catch (final IllegalOperationException e) {

							e.printStackTrace();
						}
					}

					oldLip = newLip;

					newLip = this.createNextIntracellularLipid();

					final Point3f p1 = new Point3f((Point3f) oldLip
							.getPosition());
					p1.x = p1.x + oldLip.getMaxRadius_XZ()
							+ newLip.getMaxRadius_XZ() + 2;

					newLip.setPosition(p1);
				}

				height += (int) newLip.getMaxRadius_XZ() + 2;
			}
		}
	}

	/**
	 * state changed method. Handles GUI settings.
	 */
	public void stateChanged(final ChangeEvent arg0) {

		if (arg0.getSource().equals(this.intStTries)) {
			this.firstStateTries = this.intStTries.getValue();
		}
		if (arg0.getSource().equals(this.initTemp)) {
			if (this.annealing) {
				if (((Integer) this.initTemp.getValue()).intValue() < 1) {
					this.initTemp.setValue(new Integer(1));
				}
				if (((Integer) this.initTemp.getValue()).intValue() <= ((Integer) this.endTemp
						.getValue()).intValue()) {
					this.endTemp.setValue(new Integer(((Integer) this.initTemp
							.getValue()).intValue() - 1));
				}
			} else {
				if (((Integer) this.initTemp.getValue()).intValue() < 1) {
					this.initTemp.setValue(new Integer(1));
				}
				if (((Integer) this.initTemp.getValue()).intValue() >= ((Integer) this.endTemp
						.getValue()).intValue()) {
					this.endTemp.setValue(new Integer(((Integer) this.initTemp
							.getValue()).intValue() + 1));
				}
			}
			this.initialTemperature = ((Integer) this.initTemp.getValue())
					.intValue();
		}
		if (arg0.getSource().equals(this.endTemp)) {
			if (this.annealing) {
				if (((Integer) this.endTemp.getValue()).intValue() < 0) {
					this.endTemp.setValue(new Integer(0));
				}
				if (((Integer) this.endTemp.getValue()).intValue() >= ((Integer) this.initTemp
						.getValue()).intValue()) {
					this.initTemp.setValue(new Integer(((Integer) this.endTemp
							.getValue()).intValue() + 1));
				}
			} else {
				if (((Integer) this.endTemp.getValue()).intValue() < 0) {
					this.endTemp.setValue(new Integer(0));
				}
				if (((Integer) this.endTemp.getValue()).intValue() <= ((Integer) this.initTemp
						.getValue()).intValue()) {
					this.initTemp.setValue(new Integer(((Integer) this.endTemp
							.getValue()).intValue() - 1));
				}
			}
			this.finaleTemperature = ((Integer) this.endTemp.getValue())
					.intValue();
		}
		if (arg0.getSource().equals(this.ratio)) {
			if (((Integer) this.ratio.getValue()).intValue() < 0) {
				this.ratio.setValue(new Integer(0));
			}
			if (((Integer) this.ratio.getValue()).intValue() > 100) {
				this.ratio.setValue(new Integer(100));
			}
			this.temperatureLoweringRatio = ((float) ((Integer) this.ratio
					.getValue()).intValue()) / 100f;
		}
		if (arg0.getSource().equals(this.plateau)) {
			if (((Integer) this.plateau.getValue()).intValue() < 1) {
				this.plateau.setValue(new Integer(1));
			}
			this.plateauLength = ((Integer) this.plateau.getValue()).intValue();
		}
		if (arg0.getSource().equals(this.molVars)) {
			if (((Integer) this.molVars.getValue()).intValue() < 1) {
				this.molVars.setValue(new Integer(1));
			}
			this.molVarsPerPlateauStep = ((Integer) this.molVars.getValue())
					.intValue();
		}
		if (arg0.getSource().equals(this.divSpinner)) {
			if (((Integer) this.divSpinner.getValue()).intValue() < 1) {
				this.divSpinner.setValue(new Integer(1));
			}
			if (((Integer) this.divSpinner.getValue()).intValue() > 10) {
				this.divSpinner.setValue(new Integer(10));
			}
			this.div = ((Integer) this.divSpinner.getValue()).intValue();
		}
		if (arg0.getSource().equals(this.cutOffSlider)) {
			this.cutoff = this.cutOffSlider.getValue();
		}
		if (arg0.getSource().equals(this.seedSpinner)) {
			if (((Integer) this.seedSpinner.getValue()).intValue() < 0) {
				this.seedSpinner.setValue(new Integer(0));
				this.seed = 0;
			}
			this.seed = ((Integer) this.seedSpinner.getValue()).longValue();
		}
		if (arg0.getSource().equals(this.newLipSpinner)) {
			if (((Integer) this.newLipSpinner.getValue()).intValue() < 0) {
				this.newLipSpinner.setValue(new Integer(0));
			}
			this.newLipAddingAttempts = ((Integer) this.newLipSpinner.getValue())
					.intValue();
		}
	}

	/**
	 * item state changed method. Handles GUI settings
	 */
	public void itemStateChanged(final ItemEvent arg0) {

		if (arg0.getSource().equals(this.chkInSt)) {
			if (this.chkInSt.isSelected() == true) {
				this.firstState = true;
				this.lTries.setEnabled(true);
				this.lTries2.setEnabled(true);
				this.lTries3.setEnabled(true);
				this.lTries4.setEnabled(true);
				this.intStTries.setEnabled(true);
				this.firstStateTries = this.intStTries.getValue();
				this.divLabel.setEnabled(true);
				this.divLabel2.setEnabled(true);
				this.divSpinner.setEnabled(true);
				this.div = ((Integer) this.divSpinner.getValue()).intValue();
			} else {
				this.firstState = false;
				this.lTries.setEnabled(false);
				this.lTries2.setEnabled(false);
				this.lTries3.setEnabled(false);
				this.lTries4.setEnabled(false);
				this.intStTries.setEnabled(false);
				this.firstStateTries = 50;
				this.divLabel.setEnabled(false);
				this.divLabel2.setEnabled(false);
				this.divSpinner.setEnabled(false);
				this.div = 4;
			}
		}
		if (arg0.getSource().equals(this.chkTemp)) {
			if (this.chkTemp.isSelected() == true) {
				this.temp2.setEnabled(true);
				this.initTemp.setEnabled(true);
				this.initialTemperature = ((Integer) this.initTemp.getValue())
						.intValue();

				this.temp3.setEnabled(true);
				this.endTemp.setEnabled(true);
				this.finaleTemperature = ((Integer) this.endTemp.getValue())
						.intValue();

				this.temp4.setEnabled(true);
				this.ratio.setEnabled(true);
				this.temperatureLoweringRatio = (float) ((Integer) this.ratio
						.getValue()).intValue() / 100f;

				this.anneal.setEnabled(true);
				this.heat.setEnabled(true);
				this.annealLabel.setEnabled(true);
				this.annealLabel2.setEnabled(true);
				if (this.anneal.isSelected()) {
					this.annealing = true;
				} else
					this.annealing = false;

			} else {
				this.temp2.setEnabled(false);
				this.initTemp.setEnabled(false);
				this.initialTemperature = 42;

				this.temp3.setEnabled(false);
				this.endTemp.setEnabled(false);
				this.finaleTemperature = 30;

				this.temp4.setEnabled(false);
				this.ratio.setEnabled(false);
				this.temperatureLoweringRatio = 0.1f;

				this.anneal.setEnabled(false);
				this.heat.setEnabled(false);
				this.annealLabel.setEnabled(false);
				this.annealLabel2.setEnabled(false);
				this.annealing = true;
			}

		}
		if (arg0.getSource().equals(this.chkPlat)) {
			if (this.chkPlat.isSelected()) {
				this.plat1.setEnabled(true);
				this.plat2.setEnabled(true);
				this.plateau.setEnabled(true);
				this.molVars.setEnabled(true);
				this.plateauLength = ((Integer) this.plateau.getValue())
						.intValue();
				this.molVarsPerPlateauStep = ((Integer) this.molVars.getValue())
						.intValue();

			} else {
				this.plat1.setEnabled(false);
				this.plat2.setEnabled(false);
				this.plateau.setEnabled(false);
				this.molVars.setEnabled(false);
				this.plateauLength = 20;
				this.molVarsPerPlateauStep = 100000;

			}
		}
		if (arg0.getSource().equals(this.chkNewLips)) {
			if (this.chkNewLips.isSelected()) {
				this.newLips = true;
				this.newLipNumbPanel.setVisible(true);
			} else {
				this.newLips = false;
				this.newLipNumbPanel.setVisible(false);
			}
		}
		if (arg0.getSource().equals(this.chkProtMove)) {
			if (this.chkProtMove.isSelected()) {
				this.nearProtMove = true;
			} else
				this.nearProtMove = false;
		}
		if (arg0.getSource().equals(this.chkPressure)) {
			if (this.chkPressure.isSelected()) {
				this.pressure = true;
			} else
				this.pressure = false;
		}
		if (arg0.getSource().equals(this.chkProtPressure)) {
			if (this.chkProtPressure.isSelected()) {
				this.protPressure = true;
			} else
				this.protPressure = false;
		}

	}

	/**
	 * action performed method. Handles GUI settings
	 */
	public void actionPerformed(final ActionEvent arg0) {
		if (arg0.getSource().equals(this.anneal)) {
			this.annealing = true;
			if (((Integer) this.initTemp.getValue()).intValue() <= ((Integer) this.endTemp
					.getValue()).intValue()) {
				this.endTemp.setValue(new Integer(((Integer) this.initTemp
						.getValue()).intValue() - 1));
			}
			this.initialTemperature = ((Integer) this.initTemp.getValue())
					.intValue();
			if (((Integer) this.endTemp.getValue()).intValue() >= ((Integer) this.initTemp
					.getValue()).intValue()) {
				this.initTemp.setValue(new Integer(((Integer) this.endTemp
						.getValue()).intValue() + 1));
			}
			this.finaleTemperature = ((Integer) this.endTemp.getValue())
					.intValue();
			this.temp4
					.setText("Change the temperature lowering steps (0.01 per 1):");
		}
		if (arg0.getSource().equals(this.heat)) {
			this.annealing = false;
			if (((Integer) this.initTemp.getValue()).intValue() >= ((Integer) this.endTemp
					.getValue()).intValue()) {
				this.endTemp.setValue(new Integer(((Integer) this.initTemp
						.getValue()).intValue() + 1));
			}
			this.initialTemperature = ((Integer) this.initTemp.getValue())
					.intValue();
			if (((Integer) this.endTemp.getValue()).intValue() <= ((Integer) this.initTemp
					.getValue()).intValue()) {
				this.initTemp.setValue(new Integer(((Integer) this.endTemp
						.getValue()).intValue() - 1));
			}

			this.finaleTemperature = ((Integer) this.endTemp.getValue())
					.intValue();
			this.temp4
					.setText("Change the temperature rising steps (0.01 per 1):     ");
		}
		if (arg0.getSource().equals(this.bi)) {
			this.monUp = true;
			this.monDown = true;
			this.extra.setEnabled(false);
			this.intra.setEnabled(false);
			this.lextr.setEnabled(false);
			this.lintr.setEnabled(false);
			this.layerRight.setVisible(false);
		}
		if (arg0.getSource().equals(this.mono)) {
			this.extra.setEnabled(true);
			this.intra.setEnabled(true);
			this.lextr.setEnabled(true);
			this.lintr.setEnabled(true);
			if (this.extra.isSelected()) {
				this.monUp = true;
				this.monDown = false;
			} else {
				this.monUp = false;
				this.monDown = true;
			}
			this.layerRight.setVisible(true);
		}
		if (arg0.getSource().equals(this.extra)) {
			this.monUp = true;
			this.monDown = false;
		}
		if (arg0.getSource().equals(this.intra)) {
			this.monUp = false;
			this.monDown = true;
		}
	}

	/**
	 * 
	 * returns the full information about this algorithm
	 * 
	 * @see org.cellmicrocosmos.cm2.membranepackingalgorithm.MembranePackingAlgorithmInterface#getFullInformation()
	 */
	public String getFullInformation() {
		final String s = "<html>"
				+ "Performs a simulated annealing simulation.<br>"
				+ "<br>"
				+ "This algorithm represents the power of geometrical<br>"
				+ "based membrane simulations. It has been introduced<br>"
				+ "by the diploma thesis of Sebastian Schneider in 2008.<br>"
				+ "The main idea is the simulation of a cooling down or<br>" 
				+ "heating up process. The results are a more ordered<br>"
				+ "or a more chaotic state. In both cases, the spatial<br>" 
				+ "optimization of the lipids is intended.<br>"
				+ "<br>"
				+ "This version contains small corrections by<br>" 
				+ "Bjoern Sommer strictly following the original idea<br"
				+ "of Sebastian.<br>"
				+ "<br>"
				+ "The evaluation function is based on geometrical<br>"
				+ "calculations."
				+ "<br><br>"
				+ "Author: Sebastian Schneider."
				+ "<br>"
				+ "Version: 1.03"
				+ "<br>"
				+ "</html>";

		return s;
	}

	/**
	 * returns the name of this algorithm.
	 */
	public String getName() {
		return "Simulated Annealing";
	}

	/**
	 * returns the short information about this algorithm
	 */
	public String getShortInformation() {
		return "Performs a simulated annealing simulation. (Version 1.03)";
	}

	/**
	 * returns string representation of the chosen settings.
	 */
	public String getStringRepresentationOfSetupParameters() {
		String s = new String();
		if (this.chkNoSA.isSelected())
			s += "Simulated Annealing DEACTIVATED\n";
		
		s += "Seed " + this.seed;
		s += "\n";
		s += "First state advanced " + this.firstState;
		s += "\n";
		s += "Initial state placing grid divisor " + this.div;
		s += "\n";
		s += "First state better placing " + this.firstStateTries;
		s += "\n";
		s += "Build extracellular " + this.monUp;
		s += "\n";
		s += "Build intracellular " + this.monDown;
		s += "\n";
		s += "Calculation cutoff " + this.cutoff;
		s += "\n";
		s += "Initial Temperature " + this.initialTemperature;
		s += "\n";
		s += "Final Temperature " + this.finaleTemperature;
		s += "\n";
		s += "Temperature Lowering ratio " + this.temperatureLoweringRatio;
		s += "\n";
		s += "Plateau steps " + this.plateauLength;
		s += "\n";
		s += "Molecular variations per plateu step " + this.molVarsPerPlateauStep;
		s += "\n";
		s += "Annealing " + this.annealing;
		s += "\n";
		s += "Placing new lips " + this.newLips;
		s += "\n";
		s += "Number of attempts to place new lipids " + this.newLipAddingAttempts;
		s += "\n";
		s += "Y-Translate lipids near proteins " + this.nearProtMove;
		s += "\n";
		s += "Algorithm stoped before finishing " + this.breake;
		s += "\n";
		s += "Pressure " + this.pressure;
		s += "\n";
		s += "Protein Gravity " + this.protPressure;
		return s;
	}

	/**
	 * sets the breake flag. Lets the algorithm know if it should stop soon.
	 * 
	 */
	public void stopAlgorithm() {
		this.breake = true;
	}

	/**
	 * 
	 * Private Random Class generates new Random numbers. This makes sure that
	 * you always get the same random numbers if same starting value is used.
	 * 
	 * @author Maloche
	 * 
	 */
	private class MyRandom {

		long seed;
		Random rand;

		/**
		 * Creates new MyRandom instance.
		 * 
		 * @param seed
		 */
		public MyRandom(final long seed) {
			this.seed = seed;
			this.rand = new Random();
			this.rand.setSeed(seed);
		}

		/**
		 * Returns the next int value in the given range. The absval boolean
		 * indicates if the int is in the range from -scale to +scale or the
		 * absolut value.
		 * 
		 * 
		 * @param scale
		 * @param absval
		 * @return
		 */
		public int getNextInt(final int scale, final boolean absval) {
			this.rand.setSeed(this.seed);
			int i = this.rand.nextInt(scale);
			this.seed = this.rand.nextLong();
			if (absval == false) {

				if (this.seed % 2 == 0)
					i = -i;
			}

			return i;
		}

//		/**
//		 * Returns the next float The absval boolean indicates if the float is
//		 * in the range from - to + or the absolut value.
//		 * 
//		 * 
//		 * @return
//		 */
//		public float getNextFloat(final boolean absval) {
//			this.rand.setSeed(this.seed);
//			float f = this.rand.nextFloat();
//
//			this.seed = this.rand.nextLong();
//
//			if (absval == false) {
//				if (this.seed % 2 == 0)
//					f = -f;
//			}
//
//			return f;
//		}
//
//		/**
//		 * Returns the next double The absval boolean indicates if the double is
//		 * in the range from - to + or the absolut value.
//		 * 
//		 * @return
//		 */
//		public double getNextDouble(final boolean absval) {
//			this.rand.setSeed(this.seed);
//			double d = this.rand.nextDouble();
//
//			this.seed = this.rand.nextLong();
//
//			if (absval == false) {
//				if (this.seed % 2 == 0)
//					d = -d;
//			}
//
//			return d;
//		}

	}

	// NEW>=Version 1.03
	public boolean supportsAbsoluteValues() {
		return false;
	}
	
	public boolean supportsMicrodomains() {
		return false;
	}

	public boolean overrideExistingLipids() {

		return false;
	}

	public Component getRuntimeControlPanel() {
		return null;
	}
	
}
