/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_1)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.membranepackingalgorithms.atomlevelminimizer;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.io.File;
import java.security.SecureRandom;
import java.util.Collections;
import java.util.Vector;

import javax.swing.JOptionPane;
import javax.vecmath.Matrix3f;
import javax.vecmath.Point2f;
import javax.vecmath.Point3f;
import javax.vecmath.Vector3f;

import org.cellmicrocosmos.cm2.membranepackingalgorithm.Atom;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.AutoBox;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.EnergyFunction;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.IllegalOperationException;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.Lipid;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.MembranePackingAlgorithm;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.MicroDomain;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.Molecule;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.RotationToolkit;


/**
 *
 * This is the base-class of the Atom-Level-Minimizer, described by my Diploma-Thesis. All basic functionality
 * is performed or controlled by this class 
 * 
 * @author Tim Dingersen
 * dackelnose@gmx.net
 *
 */

public class Atom_Level_Minimizer extends MembranePackingAlgorithm {

	MyData myData = new MyData();

	Energies energies;

	Gui gui;

	boolean hasHydrogens = false;

	File pluginsFile;

	private boolean stopAlgorithm = false;
	public boolean pauseAlgorithm = true;
	
	long timeTaken = 0;
	long timeInit = 0;
	
	public Vector<EnergyFunction> energyFunctions = new Vector<EnergyFunction>();
	AutoBox bounds;

	double initialValue = 0;
	int area = -1;

	public boolean buildAndShowGui() {
		
		this.setStartInBackgroundAllowed(false);

		pluginsFile = new File(this.getAlgorithm_Directory_In_Workspace()
				.getAbsolutePath()
				+ File.separator + "EnergyFunctions");
		if (!pluginsFile.exists())
			pluginsFile.mkdirs();

		if (this.getSelectedArea() != -1) {

			int val = JOptionPane
					.showConfirmDialog(
							null,
							"You have currently selected a particular area\nDo you want to consider this area only?\n",

							"Selected Area", JOptionPane.YES_NO_OPTION,
							JOptionPane.QUESTION_MESSAGE);

			if (val == JOptionPane.YES_OPTION) {
				area = this.getSelectedArea();
			}

		}

		this.energies = new Energies(this);

		myData.contents.setRelevantMolecules(this
				.getAllMoleculesInCurrentModel(), this.getCurrentLayerID(),
				area, this);

		String[] types = myData.contents
				.getAllDifferentParticleTypesInMembrane(this);

		if (this.countTotalLipidsThatAreAdded() < 1) {
			JOptionPane
					.showMessageDialog(
							null,
							"This algorithm is meant to performs refinements on existing membranes! You\n"
									+ "can, however use the 'Add Lipids' option in order to insert Lipids, but make\n"
									+ "sure to have a high setting chosen for this, at least at the beginning!",
							"No Molecules in Model",
							JOptionPane.WARNING_MESSAGE);

		}

		gui = new Gui(this);

		gui.paraPanel.buildParameterMatrices(types);
		gui.paraPanel.fillWithDefaults(0);
		gui.paraPanel.fillWithDefaults(1);
		gui.paraPanel.fillWithPreviousValues();

		gui.controlPanel.fillWithDefaults();
		gui.controlPanel.restoreSettings();
		gui.controlPanel.updateFields();

		this.bounds = new AutoBox(myData.contents.atomsToCompare);
		this.bounds.lower = new Point3f(0, this.bounds.lower.y, 0);
		this.bounds.upper = new Point3f(this.getMembraneSize().width,
				this.bounds.upper.y, this.getMembraneSize().height);
		this.bounds.center = new Point3f(this.getMembraneSize().width / 2, 0,
				this.getMembraneSize().height / 2);

		if (area != -1) {
			gui.controlPanel.cbAllLayers.setSelected(false);
			gui.controlPanel.cbAllLayers.setEnabled(false);
		}

		return true;
	}

	private void pause(boolean atStart) throws InterruptedException {
		long entranceTime = System.currentTimeMillis();
		boolean hasbeenPaused = false;

		if (this.pauseAlgorithm) {
			if (atStart) {

				// this.gui.startBtn.setForeground(Color.green.darker());
				// this.gui.startBtn.setText("Wait...");
				// this.gui.startBtn.setEnabled(false);

				this.gui.startBtn.setText("Start");
				this.gui.startBtn.setEnabled(true);
				hasbeenPaused = true;

			} else {

				this.gui.startBtn.setForeground(Color.green.darker());
				this.gui.startBtn.setText("Resume");
				this.gui.startBtn.setEnabled(true);
				hasbeenPaused = true;
			}

		} else {
			this.gui.startBtn.setForeground(Color.yellow.darker());
			this.gui.startBtn.setText("Pause");
			this.gui.startBtn.setEnabled(true);
		}
		this.gui.controlsBtn.setEnabled(this.pauseAlgorithm);
		this.gui.parametersBtn.setEnabled(this.pauseAlgorithm);

		// boolean updated = false;
		while (true) {

			this.dock();

			if (!pauseAlgorithm) {
				if (this.gui.controlsBtn.isSelected())
					this.gui.controlsBtn.doClick();
				if (this.gui.parametersBtn.isSelected())
					this.gui.parametersBtn.doClick();
			}
			this.gui.controlsBtn.setEnabled(pauseAlgorithm);
			this.gui.parametersBtn.setEnabled(pauseAlgorithm);

			if (!this.pauseAlgorithm)
				break;
			if (stopAlgorithm) {
				break;
			}
			// if (!updated) {
			//
			// // for (int i = 0; i < myData.contents.atomsToTreat.length; i++)
			// {
			// // this.myData.contents.atomsToTreat[i]
			// // .setPosition(this.myData.contents.atomsToTreat[i].position);
			// // if (stopAlgorithm) {
			// // break;
			// // }
			// // }
			//
			// // for (int i = 0; i < myData.contents.moleculesToTreat.length;
			// i++) {
			// //
			// // myData.contents.moleculesToTreat[i].updateGeometry();
			// // this.shift(myData.contents.moleculesToTreat[i]);
			// //
			// // if (stopAlgorithm) {
			// // break;
			// // }
			// // }
			// updated = true;
			// }
			Thread.sleep(300);

		}

		// /////continue after pause

		if (atStart) {

			gui.controlPanel.updateFields();
			gui.paraPanel.updateEnergies();

			myData.contents.setRelevantMolecules(this
					.getAllMoleculesInCurrentModel(), this.getCurrentLayerID(),
					area, this);

			for (int i = 0; i < myData.contents.atomsToTreat.length; i++) {
				if (stopAlgorithm) {
					break;
				}
				double en = energies
						.totalEnergy(myData.contents.atomsToTreat[i]);

				en += energies
						.totalInternalEnergy(myData.contents.atomsToTreat[i]);

				myData.contents.atomsToTreat[i].setEnergy(en);
				initialValue += en;
			}

			myData.contents.addCurrentEnergyToHistory(initialValue);
			if (this.gui.guiMode == 2) {
				this.gui.graphPanel.setValues(myData.contents
						.getEnergyHistory(), myData.contents.step);
			}

		}

		if (hasbeenPaused) {
			// boolean anyChanges=gui.anyChanges();
			gui.controlPanel.updateFields();
			gui.paraPanel.updateEnergies();

			myData.contents.setRelevantMolecules(this
					.getAllMoleculesInCurrentModel(), this.getCurrentLayerID(),
					area, this);

			energies.lowestValue = null;

			// if(anyChanges&&!atStart) {
			// double totalEn=0d;
			// for (int i = 0; i < myData.contents.atomsToTreat.length; i++) {
			// if (stopAlgorithm) {
			// break;
			// }
			// double en =
			// energies.totalEnergy(myData.contents.atomsToTreat[i]);
			//				
			// en+=energies.totalInternalEnergy(myData.contents.atomsToTreat[i]);
			//
			// myData.contents.atomsToTreat[i].setEnergy(en);
			// totalEn += en;
			// }
			//				
			// myData.contents.currentFreeEnergy=totalEn;
			//				
			// if(!myData.contents.energyHistory.isEmpty()) {
			// myData.contents.energyHistory.set(myData.contents.energyHistory.size()-1,totalEn);
			// }
			// if (this.gui.guiMode == 2) {
			// this.gui.graphPanel.setValues(myData.contents
			// .getEnergyHistory(), myData.contents.step);
			// }
			// System.out.println("AAaahh");
			// }

		}
		
		long pauseT = System.currentTimeMillis() - entranceTime;
		timeInit -= pauseT;
		timeTaken -= pauseT;
		
	}

	public void fillWithLipids() throws Exception {
		this.pause(true);
		if (stopAlgorithm) {
			return;
		}

		timeTaken = System.currentTimeMillis();
		timeInit = System.currentTimeMillis();

		String text = this.gui.controlPanel.seedField.getText();

		this.gui.controlPanel.seedField.setEnabled(false);

		text = text.trim();

		if (text.length() < 1) {
			random.setSeed(new SecureRandom().nextLong());
		} else {
			long seed = 0l;

			boolean toggl = false;
			for (int i = 0; i < text.length(); i++) {
				seed += (int) text.charAt(i);
				if (toggl)
					seed += ((int) text.charAt(i) - (int) text.charAt(i - 1))
							+ (1000 - i) * 3;
				toggl = !toggl;
			}

			random.setSeed(seed);
		}

		while (true) {

			this.pause(false);
			if (stopAlgorithm) {
				break;
			}
			myData.contents.stepsSinceLastCooling++;
			if (myData.contents.stepsSinceLastCooling > myData.settings.coolingIntervalLength) {
				myData.contents.stepsSinceLastCooling = 0;

				myData.settings.currentTemparature -= myData.settings.coolingRate;

				if (myData.settings.currentTemparature <= 0) {
					break;
				}

				gui.controlPanel.tfTemp.setText(""
						+ (myData.settings.currentTemparature));

			}

			String temp = "" + myData.settings.currentTemparature;
			if (temp.length() > 5)
				temp = temp.substring(0, 5);
			this.displayInformation("Current Temperature: " + temp);

			this.fillWithAnotherLipid();

			if (this.myData.contents.moleculesToTreat.length < 1)
				continue;

			int M = 0;

			if (myData.settings.moleculeMovement
					&& myData.settings.atomMovement) {
				int[] r = { 0, 1, 2 };
				M = r[random.nextInt(r.length)];
			} else if (myData.settings.moleculeMovement) {
				M = 0;
			} else {
				int[] r = { 1, 2 };
				M = r[random.nextInt(r.length)];
			}

			if (M == 0) {

				Vector<Molecule> shuffledList = new Vector<Molecule>();
				for (int i = 0; i < myData.contents.moleculesToTreat.length; i++) {
					shuffledList
							.addElement(myData.contents.moleculesToTreat[i]);
				}
				Collections.shuffle(shuffledList, random.getRandom());
				myData.contents.moleculesToTreat = shuffledList
						.toArray(new Molecule[0]);

				int Ml = (myData.contents.moleculesToTreat.length / 4)
						/ (random.nextInt(3) + 1);

				if (Ml < 1)
					Ml = 1;

				Ml = random.nextInt(Ml);

				double[] oldEnergies = new double[myData.contents.atomsToTreat.length];

				for (int i = 0; i < myData.contents.atomsToTreat.length; i++) {
					oldEnergies[i] = myData.contents.atomsToTreat[i]
							.getEnergy();
				}

				Point3f[] backupMoleculePositions = new Point3f[Ml];
				Matrix3f[] backupRotations = new Matrix3f[Ml];

				float tempSquared = myData.settings.currentTemparature
						* myData.settings.currentTemparature;

				boolean anyChanges = false;
				for (int i = 0; i < Ml; i++) {
					if (stopAlgorithm) {
						break;
					}
					Molecule current = myData.contents.moleculesToTreat[i];

					backupMoleculePositions[i] = current.getPosition();
					backupRotations[i] = this.copyMatrix(current
							.getRotationMatrix());

					this.moveMolecule(current);

					double oldEnergy = 0d;
					double newEnergy = 0d;

					for (int j = 0; j < current.getOriginalAtoms().length; j++) {
						oldEnergy += current.getOriginalAtoms()[j].getEnergy();
					}

					double lowest = Math.abs(energies
							.getLowestValuePossibleMultiplied());
					double oldTotalEnergy = oldEnergy
							+ ((double) current.getOriginalAtoms().length * lowest);

					double r = random.nextDouble();

					double[] newEnergies = new double[current
							.getOriginalAtoms().length];

					boolean changed = true;
					for (int j = 0; j < current.getOriginalAtoms().length; j++) {

						double ene;

						ene = energies
								.totalEnergy(myData.contents.atomsToTreat[i]);

						ene += energies
								.totalInternalEnergy(myData.contents.atomsToTreat[i]);

						newEnergies[j] = ene;

						newEnergy += ene;

						double en = newEnergy + ((double) (i + 1) * lowest);

						boolean acceptAnyways = this.stateOk(oldTotalEnergy,
								en, tempSquared, r,
								current.getOriginalAtoms().length);

						if (!acceptAnyways) {
							changed = false;
							break;
						}

					}

					if (!changed) {

						current.setPosition(backupMoleculePositions[i]);
						current.setRotationMatrix(backupRotations[i]);

					} else {

						anyChanges = true;

						for (int j = 0; j < current.getOriginalAtoms().length; j++) {
							current.getOriginalAtoms()[j]
									.setEnergy(newEnergies[j]);

						}

					}

				}

				if (anyChanges) {

					double[] newEnergies = new double[myData.contents.atomsToTreat.length];
					double lowest = Math.abs(energies
							.getLowestValuePossibleMultiplied());
					double totalEnergy = 0f;
					double oldTotalEnergy = myData.contents.currentFreeEnergy
							+ ((double) myData.contents.atomsToTreat.length * lowest);
					boolean change = true;
					double r = random.nextDouble();

					for (int i = 0; i < myData.contents.atomsToTreat.length; i++) {
						if (stopAlgorithm) {
							break;
						}
						double ene;

						ene = energies
								.totalEnergy(myData.contents.atomsToTreat[i]);

						ene += energies
								.totalInternalEnergy(myData.contents.atomsToTreat[i]);

						totalEnergy += ene;

						newEnergies[i] = ene;

						double en = totalEnergy + ((double) (i + 1) * lowest);
						boolean acceptAnyways = this.stateOk(oldTotalEnergy,
								en, tempSquared, r,
								myData.contents.atomsToTreat.length);

						if (!acceptAnyways) {
							change = false;
							break;
						}

					}

					if (change) {

						myData.contents.addCurrentEnergyToHistory(totalEnergy);

						for (int j = 0; j < myData.contents.atomsToTreat.length; j++) {

							myData.contents.atomsToTreat[j]
									.setEnergy(oldEnergies[j]);

						}

					} else {

						for (int i = 0; i < Ml; i++) {
							Molecule current = myData.contents.moleculesToTreat[i];
							current.setPosition(backupMoleculePositions[i]);
							current.setRotationMatrix(backupRotations[i]);

						}

						myData.contents
								.addCurrentEnergyToHistory(myData.contents.currentFreeEnergy);
					}

				} else {
					myData.contents
							.addCurrentEnergyToHistory(myData.contents.currentFreeEnergy);
				}

			} else if (M == 1) {

				Vector<Atom> shuffledList = new Vector<Atom>();
				for (int i = 0; i < myData.contents.atomsToTreat.length; i++) {
					shuffledList.addElement(myData.contents.atomsToTreat[i]);
				}
				Collections.shuffle(shuffledList, random.getRandom());
				myData.contents.atomsToTreat = shuffledList
						.toArray(new Atom[0]);

				Point3f[] backupAtomPositions = new Point3f[myData.contents.atomsToTreat.length];

				for (int i = 0; i < myData.contents.atomsToTreat.length; i++) {
					backupAtomPositions[i] = myData.contents.atomsToTreat[i]
							.getPosition();
				}

				int Al = myData.contents.atomsToTreat.length;

				Al = random.nextInt(Al);

				float tempSquared = myData.settings.currentTemparature
						* myData.settings.currentTemparature;

				boolean anyChanges = false;
				for (int i = 0; i < Al; i++) {
					if (stopAlgorithm) {
						break;
					}
					Atom current = myData.contents.atomsToTreat[i];

					PositionChange posChange = this.moveAtom(current, true);

					// this.forceConstraints(current);

					this.doBorderCutoff(posChange.effectedAtoms);

					if (stopAlgorithm) {
						break;
					}

					double en = 0;
					double oldEnergy = 0;
					double[] newEnergies = new double[posChange.effectedAtoms.length];
					boolean changed = true;
					double lowest = Math.abs(energies
							.getLowestValuePossibleMultiplied());

					double r = random.nextDouble();

					for (int j = 0; j < posChange.effectedAtoms.length; j++) {

						oldEnergy += posChange.effectedAtoms[j].getEnergy();
					}

					oldEnergy = oldEnergy
							+ ((double) posChange.effectedAtoms.length * lowest);

					for (int j = 0; j < posChange.effectedAtoms.length; j++) {

						double singleEn = energies
								.totalEnergy(posChange.effectedAtoms[j]);

						singleEn += energies
								.totalInternalEnergy(posChange.effectedAtoms[j]);

						newEnergies[j] = singleEn;
						en += singleEn;

						boolean acceptAnyways = this.stateOk(oldEnergy,
								(en + ((double) (j + 1) * lowest)),
								tempSquared, r, posChange.effectedAtoms.length);

						if (!acceptAnyways) {
							changed = false;
							break;
						}

					}

					if (changed) {
						for (int j = 0; j < posChange.effectedAtoms.length; j++) {

							posChange.effectedAtoms[j]
									.setEnergy(newEnergies[j]);
						}

						anyChanges = true;
					} else {
						for (int j = 0; j < posChange.effectedAtoms.length; j++) {

							posChange.effectedAtoms[j].position = posChange.oldOnes[j];
						}

					}
				}
				if (anyChanges) {

					double[] newEnergies = new double[myData.contents.atomsToTreat.length];
					double lowest = Math.abs(energies
							.getLowestValuePossibleMultiplied());
					double totalEnergy = 0f;
					double oldTotalEnergy = myData.contents.currentFreeEnergy
							+ ((double) myData.contents.atomsToTreat.length * lowest);
					boolean change = true;
					double r = random.nextDouble();

					for (int i = 0; i < myData.contents.atomsToTreat.length; i++) {
						if (stopAlgorithm) {
							break;
						}
						double ene;

						ene = energies
								.totalEnergy(myData.contents.atomsToTreat[i]);

						ene += energies
								.totalInternalEnergy(myData.contents.atomsToTreat[i]);

						totalEnergy += ene;

						newEnergies[i] = ene;

						double en = totalEnergy + ((double) (i + 1) * lowest);
						boolean acceptAnyways = this.stateOk(oldTotalEnergy,
								en, tempSquared, r,
								myData.contents.atomsToTreat.length);

						if (!acceptAnyways) {
							change = false;
							break;
						}

					}

					if (!change) {

						if (myData.settings.atomMovement) {
							for (int i = 0; i < myData.contents.atomsToTreat.length; i++) {
								Atom current = myData.contents.atomsToTreat[i];
								current.position = backupAtomPositions[i];

							}
						}
						myData.contents
								.addCurrentEnergyToHistory(myData.contents.currentFreeEnergy);

					} else {

						myData.contents.addCurrentEnergyToHistory(totalEnergy);

						for (int j = 0; j < myData.contents.atomsToTreat.length; j++) {

							myData.contents.atomsToTreat[j]
									.setEnergy(newEnergies[j]);

						}

						for (int j = 0; j < myData.contents.atomsToTreat.length; j++) {

							myData.contents.atomsToTreat[j]
									.setPosition(myData.contents.atomsToTreat[j]
											.getPosition());
							myData.contents.atomsToTreat[j].position = myData.contents.atomsToTreat[j]
									.getPosition();

						}

						if (this.isInSystemTray()) {

							for (int i = 0; i < myData.contents.moleculesToTreat.length; i++) {

								myData.contents.moleculesToTreat[i]
										.updateGeometry();
								this.shift(myData.contents.moleculesToTreat[i]);
								if (stopAlgorithm) {
									break;
								}
							}

						}

					}
				} else {
					myData.contents
							.addCurrentEnergyToHistory(myData.contents.currentFreeEnergy);
				}

			} else if (M == 2) {

				Vector<Atom> shuffledList = new Vector<Atom>();
				for (int i = 0; i < myData.contents.atomsToTreat.length; i++) {
					shuffledList.addElement(myData.contents.atomsToTreat[i]);
				}
				Collections.shuffle(shuffledList, random.getRandom());
				myData.contents.atomsToTreat = shuffledList
						.toArray(new Atom[0]);

				Point3f[] backupAtomPositions = new Point3f[myData.contents.atomsToTreat.length];

				for (int i = 0; i < myData.contents.atomsToTreat.length; i++) {
					backupAtomPositions[i] = myData.contents.atomsToTreat[i]
							.getPosition();
				}

				int Al = (myData.contents.atomsToTreat.length / 6)
						/ (random.nextInt(5) + 1);
				if (Al < 1)
					Al = 1;
				Al = random.nextInt(Al);

				for (int i = 0; i < Al; i++) {
					if (stopAlgorithm) {
						break;
					}
					Atom current = myData.contents.atomsToTreat[i];

					PositionChange posChange = this.moveAtom(current, false);

					// this.forceConstraints(current);

					this.doBorderCutoff(posChange.effectedAtoms);

					if (stopAlgorithm) {
						break;
					}

				}

				double[] newEnergies = new double[myData.contents.atomsToTreat.length];
				double lowest = Math.abs(energies
						.getLowestValuePossibleMultiplied());
				double totalEnergy = 0f;
				double oldTotalEnergy = myData.contents.currentFreeEnergy
						+ ((double) myData.contents.atomsToTreat.length * lowest);
				boolean change = true;
				double r = random.nextDouble();
				float tempSquared = myData.settings.currentTemparature
						* myData.settings.currentTemparature;

				for (int i = 0; i < myData.contents.atomsToTreat.length; i++) {
					if (stopAlgorithm) {
						break;
					}
					double ene;

					ene = energies.totalEnergy(myData.contents.atomsToTreat[i]);

					ene += energies
							.totalInternalEnergy(myData.contents.atomsToTreat[i]);

					totalEnergy += ene;

					newEnergies[i] = ene;

					double en = totalEnergy + ((double) (i + 1) * lowest);
					boolean acceptAnyways = this
							.stateOk(oldTotalEnergy, en, tempSquared, r,
									myData.contents.atomsToTreat.length);

					if (!acceptAnyways) {
						change = false;
						break;
					}

				}

				if (!change) {

					for (int i = 0; i < myData.contents.atomsToTreat.length; i++) {
						Atom current = myData.contents.atomsToTreat[i];
						current.position = backupAtomPositions[i];

					}

					myData.contents
							.addCurrentEnergyToHistory(myData.contents.currentFreeEnergy);

				} else {

					myData.contents.addCurrentEnergyToHistory(totalEnergy);

					for (int j = 0; j < myData.contents.atomsToTreat.length; j++) {

						myData.contents.atomsToTreat[j]
								.setEnergy(newEnergies[j]);

					}

					for (int j = 0; j < myData.contents.atomsToTreat.length; j++) {

						myData.contents.atomsToTreat[j]
								.setPosition(myData.contents.atomsToTreat[j]
										.getPosition());
						myData.contents.atomsToTreat[j].position = myData.contents.atomsToTreat[j]
								.getPosition();

					}

					if (this.isInSystemTray()) {

						for (int i = 0; i < myData.contents.moleculesToTreat.length; i++) {

							myData.contents.moleculesToTreat[i]
									.updateGeometry();
							this.shift(myData.contents.moleculesToTreat[i]);
							if (stopAlgorithm) {
								break;
							}
						}

					}

				}

			}

			if (this.gui.guiMode == 2) {
				this.gui.graphPanel.setValues(myData.contents
						.getEnergyHistory(), myData.contents.step);
			}
			timeTaken = System.currentTimeMillis();
		}

		// END OF THE LOOP

		for (int i = 0; i < this.myData.contents.atomsToTreat.length; i++) {
			this.myData.contents.atomsToTreat[i]
					.setPosition(this.myData.contents.atomsToTreat[i].position);
		}

		for (int i = 0; i < myData.contents.moleculesToTreat.length; i++) {

			myData.contents.moleculesToTreat[i].updateGeometry();
			this.shift(myData.contents.moleculesToTreat[i]);
		}
		gui.paraPanel.storeValues();
		gui.controlPanel.storeSettings();

		gui.dialog.dispose();

	}

	private void shift(Molecule molecule) {
		Dimension box = this.getMembraneSize();
		if (myData.settings.isBoxContinuous) {

			Vector3f add = new Vector3f();
			boolean overBorder = false;
			if (molecule.getPosition().x > box.width) {
				add.add(new Vector3f(-box.width, 0, 0));
				overBorder = true;
			} else if (molecule.getPosition().x < 0) {
				add.add(new Vector3f(box.width, 0, 0));
				overBorder = true;
			}

			if (molecule.getPosition().z > box.height) {
				add.add(new Vector3f(0, 0, -box.height));
				overBorder = true;
			} else if (molecule.getPosition().z < 0) {
				add.add(new Vector3f(0, 0, box.height));
				overBorder = true;
			}

			if (overBorder) {

				Point3f p = molecule.getPosition();
				p.add(add);
				molecule.setPosition(p);

			}
		}

	}

	
	private boolean stateOk(double oldState, double newState,
			float tempSquared, double r, int loopedAtoms) {

		if (newState < oldState)
			return true;

		if (tempSquared <= 0f)
			return false;

		double diff = oldState - newState;
		diff /= loopedAtoms;

		
		if (r < Math.exp(diff / tempSquared))
			return true;
		else
			return false;

	}

	private void moveMolecule(Molecule current) {

		Point3f oldPosition = current.getPosition();

		float moveX = (this.myData.settings.currentMovingFactor * random
				.nextFloat());
		if (random.nextBoolean())
			moveX = -moveX;

		float moveY = (this.myData.settings.currentMovingFactor * random
				.nextFloat());
		if (random.nextBoolean())
			moveY = -moveY;

		float moveZ = (this.myData.settings.currentMovingFactor * random
				.nextFloat());
		if (random.nextBoolean())
			moveZ = -moveZ;

		oldPosition.add(new Vector3f(moveX, moveY, moveZ));

		if (myData.settings.pressureXZ != 0 || myData.settings.pressureY != 0) {

			Vector3f dest;
			if (myData.settings.AllLayers) {
				dest = new Vector3f(bounds.center);
				dest.y = this.getLayerByLayerID(current.getLayerID())
						.getYHeight();
			} else {
				dest = new Vector3f(bounds.center);
			}

			dest.sub(current.getPosition());

			dest.normalize();
			double r = random.nextDouble();
			dest.x *= (myData.settings.pressureXZ * r);
			dest.z *= (myData.settings.pressureXZ * r);
			dest.y *= (myData.settings.pressureY * r);

			oldPosition.add(dest);

		}

		Matrix3f newMatrix = this.copyMatrix(current.getRotationMatrix());

		current.rotX(this.myData.settings.currentMovingFactor
				* random.nextFloat() * (random.nextBoolean() ? 1 : -1) * 3);

		current.rotZ(this.myData.settings.currentMovingFactor
				* random.nextFloat() * (random.nextBoolean() ? 1 : -1) * 3);

		if (random.nextBoolean()) {

			current.rotY(random.nextInt(360) + random.nextFloat());

		}

		Point3f old1 = current.getPosition();
		Matrix3f old2 = this.copyMatrix(current.getRotationMatrix());

		current.setPosition(oldPosition);
		current.setRotationMatrix(newMatrix);

		if (myData.settings.solidBorders) {

			if (this.intersectsBorders(current)) {

				current.setPosition(old1);
				current.setRotationMatrix(old2);
			}
		}

	}

	private PositionChange moveAtom(Atom current, boolean advanced) {

		boolean translate = false;
		Vector<Atom> effectedAtoms = new Vector<Atom>();

		if (random.nextInt(10) < (advanced ? 6 : 8)) {
			translate = true;
			effectedAtoms.addElement(current);
		} else {

			if (current.bonds.length < 1)
				translate = true;

			if (current.bonds.length == 1) {

				if (random.nextBoolean())
					current = current.bonds[0].neigbor;

			}
			if (current.bonds.length < 2) {
				translate = true;

			} else
				translate = random.nextBoolean();

			effectedAtoms.addElement(current);

			if (translate) {
				for (int i = 0; i < current.bonds.length; i++) {
					Atom other = current.bonds[i].neigbor;

					if (other.bonds.length < 2)
						effectedAtoms.addElement(other);

				}
			} else {
				for (int i = 0; i < current.bonds.length; i++) {
					Atom other = current.bonds[i].neigbor;

					this.fillOthers(current, other, effectedAtoms);

				}
			}

			if (effectedAtoms.size() < 2) {

				translate = true;
			}

		}

		PositionChange posChange = new PositionChange();
		posChange.effectedAtoms = effectedAtoms.toArray(new Atom[0]);
		posChange.oldOnes = new Point3f[posChange.effectedAtoms.length];

		for (int i = 0; i < posChange.effectedAtoms.length; i++) {
			posChange.oldOnes[i] = posChange.effectedAtoms[i].getPosition();
		}

		if (translate) {

			float moveX = (this.myData.settings.currentMovingFactor * random
					.nextFloat());
			if (random.nextBoolean())
				moveX = -moveX;

			float moveY = (this.myData.settings.currentMovingFactor * random
					.nextFloat());
			if (random.nextBoolean())
				moveY = -moveY;

			float moveZ = (this.myData.settings.currentMovingFactor * random
					.nextFloat());
			if (random.nextBoolean())
				moveZ = -moveZ;

			Vector3f move = new Vector3f(moveX, moveY, moveZ);

			if (myData.settings.pressureXZ != 0
					|| myData.settings.pressureY != 0) {

				Vector3f dest;
				if (myData.settings.AllLayers) {
					dest = new Vector3f(bounds.center);
					dest.y = this.getLayerByLayerID(
							current.getParentalMolecule().getLayerID())
							.getYHeight();
				} else {
					dest = new Vector3f(bounds.center);
				}

				dest.sub(current.position);

				dest.normalize();
				double r = random.nextDouble();
				dest.x *= (myData.settings.pressureXZ * r);
				dest.z *= (myData.settings.pressureXZ * r);
				dest.y *= (myData.settings.pressureY * r);

				move.add(dest);

			}

			for (int i = 0; i < posChange.effectedAtoms.length; i++) {
				posChange.effectedAtoms[i].position.add(move);
			}

		} else {

			Matrix3f matrix = RotationToolkit.createAUnitMatrix();
			int max = 3;

			RotationToolkit.rotate_X_Axis_Matrix(matrix,
					(random.nextInt(max) + random.nextFloat())
							* (random.nextBoolean() ? 1 : -1));
			RotationToolkit.rotate_Y_Axis_Matrix(matrix,
					(random.nextInt(max) + random.nextFloat())
							* (random.nextBoolean() ? 1 : -1));
			RotationToolkit.rotate_Z_Axis_Matrix(matrix,
					(random.nextInt(max) + random.nextFloat())
							* (random.nextBoolean() ? 1 : -1));

			for (int i = 0; i < posChange.effectedAtoms.length; i++) {
				if (posChange.effectedAtoms[i] == current)
					continue;

				posChange.effectedAtoms[i].position.sub(current.position);
				posChange.effectedAtoms[i].position = RotationToolkit
						.transformToNewPoint(matrix,
								posChange.effectedAtoms[i].position);
				posChange.effectedAtoms[i].position.add(current.position);

			}

		}

		// for(int i=0;i<effectedAtoms.size();i++) {
		// System.out.println(effectedAtoms.get(i).getAtomName());
		//			
		// }
		// System.out.println("");

		return posChange;
	}

	private boolean fillOthers(Atom first, Atom second,
			Vector<Atom> effectedAtoms) {

		if (second.bonds.length < 2) {
			effectedAtoms.addElement(second);
			return true;
		} else if (second.bonds.length == 2) {
			for (int i = 0; i < 2; i++) {
				if (second.bonds[i].neigbor == first)
					continue;
				else if (fillOthers(second, second.bonds[i].neigbor,
						effectedAtoms)) {
					effectedAtoms.addElement(second);
					return true;
				} else
					return false;
			}
		}

		return false;

	}

	// public void forceConstraints(Atom current) {
	//
	//		
	//		
	// // if (myData.settings.constraintFactor > 0) {
	// //
	// // Point3f newP = current.getPosition();
	// //
	// // for (int i = 0; i < current.bonds.length; i++) {
	// //
	// //// if(hasHydrogens)
	// //// if(current.bonds[i].neigbor.getElement().equals("H"))continue;
	// //
	// // Vector3f direction = current.bonds[i].getDirection();
	// //
	// // float origLength = current.bonds[i].getOriginalLength();
	// //
	// // // Atom n = current.bonds[i].neigbor;
	// //
	// // // Point3f oldPosition = n.getPosition();
	// //
	// // float length = direction.length();
	// // float diff = length - origLength;
	// //
	// // direction.normalize();
	// //
	// // diff *= myData.settings.constraintFactor;
	// //
	// // if (myData.settings.constraintFactor != 1) {
	// // diff *= ((random.nextFloat() / 2d) + 0.5f);
	// // }
	// //
	// // direction.scale(diff);
	// //
	// // // oldPosition.add(direction);
	// //
	// // newP.add(direction);
	// // if (newP.x == Float.NaN || newP.y == Float.NaN
	// // || newP.z == Float.NaN)
	// // continue;
	// //
	// // current.position = newP;
	// //
	// // }
	// //
	// // }
	//
	// Atom[] others = current.getParentalMolecule().getOriginalAtoms();
	//
	// for (int j = 0; j < others.length; j++) {
	// if (others[j] == current)
	// continue;
	//
	// boolean isBonded = false;
	// for (int k = 0; k < current.bonds.length; k++) {
	// if (current.bonds[k].neigbor == others[j]) {
	// isBonded = true;
	// break;
	// }
	// }
	// if (isBonded)
	// continue;
	//
	// float distance = others[j].position.distance(current.position);
	//
	// if (current.isCoarseGrainParticle()
	// || others[j].isCoarseGrainParticle()) {
	// float reqDist = 2;
	//
	// Vector3f dir = new Vector3f(current.position);
	// dir.sub(others[j].position);
	//
	// dir.normalize();
	//
	// float diff = reqDist - distance;
	// diff *= ((random.nextFloat() / 2d));
	//
	// dir.scale(diff);
	//
	// current.position.add(dir);
	//
	// break;
	// } else {
	// float reqDist = (current.getCovalentRadiusInAngstrom() + others[j]
	// .getCovalentRadiusInAngstrom()) * 1.3f;
	// if (distance < reqDist) {
	//
	// Vector3f dir = new Vector3f(current.position);
	// dir.sub(others[j].position);
	//
	// dir.normalize();
	//
	// float diff = reqDist - distance;
	// diff *= ((random.nextFloat() / 2d));
	//
	// dir.scale(diff);
	//
	// current.position.add(dir);
	//
	// break;
	// }
	// }
	//
	// }
	//
	// }

	public void doBorderCutoff(Atom[] effectedAtoms) {
		if (!myData.settings.solidBorders)
			return;

		for (int i = 0; i < effectedAtoms.length; i++) {
			if (effectedAtoms[i].position.x > this.bounds.upper.x) {
				effectedAtoms[i].position.x = this.bounds.upper.x;
			} else if (effectedAtoms[i].position.x < 0) {
				effectedAtoms[i].position.x = 0;
			}

			if (effectedAtoms[i].position.z > this.bounds.upper.z) {
				effectedAtoms[i].position.z = this.bounds.upper.z;
			} else if (effectedAtoms[i].position.z < 0) {
				effectedAtoms[i].position.z = 0;
			}

			if (effectedAtoms[i].position.y > this.bounds.upper.y) {
				effectedAtoms[i].position.y = this.bounds.upper.y;
			} else if (effectedAtoms[i].position.y < this.bounds.lower.y) {
				effectedAtoms[i].position.y = this.bounds.lower.y;
			}
		}
	}

	public String getFullInformation() {

		final String s = "\nA <b>Simulated Annealing</b> algorithm that operates\n"
				+ "explicitly at atom level. For energy evaluation,\n "
				+ "choose between the Lennard Jones 12-6 \n" 
				+ "potential or between a simpler method that\n"
				+ "calculates average distances only.\n\n"
				
				+ "Choosing the LJ potential requires you to specify\n"
				+ "the two functional parameters, \u03b5 and \u03c3 for \n"
				+ "each element or Coarse Grain Particle well.\n"
				+ "(The most common elements already have \n"
				+ "default values).\n\n"
				
				+ "The runtime of this depends critically on the  \n "
				+ "setup values and the number of atoms in the\n"
				+ "membrane. Generating an acceptable result,\n"
				+ "however, will take some time.\n\n"

				+ "Important Note: This algorithm cannot substitute\n"
				+ "a molecular dynamic simulation with Gromacs, \n"
				+ "CHARMM, or similar packages. This algorithm\n"
				+ "was written to test the single atom handling\n"
				+ "of the MembraneEditor. Therefore, it is\n"
				+ "experimental.\n\n\n"
				
				+ "Functioning: State change through random\n"
				+ "atom movement, followed by the principles of\n"
				+ "Simulated Annealing\n\n"
				+ "Orientation: Atom Level\n"
				+ "Speed: Average-Slow\n"
				+ "Biological correctness: Moderate\n\n" 
				+ "Author: Tim Dingersen\n"
				+ "Version: 1.01";
		return "<html>" + s.replaceAll("\n", "<br>") + "</html>";
	}

	public String getName() {

		return "Atom Level Minimizer";
	}

	public Component getRuntimeControlPanel() {

		return gui;
	}

	public String getShortInformation() {

		return "Atom Level Simulated Annealing Algorithm (Version 1.01)";
	}

	public String getStringRepresentationOfSetupParameters() {

		return null;
	}

	public void stopAlgorithm() {
		this.pauseAlgorithm = false;
		this.stopAlgorithm = true;
		
		this.setStartingTime(this.timeTaken==0?System.currentTimeMillis():this.timeTaken);
		
	}

	public boolean overrideExistingLipids() {
		return false;
	}

	public boolean supportsAbsoluteValues() {
		return false;
	}

	public boolean supportsMicrodomains() {
		return true;
	}

	public void fillWithAnotherLipid() {

		int attempts = myData.settings.addNewLipidsFactor;
		if (attempts < 1)
			return;

		for (int i = 0; i < this.myData.contents.atomsToTreat.length; i++) {
			this.myData.contents.atomsToTreat[i]
					.setPosition(this.myData.contents.atomsToTreat[i].position);
			if (this.stopAlgorithm || this.pauseAlgorithm) {
				return;
			}
		}

		boolean success = false;

		for (int i = 0; i < attempts; i++) {
			boolean toggle = random.nextBoolean();

			if (this.stopAlgorithm || this.pauseAlgorithm) {
				return;
			}

			Lipid newOne;

			Point2f p = this.createRandomXZPosition(new Rectangle(0, 0, this
					.getMembraneSize().width, this.getMembraneSize().height));

			int layerID;
			if (myData.settings.AllLayers) {
				layerID = this.createRandomLayerID();
			} else {
				layerID = this.getCurrentLayerID();
			}

			int domainID = this.getMicroDomainAtPoint(p, layerID);

			if (this.area == -1) {

				if (domainID == 0) {
					newOne = toggle ? this
							.createNextExtracellularLipid(layerID) : this
							.createNextIntracellularLipid(layerID);
				} else {
					MicroDomain micro = this.getMicroDomain(domainID);
					newOne = toggle ? micro.createNextExtracellularLipid()
							: micro.createNextIntracellularLipid();
				}

			} else {
				if (domainID == 0 && this.area == 0) {
					newOne = toggle ? this
							.createNextExtracellularLipid(layerID) : this
							.createNextIntracellularLipid(layerID);
				} else if (this.area > 0 && domainID == this.area) {
					MicroDomain micro = this.getMicroDomain(domainID);
					newOne = toggle ? micro.createNextExtracellularLipid()
							: micro.createNextIntracellularLipid();
				} else {
					return;
				}
			}

			newOne.rotY(random.nextInt(360));

			float max = 0f;

			for (int a = 0; a < newOne.getOriginalAtoms().length; a++) {
				max = Math.max(newOne.getOriginalAtoms()[a]
						.getVanDerWaalsRadiusInAngstrom(), max);
			}

			if (newOne.getMembraneSide() == EXTRA)
				newOne.setPosition(new Point3f(p.x, newOne.getPosition().y
						+ max, p.y));
			else
				newOne.setPosition(new Point3f(p.x, newOne.getPosition().y
						- max, p.y));

			this.moveMolecule(newOne);

			int limit = newOne.getOriginalAtoms().length / 3;
			int intersections = this.getIntersections(newOne, limit);
			if (intersections > limit) {
				continue;
			} else if (intersections == 0) {
				success = true;
				try {
					if (newOne.getMembraneSide() == EXTRA)
						this.addExtracellularLipid(newOne);
					else
						this.addIntracellularLipid(newOne);
				} catch (IllegalOperationException e) {

				}
				myData.contents.setRelevantMolecules(this
						.getAllMoleculesInCurrentModel(), this
						.getCurrentLayerID(), area, this);
			} else {

				int lowest = intersections;
				for (int a = 0; a < limit; a++) {

					Matrix3f oldM = RotationToolkit.copyMatrix(newOne
							.getRotationMatrix());
					Point3f oldP = newOne.getPosition();

					this.moveMolecule(newOne);

					int inter = this.getIntersections(newOne, limit);

					if (inter > lowest) {
						newOne.setPosition(oldP);
						newOne.setRotationMatrix(oldM);
						continue;
					} else {
						lowest = inter;
						a = 0;
					}

					if (lowest == 0) {
						success = true;
						try {
							if (newOne.getMembraneSide() == EXTRA)
								this.addExtracellularLipid(newOne);
							else
								this.addIntracellularLipid(newOne);
						} catch (IllegalOperationException e) {

						}
						myData.contents.setRelevantMolecules(this
								.getAllMoleculesInCurrentModel(), this
								.getCurrentLayerID(), area, this);
						break;
					}
				}
			}

		}

		if (success) {
			//			

			double total = 0d;

			for (int j = 0; j < myData.contents.atomsToTreat.length; j++) {
				if (stopAlgorithm) {
					break;
				}
				double ene = energies
						.totalEnergy(myData.contents.atomsToTreat[j]);
				double internal = energies
						.totalInternalEnergy(myData.contents.atomsToTreat[j]);

				total += (ene + internal);

				myData.contents.atomsToTreat[j].setEnergy(ene);

			}

			myData.contents.currentFreeEnergy = total;

			if (this.gui.guiMode == 2) {
				this.gui.graphPanel.setValues(myData.contents
						.getEnergyHistory(), myData.contents.step);
			}

		}

	}

	private int getIntersections(Lipid newOne, int limit) {

		// if (myData.contents.moleculesToTreat.length < 5)
		// return 0;
		// if(myData.contents.currentFreeEnergy>0&&myData.contents.currentFreeEnergy>initialValue)return
		// true;

		if (myData.settings.solidBorders) {
			if (this.intersectsBorders(newOne))
				return newOne.getOriginalAtoms().length;
		}
		if (this.intersectsWithProteins(newOne))
			return newOne.getOriginalAtoms().length;

		Dimension box = this.getMembraneSize();
		int intersections = 0;
		for (int a = 0; a < newOne.getOriginalAtoms().length; a++) {
			Atom current = newOne.getOriginalAtoms()[a];

			Point3f second = null;
			Point3f third = null;

			if (myData.settings.isBoxContinuous) {

				Vector3f add = new Vector3f();
				boolean overBorder = false;
				if ((current.position.x + 10) > box.width) {
					add.add(new Vector3f(-box.width, 0, 0));
					overBorder = true;
				}
				if ((current.position.z + 10) > box.height) {
					add.add(new Vector3f(0, 0, -box.height));
					overBorder = true;
				}

				if (overBorder) {

					second = new Point3f(current.position);
					second.add(add);

				}

				add = new Vector3f();
				overBorder = false;
				if ((current.position.x - 10) < 0) {
					add.add(new Vector3f(box.width, 0, 0));
					overBorder = true;
				}
				if ((current.position.z - 10) < 0) {
					add.add(new Vector3f(0, 0, box.height));
					overBorder = true;
				}

				if (overBorder) {

					third = new Point3f(current.position);
					third.add(add);

				}

			}

			for (int i = 0; i < myData.contents.atomsToCompare.length; i++) {

				Atom b = myData.contents.atomsToCompare[i];

				if (b.getParentalMolecule() == current.getParentalMolecule())
					continue;

				if (b.position.x < current.position.x - 10)
					continue;
				if (b.position.x > current.position.x + 10)
					continue;
				if (b.position.z < current.position.z - 10)
					continue;
				if (b.position.z > current.position.z + 10)
					continue;
				if (b.position.y < current.position.y - 10)
					continue;
				if (b.position.y > current.position.y + 10)
					continue;

				float distance = current.position.distance(b.position);
				float r0;

				// r0 = energies.masterSigma;
				// try {
				// r0=(energies.hashSigma.get(current.getParameterKey()).get(b.getParameterKey())).floatValue();
				// }catch(Exception e) {
				//							
				// }

				r0 = current.getVanDerWaalsRadiusInAngstrom()
						+ b.getVanDerWaalsRadiusInAngstrom();

				// r0-=(r0*(0.01f*myData.settings.currentTemparature));
				r0 += (((r0 * 1.1224f) - r0) / 2f);

				if (distance < r0)
					intersections++;
				if (intersections > limit)
					return newOne.getOriginalAtoms().length;

				if (second != null) {

					if (b.position.x < second.x - 10)
						continue;
					if (b.position.x > second.x + 10)
						continue;
					if (b.position.z < second.z - 10)
						continue;
					if (b.position.z > second.z + 10)
						continue;
					if (b.position.y < second.y - 10)
						continue;
					if (b.position.y > second.y + 10)
						continue;

					distance = b.position.distance(second);
					if (distance < r0)
						intersections++;
					if (intersections > limit)
						return newOne.getOriginalAtoms().length;
				}
				if (third != null) {

					if (b.position.x < third.x - 10)
						continue;
					if (b.position.x > third.x + 10)
						continue;
					if (b.position.z < third.z - 10)
						continue;
					if (b.position.z > third.z + 10)
						continue;
					if (b.position.y < third.y - 10)
						continue;
					if (b.position.y > third.y + 10)
						continue;

					distance = b.position.distance(third);
					if (distance < r0)
						intersections++;
					if (intersections > limit)
						return newOne.getOriginalAtoms().length;
				}
			}

		}

		return intersections;
	}

	private class PositionChange {

		Atom[] effectedAtoms;
		Point3f[] oldOnes;

	}
}
