/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_1)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.membranepackingalgorithm;

import java.awt.Dimension;
import java.awt.Point;
import java.awt.Rectangle;
import java.io.File;
import java.util.Hashtable;
import java.util.Vector;

import javax.media.j3d.BranchGroup;
import javax.vecmath.Matrix3f;
import javax.vecmath.Point2f;
import javax.vecmath.Point3f;
import javax.vecmath.Vector2f;

import org.cellmicrocosmos.cm2.coarsegrainsupport.MoleculeProfile;
import org.cellmicrocosmos.cm2.coarsegrainsupport.Molecule_Profile_IO;
import org.cellmicrocosmos.cm2.constants.CM2_CONSTANTS;
import org.cellmicrocosmos.cm2.dataobjects.Atom_3D;
import org.cellmicrocosmos.cm2.dataobjects.Constraint_3D;
import org.cellmicrocosmos.cm2.dataobjects.Layer;
import org.cellmicrocosmos.cm2.dataobjects.Lipid_3D;
import org.cellmicrocosmos.cm2.dataobjects.MembraneModel;
import org.cellmicrocosmos.cm2.dataobjects.Molecule_3D;
import org.cellmicrocosmos.cm2.dataobjects.MyBounds;
import org.cellmicrocosmos.cm2.dataobjects.Particle;
import org.cellmicrocosmos.cm2.dataobjects.Protein_3D;
import org.cellmicrocosmos.cm2.engine.DataPool;
import org.cellmicrocosmos.cm2.functionality3d.IntersectManager;
import org.cellmicrocosmos.cm2.interfaces.DatabaseViewListener;
import org.cellmicrocosmos.cm2.interfaces.GuiListener;
import org.cellmicrocosmos.cm2.interfaces.MembraneViewListener;
import org.cellmicrocosmos.cm2.misc.Cm2_Toolkit;
import org.cellmicrocosmos.cm2.pluginmanager.CalculationManager;


/**
 * This is the required superclass for all Membrane Algorithms that can be used
 * by the membrane editor!
 * 
 * Membrane calculating algorithms need to extend this class and to implement
 * all of the methods from the MembranePackingAlgorithmInterface which this class
 * implements.
 * 
 * This class provides the user with various methods required to handle
 * interactions between the membrane calculating algorithms and the
 * application's internal data structure. Many methods appear in a wide range of
 * variations to give the user the comfort to get the information he needs as
 * easy as possible.
 * 
 * All euklidic values that an algorithm developer can access or manipulate are
 * measured in angstrom!
 * 
 * To compile the package from the directory where the java and class files are 
 * located at: 
 * 
 * jar.exe cvf name_of_membranepackingalgorithm.jar name_of_membranepackingalgorithm*
 * 
 * @author Tim Dingersen, Sebastian Schneider, Sebastian Rubert, Bjoern Sommer bjoern@CELLmicrocosmos.org
 * 
 */
public abstract class MembranePackingAlgorithm implements MembranePackingAlgorithmInterface {

	public MembraneViewListener membraneViewListener;
	
	public GuiListener basicOperations;
	
	public DatabaseViewListener databaseOperations;

	private final IntersectManager IM = new IntersectManager();

	public Vector<MicroDomain> microDomains = new Vector<MicroDomain>();
	
	/**
	 * The random that may be used
	 */
	public SimpleRandom random = DataPool.AlgorithmSettings.uniqueRandom?
									new UniqueRandom(DataPool.AlgorithmSettings.uniqueRandomSeed):
									new SimpleRandom();

	/**
	 * Used to specify the extracellular membrane side
	 */
	public final static int EXTRA = 0;

	/**
	 * Used to specify the intracellular membrane side
	 */
	public final static int INTRA = 1;
	
	
	/**
	 * Used to specify the PDB type for the database.
	 */
	public final static int PROTEINTYPE = 0;
	public final static int LIPIDTYPE = 1;

	public Lipid_3D[] lipidsEC;
	public Lipid_3D[] lipidsIC;

	public CalculationManager calculationManager;

	private boolean requestRenderModeChange = false;

	private Hashtable<String, MoleculeProfile> profiles = new Hashtable<String, MoleculeProfile>();

	public int CurrentMicroDomain = -1;
	
	private long startingTime = -1;
	private long terminationTime = -1;
	
	private boolean startInBackgroundAllowed = true;
	private boolean startInBackgroundSelected = false;
	
	public static final int BORDER_NORTH = 0;

	public static final int BORDER_EAST = 1;

	public static final int BORDER_SOUTH = 2;

	public static final int BORDER_WEST = 3;

	public static final int BORDER_MAX_Y = 4;

	public static final int BORDER_MIN_Y = 5;

	/**
	 * NOT MEANT TO BE USED BY PLUGIN DEVELOPERS!
	 * ! Please use the following method because this method will be removed in the future !
	 */
	@Deprecated
	public final void setListenerAndPrepare(
			final MembraneViewListener membraneViewListener,
			final GuiListener basicOperations,
			final CalculationManager calculationManager) {
		this.membraneViewListener = membraneViewListener;
		this.basicOperations = basicOperations;
		this.calculationManager = calculationManager;
		this.lipidsEC = DataPool.getMembraneModel()
				.getAllExampleLipidsExtraInDomain(0, -1);
		this.lipidsIC = DataPool.getMembraneModel()
				.getAllExampleLipidsIntraInDomain(0, -1);
	}

	/**
	 * NOT MEANT TO BE USED BY PLUGIN DEVELOPERS!
	 */
	public final void setListenerAndPrepare(
			final MembraneViewListener membraneViewListener,
			final GuiListener basicOperations,
			final DatabaseViewListener databaseOperations,
			final CalculationManager calculationManager) {
		this.membraneViewListener = membraneViewListener;
		this.basicOperations = basicOperations;
		this.databaseOperations = databaseOperations;
		this.calculationManager = calculationManager;
		this.lipidsEC = DataPool.getMembraneModel()
				.getAllExampleLipidsExtraInDomain(0, -1);
		this.lipidsIC = DataPool.getMembraneModel()
				.getAllExampleLipidsIntraInDomain(0, -1);
	}
	
	/**
	 * Use this method to inherit listeners from one algorithm to the next one.
	 * In addition, the lipid 3d models are copied into the new algorithm.
	 * Use the method the following way:
	 *  child_algorithm.setListenerAndPrepare(this);
	 * where this is the parent_algorithm
	 */
	public final void setListenerAndPrepare(MembranePackingAlgorithm algorithmToGetListeners) {
		this.membraneViewListener = algorithmToGetListeners.membraneViewListener;
		this.basicOperations = algorithmToGetListeners.basicOperations;
		this.databaseOperations = algorithmToGetListeners.databaseOperations; 
		this.calculationManager = algorithmToGetListeners.calculationManager;
		this.lipidsEC = algorithmToGetListeners.lipidsEC;
		this.lipidsIC = algorithmToGetListeners.lipidsIC;
		this.lipidsEC = DataPool.getMembraneModel()
				.getAllExampleLipidsExtraInDomain(0, -1);
		this.lipidsIC = DataPool.getMembraneModel()
				.getAllExampleLipidsIntraInDomain(0, -1);
	}
	
	/**
	 * Use this method to change the Membrane Editor distribution mode to
	 * - support absolute values if true
	 * - support percental values if false
	 * 
	 * @param computeAbsoluteValues
	 *            boolean
	 */
	public void setComputeAbsoluteValues(boolean computeAbsoluteValues){
		this.basicOperations.setComputeAbsoluteValues(computeAbsoluteValues);
	}
	
	/**
	 * Set the percentages of the ComponentView.

	 * @param mode
	 *            1=EC, 2=IC
	 * @param newval
	 *            the new percentage to accord to
	 * @param code
	 *            the code for the type of the regarded lipid
	 * @param domainID
	 *            the domainID of the value to change
	 */
	public void setPercentages(int mode, int newval, String code, int domainID) {
		this.basicOperations.setPercentages(mode, newval, code, domainID);
	}
	
	/**
	 * Get the percentages of the defined layer and domainID on the extracellular side.
	 * 
	 * Note: If isAbsoluteValuesSelected() is false (default mode of the MembraneEditor), then 
	 *       percentages are used. In this case, use getDecimalPlaces() to get the precision
	 *       of the percentages.
	 * 
	 * @param layerID 
	 *                  the ID of the layer
	 * @param domainID
	 *                  the ID of the domain
	 * @return Integer[]
	 *                  an array containing all percentages of the defined layer and domainID
	 */
	public Integer[] getPercentages_ExtraCellular(final int layerID, final int domainID) {
		Vector<Integer> ret = new Vector<Integer>();
		Lipid_3D[] lipidsEC_1Domain = DataPool.getMembraneModel().getAllExampleLipidsExtraInDomain(domainID, layerID);
		for (int i = 0; i < lipidsEC_1Domain.length; i++) {
			if (lipidsEC_1Domain[i].getLayerID() == layerID && (lipidsEC_1Domain[i].getDomainID() == domainID || domainID == -1) )
				ret.addElement(lipidsEC_1Domain[i].getPercent());
		}
		return ret.toArray(new Integer[0]);
	}
	
	/**
	 * Get the percentages of the defined layer and domainID on the intracellular side.
	 * 
	 * Note: If isAbsoluteValuesSelected() is false (default mode of the MembraneEditor), then 
	 *       percentages are used. In this case, use getDecimalPlaces() to get the precision
	 *       of the percentages.
	 * 
	 * @param layerID 
	 *                  the ID of the layer
	 * @param domainID
	 *                  the ID of the domain
	 * @return Integer[]
	 *                  an array containing all percentages of the defined layer and domainID
	 */
	public Integer[] getPercentages_IntraCellular(final int layerID, final int domainID) {
		Vector<Integer> ret = new Vector<Integer>();
		Lipid_3D[] lipidsIC_1Domain = DataPool.getMembraneModel().getAllExampleLipidsIntraInDomain(domainID, layerID);
		for (int i = 0; i < lipidsIC_1Domain.length; i++) {
			if (lipidsIC_1Domain[i].getLayerID() == layerID && (lipidsIC_1Domain[i].getDomainID() == domainID || domainID == -1) )
				ret.addElement(lipidsIC_1Domain[i].getPercent());
		}
		return ret.toArray(new Integer[0]);
	}
	
	
	/**
	 * Use this method to get the precision of the computation of the percentages 
	 * (if isAbsoluteValuesSelected() is false, default mode of the MembraneEditor).
	 * 
	 * @return int 
	 *            number of decimal places [0..3]
	 */
	public int getDecimalPlaces() {
		return DataPool.getMembraneModel().getDecimalPlaces();
	}
	
	/**
	 * Use this method if you changed some values relevant for the GUI of CmME.
	 * 
	 * @param compView
	 * @param memViewPanels
	 * @param database
	 * @param stats
	 * @param props
	 * @param intersections
	 */
	public synchronized void refreshViews(final boolean compView,
			final boolean memViewPanels, final boolean database,
			final boolean stats, final boolean props,
			final boolean intersections) {
		this.membraneViewListener.refreshViews(compView, memViewPanels, database, stats, props, intersections);
	}
	
	/**
	 * Gets the protein model of the given Protein_3D file.
	 * @param prot
	 * @return Protein 
	 */
	private Protein getProtein(Protein_3D prot) {
		if (prot.teporaryMolecule == null) {
			Protein p = new Protein(prot, this);
			prot.teporaryMolecule = p;
			return p;
		} else
			return (Protein) prot.teporaryMolecule;
	}

	/**
	 * Gets the lipid model of the given Lipid_3D file.
	 * @param lip
	 * @return Lipid
	 */
	private Lipid getLipid(Lipid_3D lip) {
		if (lip.teporaryMolecule == null) {
			Lipid p = new Lipid(lip, this);
			lip.teporaryMolecule = p;
			return p;
		} else
			return (Lipid) lip.teporaryMolecule;
	}

	/**
	 * Returns the size of the membrane
	 * 
	 * @return a Dimension Object that represents the size of the mebrane, as it
	 *         has been set by the user
	 */
	public Dimension getMembraneSize() {
		return DataPool.getMembraneModel().getSize();
	}

	/**
	 * Similar to getMembraneSize(), but this returns the bounds as a rectangle
	 * object
	 * 
	 * @return the membrane bounds as a rectangle
	 */
	public Rectangle getMembraneBounds2D() {
		return new Rectangle(0, 0, this.getMembraneSize().width, this
				.getMembraneSize().height);
	}

	/**
	 * Returns the Area of the whole Membrane in A².
	 * This area will also include the areas of additional Microdomains.
	 * @return the membrane area as an integer
	 */
	public int getMembraneArea() {
		return DataPool.getMembraneModel().getArea();
	}
	
	/**
	 * Returns a Vector with all lipids in the membrane of one side. The
	 * membranside is given by mode. Will return all lipids that belong to the
	 * currently selected bilayer
	 * 
	 * @param mode 0 = static field EXTRA = Extracellular, 1 = static field INTRA =
	 *        Intracellular
	 * 
	 * @return a list of all lipid instances on the specified membrane side
	 */
	public Vector<Lipid> getAllLipids(final int mode) {
		final Vector<Lipid> lips = new Vector<Lipid>();
		final MembraneModel model = DataPool.getMembraneModel();
		if (mode == MembranePackingAlgorithm.EXTRA) {

			final Lipid_3D[] mols = model.getAllLipidsExtraInDomain(-1, this
					.getCurrentLayerID());
			for (int i = 0; i < mols.length; i++) {
				lips.add(this.getLipid(mols[i]));
			}

		} else {
			final Lipid_3D[] mols = model.getAllLipidsIntraInDomain(-1, this
					.getCurrentLayerID());
			for (int i = 0; i < mols.length; i++) {
				lips.add(this.getLipid(mols[i]));
			}

		}

		return lips;
	}

	/**
	 * Returns a Vector with all lipids in the membrane of one domain and one side. The
	 * domain is given by domainID, the membranside is given by mode. 
	 * Will return all lipids that belong to the currently selected bilayer
	 * 
	 * @param domainID = the ID of the selected microdomain
	 *        mode 0 = static field EXTRA = Extracellular, 1 = static field INTRA =
	 *        Intracellular
	 * 
	 * @return a list of all lipid instances on the specified membrane side
	 */
	public Vector<Lipid> getAllLipidsInDomain(final int domainID, final int mode) {
		final Vector<Lipid> lips = new Vector<Lipid>();
		final MembraneModel model = DataPool.getMembraneModel();
		if (mode == MembranePackingAlgorithm.EXTRA) {

			final Lipid_3D[] mols = model.getAllLipidsExtraInDomain(domainID, this
					.getCurrentLayerID());
			for (int i = 0; i < mols.length; i++) {
				lips.add(this.getLipid(mols[i]));
			}

		} else {
			final Lipid_3D[] mols = model.getAllLipidsIntraInDomain(domainID, this
					.getCurrentLayerID());
			for (int i = 0; i < mols.length; i++) {
				lips.add(this.getLipid(mols[i]));
			}

		}

		return lips;
	}
	
	/**
	 * Returns a Vector with all proteins in the membrane Will return all proteins
	 * that belong to the currently selected bilayer
	 * 
	 * @return a list of all protein instances
	 */
	public Vector<Protein> getAllProteins() {
		final Vector<Protein> prots = new Vector<Protein>();
		final MembraneModel model = DataPool.getMembraneModel();

		final Protein_3D[] mols = model.getAllProteinsInDomain(-1, this
				.getCurrentLayerID());
		for (int i = 0; i < mols.length; i++) {
			prots.add(getProtein(mols[i]));
		}

		return prots;
	}
	
	/**
	 * Returns a Vector with all proteins in the given domain 
	 * Will return all proteins that belong to the currently 
	 * selected bilayer domain.
	 * 
	 * @param domainID = the ID of the selected microdomain
	 * 
	 * @return a list of all protein instances
	 */
	public Vector<Protein> getAllProteinsInDomain(int domainID) {
		final Vector<Protein> prots = new Vector<Protein>();
		final MembraneModel model = DataPool.getMembraneModel();

		final Protein_3D[] mols = model.getAllProteinsInDomain(domainID, this
				.getCurrentLayerID());
		for (int i = 0; i < mols.length; i++) {
			prots.add(getProtein(mols[i]));
		}

		return prots;
	}

	/**
	 * 
	 * Creates a new Lipid that can be placed in the membrane. The type of lipid
	 * you will get from this method depends on the lipid template setup of the
	 * intracellular membrane side. The initial rotation will be the way it has
	 * been set up there too. The initial y-translation will be set in a way
	 * that the whole lipid is just underneath the y-plane
	 * 
	 * 
	 * If you want to add the lipid after you set up position and/or rotation,
	 * use the addIntracellularLipid() method
	 * 
	 * @return the new lipid object or null if no template lipids exist on this
	 *         side
	 */
	public final Lipid createNextIntracellularLipid() {

		final int id = DataPool.getCurrentLayerID();

		final Lipid_3D[] lips = DataPool.getMembraneModel()
				.getAllExampleLipidsIntraInDomain(0, id);
		return DataPool.getMembraneModel().createNextLipid(
				MembranePackingAlgorithm.INTRA, lips, 0, id, this);

	}

	/**
	 * 
	 * Creates a new Lipid that can be placed in the membrane. The type of lipid
	 * you will get from this method depends on the lipid template setup of the
	 * intracellular membrane side. The initial rotation will be the way it has
	 * been set up there too. The initial y-translation will be set in a way
	 * that the whole lipid is just underneath the y-plane
	 * 
	 * 
	 * If you want to add the lipid after you set up position and/or rotation,
	 * use the addIntracellularLipid() method
	 * 
	 * @param layerID the
	 *            id of the bilayer you want take into account
	 * @return the new lipid object or null if no template lipids exist on this
	 *         side
	 */
	public final Lipid createNextIntracellularLipid(final int layerID) {

		final Lipid_3D[] lips = DataPool.getMembraneModel()
				.getAllExampleLipidsIntraInDomain(0, layerID);
		return DataPool.getMembraneModel().createNextLipid(
				MembranePackingAlgorithm.INTRA, lips, 0, layerID, this);

	}

	/**
	 * 
	 * Creates a new Lipid that can be placed in the membrane. The type of lipid
	 * you will get from this method depends on the lipid template setup of the
	 * extracellular membrane side. The initial rotation will be the way it has
	 * been set up there too. The initial y-translation will be set in a way
	 * that the whole lipid is just above the y-plane
	 * 
	 * It will get the domain identifier of the specified layer (layerID)
	 * 
	 * If you want to add the lipid after you set up position and/or rotation,
	 * use the addExtracellularLipid() method
	 * 
	 * @return the new lipid object or null if no template lipids exist on this
	 *         side
	 */
	public final Lipid createNextExtracellularLipid() {

		final int id = DataPool.getCurrentLayerID();

		final Lipid_3D[] lips = DataPool.getMembraneModel()
				.getAllExampleLipidsExtraInDomain(0, id);
		return DataPool.getMembraneModel().createNextLipid(
				MembranePackingAlgorithm.EXTRA, lips, 0, id, this);

	}

	/**
	 * 
	 * Creates a new Lipid that can be placed in the membrane. The type of lipid
	 * you will get from this method depends on the lipid template setup of the
	 * extracellular membrane side. The initial rotation will be the way it has
	 * been set up there too. The initial y-translation will be set in a way
	 * that the whole lipid is just above the y-plane
	 * 
	 * 
	 * If you want to add the lipid after you set up position and/or rotation,
	 * use the addExtracellularLipid() method
	 * 
	 * @param layerID the
	 *            id of the bilayer you want take into account
	 * @return the new lipid object or null if no template lipids exist on this
	 *         side
	 */
	public final Lipid createNextExtracellularLipid(final int layerID) {

		final Lipid_3D[] lips = DataPool.getMembraneModel()
				.getAllExampleLipidsExtraInDomain(0, layerID);
		return DataPool.getMembraneModel().createNextLipid(
				MembranePackingAlgorithm.EXTRA, lips, 0, layerID, this);
	}

	/**
	 * Chooses a random layerID of any layer that is in the model
	 * 
	 * @return a randomly chosen layerID
	 */
	public int createRandomLayerID() {
		final MembraneModel model = DataPool.getMembraneModel();
		if (model.getLayerCount() < 1)
			return 0;
		final int i = this.random.nextInt(model.getLayerCount() + 1);
		if (i == 0)
			return 0;
		else
			return model.getLayer(i - 1).getLayerId();

	}

	/**
	 * Returns a new extracellular lipid by given PDB-Code. If the lipid is not
	 * in the current membrane model, then null is returned. After calling this,
	 * the lipid is still not added to the membrane model but can be used to
	 * check for intersection with other objects in the membrane. If you want to
	 * add the lipid after placing, use the addExtracellularLipid() method
	 * 
	 * @param pdbcode
	 *            the code to choose after
	 * @return the new lipid
	 */

	public final Lipid getExtracellularLipidByPDBCode(final String pdbcode) {
		Lipid lip;
		final MembraneModel model = DataPool.getMembraneModel();

		final Lipid_3D[] source = model.getAllExampleLipidsExtraInDomain(0, 0);

		for (int i = 0; i < source.length; i++) {
			if (source[i].getPDBcode().equalsIgnoreCase(pdbcode)) {

				final Lipid_3D l3d = (Lipid_3D) source[i].copy(true, false);
				l3d.setYTranslation(MembranePackingAlgorithm.getTotalHeight(l3d) / 2f
						/ CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE);

				lip = this.getLipid(l3d);
				return lip;
			}
		}

		return null;
	}

	/**
	 * Returns a newintracellular lipid by given PDB-Code. If the lipid is not
	 * in the current membrane model, then null is returned. After calling this,
	 * the lipid is still not added to the membrane model but can be used to
	 * check for intersection with other objects in the membrane. If you want to
	 * add the lipid after placing, use the addIntracellularLipid() method
	 * 
	 * @param pdbcode
	 *            the code to choose after
	 * @return the new lipid
	 */
	public final Lipid getIntracellularLipidByPDBCode(final String pdbcode) {
		Lipid lip;
		final MembraneModel model = DataPool.getMembraneModel();

		final Lipid_3D[] source = model.getAllExampleLipidsExtraInDomain(0, 0);

		for (int i = 0; i < source.length; i++) {
			if (source[i].getPDBcode().equalsIgnoreCase(pdbcode)) {
				final Lipid_3D l3d = (Lipid_3D) source[i].copy(true, false);
				l3d
						.setYTranslation(-(MembranePackingAlgorithm
								.getTotalHeight(l3d) / 2)
								/ CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE);

				lip = this.getLipid(l3d);

				return lip;
			}
		}

		return null;
	}

	/**
	 * Adds the given intracellular lipid to the intracellular side of the
	 * membrane.
	 * 
	 * @param lipidToAdd the
	 *            lipid you want to add to the membrane
	 * @throws IllegalOperationException
	 *             if the passed instance has alrady been added
	 */
	public final void addIntracellularLipid(final Lipid lipidToAdd)
			throws IllegalOperationException {
		this.dock();
		if (DataPool.getMembraneModel().getAllLipidsIntra().contains(
				lipidToAdd.getOriginalMolecule())) {
			this.dock();
			throw new IllegalOperationException(
					"Instance is already added to the Membrane");
		}

		this.membraneViewListener
				.addLipidIntra((Lipid_3D) lipidToAdd.getOriginalMolecule());

		this.calculationManager.updateMembraneInfo(false);
		this.dock();
	}

	/**
	 * adds the given extracellular lipid to the extracellular side of the
	 * membrane.
	 * 
	 * @param lipidToAdd
	 * @throws IllegalOperationException
	 *             if the passed instance has alrady been added
	 */
	public final void addExtracellularLipid(final Lipid lipidToAdd)
			throws IllegalOperationException {
		this.dock();
		if (DataPool.getMembraneModel().getAllLipidsExtra().contains(
				lipidToAdd.getOriginalMolecule())) {
			this.dock();
			throw new IllegalOperationException(
					"Instance is already added to the Membrane");
		}

		this.membraneViewListener
				.addLipidExtra((Lipid_3D) lipidToAdd.getOriginalMolecule());

		this.calculationManager.updateMembraneInfo(false);
		this.dock();
	}

	/**
	 * @deprecated use removeLipidsFromMembrane(Lipid[] lipid) instead
	 */
	@Deprecated
	public final void removeIntracellularLipid(final Lipid lipidToRemove)
			throws IllegalOperationException {
		this.dock();

		if (DataPool.getMembraneModel().getAllLipidsIntra().contains(
				lipidToRemove.getOriginalMolecule())) {
			this.membraneViewListener.removeLipidIntra((Lipid_3D) lipidToRemove
					.getOriginalMolecule());
		} else {
			this.dock();
			throw new IllegalOperationException(
					"Instance is not in the Membrane");

		}
		this.calculationManager.updateMembraneInfo(false);

		this.dock();
	}

	/**
	 * @deprecated use removeLipidsFromMembrane(Lipid[] lipid) instead
	 */
	@Deprecated
	public final void removeExtracellularLipid(final Lipid lipidToRemove)
			throws IllegalOperationException {
		this.dock();

		if (DataPool.getMembraneModel().getAllLipidsExtra().contains(
				lipidToRemove.getOriginalMolecule())) {
			this.membraneViewListener.removeLipidExtra((Lipid_3D) lipidToRemove
					.getOriginalMolecule());
		} else {
			this.dock();
			throw new IllegalOperationException(
					"Instance is not in the Membrane");
		}
		this.calculationManager.updateMembraneInfo(false);

		this.dock();
	}

	/**
	 * Returns true if the passed lipid intersects with the border or is located
	 * completely outside the membrane.
	 * 
	 * Intersection tests do not require passed molecule objects to be added to
	 * the membrane already! It will refer to the position and rotation that
	 * have been set
	 * 
	 * @param lip the
	 *            lipid to test for border intersection
	 * @return true if an intersection occured, false if not
	 */
	public boolean intersectsBorders(final Molecule lip) {
		this.dock();
		boolean intersect = false;
		final MembraneModel model = DataPool.getMembraneModel();

		intersect = this.IM.intercects_Molecule_Border(lip
				.getOriginalMolecule(), model.getSize());
		return intersect;
	}

	/**
	 * Returns true if the passed lipid or protein intersects with any other molecule in
	 * the membrane.
	 * 
	 * Intersection tests do not require passed molecule objects to be added to
	 * the membrane already! It will refer to the position and rotation that
	 * have been set
	 * 
	 * @param lip the
	 *            lipid to test for an intersection
	 * @return true if an intersection occured, false if not
	 */
	public boolean intersects(final Molecule lip) {
		this.dock();
		boolean intersect = false;

		intersect = this.IM.intersectsMoleculeAnyMolecules(lip
				.getOriginalMolecule(), true);
		return intersect;
	}

	/**
	 * Returns the molecules the passed lipid intersects with. Null if no
	 * intersection appeared.
	 * 
	 * Intersection tests do not require passed molecule objects to be added to
	 * the membrane already! It will refer to the position and rotation that
	 * have been set
	 * 
	 * @param mol the
	 *            molecule to test for an intersection
	 * @return others a list of intersecting molecules or null if the passed lipid is
	 *         intersection-free
	 */
	public Molecule[] getIntersectedMolecules(final Molecule mol) {
		this.dock();
		this.IM.intersectsMoleculeAnyMolecules(mol.getOriginalMolecule(), true);
		final Molecule_3D[] mols3D = this.IM.getCollisionObjects();
		final Molecule[] mols = new Molecule[mols3D.length];

		if (mols == null)
			return null;
		
		for (int i = 0; i < mols.length; i++) {
			mols[i] = new Molecule(mols3D[i], this);
		}

		return mols;
	}

	/**
	 * Returns true if the first argument's molecule intersects with one of the
	 * second argument's molecules.
	 * 
	 * Intersection tests do not require passed molecule objects to be added to
	 * the membrane already! It will refer to the position and rotation that
	 * have been set
	 * 
	 * @param mol the
	 *            molecule to test for intersection
	 * @param others a
	 *            list of other molecules to test against
	 * @return true if an intersection occured, false if not
	 */
	public boolean intersectsMoleculeWithMolecules(final Molecule mol,
			final Vector<Molecule> others) {
		this.dock();
		boolean intersect = false;

		final Vector<Molecule_3D> m = new Vector<Molecule_3D>();
		for (int i = 0; i < others.size(); i++) {
			if (others.get(i) instanceof Molecule) {
				m.add(others.get(i).getOriginalMolecule());
			}
		}

		intersect = this.IM.intersectsMoleculeMoleculesVector(mol
				.getOriginalMolecule(), m, true);
		return intersect;
	}

	/**
	 * Returns true if the first argument's molecule intersects with the
	 * molecule in the second argument
	 * 
	 * Intersection tests do not require passed molecule objects to be added to
	 * the membrane already! It will refer to the position and rotation that
	 * have been set
	 * 
	 * @param mol the
	 *            molecule to test for intersection
	 * @param mol2 the
	 *            opposing molecule
	 * @return true if an intersection occured, false if not
	 */
	public boolean intersectsMoleculeWithMolecule(final Molecule mol,
			final Molecule mol2) {
		this.dock();
		boolean intersect = false;

		final Vector<Molecule_3D> m = new Vector<Molecule_3D>();
		m.add(mol2.getOriginalMolecule());

		intersect = this.IM.intersectsMoleculeMoleculesVector(mol
				.getOriginalMolecule(), m, true);
		return intersect;
	}

	/**
	 * Returns true if the passed lipid intersects with any protein in the
	 * membrane.
	 * 
	 * Intersection tests do not require passed molecule objects to be added to
	 * the membrane already! It will refer to the position and rotation that
	 * have been set
	 * 
	 * @param mol the
	 *            molecule to test for intersection
	 * @return true if an intersection occured, false if not
	 */
	public boolean intersectsWithProteins(final Molecule mol) {
		this.dock();
		boolean intersect = false;

		intersect = this.IM.intersectsMoleculeAnyProtein(mol
				.getOriginalMolecule(), true);

		return intersect;

	}

	/**
	 * Returns true if the molecule intersects with any lipid on the specified
	 * membrane side.
	 * 
	 * Intersection tests do not require passed molecule objects to be added to
	 * the membrane already! It will refer to the position and rotation that
	 * have been set
	 * 
	 * @param mol the
	 *            molecule to test for intersection
	 * @param side the
	 *            specified membrane side, EXTRA or INTRA
	 * @return true if an intersection occured, false if not
	 */
	public boolean intersectsWithLipid(final Molecule mol, final int side) {
		this.dock();
		boolean intersects = false;

		final MembraneModel model = DataPool.getMembraneModel();

		if (side == 0) {
			intersects = this.IM.intersectsMoleculeMoleculesVector(mol
					.getOriginalMolecule(), model.getAllLipidsExtra(), true);

		} else if (side == 1) {
			intersects = this.IM.intersectsMoleculeMoleculesVector(mol
					.getOriginalMolecule(), model.getAllLipidsIntra(), true);
		}

		return intersects;
	}

	/**
	 * For Java 3D experts only! Here it is possible to add custom Java3D-based
	 * objects. The origin is (0,0,0) - the point from which the axes in the 
	 * Membrane View originate. 
	 * Note 1: This BranchGroup is only for custom objects. It is not possible to
	 *         access MembraneEditor's objects such as the shapes or the atoms
	 *         of the molecules.
	 * Note 2: You have to take care to remove the objects created here. Membrane-
	 *         Editor will not alter this node. To remove all attached Java 3D
	 *         objects, you might want to use:
	 *          this.getJ3DBranchGroup().removeAllChildren(); 
	 *         
	 * @return BranchGroup to add Java 3D objects
	 */
	public BranchGroup getJ3DBranchGroup() {
		return membraneViewListener.getJ3DBranchGroup();
	}
	
	/**
	 * Returns all borders the molecule intersects with. Use the 6 static fields
	 * (BORDER_NORTH, BORDER_EAST,...) in this class to identify the different
	 * return values
	 * 
	 * Intersection tests do not require passed molecule objects to be added to
	 * the membrane already! It will refer to the position and rotation that
	 * have been set
	 * 
	 * @param mol the
	 *            molecule to test for intersection
	 * @return a list of collided borders as int values
	 */
	public int[] getIntersectedBorders(final Molecule mol) {
		this.dock();
		final int[] or = this.IM.intercects_Molecule_Border_In_Detail(mol
				.getOriginalMolecule(), this.getMembraneSize());

		return or;
	}

	/**
	 * Returns true if the lipid intersects with the northern border.
	 * Intersection tests do not require passed molecule objects to be added to
	 * the membrane already! It will refer to the position and rotation that
	 * have been set
	 * 
	 * @param mol the
	 *            molecule to test for intersection
	 * @return true if an intersection occured, false if not
	 */
	public boolean intersectsBorderNorth(final Molecule mol) {
		this.dock();
		final boolean intersects = false;

		final int[] ii = this.getIntersectedBorders(mol);
		if (ii != null && ii.length > 0) {
			for (int i = 0; i < ii.length; i++) {
				if (ii[i] == IntersectManager.BORDER_NORTH)
					return true;
			}
		}
		return intersects;
	}

	/**
	 * Returns true if the lipid intersects with the southern border.
	 * 
	 * Intersection tests do not require passed molecule objects to be added to
	 * the membrane already! It will refer to the position and rotation that
	 * have been set
	 * 
	 * @param mol the
	 *            molecule to test for intersection
	 * @return true if an intersection occured, false if not
	 */
	public boolean intersectsBorderSouth(final Molecule mol) {
		this.dock();
		final boolean intersects = false;

		final int[] ii = this.getIntersectedBorders(mol);
		if (ii != null && ii.length > 0) {
			for (int i = 0; i < ii.length; i++) {
				if (ii[i] == IntersectManager.BORDER_SOUTH)
					return true;
			}
		}
		return intersects;
	}

	/**
	 * Returns true if the lipid intersects with the eastern border.
	 * 
	 * Intersection tests do not require passed molecule objects to be added to
	 * the membrane already! It will refer to the position and rotation that
	 * have been set
	 * 
	 * @param mol the
	 *            molecule to test for intersection
	 * @return true if an intersection occured, false if not
	 */
	public boolean intersectsBorderEast(final Molecule mol) {
		this.dock();
		final boolean intersects = false;

		final int[] ii = this.getIntersectedBorders(mol);
		if (ii != null && ii.length > 0) {
			for (int i = 0; i < ii.length; i++) {
				if (ii[i] == IntersectManager.BORDER_EAST)
					return true;
			}
		}
		return intersects;
	}

	/**
	 * Returns true if the lipid intersects with the western border.
	 * 
	 * Intersection tests do not require passed molecule objects to be added to
	 * the membrane already! It will refer to the position and rotation that
	 * have been set
	 * 
	 * @param mol the
	 *            molecule to test for intersection
	 * @return true if an intersection occured, false if not
	 */
	public boolean intersectsBorderWest(final Molecule mol) {
		this.dock();
		final boolean intersects = false;

		final int[] ii = this.getIntersectedBorders(mol);
		if (ii != null && ii.length > 0) {
			for (int i = 0; i < ii.length; i++) {
				if (ii[i] == IntersectManager.BORDER_WEST)
					return true;
			}
		}
		return intersects;
	}
	
	/**
	 * Returns true if the passed lipid intersects with the 
	 * the upper limit.
	 * 
	 * Intersection tests do not require passed molecule objects to be added to
	 * the membrane already! It will refer to the position and rotation that
	 * have been set
	 * 
	 * @param lip the
	 *            lipid to test for sky intersection
	 * @return true if an intersection occured, false if not
	 */
	public boolean intersectsSky(final Molecule lip) {
		this.dock();
		boolean intersect = false;

		intersect = this.IM.intercects_Molecule_Sky(lip
				.getOriginalMolecule());
		return intersect;
	}
	
	/**
	 * Returns true if the passed lipid intersects with the 
	 * the lower limit.
	 * 
	 * Intersection tests do not require passed molecule objects to be added to
	 * the membrane already! It will refer to the position and rotation that
	 * have been set
	 * 
	 * @param lip the
	 *            lipid to test for earth intersection
	 * @return true if an intersection occured, false if not
	 */
	public boolean intersectsEarth(final Molecule lip) {
		this.dock();
		boolean intersect = false;

		intersect = this.IM.intercects_Molecule_Earth(lip
				.getOriginalMolecule());
		return intersect;
	}
	
	/**
	 * Returns true if the passed lipid intersects with the 
	 * the lower or upper limit.
	 * 
	 * Intersection tests do not require passed molecule objects to be added to
	 * the membrane already! It will refer to the position and rotation that
	 * have been set
	 * 
	 * @param lip the
	 *            lipid to test for earth or sky intersection
	 * @return true if an intersection occured, false if not
	 */
	public boolean intersectsEarthOrSky(final Molecule lip) {
		return (this.intersectsEarth(lip) || this.intersectsSky(lip));
	}
	
	/**
	 * Displays the passed information in string representation beneath the
	 * MembraneView during Algorithm Performance. It will remain beeing
	 * displayed unless this method gets called another time.
	 * 
	 * @param info the
	 *            info string (not too long please)
	 */
	public void displayInformation(final String info) {
		this.dock();
		this.membraneViewListener.displayAlgoritmInfo(info);
	}

	/**
	 * Displays the given progress beneath the MembraneView during Algorithm
	 * performance.
	 * 
	 * @param progress a
	 *            progress value between 0 and 100
	 */
	public void setDisplayedProgress(final int progress) {
		this.dock();
		this.membraneViewListener.displayAlgorithmProgress(progress);
	}

	/**
	 * NOT MEANT TO BE USED BY PLUGIN DEVELOPERS!
	 */
	public static float getTotalHeight(final Molecule_3D lip) {
		final Point3f lower = lip.getLower(false);

		final Point3f upper = lip.getUpper(false);

		return (upper.y - lower.y);
	}

	/**
	 * NOT MEANT TO BE USED BY PLUGIN DEVELOPERS!
	 */
	public float getTotalHeight(final Lipid lip) {
		final Point3f lower = lip.getOriginalMolecule().getLower(false);

		final Point3f upper = lip.getOriginalMolecule().getUpper(false);

		return (upper.y - lower.y);
	}

	/**
	 * Returns the scaled height in the membrane of the given lipid. Use this
	 * method to get the height of an placed lipid.
	 * 
	 * @param lip The
	 *            lipid you want to test
	 * @return The current y-height of the lipid
	 */
	public float getLipidInMembraneHeight(final Lipid lip) {
		return (this.getTotalHeight(lip) / 2f / CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE);
	}

	/**
	 * Returns the number of all placed lipids in the membrane
	 * 
	 * @return the current lipid count
	 */
	public int countTotalLipidsThatAreAdded() {
		final MembraneModel model = DataPool.getMembraneModel();
		return model.getLipidCountExtra() + model.getLipidCountIntra();
	}

	/**
	 * Returns the closest molecule to the passed one. If no other molecule is
	 * added to the membrane model, the return value will be null. Distance is
	 * measured from the center points of the molecules. Sometimes this leads to
	 * wrong outcomes.
	 * 
	 * 
	 */
	public Molecule getNearestMolecule(final Molecule mol) {
		final MembraneModel model = DataPool.getMembraneModel();
		Molecule nearest = null;
		double nearestDistance = 10000;

		final float[] m = new float[3];
		m[0] = mol.getPosition().x;
		m[1] = mol.getPosition().y;
		m[2] = mol.getPosition().z;

		for (int i = 0; i < model.getAllLipidsExtra().size(); i++) {

			final Molecule test = new Molecule(model.getLipidAtExtra(i), this);
			final float[] t = new float[3];
			t[0] = test.getPosition().x;
			t[1] = test.getPosition().y;
			t[2] = test.getPosition().z;

			final float[] rv = new float[3];
			rv[0] = t[0] - m[0];
			rv[1] = t[1] - m[1];
			rv[2] = t[2] - m[2];

			final double dist = Math.sqrt(Math.pow(rv[0], 2)
					+ Math.pow(rv[1], 2) + Math.pow(rv[2], 2));
			if (dist < nearestDistance) {
				nearestDistance = dist;
				nearest = test;
			}

		}

		for (int i = 0; i < model.getAllLipidsIntra().size(); i++) {

			final Molecule test = new Molecule(model.getLipidAtIntra(i), this);
			final float[] t = new float[3];
			t[0] = test.getPosition().x;
			t[1] = test.getPosition().y;
			t[2] = test.getPosition().z;

			final float[] rv = new float[3];
			rv[0] = t[0] - m[0];
			rv[1] = t[1] - m[1];
			rv[2] = t[2] - m[2];

			final double dist = Math.sqrt(Math.pow(rv[0], 2)
					+ Math.pow(rv[1], 2) + Math.pow(rv[2], 2));
			if (dist < nearestDistance) {
				nearestDistance = dist;
				nearest = test;
			}

		}

		for (int i = 0; i < model.getAllProteins().size(); i++) {

			final Molecule test = new Molecule(model.getProteinAt(i), this);
			final float[] t = new float[3];
			t[0] = test.getPosition().x;
			t[1] = test.getPosition().y;
			t[2] = test.getPosition().z;

			final float[] rv = new float[3];
			rv[0] = t[0] - m[0];
			rv[1] = t[1] - m[1];
			rv[2] = t[2] - m[2];

			final double dist = Math.sqrt(Math.pow(rv[0], 2)
					+ Math.pow(rv[1], 2) + Math.pow(rv[2], 2));
			if (dist < nearestDistance) {
				nearestDistance = dist;
				nearest = test;
			}

		}

		return nearest;
	}

	// public Vector<Protein_3D> getAllProteins() {
	// return DataPool.getMembraneModel().getAllProteins();
	// }

	/**
	 * Returns the nearest lipid to the given molecule mol on the determined
	 * membrane side. If no other lipid is added to the membrane model for this
	 * membrane side, the return value will be null. Distance is measured from
	 * the center points of the molecules. Sometimes this leads to wrong
	 * outcomes.
	 * 
	 */
	public Lipid getNearestLipidOnMySide(final Molecule mol,
			final int membraneside) {
		final MembraneModel model = DataPool.getMembraneModel();
		Lipid nearest = null;
		double nearestDistance = 10000;

		final float[] m = new float[3];
		m[0] = mol.getPosition().x;
		m[1] = mol.getPosition().y;
		m[2] = mol.getPosition().z;

		if (membraneside == MembranePackingAlgorithm.EXTRA) {
			for (int i = 0; i < model.getAllLipidsExtra().size(); i++) {

				final Lipid test = this.getLipid(model.getLipidAtExtra(i));
				final float[] t = new float[3];
				t[0] = test.getPosition().x;
				t[1] = test.getPosition().y;
				t[2] = test.getPosition().z;

				final float[] rv = new float[3];
				rv[0] = t[0] - m[0];
				rv[1] = t[1] - m[1];
				rv[2] = t[2] - m[2];

				final double dist = Math.sqrt(Math.pow(rv[0], 2)
						+ Math.pow(rv[1], 2) + Math.pow(rv[2], 2));
				if (dist < nearestDistance) {
					nearestDistance = dist;
					nearest = test;
				}

			}
		} else if (membraneside == MembranePackingAlgorithm.INTRA) {
			for (int i = 0; i < model.getAllLipidsIntra().size(); i++) {

				final Lipid test = this.getLipid(model.getLipidAtIntra(i));
				final float[] t = new float[3];
				t[0] = test.getPosition().x;
				t[1] = test.getPosition().y;
				t[2] = test.getPosition().z;

				final float[] rv = new float[3];
				rv[0] = t[0] - m[0];
				rv[1] = t[1] - m[1];
				rv[2] = t[2] - m[2];

				final double dist = Math.sqrt(Math.pow(rv[0], 2)
						+ Math.pow(rv[1], 2) + Math.pow(rv[2], 2));
				if (dist < nearestDistance) {
					nearestDistance = dist;
					nearest = test;
				}

			}
		}

		return nearest;
	}

	/**
	 * Returns the nearest molecule to the given molecule mol on the determined
	 * membrane side. If no other molecule is added to the membrane model for
	 * this membrane side, the return value will be null. Distance is measured
	 * from the center points of the molecules. Sometimes this leads to wrong
	 * outcomes.
	 * 
	 */
	public Molecule getNearestMoleculeOnMySide(final Molecule mol,
			final int membraneside) {
		final MembraneModel model = DataPool.getMembraneModel();
		Molecule nearest = null;
		double nearestDistance = 10000;

		final float[] m = new float[3];
		m[0] = mol.getPosition().x;
		m[1] = mol.getPosition().y;
		m[2] = mol.getPosition().z;

		for (int i = 0; i < model.getAllProteins().size(); i++) {

			final Molecule test = new Molecule(model.getProteinAt(i), this);
			final float[] t = new float[3];
			t[0] = test.getPosition().x;
			t[1] = test.getPosition().y;
			t[2] = test.getPosition().z;

			final float[] rv = new float[3];
			rv[0] = t[0] - m[0];
			rv[1] = t[1] - m[1];
			rv[2] = t[2] - m[2];

			final double dist = Math.sqrt(Math.pow(rv[0], 2)
					+ Math.pow(rv[1], 2) + Math.pow(rv[2], 2));
			if (dist < nearestDistance) {
				nearestDistance = dist;
				nearest = test;
			}

		}
		if (membraneside == MembranePackingAlgorithm.EXTRA) {
			for (int i = 0; i < model.getAllLipidsExtra().size(); i++) {

				final Molecule test = new Molecule(model.getLipidAtExtra(i),
						this);
				final float[] t = new float[3];
				t[0] = test.getPosition().x;
				t[1] = test.getPosition().y;
				t[2] = test.getPosition().z;

				final float[] rv = new float[3];
				rv[0] = t[0] - m[0];
				rv[1] = t[1] - m[1];
				rv[2] = t[2] - m[2];

				final double dist = Math.sqrt(Math.pow(rv[0], 2)
						+ Math.pow(rv[1], 2) + Math.pow(rv[2], 2));
				if (dist < nearestDistance) {
					nearestDistance = dist;
					nearest = test;
				}

			}
		} else if (membraneside == MembranePackingAlgorithm.INTRA) {
			for (int i = 0; i < model.getAllLipidsIntra().size(); i++) {

				final Molecule test = new Molecule(model.getLipidAtIntra(i),
						this);
				final float[] t = new float[3];
				t[0] = test.getPosition().x;
				t[1] = test.getPosition().y;
				t[2] = test.getPosition().z;

				final float[] rv = new float[3];
				rv[0] = t[0] - m[0];
				rv[1] = t[1] - m[1];
				rv[2] = t[2] - m[2];

				final double dist = Math.sqrt(Math.pow(rv[0], 2)
						+ Math.pow(rv[1], 2) + Math.pow(rv[2], 2));
				if (dist < nearestDistance) {
					nearestDistance = dist;
					nearest = test;
				}

			}
		}

		return nearest;
	}

	/**
	 * Returns the nearest molecule to the given molecule mol on the determined
	 * membrane side. If no other molecule is added to the membrane model for
	 * this membrane side, the return value will be null. This is the detailed
	 * version of the getNearestMoleculeOnMySide method. The boolean detailed
	 * indicates if you want detailed calculation or even more detailed
	 * calculation. Setting detailed = false will help you out in most cases. If
	 * you need it very precise you can set detailed = true. This provides more
	 * detailed distance calculation at the cost of time.
	 * 
	 */
	public Molecule getNearestMoleculeOnMySideDetailed(final Molecule mol,
			final int membraneside, final boolean detailed) {
		final MembraneModel model = DataPool.getMembraneModel();
		Molecule nearest = null;
		float nearestDistance = 10000f;

		final float[] m = new float[3];
		m[0] = mol.getPosition().x;
		m[1] = mol.getPosition().y;
		m[2] = mol.getPosition().z;

		for (int i = 0; i < model.getAllProteins().size(); i++) {

			float dist;
			if (detailed == false) {
				dist = this.nearestTrianglePoints(mol, getProtein(model
						.getProteinAt(i)));
			} else {
				dist = this.nearestAtomPoints(mol, getProtein(model
						.getProteinAt(i)));
			}

			if (dist < nearestDistance) {
				nearestDistance = dist;
				nearest = new Molecule(model.getProteinAt(i), this);
			}

		}
		if (membraneside == MembranePackingAlgorithm.EXTRA) {
			for (int i = 0; i < model.getAllLipidsExtra().size(); i++) {

				final Molecule test = new Molecule(model.getLipidAtExtra(i),
						this);
				float dist;
				if (detailed == false) {
					dist = this.nearestTrianglePoints(mol, test);
				} else {
					dist = this.nearestAtomPoints(mol, test);
				}
				if (dist < nearestDistance) {
					nearestDistance = dist;
					nearest = test;
				}

			}
		} else if (membraneside == MembranePackingAlgorithm.INTRA) {
			for (int i = 0; i < model.getAllLipidsIntra().size(); i++) {

				final Molecule test = new Molecule(model.getLipidAtIntra(i),
						this);
				float dist;
				if (detailed == false) {
					dist = this.nearestTrianglePoints(mol, test);
				} else {
					dist = this.nearestAtomPoints(mol, test);
				}

				if (dist < nearestDistance) {
					nearestDistance = dist;
					nearest = test;
				}

			}
		}

		return nearest;
	}

	/**
	 * Returns the nearest protein.
	 * 
	 * Distance is measured from the center points of the molecules. Sometimes
	 * this leads to wrong outcomes.
	 * 
	 */
	public Protein getNearestProtein(final Molecule mol) {
		final MembraneModel model = DataPool.getMembraneModel();
		Protein nearest = null;
		double nearestDistance = 10000;

		final float[] m = new float[3];
		m[0] = mol.getPosition().x;
		m[1] = mol.getPosition().y;
		m[2] = mol.getPosition().z;

		final Protein_3D[] prots = model.getAllProteinsInDomain(mol
				.getDomainIdentifier(), mol.getLayerID());
		for (int i = 0; i < prots.length; i++) {

			final Protein test = getProtein(prots[i]);
			final float[] t = new float[3];
			t[0] = test.getPosition().x;
			t[1] = test.getPosition().y;
			t[2] = test.getPosition().z;

			final float[] rv = new float[3];
			rv[0] = t[0] - m[0];
			rv[1] = t[1] - m[1];
			rv[2] = t[2] - m[2];

			final double dist = Math.sqrt(Math.pow(rv[0], 2)
					+ Math.pow(rv[1], 2) + Math.pow(rv[2], 2));
			if (dist < nearestDistance) {
				nearestDistance = dist;
				nearest = test;
			}

		}
		return nearest;
	}

	/**
	 * Returns the nearest protein.
	 * 
	 * This is the detailed version of the getNearestProtein method. The boolean
	 * detailed indicates if you want detailed calculation or even more detailed
	 * calculation. Setting detailed = false will help you out in most cases. If
	 * you need it very precise you can set detailed = true. This provides more
	 * detailed distance calculation at the cost of time.
	 * 
	 * @author Sebastian Schneider
	 */
	public Protein getNearestProteinDetailed(final Molecule mol,
			final boolean detailed) {
		final MembraneModel model = DataPool.getMembraneModel();
		Protein nearest = null;
		float nearestDistance = 10000f;

		for (int i = 0; i < model.getAllProteins().size(); i++) {

			final Protein test = getProtein(model.getProteinAt(i));
			float dist;
			if (detailed == false) {
				dist = this.nearestTrianglePoints(mol, test);
			} else {
				dist = this.nearestAtomPoints(mol, test);
			}

			if (dist < nearestDistance) {
				nearestDistance = dist;
				nearest = test;
			}

		}
		return nearest;
	}

	/**
	 * Returns the nearest molecule to the given molecule mol. Molecules in the
	 * except Vector are ignored. If no other molecule is added to the membrane
	 * model, the return value will be null. Distance is measured from the
	 * center points of the molecules. Sometimes this leads to wrong outcomes.
	 * 
	 * @author Sebastian Schneider
	 */
	public Molecule getNearestMoleculeExcept(final Molecule mol,
			final Vector<Molecule> except) {
		final MembraneModel model = DataPool.getMembraneModel();
		Molecule nearest = null;
		double nearestDistance = 10000;

		final Vector<Molecule_3D> allMolecules = new Vector<Molecule_3D>();
		for (int i = 0; i < model.getLipidCountExtra(); i++) {
			Molecule_3D test = (model.getLipidAtExtra(i));
			for (int j = 0; j < except.size(); j++) {
				if (test == (except.get(j).getOriginalMolecule())) {
					test = null;
					break;
				}
			}
			if (test != null) {
				allMolecules.add(test);
			}
		}

		for (int i = 0; i < model.getLipidCountIntra(); i++) {
			Molecule_3D test = (model.getLipidAtIntra(i));
			for (int j = 0; j < except.size(); j++) {
				if (test == (except.get(j).getOriginalMolecule())) {
					test = null;
					break;
				}
			}
			if (test != null) {
				allMolecules.add(test);
			}
		}

		for (int i = 0; i < model.getProteinCount(); i++) {
			Molecule_3D test = (model.getProteinAt(i));
			for (int j = 0; j < except.size(); j++) {
				if (test == (except.get(j).getOriginalMolecule())) {
					test = null;
					break;
				}
			}
			if (test != null) {
				allMolecules.add(test);
			}
		}

		final float[] m = new float[3];
		m[0] = mol.getPosition().x;
		m[1] = mol.getPosition().y;
		m[2] = mol.getPosition().z;

		for (int i = 0; i < allMolecules.size(); i++) {

			final Molecule_3D test = allMolecules.get(i);

			if (test != null) {
				final float[] t = new float[3];
				t[0] = test.getPosition().x;
				t[1] = test.getPosition().y;
				t[2] = test.getPosition().z;

				final float[] rv = new float[3];
				rv[0] = t[0] - m[0];
				rv[1] = t[1] - m[1];
				rv[2] = t[2] - m[2];

				final double dist = Math.sqrt(Math.pow(rv[0], 2)
						+ Math.pow(rv[1], 2) + Math.pow(rv[2], 2));
				if (dist < nearestDistance) {
					nearestDistance = dist;
					nearest = new Molecule(test, this);
				}
			}
		}
		return nearest;

	}

	/**
	 * Returns the nearest molecule to the given molecule mol. Molecules in the
	 * except Vector are ignored. If no other molecule is added to the membrane
	 * model, the return value will be null. This is the detailed version of the
	 * getNearestMoleculeExcept method. The boolean detailed indicates if you
	 * want detailed calculation or even more detailed calculation. Setting
	 * detailed = false will help you out in most cases. If you need it very
	 * precise you can set detailed = true. This provides more detailed distance
	 * calculation at the cost of time.
	 * 
	 * @author Sebastian Schneider
	 */
	public Molecule getNearestMoleculeExceptDetailed(final Molecule mol,
			final Vector<Molecule> except, final boolean detailed) {
		final MembraneModel model = DataPool.getMembraneModel();
		Molecule nearest = null;
		float nearestDistance = 10000f;

		final Vector<Molecule_3D> allMolecules = new Vector<Molecule_3D>();
		for (int i = 0; i < model.getLipidCountExtra(); i++) {
			Molecule_3D test = (model.getLipidAtExtra(i));
			for (int j = 0; j < except.size(); j++) {
				if (test == (except.get(j).getOriginalMolecule())) {
					test = null;
					break;
				}
			}
			if (test != null) {
				allMolecules.add(test);
			}
		}

		for (int i = 0; i < model.getLipidCountIntra(); i++) {
			Molecule_3D test = (model.getLipidAtIntra(i));
			for (int j = 0; j < except.size(); j++) {
				if (test == (except.get(j).getOriginalMolecule())) {
					test = null;
					break;
				}
			}
			if (test != null) {
				allMolecules.add(test);
			}
		}

		for (int i = 0; i < model.getProteinCount(); i++) {
			Molecule_3D test = (model.getProteinAt(i));
			for (int j = 0; j < except.size(); j++) {
				if (test == (except.get(j).getOriginalMolecule())) {
					test = null;
					break;
				}
			}
			if (test != null) {
				allMolecules.add(test);
			}
		}

		for (int i = 0; i < allMolecules.size(); i++) {

			final Molecule_3D test = allMolecules.get(i);

			if (test != null) {
				float dist;
				if (detailed == false) {
					dist = this.nearestTrianglePoints(mol, new Molecule(test,
							this));
				} else {
					dist = this.nearestTrianglePoints(mol, new Molecule(test,
							this));
				}

				if (dist < nearestDistance) {
					nearestDistance = dist;
					nearest = new Molecule(test, this);
				}
			}
		}
		return nearest;

	}

	/**
	 * Returns the nearest molecule to the given molecule mol on the determined
	 * membrane side. Molecules in the except Vector are ignored. If no other
	 * molecule is added to the membrane model for this membrane side, the
	 * return value will be null. Distance is measured from the center points of
	 * the molecules. Sometimes this leads to wrong outcomes.
	 * 
	 * @author Sebastian Schneider
	 */
	public Molecule getNearestMoleculeExceptOnMySide(final Molecule mol,
			final Vector<Molecule> except, final int membraneside) {
		final MembraneModel model = DataPool.getMembraneModel();
		Molecule nearest = null;
		double nearestDistance = 10000;

		final Vector<Molecule_3D> allMolecules = new Vector<Molecule_3D>();
		if (membraneside == MembranePackingAlgorithm.EXTRA) {
			for (int i = 0; i < model.getLipidCountExtra(); i++) {
				Molecule_3D test = model.getLipidAtExtra(i);
				for (int j = 0; j < except.size(); j++) {
					if (test == (except.get(j).getOriginalMolecule())) {
						test = null;
						break;
					}
				}
				if (test != null) {
					allMolecules.add(test);
				}
			}
		} else if (membraneside == MembranePackingAlgorithm.INTRA) {
			for (int i = 0; i < model.getLipidCountIntra(); i++) {
				Molecule_3D test = (model.getLipidAtIntra(i));
				for (int j = 0; j < except.size(); j++) {
					if (test == (except.get(j).getOriginalMolecule())) {
						test = null;
						break;
					}
				}
				if (test != null) {
					allMolecules.add(test);
				}
			}
		}
		for (int i = 0; i < model.getProteinCount(); i++) {
			Molecule_3D test = model.getProteinAt(i);
			for (int j = 0; j < except.size(); j++) {
				if (test == (except.get(j).getOriginalMolecule())) {
					test = null;
					break;
				}
			}
			if (test != null) {
				allMolecules.add(test);
			}
		}

		final float[] m = new float[3];
		m[0] = mol.getPosition().x;
		m[1] = mol.getPosition().y;
		m[2] = mol.getPosition().z;

		for (int i = 0; i < allMolecules.size(); i++) {

			final Molecule_3D test = allMolecules.get(i);

			if (test != null) {
				final float[] t = new float[3];
				t[0] = test.getPosition().x;
				t[1] = test.getPosition().y;
				t[2] = test.getPosition().z;

				final float[] rv = new float[3];
				rv[0] = t[0] - m[0];
				rv[1] = t[1] - m[1];
				rv[2] = t[2] - m[2];

				final double dist = Math.sqrt(Math.pow(rv[0], 2)
						+ Math.pow(rv[1], 2) + Math.pow(rv[2], 2));
				if (dist < nearestDistance) {
					nearestDistance = dist;
					nearest = new Molecule(test, this);
				}
			}
		}
		return nearest;

	}

	/**
	 * Returns the nearest molecule to the given molecule mol on the determined
	 * membrane side. Molecules in the except Vector are ignored. If no other
	 * molecule is added to the membrane model for this membrane side, the
	 * return value will be null. This is the detailed version of the
	 * getNearestMoleculeExceptOnMySide method. The boolean detailed indicates
	 * if you want detailed calculation or even more detailed calculation.
	 * Setting detailed = false will help you out in most cases. If you need it
	 * very precise you can set detailed = true. This provides more detailed
	 * distance calculation at the cost of time.
	 * 
	 * @author Sebastian Schneider
	 */
	public Molecule getNearestMoleculeExceptOnMySideDetailed(
			final Molecule mol, final Vector<Molecule> except,
			final int membraneside, final boolean detailed) {
		final MembraneModel model = DataPool.getMembraneModel();
		Molecule nearest = null;
		float nearestDistance = 10000f;

		final Vector<Molecule_3D> allMolecules = new Vector<Molecule_3D>();
		if (membraneside == MembranePackingAlgorithm.EXTRA) {
			for (int i = 0; i < model.getLipidCountExtra(); i++) {
				Molecule_3D test = model.getLipidAtExtra(i);
				for (int j = 0; j < except.size(); j++) {
					if (test == (except.get(j).getOriginalMolecule())) {
						test = null;
						break;
					}
				}
				if (test != null) {
					allMolecules.add(test);
				}
			}
		} else if (membraneside == MembranePackingAlgorithm.INTRA) {
			for (int i = 0; i < model.getLipidCountIntra(); i++) {
				Molecule_3D test = (model.getLipidAtIntra(i));
				for (int j = 0; j < except.size(); j++) {
					if (test == (except.get(j).getOriginalMolecule())) {
						test = null;
						break;
					}
				}
				if (test != null) {
					allMolecules.add(test);
				}
			}
		}
		for (int i = 0; i < model.getProteinCount(); i++) {
			Molecule_3D test = model.getProteinAt(i);
			for (int j = 0; j < except.size(); j++) {
				if (test == (except.get(j).getOriginalMolecule())) {
					test = null;
					break;
				}
			}
			if (test != null) {
				allMolecules.add(test);
			}
		}

		for (int i = 0; i < allMolecules.size(); i++) {

			final Molecule test = new Molecule(allMolecules.get(i), this);

			if (test != null) {
				float dist;
				if (detailed == false) {
					dist = this.nearestTrianglePoints(mol, test);
				} else {
					dist = this.nearestAtomPoints(mol, test);
				}

				if (dist < nearestDistance) {
					nearestDistance = dist;
					nearest = (test);
				}
			}
		}
		return nearest;

	}

	/**
	 * Returns the nearest protein. Proteins in the except vector are ignored.
	 * 
	 * Distance is measured from the center points of the molecules. Sometimes
	 * this leads to wrong outcomes.
	 * 
	 * @author Sebastian Schneider
	 */
	public Protein getNearestProteinExcept(final Molecule mol,
			final Vector<Protein> except) {
		final MembraneModel model = DataPool.getMembraneModel();
		Protein nearest = null;
		double nearestDistance = 10000;

		final Vector<Protein_3D> allMolecules = new Vector<Protein_3D>();

		for (int i = 0; i < model.getProteinCount(); i++) {
			Protein_3D test = model.getProteinAt(i);
			for (int j = 0; j < except.size(); j++) {
				if (test == (except.get(j).getOriginalMolecule())) {
					test = null;
					break;
				}
			}
			if (test != null) {
				allMolecules.add(test);
			}
		}

		final float[] m = new float[3];
		m[0] = mol.getPosition().x;
		m[1] = mol.getPosition().y;
		m[2] = mol.getPosition().z;

		for (int i = 0; i < allMolecules.size(); i++) {

			final Protein_3D test = allMolecules.get(i);

			if (test != null) {
				final float[] t = new float[3];
				t[0] = test.getPosition().x;
				t[1] = test.getPosition().y;
				t[2] = test.getPosition().z;

				final float[] rv = new float[3];
				rv[0] = t[0] - m[0];
				rv[1] = t[1] - m[1];
				rv[2] = t[2] - m[2];

				final double dist = Math.sqrt(Math.pow(rv[0], 2)
						+ Math.pow(rv[1], 2) + Math.pow(rv[2], 2));
				if (dist < nearestDistance) {
					nearestDistance = dist;
					nearest = getProtein(test);
				}
			}
		}
		return nearest;
	}

	/**
	 * 
	 * Returns the nearest protein. Proteins in the except vector are ignored.
	 * 
	 * This is the detailed version of the getNearestProteinExcept method. The
	 * boolean detailed indicates if you want detailed calculation or even more
	 * detailed calculation. Setting detailed = false will help you out in most
	 * cases. If you need it very precise you can set detailed = true. This
	 * provides more detailed distance calculation at the cost of time.
	 * 
	 * @author Sebastian Schneider
	 */
	public Protein getNearestProteinExceptDetailed(final Molecule mol,
			final Vector<Protein> except, final boolean detailed) {
		final MembraneModel model = DataPool.getMembraneModel();
		Protein nearest = null;
		float nearestDistance = 10000f;

		final Vector<Protein_3D> allMolecules = new Vector<Protein_3D>();

		for (int i = 0; i < model.getProteinCount(); i++) {
			Protein_3D test = model.getProteinAt(i);
			for (int j = 0; j < except.size(); j++) {
				if (test == (except.get(j).getOriginalMolecule())) {
					test = null;
					break;
				}
			}
			if (test != null) {
				allMolecules.add(test);
			}
		}

		for (int i = 0; i < allMolecules.size(); i++) {

			final Protein_3D test = allMolecules.get(i);

			if (test != null) {
				float dist;
				if (detailed == false) {
					dist = this.nearestTrianglePoints(mol, getProtein(test));
				} else {
					dist = this.nearestAtomPoints(mol, getProtein(test));
				}

				if (dist < nearestDistance) {
					nearestDistance = dist;
					nearest = getProtein(test);
				}
			}
		}
		return nearest;
	}

	/**
	 * Returns all molecules within the given radius of the molecule mol.
	 * 
	 * Distance is measured from the center points of the molecules. Sometimes
	 * this leads to wrong outcomes.
	 * 
	 * @author Sebastian Schneider
	 */
	public Vector<Molecule> getMoleculesInRadius(final Molecule mol,
			final int radius) {
		final MembraneModel model = DataPool.getMembraneModel();

		final Vector<Molecule> ret = new Vector<Molecule>();

		final float[] m = new float[3];
		m[0] = mol.getPosition().x;
		m[1] = mol.getPosition().y;
		m[2] = mol.getPosition().z;

		for (int i = 0; i < model.getAllLipidsExtra().size(); i++) {

			final Molecule test = new Molecule(model.getLipidAtExtra(i), this);
			final float[] t = new float[3];
			t[0] = test.getPosition().x;
			t[1] = test.getPosition().y;
			t[2] = test.getPosition().z;

			final float[] rv = new float[3];
			rv[0] = t[0] - m[0];
			rv[1] = t[1] - m[1];
			rv[2] = t[2] - m[2];

			final double dist = Math.sqrt(Math.pow(rv[0], 2)
					+ Math.pow(rv[1], 2) + Math.pow(rv[2], 2));
			if (dist <= radius) {
				ret.add(test);
			}

		}

		for (int i = 0; i < model.getAllLipidsIntra().size(); i++) {

			final Molecule test = new Molecule(model.getLipidAtIntra(i), this);
			final float[] t = new float[3];
			t[0] = test.getPosition().x;
			t[1] = test.getPosition().y;
			t[2] = test.getPosition().z;

			final float[] rv = new float[3];
			rv[0] = t[0] - m[0];
			rv[1] = t[1] - m[1];
			rv[2] = t[2] - m[2];

			final double dist = Math.sqrt(Math.pow(rv[0], 2)
					+ Math.pow(rv[1], 2) + Math.pow(rv[2], 2));
			if (dist <= radius) {
				ret.add(test);
			}

		}

		for (int i = 0; i < model.getAllProteins().size(); i++) {

			final Molecule test = new Molecule(model.getProteinAt(i), this);
			final float[] t = new float[3];
			t[0] = test.getPosition().x;
			t[1] = test.getPosition().y;
			t[2] = test.getPosition().z;

			final float[] rv = new float[3];
			rv[0] = t[0] - m[0];
			rv[1] = t[1] - m[1];
			rv[2] = t[2] - m[2];

			final double dist = Math.sqrt(Math.pow(rv[0], 2)
					+ Math.pow(rv[1], 2) + Math.pow(rv[2], 2));
			if (dist <= radius) {
				ret.add(test);
			}

		}

		return ret;
	}

	/**
	 * Returns all molecules within the given radius of the Point3f on given
	 * membrane side.
	 * 
	 * Distance is measured from the center points of the molecules. Sometimes
	 * this leads to wrong outcomes.
	 * 
	 * @author Sebastian Schneider
	 */
	public Vector<Molecule> getMoleculesInRadiusOnMySide(final Point3f mol,
			final int radius, final int membraneside) {
		final MembraneModel model = DataPool.getMembraneModel();

		final Vector<Molecule> ret = new Vector<Molecule>();

		final float[] m = new float[3];
		m[0] = mol.x;
		m[1] = mol.y;
		m[2] = mol.z;

		if (membraneside == MembranePackingAlgorithm.EXTRA) {
			for (int i = 0; i < model.getAllLipidsExtra().size(); i++) {

				final Molecule test = new Molecule(model.getLipidAtExtra(i),
						this);
				final float[] t = new float[3];
				t[0] = test.getPosition().x;
				t[1] = test.getPosition().y;
				t[2] = test.getPosition().z;

				final float[] rv = new float[3];
				rv[0] = t[0] - m[0];
				rv[1] = t[1] - m[1];
				rv[2] = t[2] - m[2];

				final double dist = Math.sqrt(Math.pow(rv[0], 2)
						+ Math.pow(rv[1], 2) + Math.pow(rv[2], 2));
				if (dist <= radius) {
					ret.add(test);
				}

			}
		} else if (membraneside == MembranePackingAlgorithm.INTRA) {
			for (int i = 0; i < model.getAllLipidsIntra().size(); i++) {

				final Molecule test = new Molecule(model.getLipidAtIntra(i),
						this);
				final float[] t = new float[3];
				t[0] = test.getPosition().x;
				t[1] = test.getPosition().y;
				t[2] = test.getPosition().z;

				final float[] rv = new float[3];
				rv[0] = t[0] - m[0];
				rv[1] = t[1] - m[1];
				rv[2] = t[2] - m[2];

				final double dist = Math.sqrt(Math.pow(rv[0], 2)
						+ Math.pow(rv[1], 2) + Math.pow(rv[2], 2));
				if (dist <= radius) {
					ret.add(test);
				}

			}
		}
		for (int i = 0; i < model.getAllProteins().size(); i++) {

			final Molecule test = new Molecule(model.getProteinAt(i), this);
			final float[] t = new float[3];
			t[0] = test.getPosition().x;
			t[1] = test.getPosition().y;
			t[2] = test.getPosition().z;

			final float[] rv = new float[3];
			rv[0] = t[0] - m[0];
			rv[1] = t[1] - m[1];
			rv[2] = t[2] - m[2];

			final double dist = Math.sqrt(Math.pow(rv[0], 2)
					+ Math.pow(rv[1], 2) + Math.pow(rv[2], 2));
			if (dist <= radius) {
				ret.add(test);
			}

		}

		return ret;
	}

	/**
	 * Returns all molecules within the given radius of the molecule mol on the
	 * given membrane side.
	 * 
	 * Distance is measured from the center points of the molecules. Sometimes
	 * this leads to wrong outcomes.
	 * 
	 * @author Sebastian Schneider
	 */
	public Vector<Molecule> getMoleculesInRadiusOnMySide(final Molecule mol,
			final int radius, final int membraneside) {
		final MembraneModel model = DataPool.getMembraneModel();

		final Vector<Molecule> ret = new Vector<Molecule>();

		final float[] m = new float[3];
		m[0] = mol.getPosition().x;
		m[1] = mol.getPosition().y;
		m[2] = mol.getPosition().z;

		if (membraneside == MembranePackingAlgorithm.EXTRA) {
			for (int i = 0; i < model.getAllLipidsExtra().size(); i++) {

				final Molecule test = new Molecule(model.getLipidAtExtra(i),
						this);
				final float[] t = new float[3];
				t[0] = test.getPosition().x;
				t[1] = test.getPosition().y;
				t[2] = test.getPosition().z;

				final float[] rv = new float[3];
				rv[0] = t[0] - m[0];
				rv[1] = t[1] - m[1];
				rv[2] = t[2] - m[2];

				final double dist = Math.sqrt(Math.pow(rv[0], 2)
						+ Math.pow(rv[1], 2) + Math.pow(rv[2], 2));
				if (dist <= radius) {
					ret.add(test);
				}

			}
		} else if (membraneside == MembranePackingAlgorithm.INTRA) {
			for (int i = 0; i < model.getAllLipidsIntra().size(); i++) {

				final Molecule test = new Molecule(model.getLipidAtIntra(i),
						this);
				final float[] t = new float[3];
				t[0] = test.getPosition().x;
				t[1] = test.getPosition().y;
				t[2] = test.getPosition().z;

				final float[] rv = new float[3];
				rv[0] = t[0] - m[0];
				rv[1] = t[1] - m[1];
				rv[2] = t[2] - m[2];

				final double dist = Math.sqrt(Math.pow(rv[0], 2)
						+ Math.pow(rv[1], 2) + Math.pow(rv[2], 2));
				if (dist <= radius) {
					ret.add(test);
				}

			}
		}
		for (int i = 0; i < model.getAllProteins().size(); i++) {

			final Molecule test = new Molecule(model.getProteinAt(i), this);
			final float[] t = new float[3];
			t[0] = test.getPosition().x;
			t[1] = test.getPosition().y;
			t[2] = test.getPosition().z;

			final float[] rv = new float[3];
			rv[0] = t[0] - m[0];
			rv[1] = t[1] - m[1];
			rv[2] = t[2] - m[2];

			final double dist = Math.sqrt(Math.pow(rv[0], 2)
					+ Math.pow(rv[1], 2) + Math.pow(rv[2], 2));
			if (dist <= radius) {
				ret.add(test);
			}

		}

		return ret;
	}

	/**
	 * 
	 * Returns all lipids within the given radius of the molecule mol. The
	 * lipids will be returned cast to molecules. Molecules in the except vector
	 * are ignored.
	 * 
	 * Distance is measured from the center points of the molecules. Sometimes
	 * this leads to wrong outcomes.
	 * 
	 * @author Sebastian Schneider
	 */
	public Vector<Molecule> getLipidsInRadiusExcept(final Molecule mol,
			final int radius, final Vector<Molecule> except) {
		final MembraneModel model = DataPool.getMembraneModel();
		final Vector<Molecule> ret = new Vector<Molecule>();

		final Vector<Molecule_3D> allMolecules = new Vector<Molecule_3D>();
		for (int i = 0; i < model.getLipidCountExtra(); i++) {
			Molecule_3D test = model.getLipidAtExtra(i);
			for (int j = 0; j < except.size(); j++) {
				if (test == (except.get(j).getOriginalMolecule())) {
					test = null;
					break;
				}
			}
			if (test != null) {
				allMolecules.add(test);
			}
		}

		for (int i = 0; i < model.getLipidCountIntra(); i++) {
			Molecule_3D test = (model.getLipidAtIntra(i));
			for (int j = 0; j < except.size(); j++) {
				if (test == (except.get(j).getOriginalMolecule())) {
					test = null;
					break;
				}
			}
			if (test != null) {
				allMolecules.add(test);
			}
		}

		final float[] m = new float[3];
		m[0] = mol.getPosition().x;
		m[1] = mol.getPosition().y;
		m[2] = mol.getPosition().z;

		for (int i = 0; i < allMolecules.size(); i++) {

			final Molecule_3D test = allMolecules.get(i);

			if (test != null) {
				final float[] t = new float[3];
				t[0] = test.getPosition().x;
				t[1] = test.getPosition().y;
				t[2] = test.getPosition().z;

				final float[] rv = new float[3];
				rv[0] = t[0] - m[0];
				rv[1] = t[1] - m[1];
				rv[2] = t[2] - m[2];

				final double dist = Math.sqrt(Math.pow(rv[0], 2)
						+ Math.pow(rv[1], 2) + Math.pow(rv[2], 2));
				if (dist <= radius) {
					ret.add(new Molecule(test, this));
				}
			}
		}
		return ret;

	}

	/**
	 * Returns all molecules within the given radius of the molecule mol.
	 * Molecules in the except vector are ignored.
	 * 
	 * Distance is measured from the center points of the molecules. Sometimes
	 * this leads to wrong outcomes.
	 * 
	 * @author Sebastian Schneider
	 */
	public Vector<Molecule> getMoleculesInRadiusExcept(final Molecule mol,
			final int radius, final Vector<Molecule> except) {
		final MembraneModel model = DataPool.getMembraneModel();
		final Vector<Molecule> ret = new Vector<Molecule>();

		final Vector<Molecule_3D> allMolecules = new Vector<Molecule_3D>();
		for (int i = 0; i < model.getLipidCountExtra(); i++) {
			Molecule_3D test = (model.getLipidAtExtra(i));
			for (int j = 0; j < except.size(); j++) {
				if (test == (except.get(j).getOriginalMolecule())) {
					test = null;
					break;
				}
			}
			if (test != null) {
				allMolecules.add(test);
			}
		}

		for (int i = 0; i < model.getLipidCountIntra(); i++) {
			Molecule_3D test = (model.getLipidAtIntra(i));
			for (int j = 0; j < except.size(); j++) {
				if (test == (except.get(j).getOriginalMolecule())) {
					test = null;
					break;
				}
			}
			if (test != null) {
				allMolecules.add(test);
			}
		}

		for (int i = 0; i < model.getProteinCount(); i++) {
			Molecule_3D test = (model.getProteinAt(i));
			for (int j = 0; j < except.size(); j++) {
				if (test == (except.get(j).getOriginalMolecule())) {
					test = null;
					break;
				}
			}
			if (test != null) {
				allMolecules.add(test);
			}
		}

		final float[] m = new float[3];
		m[0] = mol.getPosition().x;
		m[1] = mol.getPosition().y;
		m[2] = mol.getPosition().z;

		for (int i = 0; i < allMolecules.size(); i++) {

			final Molecule_3D test = allMolecules.get(i);

			if (test != null) {
				final float[] t = new float[3];
				t[0] = test.getPosition().x;
				t[1] = test.getPosition().y;
				t[2] = test.getPosition().z;

				final float[] rv = new float[3];
				rv[0] = t[0] - m[0];
				rv[1] = t[1] - m[1];
				rv[2] = t[2] - m[2];

				final double dist = Math.sqrt(Math.pow(rv[0], 2)
						+ Math.pow(rv[1], 2) + Math.pow(rv[2], 2));
				if (dist <= radius) {
					ret.add(new Molecule(test, this));
				}
			}
		}
		return ret;

	}

	/**
	 * Returns all molecules within the given radius of the molecule mol on the
	 * given membrane side. Molecules in the except vector are ignored.
	 * 
	 * Distance is measured from the center points of the molecules. Sometimes
	 * this leads to wrong outcomes.
	 * 
	 * @author Sebastian Schneider
	 */
	public Vector<Molecule> getMoleculesInRadiusOnMySideExcept(
			final Molecule mol, final int radius, final int membraneside,
			final Vector<Molecule> except) {
		final MembraneModel model = DataPool.getMembraneModel();
		final Vector<Molecule> ret = new Vector<Molecule>();

		final Vector<Molecule_3D> allMolecules = new Vector<Molecule_3D>();
		if (membraneside == MembranePackingAlgorithm.EXTRA) {
			for (int i = 0; i < model.getLipidCountExtra(); i++) {
				Molecule_3D test = (model.getLipidAtExtra(i));
				for (int j = 0; j < except.size(); j++) {
					if (test == (except.get(j).getOriginalMolecule())) {
						test = null;
						break;
					}
				}
				if (test != null) {
					allMolecules.add(test);
				}
			}
		} else if (membraneside == MembranePackingAlgorithm.INTRA) {
			for (int i = 0; i < model.getLipidCountIntra(); i++) {
				Molecule_3D test = (model.getLipidAtIntra(i));
				for (int j = 0; j < except.size(); j++) {
					if (test == (except.get(j).getOriginalMolecule())) {
						test = null;
						break;
					}
				}
				if (test != null) {
					allMolecules.add(test);
				}
			}
		}
		for (int i = 0; i < model.getProteinCount(); i++) {
			Molecule_3D test = (model.getProteinAt(i));
			for (int j = 0; j < except.size(); j++) {
				if (test == (except.get(j).getOriginalMolecule())) {
					test = null;
					break;
				}
			}
			if (test != null) {
				allMolecules.add(test);
			}
		}

		final float[] m = new float[3];
		m[0] = mol.getPosition().x;
		m[1] = mol.getPosition().y;
		m[2] = mol.getPosition().z;

		for (int i = 0; i < allMolecules.size(); i++) {

			final Molecule_3D test = allMolecules.get(i);

			if (test != null) {
				final float[] t = new float[3];
				t[0] = test.getPosition().x;
				t[1] = test.getPosition().y;
				t[2] = test.getPosition().z;

				final float[] rv = new float[3];
				rv[0] = t[0] - m[0];
				rv[1] = t[1] - m[1];
				rv[2] = t[2] - m[2];

				final double dist = Math.sqrt(Math.pow(rv[0], 2)
						+ Math.pow(rv[1], 2) + Math.pow(rv[2], 2));
				if (dist <= radius) {
					ret.add(new Molecule(test, this));
				}
			}
		}
		return ret;

	}

	/**
	 * Returns the distance between mol1 and mol2. If the absval flag is set
	 * true, the absolute value of the distance will be returned.
	 * 
	 * @author Sebastian Schneider
	 */
	public float distance(final Molecule mol1, final Molecule mol2,
			final boolean absval) {
		float distance = 0;
		final Point3f p1 = new Point3f(mol1.getPosition());
		final Point3f p2 = new Point3f(mol2.getPosition());

		final float[] vec1 = new float[3];
		final float[] vec2 = new float[3];

		vec1[0] = p1.x;
		vec1[1] = p1.y;
		vec1[2] = p1.z;

		vec2[0] = p2.x;
		vec2[1] = p2.y;
		vec2[2] = p2.z;

		final float[] rVec = new float[3];
		rVec[0] = vec2[0] - vec1[0];
		rVec[1] = vec2[1] - vec1[1];
		rVec[2] = vec2[2] - vec1[2];

		distance = (float) Math.sqrt(Math.pow(rVec[0], 2)
				+ Math.pow(rVec[1], 2) + Math.pow(rVec[2], 2));

		if (absval == true) {
			distance = Math.abs(distance);
		}
		return distance;
	}

	/**
	 * Returns the distance between tow positions. If the absval flag is set
	 * true, the absolute value of the distance will be returned.
	 * 
	 * @author Sebastian Schneider
	 */
	public float distance(final Point3f p1, final Point3f p2,
			final boolean absval) {
		float distance = 0;

		final float[] vec1 = new float[3];
		final float[] vec2 = new float[3];

		vec1[0] = p1.x;
		vec1[1] = p1.y;
		vec1[2] = p1.z;

		vec2[0] = p2.x;
		vec2[1] = p2.y;
		vec2[2] = p2.z;

		final float[] rVec = new float[3];
		rVec[0] = vec2[0] - vec1[0];
		rVec[1] = vec2[1] - vec1[1];
		rVec[2] = vec2[2] - vec1[2];

		distance = (float) Math.sqrt(Math.pow(rVec[0], 2)
				+ Math.pow(rVec[1], 2) + Math.pow(rVec[2], 2));

		if (absval == true) {
			distance = Math.abs(distance);
		}
		return distance;
	}

	/**
	 * Determines the distance between two molecules about the nearest triangles
	 * of each molecule to the other molecule. This is more detailed then all
	 * the other getNearest... methods but takes more time.
	 * 
	 * @author Sebastian Schneider
	 */
	public float nearestTrianglePoints(final Molecule mol1, final Molecule mol2) {
		final Point3f[] protAngle = mol2.getTrianglePoints();
		final Point3f[] molAngle = mol1.getTrianglePoints();

		float dis = 10000;
		for (int i = 0; i < protAngle.length; i += 3) {
			for (int j = 0; j < molAngle.length; j += 3) {
				final Vector<Point3f> mols = new Vector<Point3f>();
				mols.add(molAngle[j]);
				mols.add(molAngle[j + 1]);
				mols.add(molAngle[j + 2]);

				final Vector<Point3f> prots = new Vector<Point3f>();
				prots.add(protAngle[i]);
				prots.add(protAngle[i + 1]);
				prots.add(protAngle[i + 2]);
				for (int k = 0; k < mols.size(); k++) {
					for (int t = 0; t < prots.size(); t++) {
						if (this.distance(mols.get(k), prots.get(t), true) < dis) {
							dis = this
									.distance(mols.get(k), prots.get(t), true);
						}
					}
				}

			}
		}
		return dis;
	}

	/**
	 * Determines the distance between two molecules about the nearest atoms of
	 * each molecule to the other molecule. This is more detailed then all the
	 * other getNearest... methods. If the detail level of the molecules is low
	 * this method is more detailed then the nearestTrianglePoints() method but
	 * needs more time. Especially when used to get distance between one or more
	 * proteins. (Evens worse when size of proteins increases)
	 * 
	 * @author Sebastian Schneider
	 */
	public float nearestAtomPoints(final Molecule mol1, final Molecule mol2) {
		final Atom[] protAtom = mol2.getAlignedAtomCopies();
		final Atom[] atoms = mol1.getAlignedAtomCopies();

		float distanceAtom = 1000f;
		for (int q = 0; q < protAtom.length; q++) {
			for (int j = 0; j < atoms.length; j++) {
				final float dd = this.distance(protAtom[q].position,
						atoms[j].position, true);
				if (dd < distanceAtom) {
					distanceAtom = dd;
				}
			}
		}

		return distanceAtom;
	}

	/**
	 * Tests if the center of the passed molecule is located inside a
	 * microdomain
	 * 
	 * @param mol the
	 *            molecule to test against
	 * @return true if inside a microdomain, false if not (if it's inside the
	 *         default area)
	 */
	public boolean intersectsAnyMicroDomain(final Molecule mol) {
		final Point3f p = mol.getPosition();
		if (DataPool.getMembraneModel().getMicroDomainAtPoint(
				new Point((int) p.x, (int) p.z), mol.getLayerID()) == 0)
			return false;
		else
			return true;
	}

	/**
	 * Tests if the passed point is inside the area of a microdomain of the
	 * given membrane layer and returns the Domain_ID for it
	 * 
	 * @param point The
	 *            point to test for
	 * 
	 * @return the domain_ID of the microdomain that contains the passed point
	 *         or 0 if it's inside the default area or -1 if the point is
	 *         completely outside the membrane bounds
	 */
	public int getMicroDomainAtPoint(final Point2f point, final int layerID) {

		return DataPool.getMembraneModel().getMicroDomainAtPoint(
				new Point((int) point.x, (int) point.y), layerID);

	}

	/**
	 * Tests if the passed point is inside the area of a microdomain of the
	 * active membrane layer and returns the Domain_ID for it
	 * 
	 * @param point The
	 *            point to test for
	 * @return the domain_ID of the microdomain that contains the passed point
	 *         or 0 if it's inside the default area or -1 if the point is
	 *         completely outside the membrane bounds
	 */
	public int getMicroDomainAtPoint(final Point2f point) {
		final int layerID = this.getCurrentLayerID();
		return DataPool.getMembraneModel().getMicroDomainAtPoint(
				new Point((int) point.x, (int) point.y), layerID);

	}

	/**
	 * Returns the count of microdomains in the membrane
	 */
	public int getMicroDomainCount() {
		return this.microDomains.size();
	}

	/**
	 * Returns the microdomain object at the specified index
	 * 
	 * @param index the
	 *            specified index
	 * @return the microdomain object at the specified index
	 */
	public MicroDomain getMicroDomainAt(final int index) {
		return this.microDomains.get(index);
	}

	/**
	 * Returns the microdomain object that has the specified id
	 * 
	 * @param domainID
	 *            the domainID to look for
	 * @return the microdomain object with the specified id
	 */
	public MicroDomain getMicroDomain(final int domainID) {
		for (int i = 0; i < this.getMicroDomainCount(); i++) {
			if (this.getMicroDomainAt(i).getDomainID() == domainID)
				return this.getMicroDomainAt(i);
		}
		return null;
	}

	/**
	 * Randomly creates a Point that has random x/z coordinates, including
	 * decimal values
	 * 
	 * @param bounds the
	 *            bounds used as random space
	 * @return a randomly created point2f
	 */
	public Point2f createRandomXZPosition(Rectangle bounds) {

		if (bounds == null)
			bounds = new Rectangle(0, 0, this.getMembraneSize().width, this
					.getMembraneSize().height);

		if (bounds.width < 2) {
			bounds = new Rectangle(bounds.x, bounds.y, 2, bounds.width);
		}
		if (bounds.height < 2) {
			bounds = new Rectangle(bounds.x, bounds.y, bounds.width, 2);
		}

		float x = this.random.nextInt(bounds.width - 1);
		float z = this.random.nextInt(bounds.height - 1);

		if (x < 0)
			x = 0;
		if (z < 0)
			z = 0;

		x += this.random.nextFloat();
		z += this.random.nextFloat();

		x += bounds.x;
		z += bounds.y;

		return new Point2f(x, z);
	}

	/**
	 * Simply clones a matrix. Works better than the vecmath clone method that
	 * appears to be a little buggy
	 * 
	 * @param m the
	 *            matrix to copy
	 * @return a copy of the passed matrix
	 */
	public Matrix3f copyMatrix(final Matrix3f m) {
		return Cm2_Toolkit.copyMatrix(m);
	}

	/**
	 * Creates a new unit matrix
	 * 
	 * @return a new unit matrix
	 */
	public Matrix3f createAUnitMatrix() {
		return Cm2_Toolkit.createAUnitMatrix();
	}

	/**
	 * Returns the number of lipid templates on the extracellular side of the
	 * default area
	 * 
	 * @param layerID the
	 *            id of the layer taken into account
	 */
	public int countSampleLipids_Extracellular(final int layerID) {
		int c = 0;
		for (int i = 0; i < this.lipidsEC.length; i++) {
			if (this.lipidsEC[i].getLayerID() == layerID)
				c++;
		}

		return c;

	}

	/**
	 * Returns the number of lipid templates on the intracellular side of the
	 * default area
	 * 
	 * @param layerID the
	 *            id of the layer taken into account
	 */
	public int countSampleLipids_Intracellular(final int layerID) {
		int c = 0;
		for (int i = 0; i < this.lipidsIC.length; i++) {
			if (this.lipidsIC[i].getLayerID() == layerID)
				c++;
		}

		return c;
	}

	/**
	 * Returns the extracellular lipid template at the specified index.
	 * 
	 * @param index the
	 *            index of the lipid template
	 * @param layerID the
	 *            id of the layer taken into account
	 * @return a copy of the indexed lipid template
	 */
	public Lipid getSampleLipid_Extracellular(final int index, final int layerID) {
		final Vector<Lipid_3D> ret = new Vector<Lipid_3D>();
		for (int i = 0; i < this.lipidsEC.length; i++) {
			if (this.lipidsEC[i].getLayerID() == layerID)
				ret.addElement(this.lipidsEC[i]);
		}

		return this.getLipid((Lipid_3D) ret.get(index).copy(true, false));

	}

	/**
	 * Returns the intracellular lipid template at the specified index.
	 * 
	 * @param index the
	 *            index of the lipid template
	 * @param layerID the
	 *            id of the layer taken into account
	 * @return a copy of the indexed lipid template
	 */
	public Lipid getSampleLipid_Intracellular(final int index, final int layerID) {

		final Vector<Lipid_3D> ret = new Vector<Lipid_3D>();
		for (int i = 0; i < this.lipidsIC.length; i++) {
			if (this.lipidsIC[i].getLayerID() == layerID)
				ret.addElement(this.lipidsIC[i]);
		}

		return this.getLipid((Lipid_3D) ret.get(index).copy(true, false));
	}

	/**
	 * Returns all lipid templates of the extracellular side of this microdomain
	 * 
	 * @param layerID the
	 *            id of the layer taken into account
	 */
	public Lipid[] getAllSampleLipids_ExtraCellular(final int layerID) {
		final Vector<Lipid> ret = new Vector<Lipid>();
		for (int i = 0; i < this.lipidsEC.length; i++) {
			if (this.lipidsEC[i].getLayerID() == layerID)
				ret.addElement(this.getLipid((Lipid_3D) this.lipidsEC[i].copy(
						true, false)));
		}
		return ret.toArray(new Lipid[0]);
	}
	
	/**
	 * Returns all lipid templates of the extracellular side of this microdomain
	 * 
	 * @param layerID the
	 *            id of the layer taken into account
	 */
	public Lipid[] getAllSampleLipids_ExtraCellular(final int layerID, final int domainID) {
		final Vector<Lipid> ret = new Vector<Lipid>();
		final Lipid_3D[] lipidsEC_1Domain = DataPool.getMembraneModel().getAllExampleLipidsExtraInDomain(domainID, layerID);
		for (int i = 0; i < lipidsEC_1Domain.length; i++) {
			if (lipidsEC_1Domain[i].getLayerID() == layerID && (lipidsEC_1Domain[i].getDomainID() == domainID || domainID == -1) )
				ret.addElement(this.getLipid((Lipid_3D) lipidsEC_1Domain[i].copy(
						true, false)));
		}
		return ret.toArray(new Lipid[0]);
	}

	/**
	 * Returns all lipid templates of the intracellular side of this microdomain
	 * 
	 * @param layerID the
	 *            id of the layer taken into account
	 */
	public Lipid[] getAllSampleLipids_IntraCellular(final int layerID) {
		final Vector<Lipid> ret = new Vector<Lipid>();
		for (int i = 0; i < this.lipidsIC.length; i++) {
			if (this.lipidsIC[i].getLayerID() == layerID)
				ret.addElement(this.getLipid((Lipid_3D) this.lipidsIC[i].copy(
						true, false)));
		}
		return ret.toArray(new Lipid[0]);
	}
	
	/**
	 * Returns all lipid templates of the intracellular side of this microdomain
	 * 
	 * @param layerID the
	 *            id of the layer taken into account
	 * @param domainID the
	 *            id of the microdomain taken into account
	 */
	public Lipid[] getAllSampleLipids_IntraCellular(final int layerID, final int domainID) {
		final Vector<Lipid> ret = new Vector<Lipid>();
		final Lipid_3D[] lipidsIC_1Domain = DataPool.getMembraneModel().getAllExampleLipidsIntraInDomain(domainID, layerID);
		for (int i = 0; i < lipidsIC_1Domain.length; i++) {
			if (lipidsIC_1Domain[i].getLayerID() == layerID && (lipidsIC_1Domain[i].getDomainID() == domainID || domainID == -1) )
				ret.addElement(this.getLipid((Lipid_3D) lipidsIC_1Domain[i].copy(
						true, false)));
		}
		return ret.toArray(new Lipid[0]);
	}

	/**
	 * Returns a list of all (!) lipid templates in the model
	 * 
	 * @return a list of all lipid templates
	 */
	public Lipid[] getAllSampleLipidsInModel() {
		final Vector<Lipid> ret = new Vector<Lipid>();

		for (int i = 0; i < DataPool.getMembraneModel()
				.getExampleLipidsCountExtra(); i++) {

			ret.addElement(this.getLipid((Lipid_3D) DataPool.getMembraneModel()
					.getExampleLipidAtExtra(i).copy(true, false)));
		}
		for (int i = 0; i < DataPool.getMembraneModel()
				.getExampleLipidsCountIntra(); i++) {

			ret.addElement(this.getLipid((Lipid_3D) DataPool.getMembraneModel()
					.getExampleLipidAtIntra(i).copy(true, false)));
		}
		return ret.toArray(new Lipid[0]);

	}

	/**
	 * NOT MEANT TO BE USED BY PLUGIN DEVELOPERS!
	 */
	public synchronized void dock() {

		if (this.requestRenderModeChange) {
			membraneViewListener.changeRenderModeFromAlgorithm();

			this.requestRenderModeChange = false;
		}

	}
	
	/**
	 * This method returns a CmME instance which has been sent
	 * to the SystemTray back to the screen.
	 *
	 */
	public void returnFromSystemTray() {
		calculationManager.returnFromSystemTray();
	} 

	/**
	 * This method hides a CmME instance in the SystemTray.
	 *
	 */
	public void sendToSystemTray() {
		calculationManager.sendToSystemTray();
	}

	/**
	 * Removes all added lipids from the membrane at once
	 */
	public void clearAllLipids() {
		this.membraneViewListener.clearAllLipidsFrom3DandModel(-1, DataPool
				.getCurrentLayerID());
	}

	/**
	 * Removes the passed list of lipids from the membrane
	 * 
	 * @param toremove a
	 *            list of lipids that you want to remove
	 */
	public void removeLipidsFromMembrane(final Lipid[] toremove) {
		if (toremove == null || toremove.length < 1)
			return;
		final Lipid_3D[] lips = new Lipid_3D[toremove.length];
		for (int i = 0; i < lips.length; i++) {
			lips[i] = (Lipid_3D) toremove[i].getOriginalMolecule();

		}
		this.membraneViewListener.removeLipidsFrom3D(lips);
		for (int i = 0; i < lips.length; i++) {
			DataPool.getMembraneModel().deleteLipidExtra(lips[i]);
			DataPool.getMembraneModel().deleteLipidIntra(lips[i]);
		}
		this.manuallyUpdateStatistics();
	}

	/**
	 * Forces the statistics to update on current values. This usually happens
	 * when you add or remove lipids, but it can also be forced independently,
	 * if desired
	 */
	public void manuallyUpdateStatistics() {
		this.calculationManager.updateMembraneInfo(true);
	}

	/**
	 * Returns the desired distance between atoms that the user can define. The
	 * intersection methods already respect these setting but it may help you
	 * while placing your molecules
	 * 
	 * @return the desired distance between atoms, or 0 if this feature is
	 *         disabled
	 */
	public float getDesiredAtomDistance() {
		if (DataPool.J3DSettings.atomCollisionCheck)
			return DataPool.J3DSettings.minatomdistance;
		else
			return 0;
	}

	/**
	 * Returns the number of Layers in the model
	 * 
	 */
	public int getLayerCount() {
		return DataPool.getMembraneModel().getLayerCount();
	}

	/**
	 * Returns the Layer Object at the specified index
	 * 
	 * @param index the
	 *            index of the layer object
	 */
	public Layer getLayerAtIndex(final int index) {
		return DataPool.getMembraneModel().getLayer(index);
	}

	/**
	 * Returns the Layer that has the specified Layer ID or null if the Layer
	 * doesn't exist
	 * 
	 * @param LayerID the
	 *            layerID to look for
	 */
	public Layer getLayerByLayerID(final int LayerID) {
		return DataPool.getMembraneModel().getLayerByID(LayerID);
	}

 	/**
	 * Returns true if the membrane contains extracellular lipids. False
	 * otherwise. This takes all domains of the selected membrane layer
	 * into account.
	 * 
	 * @return true if extracellular lipids exist, false otherwise 
	 */
 
	public boolean hasMembraneAnyExtracellularLipids() {
		if (DataPool.getMembraneModel().getAllExampleLipidsExtraInDomain
				(0,this.getCurrentLayerID()).length > 0)
			return true;
		for (int i=0;i<this.getMicroDomainCount();i++) {
			if (DataPool.getMembraneModel().getAllExampleLipidsExtraInDomain
					(i+1,this.getCurrentLayerID()).length > 0)
				return true;
		}
		return false;
	}
	
	/**
	 * @deprecated use hasMembraneAnyExtracellularLipids instead!
	 * @return boolean
	 */
	public boolean MembraneHasAnyExtracellularLipids() {
		return hasMembraneAnyExtracellularLipids();
	}

	/**
	 * Returns true if the membrane contains intracellular lipids. False
	 * otherwise. This takes all domains of the selected membrane layer
	 * into account.
	 * @return true if intracellular lipids exist, false otherwise
	 */
	public boolean hasMembraneAnyIntracellularLipids() {
		if (DataPool.getMembraneModel().getAllExampleLipidsIntraInDomain
				(0,this.getCurrentLayerID()).length > 0)
			return true;
		for (int i=0;i<this.getMicroDomainCount();i++) {
			if (DataPool.getMembraneModel().getAllExampleLipidsIntraInDomain
					(i+1,this.getCurrentLayerID()).length > 0)
				return true;
		}
		return false;
	}
	
	/**
	 * @deprecated use hasMembraneAnyIntracellularLipids instead!
	 * @return boolean
	 */
	public boolean MembraneHasAnyIntracellularLipids() {
		return hasMembraneAnyIntracellularLipids();
	}

	/**
	 * Returns true if the membrane contains proteins. False
	 * otherwise. This takes all domains of the selected membrane layer
	 * into account.
	 * 
	 */
	public boolean hasMembraneAnyProteins() {
		if (DataPool.getMembraneModel().getAllProteinsInDomain
				(0,this.getCurrentLayerID()).length > 0)
			return true;
		for (int i=0;i<this.getMicroDomainCount();i++) {
			if (DataPool.getMembraneModel().getAllProteinsInDomain
					(i+1,this.getCurrentLayerID()).length > 0)
				return true;
		}
		return false;
	}
	
	/**
	 * Checks if the model has more than the default layer
	 * 
	 */
	public boolean hasMultipleLayers() {
		return DataPool.getMembraneModel().hasMultipleLayers();
	}

	/**
	 * Returns the ID of the currently active layer
	 * 
	 * @return the id of the current layer
	 */
	public int getCurrentLayerID() {

		return DataPool.getCurrentLayerID();
	}

	/**
	 * Returns all Molecule Instances that are currently in the Membrane Model,
	 * including all Layers and Microdomains. Excluded are only the template
	 * Lipids
	 * 
	 * @return a list of all actual (!) members in the membrane
	 */
	public Molecule[] getAllMoleculesInCurrentModel() {
		Vector<Molecule> ret = new Vector<Molecule>();
		final MembraneModel model = DataPool.getMembraneModel();

		final Molecule_3D[] mols = model.getAllMoleculesInMembrane();
		for (int i = 0; i < mols.length; i++) {
			ret
					.add(mols[i] instanceof Protein_3D ? getProtein((Protein_3D) mols[i])
							: this.getLipid((Lipid_3D) mols[i]));
		}

		return ret.toArray(new Molecule[0]);
	}

	/**
	 * Returns the Directory inside the Workspace Location for this Algorithm.
	 * For each Algorithm, such a directory will be created at first startup. It
	 * contains a copy of all additional files that have been put into the jar
	 * file.
	 * 
	 * This directory should also serve as a root directory for algorithm
	 * specific setup files.
	 * 
	 */
	public File getAlgorithm_Directory_In_Workspace() {

		return new File(DataPool.workspace.getAbsolutePath() + File.separator
				+ CM2_CONSTANTS.IO.ALGORITHM_SETTINGS.getName() + File.separator
				+ this.getName());
	}

	/**
	 * NOT MEANT TO BE USED BY PLUGIN DEVELOPERS!
	 */
	public void requestChangeOfRenderMode() {

		this.requestRenderModeChange = true;

	}

	/**
	 * Returns the current mode of rendering
	 * 
	 * @return the current render mode
	 */
	public int getCurrentRenderMode() {
		return DataPool.J3DSettings.overallRenderMode;
	}

	/**
	 * Gets the starting time of the algorithm.
	 * Usually this is set to -1, indicating that the algorithm
	 * is not handling the time internally. In this case, the
	 * timing is computed by CmME. The starting time is the time, 
	 * the algorithm starts running. If the buildAndShowGUI()
	 * method is used, the timer is started after GUI is closed.
	 * 
	 * @return long starting time
	 */
	public long getStartingTime() {
		return this.startingTime;
	}
	
	/**
	 * Get the optionally defined starting time.
	 * See getStartingTime() for more info.
	 * Set this to -1 [Default], if the starting time
	 * should not be handled by the algorithm. 
	 * Otherwise the method System.currentTimeMillis()
	 * should be used to set the starting time.
	 * 
	 * @param startingTime
	 */
	public void setStartingTime(long startingTime) {
		this.startingTime = startingTime;
	}
	
	
	/**
	 * Gets the termination time of the algorithm.
	 * Usually this is set to -1, indicating that the algorithm
	 * is not handling the time internally. In this case, the
	 * timing is computed by CmME. The termination time is the time, 
	 * the algorithm is finished completely and is returning to 
	 * the regular CmME mode.
	 * 
	 * @return long starting time
	 */
	public long getTerminationTime() {
		return this.terminationTime;
	}
	
	/**
	 * Get the optionally defined termination time.
	 * See getTerminationTime() for more info.
	 * Set this to -1 [Default], if the termination time
	 * should not be handled by the algorithm.
	 * Otherwise the method System.currentTimeMillis()
	 * should be used to set the termination time.
	 * 
	 * @param terminationTime
	 */
	public void setTerminationTime(long terminationTime) {
		this.terminationTime = terminationTime;
	}

	/**
	 * Returns the Van-der-Waals-radius that is stored by this application under
	 * the specified key. This value is also accessible directly by the atom
	 * instances
	 * 
	 * @param key
	 *            use possible parameter-keys of atoms here
	 * @return the radius stored by the key or -1 if no radius haa been found
	 */
	public float getSystemVanDerWaalsRadius(String key) {

		Vector2f radii = DataPool.AtomValues.radii.get(key);

		if (radii == null)
			return -1;
		else
			return radii.y;

	}

	/**
	 * Returns the covalent radius that is stored by this application under the
	 * specified key. This value is also accessible directly by the atom
	 * instances
	 * 
	 * @param key
	 *            use possible parameter-keys of atoms here
	 * @return the radius stored by the key or -1 if no radius haa been found
	 */
	public float getSystemCovalentRadius(String key) {
		Vector2f radii = DataPool.AtomValues.radii.get(key);

		if (radii == null)
			return -1;
		else
			return radii.x;

	}

	/**
	 * Returns the ID of the microdomain that has been highlighted at startup.
	 * 
	 * @return the ID-number of the selected Microdomain, 0 if the default area
	 *         of the current Layer has been selected, -1 of no area has been
	 *         selected
	 */
	public int getSelectedArea() {
		return this.CurrentMicroDomain;
	}

	/**
	 * Returns the desired distance between atoms that has been set in the
	 * settings dialog of the applycation
	 * 
	 * @return the desired minimum distance
	 */
	public float getMinAtomDistance() {
		return DataPool.J3DSettings.minatomdistance;
	}
	
	/**
	 * Determines wether or not the use of absolute values is selected in the
	 * component view MembraneEditor
	 * 
	 * @return true if absolute values is selected, false otherwise
	 */
	public boolean isComputeAbsoluteValuesSelected() {
		return DataPool.getMembraneModel().getComputeAbsoluteValues();
	}

	/**
	 * Determines wether or not atom collision is activated in the
	 * options-dialog of the MembraneEditor
	 * 
	 * @return true if atom-collision is activated, false otherwise
	 */
	public boolean isAtomCollisionOn() {
		return DataPool.J3DSettings.atomCollisionCheck;
	}
	
	/**
	 * Determines wether or not the atom distance values
	 * for the atom collision is absolute.
	 * 
	 * @return true if absolute-atom-distance is activated, false otherwise
	 */
	public boolean isAtomDistanceAbsolute() {
		return DataPool.J3DSettings.atomDistanceAbsolute;
	}
	
	/**
	 * @deprecated use isInSystemTray() instead
	 * 
	 * Determines, wether the MembraneEditor is currently visible or minimized
	 * into the system-tray
	 * 
	 * @return true if visible, false if inside the system-tray
	 */
	public boolean isOnAir() {
		return !DataPool.J3DSettings.isInSystemTray;
	}
	
	/**
	 * Determines, wether the MembraneEditor is currently visible or minimized
	 * into the system-tray.
	 * 
	 * Use the methods returnFromSystemTray and sendToSystemTray to change this.
	 * 
	 * @return true if in SystemTray, false otherwise 
	 */
	public boolean isInSystemTray() {
		return !DataPool.J3DSettings.isInSystemTray;
	}
	
	/**
	 * Is true if the algorithm can be started in the background.
	 * This should be true [Default] for every algorithm which does not use 
	 * the method getRuntimeControlPanel().
	 * If the method getRuntimeControlPanel() is implemented, it might make
	 * sense to set this to false, especially if no additional starting GUI
	 * is shown.
	 * @return true if starting in background is allowed, false otherwise
	 */
	public boolean isStartInBackgroundAllowed() {
		return this.startInBackgroundAllowed;
	}
	
	/**
	 * This should be true [Default] for every algorithm which does not use 
	 * the method getRuntimeControlPanel().
	 * If the method getRuntimeControlPanel() is implemented, it might make
	 * sense to set this to false, especially if no additional starting GUI
	 * is shown.
	 * Use this method in buildAndShowGUI().
	 */
	public void setStartInBackgroundAllowed(boolean startInBackground) {
		this.startInBackgroundAllowed = startInBackground;
	}
	
	/**
	 * This is true if the user selected to start the algorithm immediately
	 * in background after the calculation was started. This means, that
	 * the regular GUI Window created with buildAndShowGUI() is shown,
	 * but after pressing OK, the CmME instance is minimized to SystemTray.
	 * 
	 * If the programmer of an algorithm wants to prevent this option, then
	 * he can use the method setStartInBackgroundAllowed(false). In this case
	 * the startInBackground option is ignored and may be handled by the 
	 * algorithm programmer.
	 * 
	 * @return true if starting in background is selected, false otherwise
	 */
	public boolean isStartInBackgroundSelected() {
		return this.startInBackgroundSelected;
	}
	
	/**
	 * 
	 * NOT MEANT TO BE USED BY PLUGIN DEVELOPERS!
	 * 
	 */
	public void setStartInBackgroundSelected(boolean startInBackground) {
		this.startInBackgroundSelected = startInBackground;
	}
	
	/**
	 * Set the Random Number Generator to SimpleRandom.
	 */
	public void setSimpleRandom() {
		this.random = new SimpleRandom();
	}

	/**
	 * Set the Random Number Generator to UniqueRandom 
	 * based on the defined seed. 
	 * @param seed long
	 */
	public void setUniqueRandom(long seed) {
		this.random = new UniqueRandom(seed);
	}
	
	
	/**
	 * Restore a membrane from a PDB file with the following options
	 * (Set optional parameters to null if not needed):
	 * 
	 * @param absolutePathOfPDB mandatory: the absolute path of the PDB 
	 * @param absolutePathToCm2 optional: the absolute path of the Cm2 against which the PDB has to be parsed
	 * @param overwriteHints optional: the transformation hints containing information about how to transform the membrane
	 * @param checkMembraneState mandatory: true if a dialog should appear if there were changes on the membrane, false if the process should be started immediatelyE.
	 */
	public boolean restoreMembraneFromPDB(String absolutePathOfPDB, String absolutePathToCm2, CoordinateTransformationHints overwriteHints, boolean checkMembraneState) {
		return basicOperations.restoreMembraneFromPDB(absolutePathOfPDB, absolutePathToCm2, overwriteHints, checkMembraneState);
	}
	
	/**
	 * Add a PDB file to the local database.
	 * 
	 * @param absolutePathOfPDB the file of the PDB
	 * @param pdbType 0 for protein, 1 for lipid
	 */
	public void addEntryToDB(File absolutePathOfPDB, int pdbType) {
		if (this.databaseOperations != null)
			this.databaseOperations.addEntryToDB(absolutePathOfPDB, pdbType);
	}

	@Deprecated
	/**
	 * Please use getOriginalConstraintLengths(Molecule molecule) instead.
	 */
	public float[] getOriginalConstraintLenghts(Molecule molecule) {
		return this.getOriginalConstraintLengths(molecule);
	}
	
	/**
	 * NOT MEANT TO BE USED BY PLUGIN DEVELOPERS!
	 */
	public float[] getOriginalConstraintLengths(Molecule molecule) {

		MoleculeProfile profile = this.profiles.get(molecule
				.getOriginalMolecule().getPDBcode());

		try {
			if (profile == null) {
				String code = molecule.getOriginalMolecule().getPDBcode();
				if (code == null || code.length() < 1)
					code = Cm2_Toolkit.getSimpleName(molecule
							.getOriginalMolecule().getPDBFile());

				profile = Molecule_Profile_IO.readProfile(molecule
						.getOriginalMolecule(), molecule.getOriginalMolecule()
						.getCurrentProfileName(), code);

				this.profiles.put(molecule.getOriginalMolecule().getPDBcode(),
						profile);
			}

			Vector<Constraint_3D> conts = molecule.getOriginalMolecule().getActiveConstraints();

			Vector<Object> v = Cm2_Toolkit.fromArrayToVector(molecule
					.getOriginalMolecule().allAtomsSorted);

			float[] ret = new float[conts.size()];
			for (int i = 0; i < conts.size(); i++) {

				Atom_3D a = conts.get(i).atomA;
				Atom_3D b = conts.get(i).atomB;

				int indexA = v.indexOf(a);
				int indexB = v.indexOf(b);

				Point3f posA = null;
				Point3f posB = null;

				if (indexA == -1) {
					Particle p = (Particle) a;
					if (p.includes(a)) {

						MyBounds mb = new MyBounds(p.includedAtoms);
						posA = mb.center;
					}

				} else {
					posA = profile.positions[indexA];
				}

				if (posA == null)
					return null;

				if (indexB == -1) {
					Particle p = (Particle) b;
					if (p.includes(b)) {

						MyBounds mb = new MyBounds(p.includedAtoms);
						posB = mb.center;
					}

				} else {
					posB = profile.positions[indexB];
				}

				if (posB == null)
					return null;

				ret[i] = posA.distance(posB);

			}

			return ret;

		} catch (Exception e) {
			return null;

		}

	}

}
